@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.248 2004/03/23 06:48:31 guy Exp $
+ * $Id: capture.c,v 1.254 2004/06/30 06:58:56 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,6 +22,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+/* With MSVC and a libethereal.dll this file needs to import some variables 
+   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */
+#define _NEED_VAR_IMPORT_
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -220,7 +224,6 @@ typedef struct _loop_data {
 #define O_BINARY	0
 #endif
 
-
 static gboolean sync_pipe_do_capture(gboolean is_tempfile);
 static gboolean sync_pipe_input_cb(gint source, gpointer user_data);
 static void sync_pipe_wait_for_child(gboolean);
@@ -257,6 +260,7 @@ do_capture(const char *save_file)
   char tmpname[128+1];
   gboolean is_tempfile;
   gchar *capfile_name;
+  gboolean ret;
 
   if (save_file != NULL) {
     /* If the Sync option is set, we return to the caller while the capture
@@ -299,16 +303,21 @@ do_capture(const char *save_file)
   cfile.save_file = capfile_name;
   /* cfile.save_file is "g_free"ed below, which is equivalent to
      "g_free(capfile_name)". */
+  fork_child = -1;
 
   if (capture_opts.sync_mode) {	
     /* sync mode: do the capture in a child process */
-    return sync_pipe_do_capture(is_tempfile);
+    ret = sync_pipe_do_capture(is_tempfile);
     /* capture is still running */
+    set_main_window_name("(Live Capture in Progress) - Ethereal");
   } else {
     /* normal mode: do the capture synchronously */
-    return normal_do_capture(is_tempfile);
+    set_main_window_name("(Live Capture in Progress) - Ethereal");
+    ret = normal_do_capture(is_tempfile);
     /* capture is finished here */
   }
+
+  return ret;
 }
 
 
@@ -427,6 +436,10 @@ sync_pipe_do_capture(gboolean is_tempfile) {
       argv = sync_pipe_add_arg(argv, &argc, sautostop_duration);
     }
 
+    if (!capture_opts.show_info) {
+      argv = sync_pipe_add_arg(argv, &argc, "-H");
+    }
+
     if (!capture_opts.promisc_mode)
       argv = sync_pipe_add_arg(argv, &argc, "-p");
 
@@ -573,7 +586,7 @@ sync_pipe_do_capture(gboolean is_tempfile) {
 	unlink(cfile.save_file);
 	g_free(cfile.save_file);
 	cfile.save_file = NULL;
-	simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			"Capture child process sent us a bad message");
 	return FALSE;
       }
@@ -584,22 +597,22 @@ sync_pipe_do_capture(gboolean is_tempfile) {
 	 what the problem was. */
       if (byte_count == 0) {
 	/* Zero-length message? */
-	simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		"Capture child process failed, but its error message was empty.");
       } else {
 	msg = g_malloc(byte_count + 1);
 	if (msg == NULL) {
-	  simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+	  simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		"Capture child process failed, but its error message was too big.");
 	} else {
 	  i = read(sync_pipe[READ], msg, byte_count);
 	  msg[byte_count] = '\0';
 	  if (i < 0) {
-	    simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		  "Capture child process failed: Error %s reading its error message.",
 		  strerror(errno));
 	  } else if (i == 0) {
-	    simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		  "Capture child process failed: EOF reading its error message.");
 	    sync_pipe_wait_for_child(FALSE);
 	  } else
@@ -787,7 +800,7 @@ sync_pipe_wait_for_child(gboolean always_report)
      in the dialog box?
      XXX - set "fork_child" to -1 if we find it exited? */
   if (_cwait(&wstatus, fork_child, _WAIT_CHILD) == -1) {
-    simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		"Child capture process stopped unexpectedly");
   }
 #else
@@ -796,25 +809,25 @@ sync_pipe_wait_for_child(gboolean always_report)
       /* The child exited; display its exit status, if it's not zero,
          and even if it's zero if "always_report" is true. */
       if (always_report || WEXITSTATUS(wstatus) != 0) {
-        simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		      "Child capture process exited: exit status %d",
 		      WEXITSTATUS(wstatus));
       }
     } else if (WIFSTOPPED(wstatus)) {
       /* It stopped, rather than exiting.  "Should not happen." */
-      simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Child capture process stopped: %s",
 		    sync_pipe_signame(WSTOPSIG(wstatus)));
     } else if (WIFSIGNALED(wstatus)) {
       /* It died with a signal. */
-      simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Child capture process died: %s%s",
 		    sync_pipe_signame(WTERMSIG(wstatus)),
 		    WCOREDUMP(wstatus) ? " - core dumped" : "");
     } else {
       /* What?  It had to either have exited, or stopped, or died with
          a signal; what happened here? */
-      simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+      simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Child capture process died: wait status %#o", wstatus);
     }
   }
@@ -1431,6 +1444,7 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
 #ifdef MUST_DO_SELECT
   int         pcap_fd = 0;
 #endif
+  gboolean    show_info = capture_opts.show_info || !capture_opts.sync_mode;
 
   /* Initialize Windows Socket if we are in a WIN32 OS
      This needs to be done before querying the interface for network/netmask */
@@ -1754,9 +1768,11 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
   }
 
   /* start capture info dialog */
-  capture_ui.callback_data  = &ld;
-  capture_ui.counts         = &ld.counts;
-  capture_info_create(&capture_ui);
+  if(show_info) {
+      capture_ui.callback_data  = &ld;
+      capture_ui.counts         = &ld.counts;
+      capture_info_create(&capture_ui, cfile.iface);
+  }
 
   start_time = time(NULL);
   upd_time = time(NULL);
@@ -1825,7 +1841,7 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
       }
     }
     else
-#endif
+#endif /* _WIN32 */
     {
 #ifdef MUST_DO_SELECT
       /*
@@ -1885,7 +1901,7 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
         ld.pcap_err = TRUE;
         ld.go = FALSE;
       }
-#endif
+#endif /* MUST_DO_SELECT */
     }
 
     if (inpkts > 0) {
@@ -1933,10 +1949,12 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
 
       	/* Let the parent process know. */
       /* calculate and display running time */
-      cur_time -= start_time;
-      capture_ui.running_time   = cur_time;
-      capture_ui.new_packets    = ld.sync_packets;
-      capture_info_update(&capture_ui);
+      if(show_info) {
+          cur_time -= start_time;
+          capture_ui.running_time   = cur_time;
+          capture_ui.new_packets    = ld.sync_packets;
+          capture_info_update(&capture_ui);
+      }
 
       if (ld.sync_packets) {
         /* do sync here */
@@ -2071,7 +2089,9 @@ capture(gboolean *stats_known, struct pcap_stat *stats)
   WSACleanup();
 #endif
 
-  capture_info_destroy(&capture_ui);
+  if(show_info) {
+    capture_info_destroy(&capture_ui);
+  }
 
   return write_ok;
 
