@@ -1,5 +1,5 @@
 /*
- * $Id: sttype-test.h,v 1.3 2002/08/28 20:40:56 jmayer Exp $
+ * $Id: sttype-test.h,v 1.6 2004/02/27 12:00:31 obiot Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,7 +35,10 @@ typedef enum {
 	TEST_OP_GT,
 	TEST_OP_GE,
 	TEST_OP_LT,
-	TEST_OP_LE
+	TEST_OP_LE,
+	TEST_OP_BITWISE_AND,
+	TEST_OP_CONTAINS,
+	TEST_OP_MATCHES
 } test_op_t;
 
 void
