@@ -1,7 +1,7 @@
 /* ftypes.h
  * Definitions for field types
  *
- * $Id: ftypes.h,v 1.31 2004/02/27 12:00:32 obiot Exp $
+ * $Id: ftypes.h,v 1.35 2004/07/04 02:29:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -63,6 +63,11 @@ enum ftenum {
 	FT_NUM_TYPES /* last item number plus one */
 };
 
+#define IS_FT_INT(ft)    ((ft)==FT_INT8||(ft)==FT_INT16||(ft)==FT_INT24||(ft)==FT_INT32)
+#define IS_FT_UINT(ft)   ((ft)==FT_UINT8||(ft)==FT_UINT16||(ft)==FT_UINT24||(ft)==FT_UINT32||(ft)==FT_FRAMENUM)
+#define IS_FT_TIME(ft)   ((ft)==FT_ABSOLUTE_TIME||(ft)==FT_RELATIVE_TIME)
+#define IS_FT_STRING(ft) ((ft)==FT_STRING||(ft)==FT_STRINGZ)
+
 typedef enum ftenum ftenum_t;
 typedef struct _ftype_t ftype_t;
 
@@ -226,8 +231,15 @@ void
 fvalue_init(fvalue_t *fv, ftenum_t ftype);
 
 
-/* Free all memory used by an fvalue_t */
-extern fvalue_t *fvalue_free_list;
+/* Define type needed for the fvalue_t free list. */
+SLAB_ITEM_TYPE_DEFINE(fvalue_t)
+
+/* Free all memory used by an fvalue_t. With MSVC and a 
+ * libethereal.dll, we need a special declaration.
+ */
+ETH_VAR_IMPORT SLAB_FREE_LIST_DECLARE(fvalue_t)
+
+
 #define FVALUE_CLEANUP(fv)					\
 	{							\
 		register FvalueFreeFunc	free_value;		\
@@ -240,7 +252,7 @@ extern fvalue_t *fvalue_free_list;
 #define FVALUE_FREE(fv)						\
 	{							\
 		FVALUE_CLEANUP(fv)				\
-		SLAB_FREE(fv, fvalue_free_list);		\
+		SLAB_FREE(fv, fvalue_t);			\
 	}
 
 
