@@ -1,7 +1,7 @@
 /* proto.c
  * Routines for protocol tree
  *
- * $Id: proto.c,v 1.131 2004/03/25 23:55:21 guy Exp $
+ * $Id: proto.c,v 1.135 2004/07/05 16:42:19 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -153,34 +153,37 @@ static GMemChunk *gmc_hfinfo = NULL;
 
 /* Contains information about a field when a dissector calls
  * proto_tree_add_item.  */
-static field_info *field_info_free_list=NULL;
+SLAB_ITEM_TYPE_DEFINE(field_info)
+static SLAB_FREE_LIST_DEFINE(field_info)
 static field_info *field_info_tmp=NULL;
 #define FIELD_INFO_NEW(fi)					\
-	SLAB_ALLOC(fi, field_info_free_list)
+	SLAB_ALLOC(fi, field_info)
 #define FIELD_INFO_FREE(fi)					\
-	SLAB_FREE(fi, field_info_free_list)
+	SLAB_FREE(fi, field_info)
 
 
 
 /* Contains the space for proto_nodes. */
-static proto_node *proto_node_free_list=NULL;
+SLAB_ITEM_TYPE_DEFINE(proto_node)
+static SLAB_FREE_LIST_DEFINE(proto_node)
 #define PROTO_NODE_NEW(node)				\
-	SLAB_ALLOC(node, proto_node_free_list)		\
+	SLAB_ALLOC(node, proto_node)			\
 	node->first_child = NULL;			\
 	node->last_child = NULL;			\
 	node->next = NULL;
 
 #define PROTO_NODE_FREE(node)				\
-	SLAB_FREE(node, proto_node_free_list)
+	SLAB_FREE(node, proto_node)
 
 
 
 /* String space for protocol and field items for the GUI */
-static item_label_t *item_label_free_list = NULL;
+SLAB_ITEM_TYPE_DEFINE(item_label_t)
+static SLAB_FREE_LIST_DEFINE(item_label_t)
 #define ITEM_LABEL_NEW(il)				\
-	SLAB_ALLOC(il, item_label_free_list)
+	SLAB_ALLOC(il, item_label_t)
 #define ITEM_LABEL_FREE(il)				\
-	SLAB_FREE(il, item_label_free_list)
+	SLAB_FREE(il, item_label_t)
 
 
 
@@ -658,7 +661,7 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 		 * good thing we saved it, now we can reverse the
 		 * memory leak and reclaim it.
 		 */
-		SLAB_FREE(field_info_tmp, field_info_free_list);
+		SLAB_FREE(field_info_tmp, field_info);
 	}
 	/* we might throw an exception, keep track of this one
 	 * across the "dangerous" section below.
@@ -858,14 +861,12 @@ proto_tree_add_item_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb,
     gint start, gint length, gboolean little_endian)
 {
 	proto_item	*pi;
-	field_info	*fi;
 
 	pi = proto_tree_add_item(tree, hfindex, tvb, start, length, little_endian);
 	if (pi == NULL)
 		return(NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -961,14 +962,12 @@ proto_tree_add_bytes_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint s
 		gint length, const guint8 *start_ptr)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_bytes(tree, hfindex, tvb, start, length, start_ptr);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1037,14 +1036,12 @@ proto_tree_add_time_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 		nstime_t *value_ptr)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_time(tree, hfindex, tvb, start, length, value_ptr);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1100,14 +1097,12 @@ proto_tree_add_ipxnet_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 		guint32 value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_ipxnet(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1163,14 +1158,12 @@ proto_tree_add_ipv4_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 		guint32 value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_ipv4(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1226,14 +1219,12 @@ proto_tree_add_ipv6_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 		const guint8* value_ptr)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_ipv6(tree, hfindex, tvb, start, length, value_ptr);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1318,14 +1309,12 @@ proto_tree_add_string_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 		gint length, const char* value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_string(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1421,14 +1410,12 @@ proto_tree_add_ether_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint s
 		const guint8* value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_ether(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1490,14 +1477,12 @@ proto_tree_add_boolean_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 		guint32 value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_boolean(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1553,14 +1538,12 @@ proto_tree_add_float_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint s
 		float value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_float(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1616,14 +1599,12 @@ proto_tree_add_double_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint
 		double value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_double(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1689,14 +1670,12 @@ proto_tree_add_uint_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint st
 		guint32 value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_uint(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -1776,14 +1755,12 @@ proto_tree_add_int_hidden(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint sta
 		gint32 value)
 {
 	proto_item		*pi;
-	field_info 		*fi;
 
 	pi = proto_tree_add_int(tree, hfindex, tvb, start, length, value);
 	if (pi == NULL)
 		return (NULL);
 
-	fi = PITEM_FINFO(pi);
-	fi->visible = FALSE;
+	PROTO_ITEM_SET_HIDDEN(pi);
 
 	return pi;
 }
@@ -2003,7 +1980,10 @@ alloc_field_info(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start,
 	fi->start+=(tvb)?TVB_RAW_OFFSET(tvb):0;
 	fi->length = *length;
 	fi->tree_type = -1;
-	fi->visible = PTREE_DATA(tree)->visible;
+	fi->flags = 0;
+    if(!PTREE_DATA(tree)->visible) {
+        FI_SET_FLAG(fi, FI_HIDDEN);
+    }
 	fvalue_init(&fi->value, fi->hfinfo->type);
 	fi->rep = NULL;
 
@@ -2021,7 +2001,7 @@ proto_tree_set_representation(proto_item *pi, const char *format, va_list ap)
 	int					ret;	/*tmp return value */
 	field_info *fi = PITEM_FINFO(pi);
 
-	if (fi->visible) {
+	if (!PROTO_ITEM_IS_HIDDEN(pi)) {
 		ITEM_LABEL_NEW(fi->rep);
 		ret = vsnprintf(fi->rep->representation, ITEM_LABEL_LENGTH, format, ap);
 		if ((ret == -1) || (ret >= ITEM_LABEL_LENGTH))
@@ -2066,7 +2046,7 @@ proto_item_append_text(proto_item *pi, const char *format, ...)
 
 	fi = PITEM_FINFO(pi);
 
-	if (fi->visible) {
+    if (!PROTO_ITEM_IS_HIDDEN(pi)) {
 		va_start(ap, format);
 
 		/*
@@ -3426,7 +3406,7 @@ check_for_offset(proto_node *node, gpointer data)
 	offset_search_t		*offsearch = data;
 
 	/* !fi == the top most container node which holds nothing */
-	if (fi && fi->visible && fi->ds_tvb && offsearch->tvb == fi->ds_tvb) {
+	if (fi && !PROTO_ITEM_IS_HIDDEN(node) && fi->ds_tvb && offsearch->tvb == fi->ds_tvb) {
 		if (offsearch->offset >= (guint) fi->start &&
 				offsearch->offset < (guint) (fi->start + fi->length)) {
 
@@ -3911,8 +3891,8 @@ proto_construct_dfilter_string(field_info *finfo, epan_dissect_t *edt)
 			/*
 			 * Don't go past the end of that tvbuff.
 			 */
-			if ((guint)length > tvb_length(finfo->ds_tvb))
-				length = tvb_length(finfo->ds_tvb);
+			if ((gint)length > tvb_length_remaining(finfo->ds_tvb, finfo->start))
+				length = tvb_length_remaining(finfo->ds_tvb, finfo->start);
 			if (length <= 0)
 				return NULL;
 			
@@ -3923,7 +3903,7 @@ proto_construct_dfilter_string(field_info *finfo, epan_dissect_t *edt)
 			sprintf(ptr, "frame[%d:%d] == ", finfo->start, length);
 			ptr = buf+strlen(buf);
 
-			for (i=0;i<length; i++) {
+			for (i=0;i<length-finfo->start; i++) {
 				c = tvb_get_guint8(finfo->ds_tvb, start);
 				start++;
 				if (i == 0 ) {
