@@ -1,7 +1,7 @@
 /* file.c
  * File I/O routines
  *
- * $Id: file.c,v 1.370 2004/03/08 23:45:24 guy Exp $
+ * $Id: file.c,v 1.386 2004/06/30 06:58:56 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,6 +22,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+/* With MSVC and a libethereal.dll this file needs to import some variables 
+   in a special way. Therefore _NEED_VAR_IMPORT_ is defined. */
+#define _NEED_VAR_IMPORT_
+
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -84,6 +88,11 @@
 #include "tap_dfilter_dlg.h"
 #include "packet-data.h"
 
+/* Win32 needs the O_BINARY flag for open() */
+#ifndef O_BINARY
+#define O_BINARY	0
+#endif
+
 #ifdef HAVE_LIBPCAP
 gboolean auto_scroll_live;
 #endif
@@ -131,15 +140,6 @@ static   gboolean copy_binary_file(char *from_filename, char *to_filename);
    XXX - is this the right number? */
 #define	FRAME_DATA_CHUNK_SIZE	1024
 
-typedef struct {
-	int		level;
-	FILE		*fh;
-	GSList		*src_list;
-	gboolean	print_all_levels;
-	gboolean	print_hex_for_data;
-	char_enc	encoding;
-	gint		format;		/* text or PostScript */
-} print_data;
 
 int
 cf_open(char *fname, gboolean is_tempfile, capture_file *cf)
@@ -288,6 +288,8 @@ cf_close(capture_file *cf)
   set_menus_for_capture_in_progress(FALSE);
   set_menus_for_selected_tree_row(cf);
 
+  reset_tap_listeners();
+
   /* We have no file open. */
   cf->state = FILE_CLOSED;
 }
@@ -381,10 +383,6 @@ cf_read(capture_file *cf)
      bump that value by this amount. */
   progbar_quantum = cf->f_len/N_PROGBAR_UPDATES;
 
-#ifndef O_BINARY
-#define O_BINARY 	0
-#endif
-
   packet_list_freeze();
 
   stop_flag = FALSE;
@@ -529,6 +527,7 @@ int
 cf_start_tail(char *fname, gboolean is_tempfile, capture_file *cf)
 {
   int     err;
+  gchar *capture_msg;
 
   err = cf_open(fname, is_tempfile, cf);
   if (err == 0) {
@@ -540,7 +539,11 @@ cf_start_tail(char *fname, gboolean is_tempfile, capture_file *cf)
        packets (yes, I know, we don't have any *yet*). */
     set_menus_for_captured_packets(TRUE);
 
-    statusbar_push_file_msg(" <live capture in progress>");
+    capture_msg = g_strdup_printf(" %s: <live capture in progress>", cf->iface);
+
+    statusbar_push_file_msg(capture_msg);
+
+    g_free(capture_msg);
   }
   return err;
 }
@@ -983,13 +986,18 @@ filter_packets(capture_file *cf, gchar *dftext, gboolean force)
     dftext = g_strdup(dftext);
     if (!dfilter_compile(dftext, &dfcode)) {
       /* The attempt failed; report an error. */
+      gchar *safe_dftext = simple_dialog_format_message(dftext);
+      gchar *safe_dfilter_error_msg = simple_dialog_format_message(
+	  dfilter_error_msg);
       simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, 
           "%s%s%s\n"
           "\n"
-          "The display filter \"%s\" is not a valid display filter.\n"
+          "The following display filter is not a valid display filter:\n%s\n"
           "See the help for a description of the display filter syntax.",
-          simple_dialog_primary_start(), dfilter_error_msg,
-          simple_dialog_primary_end(), dftext);
+          simple_dialog_primary_start(), safe_dfilter_error_msg,
+          simple_dialog_primary_end(), safe_dftext);
+      g_free(safe_dfilter_error_msg);
+      g_free(safe_dftext);
       g_free(dftext);
       return FALSE;
     }
@@ -1414,6 +1422,7 @@ process_specified_packets(capture_file *cf, packet_range_t *range,
       ret = PSP_FAILED;
       break;
     }
+    /* Process the packet */
     if (!callback(cf, fdata, &pseudo_header, pd, callback_args)) {
       /* Callback failed.  We assume it reported the error appropriately. */
       ret = PSP_FAILED;
@@ -1483,6 +1492,10 @@ retap_packets(capture_file *cf)
 typedef struct {
   print_args_t *print_args;
   FILE         *print_fh;
+  gboolean      print_header_line;
+  char         *header_line_buf;
+  int           header_line_buf_len;
+  gboolean      print_formfeed;
   gboolean      print_separator;
   char         *line_buf;
   int           line_buf_len;
@@ -1501,13 +1514,30 @@ print_packet(capture_file *cf, frame_data *fdata,
   int             line_len;
   int             column_len;
   int             cp_off;
+  gboolean        proto_tree_needed;
 
-  if (args->print_args->print_summary) {
-    /* Fill in the column information, but don't bother creating
-       the logical protocol tree. */
-    edt = epan_dissect_new(FALSE, FALSE);
-    epan_dissect_run(edt, pseudo_header, pd, fdata, &cf->cinfo);
-    epan_dissect_fill_in_columns(edt);
+  proto_tree_needed = 
+      args->print_args->print_dissections != print_dissections_none || args->print_args->print_hex;
+
+  /* Fill in the column information, but don't bother creating
+     the logical protocol tree. */
+  edt = epan_dissect_new(proto_tree_needed, proto_tree_needed);
+  epan_dissect_run(edt, pseudo_header, pd, fdata, &cf->cinfo);
+  epan_dissect_fill_in_columns(edt);
+
+  if (args->print_formfeed) {
+    print_formfeed(args->print_fh, args->print_args->format);
+  } else {
+      if (args->print_separator)
+        print_line(args->print_fh, 0, args->print_args->format, "");
+  }
+
+  if (args->print_args->print_summary || args->print_args->format == PR_FMT_PS) {
+    if (args->print_header_line) {
+      print_line(args->print_fh, 0, args->print_args->format,
+                 args->header_line_buf);
+      args->print_header_line = FALSE;	/* we might not need to print any more */
+    }
     cp = &args->line_buf[0];
     line_len = 0;
     for (i = 0; i < cf->cinfo.num_cols; i++) {
@@ -1536,30 +1566,48 @@ print_packet(capture_file *cf, frame_data *fdata,
         *cp++ = ' ';
     }
     *cp = '\0';
-    print_line(args->print_fh, 0, args->print_args->format, args->line_buf);
-  } else {
-    if (args->print_separator)
-      print_line(args->print_fh, 0, args->print_args->format, "");
 
-    /* Create the logical protocol tree, complete with the display
-       representation of the items; we don't need the columns here,
-       however. */
-    edt = epan_dissect_new(TRUE, TRUE);
-    epan_dissect_run(edt, pseudo_header, pd, fdata, NULL);
+    print_packet_header(args->print_fh, args->print_args->format, fdata->num, args->line_buf);
+
+    if (args->print_args->print_summary) {
+        print_line(args->print_fh, 0, args->print_args->format, args->line_buf);
+    }
+  } /* if (print_summary) */
+  
+  if (args->print_args->print_dissections != print_dissections_none) {
+    if (args->print_args->print_summary) {
+      /* Separate the summary line from the tree with a blank line. */
+      print_line(args->print_fh, 0, args->print_args->format, "");
+    }
 
     /* Print the information in that tree. */
     proto_tree_print(args->print_args, edt, args->print_fh);
 
-    if (args->print_args->print_hex) {
-      /* Print the full packet data as hex. */
-      print_hex_data(args->print_fh, args->print_args->format, edt);
-    }
-
-    /* Print a blank line if we print anything after this. */
+    /* Print a blank line if we print anything after this (aka more than one packet). */
     args->print_separator = TRUE;
+
+    /* Print a header line if we print any more packet summaries */
+    args->print_header_line = TRUE;
+  }
+
+  if (args->print_args->print_hex) {
+    /* Print the full packet data as hex. */
+    print_hex_data(args->print_fh, args->print_args->format, edt);
+
+    /* Print a blank line if we print anything after this (aka more than one packet). */
+    args->print_separator = TRUE;
+
+    /* Print a header line if we print any more packet summaries */
+    args->print_header_line = TRUE;
   } /* if (print_summary) */
+
   epan_dissect_free(edt);
 
+  /* do we want to have a formfeed between each packet from now on? */
+  if(args->print_args->print_formfeed) {
+    args->print_formfeed = TRUE;
+  }
+
   return !ferror(args->print_fh);
 }
 
@@ -1575,32 +1623,41 @@ print_packets(capture_file *cf, print_args_t *print_args)
   int         line_len;
   psp_return_t ret;
 
-  callback_args.print_fh = open_print_dest(print_args->to_file,
-                                           print_args->dest);
+  if(print_args->to_file) {
+      callback_args.print_fh = open_print_dest(print_args->to_file,
+                                               print_args->file);
+  } else {
+      callback_args.print_fh = open_print_dest(print_args->to_file,
+                                               print_args->cmd);
+  }
   if (callback_args.print_fh == NULL)
     return PP_OPEN_ERROR;	/* attempt to open destination failed */
 
-  print_preamble(callback_args.print_fh, print_args->format);
+  print_preamble(callback_args.print_fh, print_args->format, cf->filename);
   if (ferror(callback_args.print_fh)) {
     close_print_dest(print_args->to_file, callback_args.print_fh);
     return PP_WRITE_ERROR;
   }
 
   callback_args.print_args = print_args;
+  callback_args.print_header_line = TRUE;
+  callback_args.header_line_buf = NULL;
+  callback_args.header_line_buf_len = 256;
+  callback_args.print_formfeed = FALSE;
   callback_args.print_separator = FALSE;
   callback_args.line_buf = NULL;
   callback_args.line_buf_len = 256;
   callback_args.col_widths = NULL;
-  if (print_args->print_summary) {
-    /* We're printing packet summaries.  Allocate the line buffer at
-       its initial length. */
-    callback_args.line_buf = g_malloc(callback_args.line_buf_len + 1);
+  if (print_args->print_summary || print_args->format == PR_FMT_PS) {
+    /* We're printing packet summaries.  Allocate the header line buffer
+       and get the column widths. */
+    callback_args.header_line_buf = g_malloc(callback_args.header_line_buf_len + 1);
 
     /* Find the widths for each of the columns - maximum of the
-       width of the title and the width of the data - and print
-       the column titles. */
+       width of the title and the width of the data - and construct
+       a buffer with a line containing the column titles. */
     callback_args.col_widths = (gint *) g_malloc(sizeof(gint) * cf->cinfo.num_cols);
-    cp = &callback_args.line_buf[0];
+    cp = &callback_args.header_line_buf[0];
     line_len = 0;
     for (i = 0; i < cf->cinfo.num_cols; i++) {
       /* Don't pad the last column. */
@@ -1621,26 +1678,29 @@ print_packets(capture_file *cf, print_args_t *print_args)
       /* Make sure there's room in the line buffer for the column; if not,
          double its length. */
       line_len += column_len + 1;	/* "+1" for space */
-      if (line_len > callback_args.line_buf_len) {
-        cp_off = cp - callback_args.line_buf;
-        callback_args.line_buf_len = 2 * line_len;
-        callback_args.line_buf = g_realloc(callback_args.line_buf,
-                                           callback_args.line_buf_len + 1);
-        cp = callback_args.line_buf + cp_off;
+      if (line_len > callback_args.header_line_buf_len) {
+        cp_off = cp - callback_args.header_line_buf;
+        callback_args.header_line_buf_len = 2 * line_len;
+        callback_args.header_line_buf = g_realloc(callback_args.header_line_buf,
+                                                  callback_args.header_line_buf_len + 1);
+        cp = callback_args.header_line_buf + cp_off;
       }
 
       /* Right-justify the packet number column. */
-      if (cf->cinfo.col_fmt[i] == COL_NUMBER)
+/*      if (cf->cinfo.col_fmt[i] == COL_NUMBER)
         sprintf(cp, "%*s", callback_args.col_widths[i], cf->cinfo.col_title[i]);
-      else
+      else*/
         sprintf(cp, "%-*s", callback_args.col_widths[i], cf->cinfo.col_title[i]);
       cp += column_len;
       if (i != cf->cinfo.num_cols - 1)
         *cp++ = ' ';
     }
     *cp = '\0';
-    print_line(callback_args.print_fh, 0, print_args->format,
-               callback_args.line_buf);
+
+    /* Now start out the main line buffer with the same length as the
+       header line buffer. */
+    callback_args.line_buf_len = callback_args.header_line_buf_len;
+    callback_args.line_buf = g_malloc(callback_args.line_buf_len + 1);
   } /* if (print_summary) */
 
   /* Iterate through the list of packets, printing the packets we were
@@ -1649,10 +1709,12 @@ print_packets(capture_file *cf, print_args_t *print_args)
                                   "selected packets", print_packet,
                                   &callback_args);
 
-  if (callback_args.col_widths != NULL)
-    g_free(callback_args.col_widths);
+  if (callback_args.header_line_buf != NULL)
+    g_free(callback_args.header_line_buf);
   if (callback_args.line_buf != NULL)
     g_free(callback_args.line_buf);
+  if (callback_args.col_widths != NULL)
+    g_free(callback_args.col_widths);
 
   switch (ret) {
 
@@ -1910,7 +1972,7 @@ match_subtree_text(proto_node *node, gpointer data)
   }
 
   /* Don't match invisible entries. */
-  if (!fi->visible)
+  if (PROTO_ITEM_IS_HIDDEN(node))
     return;
 
   /* was a free format label produced? */
@@ -2014,7 +2076,7 @@ find_packet_data(capture_file *cf, const guint8 *string, size_t string_size)
   info.data_len = string_size;
 
   /* String or hex search? */
-  if (cf->ascii) {
+  if (cf->string) {
     /* String search - what type of string? */
     switch (cf->scs_type) {
 
