@@ -1,7 +1,7 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c,v 1.121 2004/03/13 22:49:30 ulfl Exp $
+ * $Id: capture_dlg.c,v 1.137 2004/06/30 18:24:56 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -50,6 +50,7 @@
 #include "compat_macros.h"
 #include "file_dlg.h"
 #include "help_dlg.h"
+#include "gtkglobals.h"
 
 #ifdef _WIN32
 #include "capture-wpcap.h"
@@ -82,6 +83,7 @@
 #define E_CAP_STOP_FILES_LB_KEY     "cap_stop_files_lb"
 #define E_CAP_SYNC_KEY              "cap_sync"
 #define E_CAP_AUTO_SCROLL_KEY       "cap_auto_scroll"
+#define E_CAP_HIDE_INFO_KEY         "cap_hide_info"
 #define E_CAP_STOP_PACKETS_CB_KEY   "cap_stop_packets_cb"
 #define E_CAP_STOP_PACKETS_SB_KEY   "cap_stop_packets_sb"
 #define E_CAP_STOP_PACKETS_LB_KEY   "cap_stop_packets_lb"
@@ -97,33 +99,18 @@
 
 #define E_CAP_OM_LT_VALUE_KEY       "cap_om_lt_value"
 
-#define E_FS_CALLER_PTR_KEY         "fs_caller_ptr"
-#define E_FILE_SEL_DIALOG_PTR_KEY   "file_sel_dialog_ptr"
-
 static void
-capture_prep_file_cb(GtkWidget *w, gpointer te);
+capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te);
 
 static void
 select_link_type_cb(GtkWidget *w, gpointer data);
 
-static void
-cap_prep_fs_ok_cb(GtkWidget *w, gpointer data);
-
-static void
-cap_prep_fs_cancel_cb(GtkWidget *w, gpointer data);
-
-static void
-cap_prep_fs_destroy_cb(GtkWidget *win, GtkWidget* file_te);
-
 static void
 capture_prep_adjust_sensitivity(GtkWidget *tb, gpointer parent_w);
 
 static void
 capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w);
 
-static void
-capture_prep_close_cb(GtkWidget *close_bt, gpointer parent_w);
-
 static void
 capture_prep_destroy_cb(GtkWidget *win, gpointer user_data);
 
@@ -168,13 +155,35 @@ get_if_name(char *if_text)
    * return the entire string; otherwise, skip the colon and any blanks
    * after it, and return that string.
    */
-   if_name = strrchr(if_text, ':');
-   if (if_name == NULL) {
-     if_name = if_text;
-   } else {
-     if_name++;
-     while (*if_name == ' ')
-       if_name++;
+   if_name = if_text + strlen(if_text);
+   for (;;) {
+     if (if_name == if_text) {
+       /* We're at the beginning of the string; return it. */
+       break;
+     }
+     if_name--;
+     if (*if_name == ':') {
+       /*
+        * We've found a colon.
+        * Unfortunately, a colon is used in the string "rpcap://",
+        * which is used in case of a remote capture.
+        * So we'll check to make sure the colon isn't followed by "//";
+        * it'll be followed by a blank if it separates the description
+        * and the interface name.  (We don't wire in "rpcap", in case we
+        * support other protocols in the same syntax.)
+        */
+       if (strncmp(if_name, "://", 3) != 0) {
+         /*
+          * OK, we've found a colon not followed by "//".  Skip blanks
+          * following it.
+          */
+         if_name++;
+         while (*if_name == ' ')
+           if_name++;
+         break;
+       }
+     }
+     /* Keep looking for a colon not followed by "//". */
    }
 #else
   /*
@@ -303,7 +312,7 @@ static char *time_unit_name[MAX_TIME_UNITS] = {
 	"day(s)",
 };
 
-GtkWidget *time_unit_option_menu_new(void) {
+static GtkWidget *time_unit_option_menu_new(guint32 value) {
     GtkWidget *unit_om, *menu, *menu_item;
     int i;
 
@@ -314,14 +323,55 @@ GtkWidget *time_unit_option_menu_new(void) {
 		OBJECT_SET_DATA(menu_item, "time_unit", GINT_TO_POINTER(i));
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 	}
-	gtk_menu_set_active(GTK_MENU(menu), TIME_UNIT_MINUTE);
+
+    /* the selected menu item can't be changed, once the option_menu 
+       is created, so set the matching menu item now */
+    /* days */
+    if(value >= 60 * 60 * 24) {
+	    gtk_menu_set_active(GTK_MENU(menu), TIME_UNIT_DAY);
+    } else {
+        /* hours */
+        if(value >= 60 * 60) {
+	        gtk_menu_set_active(GTK_MENU(menu), TIME_UNIT_HOUR);
+        } else {
+            /* minutes */
+            if(value >= 60) {
+	            gtk_menu_set_active(GTK_MENU(menu), TIME_UNIT_MINUTE);
+            } else {
+                /* seconds */
+	            gtk_menu_set_active(GTK_MENU(menu), TIME_UNIT_SECOND);
+            }
+        }
+    }
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(unit_om), menu);
 
     return unit_om;
 }
 
-guint32 time_unit_option_menu_get_value(
+static guint32 time_unit_option_menu_convert_value(
+guint32 value)
+{
+    /* days */
+    if(value >= 60 * 60 * 24) {
+        return value / (60 * 60 * 24);
+    }
+
+    /* hours */
+    if(value >= 60 * 60) {
+        return value / (60 * 60);
+    }
+
+    /* minutes */
+    if(value >= 60) {
+        return value / 60;
+    }
+
+    /* seconds */
+    return value;
+}
+
+static guint32 time_unit_option_menu_get_value(
 GtkWidget *unit_om,
 guint32 value)
 {
@@ -365,7 +415,7 @@ static char *size_unit_name[MAX_SIZE_UNITS] = {
 	"gigabyte(s)",
 };
 
-GtkWidget *size_unit_option_menu_new(void) {
+static GtkWidget *size_unit_option_menu_new(guint32 value) {
     GtkWidget *unit_om, *menu, *menu_item;
     int i;
 
@@ -376,14 +426,55 @@ GtkWidget *size_unit_option_menu_new(void) {
 		OBJECT_SET_DATA(menu_item, "size_unit", GINT_TO_POINTER(i));
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 	}
-	gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_MEGABYTES);
 
+    /* the selected menu item can't be changed, once the option_menu 
+       is created, so set the matching menu item now */
+    /* gigabytes */
+    if(value >= 1024 * 1024 * 1024) {
+	    gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_GIGABYTES);
+    } else {
+        /* megabytes */
+        if(value >= 1024 * 1024) {
+	        gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_MEGABYTES);
+        } else {
+            /* kilobytes */
+            if(value >= 1024) {
+	            gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_KILOBYTES);
+            } else {
+                /* bytes */
+	            gtk_menu_set_active(GTK_MENU(menu), SIZE_UNIT_BYTES);
+            }
+        }
+    }
+        
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(unit_om), menu);
 
     return unit_om;
 }
 
-guint32 size_unit_option_menu_get_value(
+static guint32 size_unit_option_menu_set_value(
+guint32 value)
+{
+    /* gigabytes */
+    if(value >= 1024 * 1024 * 1024) {
+        return value / (1024 * 1024 * 1024);
+    }
+
+    /* megabytes */
+    if(value >= 1024 * 1024) {
+        return value / (1024 * 1024);
+    }
+
+    /* kilobytes */
+    if(value >= 1024) {
+        return value / 1024;
+    }
+
+    /* bytes */
+    return value;
+}
+
+static guint32 size_unit_option_menu_convert_value(
 GtkWidget *unit_om,
 guint32 value)
 {
@@ -454,7 +545,7 @@ capture_prep(void)
                 *stop_duration_cb, *stop_duration_sb, *stop_duration_om,
 
                 *display_fr, *display_vb,
-                *sync_cb, *auto_scroll_cb,
+                *sync_cb, *auto_scroll_cb, *hide_info_cb,
 
                 *resolv_fr, *resolv_vb,
                 *m_resolv_cb, *n_resolv_cb, *t_resolv_cb,
@@ -463,16 +554,19 @@ capture_prep(void)
 #if GTK_MAJOR_VERSION < 2
   GtkAccelGroup *accel_group;
 #endif
+  GtkTooltips   *tooltips;
   GtkAdjustment *snap_adj, *ringbuffer_nbf_adj,
 		*stop_packets_adj, *stop_filesize_adj, *stop_duration_adj, *stop_files_adj, *ring_filesize_adj, *file_duration_adj;
   GList         *if_list, *combo_list;
   int           err;
   int           row;
   char          err_str[PCAP_ERRBUF_SIZE];
+  gchar         *cant_get_if_list_errstr;
 #ifdef _WIN32
   GtkAdjustment *buffer_size_adj;
   GtkWidget     *buffer_size_lb, *buffer_size_sb;
 #endif
+  guint32       value;
 
   if (cap_open_w != NULL) {
     /* There's already a "Capture Options" dialog box; reactivate it. */
@@ -506,12 +600,15 @@ capture_prep(void)
 
   if_list = get_interface_list(&err, err_str);
   if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
-    simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK, "Can't get list of interfaces: %s",
-			err_str);
+    cant_get_if_list_errstr = cant_get_if_list_error_message(err_str);
+    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s",
+                  cant_get_if_list_errstr);
+    g_free(cant_get_if_list_errstr);
   }
 
   cap_open_w = dlg_window_new("Ethereal: Capture Options");
-  SIGNAL_CONNECT(cap_open_w, "destroy", capture_prep_destroy_cb, NULL);
+
+  tooltips = gtk_tooltips_new();
 
 #if GTK_MAJOR_VERSION < 2
   /* Accelerator group for the accelerators (or, as they're called in
@@ -558,6 +655,9 @@ capture_prep(void)
   }
   free_capture_combo_list(combo_list);
   free_interface_list(if_list);
+  gtk_tooltips_set_tip(tooltips, GTK_COMBO(if_cb)->entry, 
+    "Choose which interface (network card) will be used to capture packets from. "
+    "Be sure to select the correct one, as it's a common mistake to select the wrong interface.", NULL);
   gtk_box_pack_start(GTK_BOX(if_hb), if_cb, TRUE, TRUE, 6);
 
   /* Linktype row */
@@ -572,6 +672,23 @@ capture_prep(void)
   /* Default to "use the default" */
   OBJECT_SET_DATA(linktype_om, E_CAP_OM_LT_VALUE_KEY, GINT_TO_POINTER(-1));
   set_link_type_list(linktype_om, GTK_COMBO(if_cb)->entry);
+  /*
+   * XXX - in some cases, this is "multiple link-layer header types", e.g.
+   * some 802.11 interfaces on FreeBSD 5.2 and later, where you can request
+   * fake Ethernet, 802.11, or 802.11-plus-radio-information headers.
+   *
+   * In other cases, it's "multiple link-layer types", e.g., with recent
+   * versions of libpcap, a DAG card on an "HDLC" WAN, where you can
+   * request Cisco HDLC or PPP depending on what type of traffic is going
+   * over the WAN, or an Ethernet interface, where you can request Ethernet
+   * or DOCSIS, the latter being for some Cisco cable modem equipment that
+   * can be configured to send raw DOCSIS frames over an Ethernet inside
+   * Ethernet low-level framing, for traffic capture purposes.
+   *
+   * We leave it as "multiple link-layer types" for now.
+   */
+  gtk_tooltips_set_tip(tooltips, linktype_om, 
+    "The selected interface supports multiple link-layer types; select the desired one.", NULL);
   gtk_box_pack_start (GTK_BOX(linktype_hb), linktype_om, FALSE, FALSE, 0);
   SIGNAL_CONNECT(GTK_ENTRY(GTK_COMBO(if_cb)->entry), "changed",
                  capture_prep_interface_changed_cb, linktype_om);
@@ -586,6 +703,8 @@ capture_prep(void)
   gtk_spin_button_set_value(GTK_SPIN_BUTTON (buffer_size_sb), (gfloat) capture_opts.buffer_size);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (buffer_size_sb), TRUE);
   WIDGET_SET_SIZE(buffer_size_sb, 80, -1);
+  gtk_tooltips_set_tip(tooltips, buffer_size_sb, 
+    "The memory buffer size used while capturing. If you notice packet drops, you can try to increase this size.", NULL);
   gtk_box_pack_start (GTK_BOX(linktype_hb), buffer_size_sb, FALSE, FALSE, 0);
 
   buffer_size_lb = gtk_label_new("megabyte(s)");
@@ -597,6 +716,10 @@ capture_prep(void)
       "Capture packets in _promiscuous mode", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(promisc_cb),
 		capture_opts.promisc_mode);
+  gtk_tooltips_set_tip(tooltips, promisc_cb, 
+    "Usually a network card will only capture the traffic sent to its own network address. "
+    "If you want to capture all traffic that the network card can \"see\", mark this option. "
+    "See the FAQ for some more details of capturing packets from a switched network.", NULL);
   gtk_container_add(GTK_CONTAINER(capture_vb), promisc_cb);
 
   /* Capture length row */
@@ -607,6 +730,9 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(snap_cb),
 		capture_opts.has_snaplen);
   SIGNAL_CONNECT(snap_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, snap_cb, 
+    "Limit the maximum number of bytes to be captured from each packet. This size includes the "
+    "link-layer header and all subsequent headers. ", NULL);
   gtk_box_pack_start(GTK_BOX(snap_hb), snap_cb, FALSE, FALSE, 0);
 
   snap_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat) capture_opts.snaplen,
@@ -627,11 +753,19 @@ capture_prep(void)
   filter_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_CAPTURE_FILTER_ENTRY);
   SIGNAL_CONNECT(filter_bt, "clicked", capture_filter_construct_cb, NULL);
   SIGNAL_CONNECT(filter_bt, "destroy", filter_button_destroy_cb, NULL);
+  gtk_tooltips_set_tip(tooltips, filter_bt, 
+    "Select a capture filter to reduce the amount of packets to be captured. "
+    "See \"Capture Filters\" in the online help for further information how to use it.", 
+    NULL);
   gtk_box_pack_start(GTK_BOX(filter_hb), filter_bt, FALSE, FALSE, 3);
 
   filter_te = gtk_entry_new();
   if (cfile.cfilter) gtk_entry_set_text(GTK_ENTRY(filter_te), cfile.cfilter);
   OBJECT_SET_DATA(filter_bt, E_FILT_TE_PTR_KEY, filter_te);
+  gtk_tooltips_set_tip(tooltips, filter_te, 
+    "Enter a capture filter to reduce the amount of packets to be captured. "
+    "See \"Capture Filters\" in the online help for further information how to use it.", 
+    NULL);
   gtk_box_pack_start(GTK_BOX(filter_hb), filter_te, TRUE, TRUE, 3);
 
   main_hb = gtk_hbox_new(FALSE, 5);
@@ -663,9 +797,17 @@ capture_prep(void)
   gtk_box_pack_start(GTK_BOX(file_hb), file_lb, FALSE, FALSE, 3);
 
   file_te = gtk_entry_new();
+  gtk_tooltips_set_tip(tooltips, file_te, 
+    "Enter the file name to which captured data will be written. "
+    "If you don't enter something here, a temporary file will be used.",
+    NULL);
   gtk_box_pack_start(GTK_BOX(file_hb), file_te, TRUE, TRUE, 3);
 
   file_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_BROWSE);
+  gtk_tooltips_set_tip(tooltips, file_bt, 
+    "Select a file to which captured data will be written, "
+    "instead of entering the file name directly. ",
+    NULL);
   gtk_box_pack_start(GTK_BOX(file_hb), file_bt, FALSE, FALSE, 3);
 
   SIGNAL_CONNECT(file_bt, "clicked", capture_prep_file_cb, file_te);
@@ -683,6 +825,9 @@ capture_prep(void)
 		capture_opts.multi_files_on);
   SIGNAL_CONNECT(multi_files_on_cb, "toggled", capture_prep_adjust_sensitivity,
                  cap_open_w);
+  gtk_tooltips_set_tip(tooltips, multi_files_on_cb, 
+    "Instead of using a single capture file, multiple files will be created. "
+    "The generated file names will contain an incrementing number and the start time of the capture.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), multi_files_on_cb, 0, 1, row, row+1);
   row++;
 
@@ -691,17 +836,24 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ring_filesize_cb),
 		capture_opts.has_autostop_filesize);
   SIGNAL_CONNECT(ring_filesize_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, ring_filesize_cb, 
+    "If the selected file size is exceeded, capturing switches to the next file.",
+    NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ring_filesize_cb, 0, 1, row, row+1);
 
-  ring_filesize_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)capture_opts.autostop_filesize,
+  ring_filesize_adj = (GtkAdjustment *) gtk_adjustment_new(0.0,
     1, (gfloat)INT_MAX, 1.0, 10.0, 0.0);
   ring_filesize_sb = gtk_spin_button_new (ring_filesize_adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (ring_filesize_sb), TRUE);
   WIDGET_SET_SIZE(ring_filesize_sb, 80, -1);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ring_filesize_sb, 1, 2, row, row+1);
 
-  ring_filesize_om = size_unit_option_menu_new();
+  ring_filesize_om = size_unit_option_menu_new(capture_opts.autostop_filesize);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ring_filesize_om, 2, 3, row, row+1);
+
+  value = size_unit_option_menu_set_value(capture_opts.autostop_filesize);
+  gtk_adjustment_set_value(ring_filesize_adj, (gfloat) value);
+
   row++;
 
   /* Ring buffer duration row */
@@ -710,17 +862,23 @@ capture_prep(void)
 			      capture_opts.has_file_duration);
   SIGNAL_CONNECT(file_duration_cb, "toggled", 
 		 capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, file_duration_cb, 
+    "If the selected duration is exceeded, capturing switches to the next file.",
+    NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), file_duration_cb, 0, 1, row, row+1);
 
-  file_duration_adj = (GtkAdjustment *)gtk_adjustment_new((gfloat)capture_opts.file_duration,
+  file_duration_adj = (GtkAdjustment *)gtk_adjustment_new(0.0,
     1, (gfloat)INT_MAX, 1.0, 10.0, 0.0);
   file_duration_sb = gtk_spin_button_new (file_duration_adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (file_duration_sb), TRUE);
   WIDGET_SET_SIZE(file_duration_sb, 80, -1);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), file_duration_sb, 1, 2, row, row+1);
 
-  file_duration_om = time_unit_option_menu_new();
+  file_duration_om = time_unit_option_menu_new(capture_opts.file_duration);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), file_duration_om, 2, 3, row, row+1);
+
+  value = time_unit_option_menu_convert_value(capture_opts.file_duration);
+  gtk_adjustment_set_value(file_duration_adj, (gfloat) value);
   row++;
 
   /* Ring buffer files row */
@@ -728,6 +886,10 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ringbuffer_nbf_cb),
 		capture_opts.has_ring_num_files);
   SIGNAL_CONNECT(ringbuffer_nbf_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, ringbuffer_nbf_cb, 
+    "After capturing has switched to the next file and the given number of files has exceeded, "
+    "the oldest file will be removed.",
+    NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), ringbuffer_nbf_cb, 0, 1, row, row+1);
 
   ringbuffer_nbf_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat) capture_opts.ring_num_files,
@@ -748,6 +910,8 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_files_cb),
 		capture_opts.has_autostop_files);
   SIGNAL_CONNECT(stop_files_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, stop_files_cb, 
+    "Stop capturing after the given number of \"file switches\" have been done.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(multi_tb), stop_files_cb, 0, 1, row, row+1);
 
   stop_files_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)capture_opts.autostop_files,
@@ -782,6 +946,8 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_packets_cb),
 		capture_opts.has_autostop_packets);
   SIGNAL_CONNECT(stop_packets_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, stop_packets_cb, 
+    "Stop capturing after the given number of packets have been captured.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_packets_cb, 0, 1, row, row+1);
 
   stop_packets_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)capture_opts.autostop_packets,
@@ -801,17 +967,23 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_filesize_cb),
 		capture_opts.has_autostop_filesize);
   SIGNAL_CONNECT(stop_filesize_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, stop_filesize_cb, 
+    "Stop capturing after the given amount of capture data has been captured.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_filesize_cb, 0, 1, row, row+1);
 
-  stop_filesize_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)capture_opts.autostop_filesize,
+  stop_filesize_adj = (GtkAdjustment *) gtk_adjustment_new(0.0,
     1, (gfloat)INT_MAX, 1.0, 10.0, 0.0);
   stop_filesize_sb = gtk_spin_button_new (stop_filesize_adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (stop_filesize_sb), TRUE);
   WIDGET_SET_SIZE(stop_filesize_sb, 80, -1);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_filesize_sb, 1, 2, row, row+1);
 
-  stop_filesize_om = size_unit_option_menu_new();
+  stop_filesize_om = size_unit_option_menu_new(capture_opts.autostop_filesize);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_filesize_om, 2, 3, row, row+1);
+
+  value = size_unit_option_menu_set_value(capture_opts.autostop_filesize);
+  gtk_adjustment_set_value(stop_filesize_adj, (gfloat) value);
+
   row++;
 
   /* Duration row */
@@ -819,17 +991,22 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stop_duration_cb),
 		capture_opts.has_autostop_duration);
   SIGNAL_CONNECT(stop_duration_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, stop_duration_cb, 
+    "Stop capturing after the given time is exceeded.", NULL);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_duration_cb, 0, 1, row, row+1);
 
-  stop_duration_adj = (GtkAdjustment *) gtk_adjustment_new((gfloat)capture_opts.autostop_duration,
+  stop_duration_adj = (GtkAdjustment *) gtk_adjustment_new(0.0,
     1, (gfloat)INT_MAX, 1.0, 10.0, 0.0);
   stop_duration_sb = gtk_spin_button_new (stop_duration_adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (stop_duration_sb), TRUE);
   WIDGET_SET_SIZE(stop_duration_sb, 80, -1);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_duration_sb, 1, 2, row, row+1);
 
-  stop_duration_om = time_unit_option_menu_new();
+  stop_duration_om = time_unit_option_menu_new(capture_opts.autostop_duration);
   gtk_table_attach_defaults(GTK_TABLE(limit_tb), stop_duration_om, 2, 3, row, row+1);
+
+  value = time_unit_option_menu_convert_value(capture_opts.autostop_duration);
+  gtk_adjustment_set_value(stop_duration_adj, (gfloat) value);
   row++;
 
   /* Display-related options frame */
@@ -846,14 +1023,28 @@ capture_prep(void)
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sync_cb),
 		capture_opts.sync_mode);
   SIGNAL_CONNECT(sync_cb, "toggled", capture_prep_adjust_sensitivity, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, sync_cb, 
+    "Using this option will show the captured packets immediately on the main screen. "
+    "Please note: this will slow down capturing, so increased packet drops might appear.", NULL);
   gtk_container_add(GTK_CONTAINER(display_vb), sync_cb);
 
   /* "Auto-scroll live update" row */
   auto_scroll_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
 		"_Automatic scrolling in live capture", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(auto_scroll_cb), auto_scroll_live);
+  gtk_tooltips_set_tip(tooltips, auto_scroll_cb, 
+    "This will scroll the \"Packet List\" automatically to the latest captured packet, "
+    "when the \"Update List of packets in real time\" option is used.", NULL);
   gtk_container_add(GTK_CONTAINER(display_vb), auto_scroll_cb);
 
+  /* "Hide capture info" row */
+  hide_info_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
+		"_Hide capture info dialog", accel_group);
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(hide_info_cb), !capture_opts.show_info);
+  gtk_tooltips_set_tip(tooltips, hide_info_cb, 
+    "Hide the capture info dialog while capturing.", NULL);
+  gtk_container_add(GTK_CONTAINER(display_vb), hide_info_cb);
+
   /* Name Resolution frame */
   resolv_fr = gtk_frame_new("Name Resolution");
   gtk_container_add(GTK_CONTAINER(right_vb), resolv_fr);
@@ -866,18 +1057,24 @@ capture_prep(void)
 		"Enable _MAC name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_resolv_cb),
 		g_resolv_flags & RESOLV_MAC);
+  gtk_tooltips_set_tip(tooltips, m_resolv_cb, 
+    "Perform MAC layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), m_resolv_cb);
 
   n_resolv_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
 		"Enable _network name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(n_resolv_cb),
 		g_resolv_flags & RESOLV_NETWORK);
+  gtk_tooltips_set_tip(tooltips, n_resolv_cb, 
+    "Perform network layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), n_resolv_cb);
 
   t_resolv_cb = CHECK_BUTTON_NEW_WITH_MNEMONIC(
 		"Enable _transport name resolution", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(t_resolv_cb),
 		g_resolv_flags & RESOLV_TRANSPORT);
+  gtk_tooltips_set_tip(tooltips, t_resolv_cb, 
+    "Perform transport layer name resolution while capturing.", NULL);
   gtk_container_add(GTK_CONTAINER(resolv_vb), t_resolv_cb);
 
   /* Button row: OK and cancel buttons */
@@ -886,14 +1083,21 @@ capture_prep(void)
 
   ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
   SIGNAL_CONNECT(ok_bt, "clicked", capture_prep_ok_cb, cap_open_w);
-  gtk_widget_grab_default(ok_bt);
+  gtk_tooltips_set_tip(tooltips, ok_bt, 
+    "Start the capture process.", NULL);
 
   cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-  SIGNAL_CONNECT(cancel_bt, "clicked", capture_prep_close_cb, cap_open_w);
+  gtk_tooltips_set_tip(tooltips, cancel_bt, 
+    "Cancel and exit dialog.", NULL);
+  window_set_cancel_button(cap_open_w, cancel_bt, window_cancel_button_cb);
 
   help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
+  gtk_tooltips_set_tip(tooltips, help_bt, 
+    "Show help about capturing.", NULL);
   SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Capturing");
 
+  gtk_widget_grab_default(ok_bt);
+
   /* Attach pointers to needed widgets to the capture prefs window/object */
   OBJECT_SET_DATA(cap_open_w, E_CAP_IFACE_KEY, if_cb);
   OBJECT_SET_DATA(cap_open_w, E_CAP_SNAP_CB_KEY, snap_cb);
@@ -917,6 +1121,7 @@ capture_prep(void)
   OBJECT_SET_DATA(cap_open_w, E_CAP_FILE_DURATION_OM_KEY,  file_duration_om);
   OBJECT_SET_DATA(cap_open_w, E_CAP_SYNC_KEY,  sync_cb);
   OBJECT_SET_DATA(cap_open_w, E_CAP_AUTO_SCROLL_KEY, auto_scroll_cb);
+  OBJECT_SET_DATA(cap_open_w, E_CAP_HIDE_INFO_KEY, hide_info_cb);
   OBJECT_SET_DATA(cap_open_w, E_CAP_STOP_PACKETS_CB_KEY, stop_packets_cb);
   OBJECT_SET_DATA(cap_open_w, E_CAP_STOP_PACKETS_SB_KEY, stop_packets_sb);
   OBJECT_SET_DATA(cap_open_w, E_CAP_STOP_PACKETS_LB_KEY, stop_packets_lb);
@@ -937,18 +1142,14 @@ capture_prep(void)
      widgets. */
   capture_prep_adjust_sensitivity(NULL, cap_open_w);
 
-  /* Catch the "activate" signal on the filter and file name text
+  /* Catch the "activate" signal on the text
      entries, so that if the user types Return there, we act as if the
      "OK" button had been selected, as happens if Return is typed if some
      widget that *doesn't* handle the Return key has the input focus. */
+  dlg_set_activate(GTK_COMBO(if_cb)->entry, ok_bt);
   dlg_set_activate(filter_te, ok_bt);
   dlg_set_activate(file_te, ok_bt);
 
-  /* Catch the "key_press_event" signal in the window, so that we can catch
-     the ESC key being pressed and act as if the "Cancel" button had
-     been selected. */
-  dlg_set_cancel(cap_open_w, cancel_bt);
-
   /* XXX - why does not
 
      gtk_widget_grab_focus(if_cb);
@@ -972,7 +1173,11 @@ capture_prep(void)
     in it; and it now appears that you simply *can't* make a combo box
     get the initial focus, at least not in the obvious fashion. Sigh.... */
 
+  SIGNAL_CONNECT(cap_open_w, "delete_event", window_delete_event_cb, NULL);
+  SIGNAL_CONNECT(cap_open_w, "destroy", capture_prep_destroy_cb, NULL);
+
   gtk_widget_show_all(cap_open_w);
+  window_present(cap_open_w);
 }
 
 static void 
@@ -998,7 +1203,7 @@ capture_prep_cb(GtkWidget *w _U_, gpointer d _U_)
 {
   gpointer  dialog;
 
-  if((cfile.state != FILE_CLOSED) && !cfile.user_saved) {
+  if((cfile.state != FILE_CLOSED) && !cfile.user_saved && prefs.gui_ask_unsaved) {
     /* user didn't saved his current file, ask him */
     dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTNS_YES_NO_CANCEL,
                 PRIMARY_TEXT_START "Save capture file before starting a new capture?" PRIMARY_TEXT_END "\n\n"
@@ -1022,112 +1227,17 @@ select_link_type_cb(GtkWidget *w, gpointer data)
 }
 
 static void
-capture_prep_file_cb(GtkWidget *w, gpointer file_te)
+capture_prep_file_cb(GtkWidget *file_bt, GtkWidget *file_te)
 {
-  GtkWidget *caller = gtk_widget_get_toplevel(w);
-  GtkWidget *fs;
-
-  /* Has a file selection dialog box already been opened for that top-level
-     widget? */
-  fs = OBJECT_GET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY);
-
-  if (fs != NULL) {
-    /* Yes.  Just re-activate that dialog box. */
-    reactivate_window(fs);
-    return;
-  }
-
-  fs = file_selection_new ("Ethereal: Capture File");
-
-  /* If we've opened a file, start out by showing the files in the directory
-     in which that file resided. */
-  if (last_open_dir)
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), last_open_dir);
-
-  OBJECT_SET_DATA(fs, E_CAP_FILE_TE_KEY, file_te);
-
-  /* Set the E_FS_CALLER_PTR_KEY for the new dialog to point to our caller. */
-  OBJECT_SET_DATA(fs, E_FS_CALLER_PTR_KEY, caller);
-
-  /* Set the E_FILE_SEL_DIALOG_PTR_KEY for the caller to point to us */
-  OBJECT_SET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY, fs);
-
-  /* Call a handler when the file selection box is destroyed, so we can inform
-     our caller, if any, that it's been destroyed. */
-  SIGNAL_CONNECT(fs, "destroy", cap_prep_fs_destroy_cb, file_te);
-
-  SIGNAL_CONNECT(GTK_FILE_SELECTION(fs)->ok_button, "clicked", cap_prep_fs_ok_cb, fs);
-
-  /* Connect the cancel_button to destroy the widget */
-  SIGNAL_CONNECT(GTK_FILE_SELECTION(fs)->cancel_button, "clicked", cap_prep_fs_cancel_cb,
-                 fs);
-
-  /* Catch the "key_press_event" signal in the window, so that we can catch
-     the ESC key being pressed and act as if the "Cancel" button had
-     been selected. */
-  dlg_set_cancel(fs, GTK_FILE_SELECTION(fs)->cancel_button);
-
-  gtk_widget_show_all(fs);
+    file_selection_browse(file_bt, file_te, "Ethereal: Specify a Capture File", FILE_SELECTION_OPEN);
 }
 
-static void
-cap_prep_fs_ok_cb(GtkWidget *w _U_, gpointer data)
-{
-  gchar     *cf_name;
-
-  cf_name = g_strdup(gtk_file_selection_get_filename(
-    GTK_FILE_SELECTION (data)));
-
-  /* Perhaps the user specified a directory instead of a file.
-     Check whether they did. */
-  if (test_for_directory(cf_name) == EISDIR) {
-        /* It's a directory - set the file selection box to display it. */
-        set_last_open_dir(cf_name);
-        g_free(cf_name);
-        gtk_file_selection_set_filename(GTK_FILE_SELECTION(data),
-          last_open_dir);
-        return;
-  }
-
-  gtk_entry_set_text(GTK_ENTRY(OBJECT_GET_DATA(data, E_CAP_FILE_TE_KEY)), cf_name);
-
-  gtk_widget_destroy(GTK_WIDGET(data));
-  g_free(cf_name);
-}
-
-static void
-cap_prep_fs_cancel_cb(GtkWidget *w _U_, gpointer data)
-{
-  gtk_widget_destroy(GTK_WIDGET(data));
-}
-
-static void
-cap_prep_fs_destroy_cb(GtkWidget *win, GtkWidget* file_te)
-{
-  GtkWidget *caller;
-
-  /* Get the widget that requested that we be popped up.
-     (It should arrange to destroy us if it's destroyed, so
-     that we don't get a pointer to a non-existent window here.) */
-  caller = OBJECT_GET_DATA(win, E_FS_CALLER_PTR_KEY);
-
-  /* Tell it we no longer exist. */
-  OBJECT_SET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY, NULL);
-
-  /* Now nuke this window. */
-  gtk_grab_remove(GTK_WIDGET(win));
-  gtk_widget_destroy(GTK_WIDGET(win));
-
-  /* Give the focus to the file text entry widget so the user can just press
-     Return to start the capture. */
-  gtk_widget_grab_focus(file_te);
-}
 
 static void
 capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   GtkWidget *if_cb, *snap_cb, *snap_sb, *promisc_cb, *filter_te,
             *file_te, *multi_files_on_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_cb,
-            *linktype_om, *sync_cb, *auto_scroll_cb,
+            *linktype_om, *sync_cb, *auto_scroll_cb, *hide_info_cb,
             *stop_packets_cb, *stop_packets_sb,
             *stop_filesize_cb, *stop_filesize_sb, *stop_filesize_om,
             *stop_duration_cb, *stop_duration_sb, *stop_duration_om,
@@ -1169,6 +1279,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   file_duration_om = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_FILE_DURATION_OM_KEY);
   sync_cb   = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_SYNC_KEY);
   auto_scroll_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_AUTO_SCROLL_KEY);
+  hide_info_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_HIDE_INFO_KEY);
   stop_packets_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_PACKETS_CB_KEY);
   stop_packets_sb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_PACKETS_SB_KEY);
   stop_filesize_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_FILESIZE_CB_KEY);
@@ -1269,6 +1380,9 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
   auto_scroll_live =
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(auto_scroll_cb));
 
+  capture_opts.show_info =
+      !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hide_info_cb));
+
   g_resolv_flags |= g_resolv_flags & RESOLV_CONCURRENT;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_resolv_cb)))
     g_resolv_flags |= RESOLV_MAC;
@@ -1300,7 +1414,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ring_filesize_cb));
     if (capture_opts.has_autostop_filesize) {
       tmp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ring_filesize_sb));
-      tmp = size_unit_option_menu_get_value(ring_filesize_om, tmp);
+      tmp = size_unit_option_menu_convert_value(ring_filesize_om, tmp);
       if(tmp != 0) {
         capture_opts.autostop_filesize = tmp;
       } else {
@@ -1330,7 +1444,7 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stop_filesize_cb));
     if (capture_opts.has_autostop_filesize) {
       tmp = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(stop_filesize_sb));
-      tmp = size_unit_option_menu_get_value(stop_filesize_om, tmp);
+      tmp = size_unit_option_menu_convert_value(stop_filesize_om, tmp);
       if(tmp != 0) {
         capture_opts.autostop_filesize = tmp;
       } else {
@@ -1357,20 +1471,13 @@ capture_prep_ok_cb(GtkWidget *ok_bt _U_, gpointer parent_w) {
     capture_opts.autostop_files =
       gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(stop_files_sb));
 
-  gtk_widget_destroy(GTK_WIDGET(parent_w));
+  window_destroy(GTK_WIDGET(parent_w));
 
   do_capture(save_file);
   if (save_file != NULL)
     g_free(save_file);
 }
 
-static void
-capture_prep_close_cb(GtkWidget *close_bt _U_, gpointer parent_w)
-{
-  gtk_grab_remove(GTK_WIDGET(parent_w));
-  gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
 static void
 capture_prep_destroy_cb(GtkWidget *win, gpointer user_data _U_)
 {
@@ -1382,7 +1489,7 @@ capture_prep_destroy_cb(GtkWidget *win, gpointer user_data _U_)
 
   if (fs != NULL) {
     /* Yes.  Destroy it. */
-    gtk_widget_destroy(fs);
+    window_destroy(fs);
   }
 
   /* Note that we no longer have a "Capture Options" dialog box. */
@@ -1409,7 +1516,7 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
             *multi_files_on_cb, *ringbuffer_nbf_cb, *ringbuffer_nbf_sb, *ringbuffer_nbf_lb,
             *ring_filesize_cb, *ring_filesize_sb, *ring_filesize_om,
             *file_duration_cb, *file_duration_sb, *file_duration_om,
-            *sync_cb, *auto_scroll_cb,
+            *sync_cb, *auto_scroll_cb, *hide_info_cb,
             *stop_packets_cb, *stop_packets_sb, *stop_packets_lb,
             *stop_filesize_cb, *stop_filesize_sb, *stop_filesize_om,
             *stop_duration_cb, *stop_duration_sb, *stop_duration_om,
@@ -1431,6 +1538,7 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
   file_duration_om = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_FILE_DURATION_OM_KEY);
   sync_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_SYNC_KEY);
   auto_scroll_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_AUTO_SCROLL_KEY);
+  hide_info_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_HIDE_INFO_KEY);
   stop_packets_cb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_PACKETS_CB_KEY);
   stop_packets_sb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_PACKETS_SB_KEY);
   stop_packets_lb = (GtkWidget *) OBJECT_GET_DATA(parent_w, E_CAP_STOP_PACKETS_LB_KEY);
@@ -1462,6 +1570,8 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
     /* Auto-scroll mode is meaningful only in "Update list of packets
        in real time" captures, so make its toggle button sensitive. */
     gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), TRUE);
+
+    gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), TRUE);
   } else {
     /* "Update list of packets in real time" captures disabled; that
        means ring buffer mode is OK, so make its toggle button
@@ -1471,6 +1581,8 @@ capture_prep_adjust_sensitivity(GtkWidget *tb _U_, gpointer parent_w)
     /* Auto-scroll mode is meaningful only in "Update list of packets
        in real time" captures, so make its toggle button insensitive. */
     gtk_widget_set_sensitive(GTK_WIDGET(auto_scroll_cb), FALSE);
+
+    gtk_widget_set_sensitive(GTK_WIDGET(hide_info_cb), FALSE);
   }
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(multi_files_on_cb))) {
