@@ -1,7 +1,7 @@
 /* capture_prefs.c
  * Dialog box for capture preferences
  *
- * $Id: capture_prefs.c,v 1.31 2004/03/13 15:15:22 ulfl Exp $
+ * $Id: capture_prefs.c,v 1.38 2004/06/20 14:48:23 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,8 +49,9 @@
 #define PROM_MODE_KEY			"prom_mode"
 #define CAPTURE_REAL_TIME_KEY	"capture_real_time"
 #define AUTO_SCROLL_KEY			"auto_scroll"
+#define SHOW_INFO_KEY           "show_info"
 
-#define CAPTURE_TABLE_ROWS 5
+#define CAPTURE_TABLE_ROWS 6
 
 #define IFOPTS_CALLER_PTR_KEY	"ifopts_caller_ptr"
 #define IFOPTS_DIALOG_PTR_KEY	"ifopts_dialog_ptr"
@@ -60,12 +61,11 @@
 #define IFOPTS_IF_NOSEL -1
 
 /* interface options dialog */
-static GtkWidget *cur_clist, *if_descr_te, *if_hide_cb;
+static GtkWidget *cur_clist, *if_dev_lb, *if_name_lb, *if_descr_te, *if_hide_cb;
 static gint ifrow;						/* current interface row selected */
 
 static void ifopts_edit_cb(GtkWidget *w, gpointer data);
 static void ifopts_edit_ok_cb(GtkWidget *w, gpointer parent_w);
-static void ifopts_edit_close_cb(GtkWidget *close_bt, gpointer parent_w);
 static void ifopts_edit_destroy_cb(GtkWidget *win, gpointer data);
 static void ifopts_edit_ifsel_cb(GtkWidget *clist, gint row, gint column,
     GdkEventButton *event, gpointer data);
@@ -73,7 +73,7 @@ static void ifopts_edit_descr_changed_cb(GtkEditable *ed, gpointer udata);
 static void ifopts_edit_hide_changed_cb(GtkToggleButton *tbt, gpointer udata);
 static void ifopts_options_add(GtkCList *clist, if_info_t *if_info);
 static void ifopts_options_free(gchar *text[]);
-static void ifopts_if_clist_add(GtkCList *clist);
+static void ifopts_if_clist_add(void);
 static void ifopts_write_new_descr(void);
 static void ifopts_write_new_hide(void);
 
@@ -81,11 +81,13 @@ GtkWidget*
 capture_prefs_show(void)
 {
 	GtkWidget	*main_tb, *main_vb;
-	GtkWidget	*if_cb, *if_lb, *promisc_cb, *sync_cb, *auto_scroll_cb;
+	GtkWidget	*if_cb, *if_lb, *promisc_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
 	GtkWidget	*ifopts_lb, *ifopts_bt;
 	GList		*if_list, *combo_list;
 	int		err;
 	char		err_str[PCAP_ERRBUF_SIZE];
+    int         row = 0;
+    GtkTooltips *tooltips = gtk_tooltips_new();
 
 	/* Main vertical box */
 	main_vb = gtk_vbox_new(FALSE, 7);
@@ -100,7 +102,7 @@ capture_prefs_show(void)
 
 	/* Default device */
 	if_lb = gtk_label_new("Default interface:");
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_lb, 0, 1, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_lb, 0, 1, row, row+1);
 	gtk_misc_set_alignment(GTK_MISC(if_lb), 1.0, 0.5);
 	gtk_widget_show(if_lb);
 
@@ -118,38 +120,63 @@ capture_prefs_show(void)
 	if (prefs.capture_device)
 		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry),
 		    prefs.capture_device);
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_cb, 1, 2, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_cb, 1, 2, row, row+1);
+    gtk_tooltips_set_tip(tooltips, GTK_COMBO(if_cb)->entry, 
+        "The default interface to be captured from.", NULL);
 	gtk_widget_show(if_cb);
 	OBJECT_SET_DATA(main_vb, DEVICE_KEY, if_cb);
+    row++;
 
 	/* Interface properties */
-	ifopts_lb = gtk_label_new("Interface:");
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), ifopts_lb, 0, 1, 1, 2);
+	ifopts_lb = gtk_label_new("Interfaces:");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), ifopts_lb, 0, 1, row, row+1);
 	gtk_misc_set_alignment(GTK_MISC(ifopts_lb), 1.0, 0.5);
 	gtk_widget_show(ifopts_lb);
 
 	ifopts_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_EDIT);
+    gtk_tooltips_set_tip(tooltips, ifopts_bt, 
+        "Open a dialog box to set various interface options.", NULL);
 	SIGNAL_CONNECT(ifopts_bt, "clicked", ifopts_edit_cb, NULL);
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), ifopts_bt, 1, 2, 1, 2 );
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), ifopts_bt, 1, 2, row, row+1);
+    row++;
 
 	/* Promiscuous mode */
-	promisc_cb = create_preference_check_button(main_tb, 2,
+	promisc_cb = create_preference_check_button(main_tb, row++,
 	    "Capture packets in promiscuous mode:", NULL,
 	    prefs.capture_prom_mode);
+    gtk_tooltips_set_tip(tooltips, promisc_cb, 
+        "Usually a network card will only capture the traffic sent to its own network address. "
+        "If you want to capture all traffic that the network card can \"see\", mark this option. "
+        "See the FAQ for some more details of capturing packets from a switched network.", NULL);
 	OBJECT_SET_DATA(main_vb, PROM_MODE_KEY, promisc_cb);
 
 	/* Real-time capture */
-	sync_cb = create_preference_check_button(main_tb, 3,
+	sync_cb = create_preference_check_button(main_tb, row++,
 	    "Update list of packets in real time:", NULL,
 	    prefs.capture_real_time);
+    gtk_tooltips_set_tip(tooltips, sync_cb,
+        "Update the list of packets while capture is in progress. "
+        "Don't use this option if you notice packet drops.", NULL);
 	OBJECT_SET_DATA(main_vb, CAPTURE_REAL_TIME_KEY, sync_cb);
 
 	/* Auto-scroll real-time capture */
-	auto_scroll_cb = create_preference_check_button(main_tb, 4,
+	auto_scroll_cb = create_preference_check_button(main_tb, row++,
 	    "Automatic scrolling in live capture:", NULL,
 	    prefs.capture_auto_scroll);
+    gtk_tooltips_set_tip(tooltips, auto_scroll_cb,
+        "Automatic scrolling of the packet list while live capture is in progress. ", NULL);
 	OBJECT_SET_DATA(main_vb, AUTO_SCROLL_KEY, auto_scroll_cb);
 
+	/* Show capture info dialog */
+	show_info_cb = create_preference_check_button(main_tb, row++,
+	    "Hide capture info dialog:", NULL,
+	    !prefs.capture_show_info);
+    gtk_tooltips_set_tip(tooltips, show_info_cb,
+        "Hide the capture info dialog while capturing. "
+        "Will only take effect, if the \"Update list of packets in real time\" "
+        "option is also used.", NULL);
+	OBJECT_SET_DATA(main_vb, SHOW_INFO_KEY, show_info_cb);
+
 	/* Show 'em what we got */
 	gtk_widget_show_all(main_vb);
 
@@ -159,13 +186,14 @@ capture_prefs_show(void)
 void
 capture_prefs_fetch(GtkWidget *w)
 {
-	GtkWidget *if_cb, *promisc_cb, *sync_cb, *auto_scroll_cb;
+	GtkWidget *if_cb, *promisc_cb, *sync_cb, *auto_scroll_cb, *show_info_cb;
 	gchar	*if_text;
 
 	if_cb = (GtkWidget *)OBJECT_GET_DATA(w, DEVICE_KEY);
 	promisc_cb = (GtkWidget *)OBJECT_GET_DATA(w, PROM_MODE_KEY);
 	sync_cb = (GtkWidget *)OBJECT_GET_DATA(w, CAPTURE_REAL_TIME_KEY);
 	auto_scroll_cb = (GtkWidget *)OBJECT_GET_DATA(w, AUTO_SCROLL_KEY);
+    show_info_cb = (GtkWidget *)OBJECT_GET_DATA(w, SHOW_INFO_KEY);
 
 	if (prefs.capture_device != NULL) {
 		g_free(prefs.capture_device);
@@ -188,6 +216,8 @@ capture_prefs_fetch(GtkWidget *w)
 	prefs.capture_real_time = GTK_TOGGLE_BUTTON (sync_cb)->active;
 
 	prefs.capture_auto_scroll = GTK_TOGGLE_BUTTON (auto_scroll_cb)->active;
+
+    prefs.capture_show_info = !(GTK_TOGGLE_BUTTON (show_info_cb)->active);
 }
 
 void
@@ -207,7 +237,7 @@ capture_prefs_destroy(GtkWidget *w)
 
 	if (dlg != NULL) {
 		/* Yes.  Destroy it. */
-		gtk_widget_destroy(dlg);
+		window_destroy(dlg);
 	}
 }
 
@@ -217,13 +247,13 @@ capture_prefs_destroy(GtkWidget *w)
 static void
 ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 {
-	GtkWidget	*ifopts_edit_dlg, *cur_scr_win, *if_scr_win, *main_hb, *main_tb,
+	GtkWidget	*ifopts_edit_dlg, *cur_scr_win, *main_hb, *main_tb,
 				*cur_opts_fr, *ed_opts_fr, *main_vb,
-				*if_clist, *if_descr_lb, *if_hide_lb,
+				*if_descr_lb, *if_hide_lb,
 				*bbox, *ok_bt, *cancel_bt;
-	gchar *cur_titles[IFOPTS_CLIST_COLS] = { "dev-nodisp", "Interface", 
-												"Description", "Hide?" };
-	gchar *if_title[1] = { "Interface" };
+	gchar *cur_titles[] = { "Device", "Description", 
+												"Comment", "Hide?" };
+    int row = 0;
 
 	GtkWidget *caller = gtk_widget_get_toplevel(w);
 	
@@ -238,14 +268,15 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	
 	/* create a new dialog */
 	ifopts_edit_dlg = dlg_window_new("Ethereal: Preferences: Interface Options");
-	SIGNAL_CONNECT(ifopts_edit_dlg, "destroy", ifopts_edit_destroy_cb, NULL);
-	main_vb = gtk_vbox_new(FALSE, 1);
+    gtk_window_set_default_size(GTK_WINDOW(ifopts_edit_dlg), DEF_WIDTH, 300);
+
+    main_vb = gtk_vbox_new(FALSE, 1);
 	gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
 	gtk_container_add(GTK_CONTAINER(ifopts_edit_dlg), main_vb);
 	gtk_widget_show(main_vb);
 	
 	/* create current options frame */
-	cur_opts_fr = gtk_frame_new("Current options");
+	cur_opts_fr = gtk_frame_new("Interfaces");
 	gtk_container_add(GTK_CONTAINER(main_vb), cur_opts_fr);
 	gtk_widget_show(cur_opts_fr);
 	
@@ -256,61 +287,66 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	gtk_widget_show(cur_scr_win);
 	
 	/*
-	 * Create current options CList. Column "0" is the device name only and
-	 * is not displayed. It's used when writing new preferences strings.
-	 * This avoids having to remove OS (pcap) descriptions.
+	 * Create current options CList.
 	 */
 	cur_clist = gtk_clist_new_with_titles(IFOPTS_CLIST_COLS, cur_titles);
-	WIDGET_SET_SIZE(GTK_WIDGET(cur_clist), 570, 120);
-	gtk_clist_set_column_visibility(GTK_CLIST(cur_clist), 0, FALSE);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 1, 230);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 2, 260);
 	gtk_clist_set_column_width(GTK_CLIST(cur_clist), 3, 40);
 	gtk_clist_column_titles_passive(GTK_CLIST(cur_clist));
 	gtk_container_add(GTK_CONTAINER(cur_scr_win), cur_clist);
+	SIGNAL_CONNECT(cur_clist, "select_row", ifopts_edit_ifsel_cb, NULL);
 	gtk_widget_show(cur_clist);
 	
+	/* add interface names to cell */
+	ifopts_if_clist_add();
+    gtk_clist_columns_autosize(GTK_CLIST(cur_clist));
+	
+	/* initialize variable that saves currently selected row in "if_clist" */
+	ifrow = IFOPTS_IF_NOSEL;
+	
 	/* create edit options frame */
-	ed_opts_fr = gtk_frame_new("Edit interface options");
-	gtk_container_add(GTK_CONTAINER(main_vb),  ed_opts_fr);
+	ed_opts_fr = gtk_frame_new("Properties");
+	gtk_box_pack_start(GTK_BOX(main_vb), ed_opts_fr, FALSE, FALSE, 0);
 	gtk_widget_show(ed_opts_fr);
 	
 	main_hb = gtk_hbox_new(TRUE, 5);
 	gtk_container_border_width(GTK_CONTAINER(main_hb), 3);
 	gtk_container_add(GTK_CONTAINER(ed_opts_fr), main_hb);
 	gtk_widget_show(main_hb);
-	
-	/* create a scrolled window to pack the interface CList widget into */
-	if_scr_win = scrolled_window_new(NULL, NULL);
-	gtk_box_pack_start(GTK_BOX(main_hb), if_scr_win, TRUE, TRUE, 0);
-	gtk_widget_show(if_scr_win);
-	
-	/* create interface CList */
-	if_clist = gtk_clist_new_with_titles(1, if_title);
-	SIGNAL_CONNECT(if_clist, "select_row", ifopts_edit_ifsel_cb, if_clist);
-	WIDGET_SET_SIZE(GTK_WIDGET(if_clist), 100, 120);
-	gtk_clist_set_column_width(GTK_CLIST(if_clist), 0, 100);
-	gtk_clist_column_titles_passive(GTK_CLIST(if_clist));
-	gtk_clist_set_selection_mode(GTK_CLIST(if_clist), GTK_SELECTION_SINGLE);
-	gtk_container_add(GTK_CONTAINER(if_scr_win), if_clist);
-	
-	/* initialize variable that saves currently selected row in "if_clist" */
-	ifrow = IFOPTS_IF_NOSEL;
-	
-	/* add interface names to cell */
-	ifopts_if_clist_add(GTK_CLIST(if_clist));
-	gtk_widget_show(if_clist);
-	
+		
 	/* table to hold description text entry and hide button */
-	main_tb = gtk_table_new(IFOPTS_TABLE_ROWS, 2, FALSE);
+	main_tb = gtk_table_new(IFOPTS_TABLE_ROWS, 4, FALSE);
 	gtk_box_pack_start(GTK_BOX(main_hb), main_tb, TRUE, FALSE, 10);
 	gtk_table_set_row_spacings(GTK_TABLE(main_tb), 10);
 	gtk_table_set_col_spacings(GTK_TABLE(main_tb), 10);
 	gtk_widget_show(main_tb);
-	
+
+	if_dev_lb = gtk_label_new("Device:");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_dev_lb, 0, 1, row, row+1);
+	gtk_misc_set_alignment(GTK_MISC(if_dev_lb), 1.0, 0.5);
+	gtk_widget_show(if_dev_lb);
+    
+	if_dev_lb = gtk_label_new("");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_dev_lb, 1, 2, row, row+1);
+	gtk_misc_set_alignment(GTK_MISC(if_dev_lb), 0.0, 0.5);
+	gtk_widget_show(if_dev_lb);
+    row++;
+    
+	if_name_lb = gtk_label_new("Description:");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_name_lb, 0, 1, row, row+1);
+	gtk_misc_set_alignment(GTK_MISC(if_name_lb), 1.0, 0.5);
+	gtk_widget_show(if_name_lb);
+    
+	if_name_lb = gtk_label_new("");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_name_lb, 1, 2, row, row+1);
+	gtk_misc_set_alignment(GTK_MISC(if_name_lb), 0.0, 0.5);
+	gtk_widget_show(if_name_lb);
+    row++;
+    
 	/* create interface description label and text entry */
-	if_descr_lb = gtk_label_new("Description:");
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_descr_lb, 0, 1, 0, 1);
+	if_descr_lb = gtk_label_new("Comment:");
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_descr_lb, 0, 1, row, row+1);
 	gtk_misc_set_alignment(GTK_MISC(if_descr_lb), 1.0, 0.5);
 	gtk_widget_show(if_descr_lb);
 	
@@ -318,20 +354,22 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	SIGNAL_CONNECT(if_descr_te, "changed", ifopts_edit_descr_changed_cb, 
 			cur_clist);
 	gtk_entry_set_max_length(GTK_ENTRY(if_descr_te), IFOPTS_MAX_DESCR_LEN);
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_descr_te, 1, 2, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_descr_te, 1, 2, row, row+1);
 	gtk_widget_show(if_descr_te);
+    row++;
 	
 	/* create hide interface label and button */
 	if_hide_lb = gtk_label_new("Hide interface?:");
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_hide_lb, 0, 1, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_hide_lb, 0, 1, row, row+1);
 	gtk_misc_set_alignment(GTK_MISC(if_hide_lb), 1.0, 0.5);
 	gtk_widget_show(if_hide_lb);
 	
 	if_hide_cb = gtk_check_button_new();
 	SIGNAL_CONNECT(if_hide_cb, "toggled", ifopts_edit_hide_changed_cb, 
 			cur_clist);
-	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_hide_cb, 1, 2, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(main_tb), if_hide_cb, 1, 2, row, row+1);
 	gtk_widget_show(if_hide_cb);
+    row++;
 	
 	/* button row: OK and Cancel buttons */
 	bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL);
@@ -339,12 +377,15 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	gtk_widget_show(bbox);
 
 	ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
-	gtk_widget_grab_default(ok_bt);
 	SIGNAL_CONNECT(ok_bt, "clicked", ifopts_edit_ok_cb, ifopts_edit_dlg);
 
 	cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-	SIGNAL_CONNECT(cancel_bt, "clicked", ifopts_edit_close_cb, ifopts_edit_dlg);
+    window_set_cancel_button(ifopts_edit_dlg, cancel_bt, window_cancel_button_cb);
 
+	gtk_widget_grab_default(ok_bt);
+
+    SIGNAL_CONNECT(ifopts_edit_dlg, "delete_event", window_delete_event_cb,
+                 NULL);
 	/* Call a handler when we're destroyed, so we can inform
 	   our caller, if any, that we've been destroyed. */
 	SIGNAL_CONNECT(ifopts_edit_dlg, "destroy", ifopts_edit_destroy_cb, NULL);
@@ -353,13 +394,12 @@ ifopts_edit_cb(GtkWidget *w, gpointer data _U_)
 	OBJECT_SET_DATA(ifopts_edit_dlg, IFOPTS_CALLER_PTR_KEY, caller);
 	/* Set the key for the caller to point to us */
 	OBJECT_SET_DATA(caller, IFOPTS_DIALOG_PTR_KEY, ifopts_edit_dlg);
-	
-	/* Catch the "key_press_event" signal in the window, so that we can
-	   catch the ESC key being pressed and act as if the "Cancel" button
-	   had been selected. */
-	dlg_set_cancel(ifopts_edit_dlg, cancel_bt);
-	
+
+    /* select the first row in if list, all option fields must exist for this */
+	gtk_clist_select_row(GTK_CLIST(cur_clist), 0, -1);
+    
 	gtk_widget_show(ifopts_edit_dlg);
+    window_present(ifopts_edit_dlg);
 }
 
 /*
@@ -378,14 +418,7 @@ ifopts_edit_ok_cb(GtkWidget *w _U_, gpointer parent_w)
 	
 	/* Now nuke this window. */
 	gtk_grab_remove(GTK_WIDGET(parent_w));
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-static void
-ifopts_edit_close_cb(GtkWidget *close_bt _U_, gpointer parent_w)
-{
-	gtk_grab_remove(GTK_WIDGET(parent_w));
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
+	window_destroy(GTK_WIDGET(parent_w));
 }
 
 static void
@@ -402,10 +435,6 @@ ifopts_edit_destroy_cb(GtkWidget *win, gpointer data _U_)
 		/* Tell it we no longer exist. */
 		OBJECT_SET_DATA(caller, IFOPTS_DIALOG_PTR_KEY, NULL);
 	}
-
-	/* Now nuke this window. */
-	gtk_grab_remove(GTK_WIDGET(win));
-	gtk_widget_destroy(GTK_WIDGET(win));
 }
 
 /*
@@ -423,9 +452,26 @@ ifopts_edit_ifsel_cb(GtkWidget		*clist _U_,
 	/* save currently selected row */
 	ifrow = row;
 	
+	/* get/display the interface device from current CList */
+	gtk_clist_get_text(GTK_CLIST(cur_clist), row, 0, &text);
+    /* is needed, as gtk_entry_set_text() will change text again (bug in GTK?) */
+    text = strdup(text);
+	gtk_label_set_text(GTK_LABEL(if_dev_lb), text);
+    g_free(text);
+	
+	/* get/display the interface name from current CList */
+	gtk_clist_get_text(GTK_CLIST(cur_clist), row, 1, &text);
+    /* is needed, as gtk_entry_set_text() will change text again (bug in GTK?) */
+    text = strdup(text);
+	gtk_label_set_text(GTK_LABEL(if_name_lb), text);
+    g_free(text);
+	
 	/* get/display the interface description from current CList */
 	gtk_clist_get_text(GTK_CLIST(cur_clist), row, 2, &text);
+    /* is needed, as gtk_entry_set_text() will change text again (bug in GTK?) */
+    text = strdup(text);
 	gtk_entry_set_text(GTK_ENTRY(if_descr_te), text);
+    g_free(text);
 	
 	/* get/display the "hidden" button state from current CList */
 	gtk_clist_get_text(GTK_CLIST(cur_clist), row, 3, &text);
@@ -436,7 +482,7 @@ ifopts_edit_ifsel_cb(GtkWidget		*clist _U_,
 }
 
 /*
- * Description text entry changed callback; update current CList.
+ * Comment text entry changed callback; update current CList.
  */
 static void
 ifopts_edit_descr_changed_cb(GtkEditable *ed, gpointer udata)
@@ -487,7 +533,7 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 	gchar	*ifnm;
 	gchar	*desc;
 	gchar	*pr_descr;
-	gchar	*text[IFOPTS_CLIST_COLS] = { NULL, NULL, NULL, NULL };
+	gchar	*text[] = { NULL, NULL, NULL, NULL };
 	
 	/* add interface descriptions and "hidden" flag */
 	if (prefs.capture_devices_descr != NULL) {
@@ -504,9 +550,9 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 					text[0] = g_strdup(if_info->name);
 					/* set OS description + device name text */
 					if (if_info->description != NULL)
-						text[1] = g_strdup_printf("%s: %s", if_info->description, if_info->name);
+						text[1] = g_strdup(if_info->description);
 					else
-						text[1] = g_strdup(if_info->name);
+						text[1] = "";
 					/* check if interface is "hidden" */
 					if (prefs.capture_devices_hide != NULL) {
 						if (strstr(prefs.capture_devices_hide, if_info->name) != NULL)
@@ -559,9 +605,9 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 			text[0] = g_strdup(if_info->name);
 			/* set OS description + device name text */
 			if (if_info->description != NULL)
-				text[1] = g_strdup_printf("%s: %s", if_info->description, if_info->name);
+				text[1] = g_strdup(if_info->description);
 			else
-				text[1] = g_strdup(if_info->name);
+				text[1] = "";
 			/* set empty description */
 			text[2] = g_strdup("");
 			/* check if interface is "hidden" */
@@ -590,9 +636,9 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 		text[0] = g_strdup(if_info->name);
 		/* set OS description + device name text */
 		if (if_info->description != NULL)
-			text[1] = g_strdup_printf("%s: %s", if_info->description, if_info->name);
+			text[1] = g_strdup(if_info->description);
 		else
-			text[1] = g_strdup(if_info->name);
+			text[1] = "";
 		/* set empty description */
 		text[2] = g_strdup("");
 		/* check if interface is "hidden" */
@@ -614,9 +660,9 @@ ifopts_options_add(GtkCList *clist, if_info_t *if_info)
 		text[0] = g_strdup(if_info->name);
 		/* set OS description + device name text */
 		if (if_info->description != NULL)
-			text[1] = g_strdup_printf("%s: %s", if_info->description, if_info->name);
+			text[1] = g_strdup(if_info->description);
 		else
-			text[1] = g_strdup(if_info->name);
+			text[1] = "";
 		/* set empty description */
 		text[2] = g_strdup("");
 		/* interface is not "hidden" */
@@ -643,24 +689,26 @@ ifopts_options_free(gchar *text[])
 }
 
 /*
- * Add interfaces to displayed interfaces CList. Also, fill current 
- * options CList with current preference values.
+ * Add all interfaces to interfaces CList.
  */
 static void
-ifopts_if_clist_add(GtkCList *clist)
+ifopts_if_clist_add(void)
 {
 	GList		*if_list;
-	int			err;
+	int		err;
 	char		err_str[PCAP_ERRBUF_SIZE];
+	gchar		*cant_get_if_list_errstr;
 	if_info_t	*if_info;
-	gchar		*text[1];
 	guint		i;
 	guint		nitems;
 	
 	if_list = get_interface_list(&err, err_str);
 	if (if_list == NULL && err == CANT_GET_INTERFACE_LIST) {
-		simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
-				"Can't get list of interfaces: %s", err_str);
+		cant_get_if_list_errstr =
+		    cant_get_if_list_error_message(err_str);
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "%s",
+		    cant_get_if_list_errstr);
+		g_free(cant_get_if_list_errstr);
 		return;
 	}
 	
@@ -674,14 +722,9 @@ ifopts_if_clist_add(GtkCList *clist)
 		/* should never happen, but just in case */
 		if (if_info == NULL)
 			continue;
-		if (if_info->description != NULL)
-			text[0] = g_strdup_printf("%s: %s", if_info->description, if_info->name);
-		else
-			text[0] = g_strdup(if_info->name);
-		gtk_clist_append(GTK_CLIST(clist), text);
-		/* fill current options CList with current preference values */
+
+        /* fill current options CList with current preference values */
 		ifopts_options_add(GTK_CLIST(cur_clist), if_info);
-		g_free(text[0]);
 	}
 	
 	free_interface_list(if_list);
@@ -689,12 +732,13 @@ ifopts_if_clist_add(GtkCList *clist)
 
 /*
  * Create/write new interfaces description string based on current CList.
+ * Put it into the preferences value.
  */
 static void
 ifopts_write_new_descr(void)
 {
 	gint	i;
-	gint	first_if = 1;				/* flag to check if 1st in list */
+	gboolean	first_if = TRUE;				/* flag to check if first in list */
 	gchar	*ifnm;
 	gchar	*desc;
 	gchar	*tmp_descr;
@@ -702,28 +746,6 @@ ifopts_write_new_descr(void)
 	
 	/* new preferences interfaces description string */
 	new_descr = g_malloc0(MAX_VAL_LEN);
-	if (new_descr == NULL) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-				"Error (1) saving interface "
-				"descriptions: malloc failure");
-		return;
-	}
-	
-	/*
-	 * current row's interface description string
-	 * (leave space for parens, comma and terminator)
-	 *
-	 * XXX - Currently, MAX_WIN_IF_NAME_LEN is 511. This should be large 
-	 * enough for *nix. ;o)
-	 */
-	tmp_descr = g_malloc0(IFOPTS_MAX_DESCR_LEN + MAX_WIN_IF_NAME_LEN + 4);
-	if (tmp_descr == NULL) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-				"Error (2) saving interface "
-				"descriptions: malloc failure");
-		g_free(new_descr);
-		return;
-	}
 	
 	/* get description for each row (interface) */
 	for (i = 0; ;i++) {
@@ -733,33 +755,22 @@ ifopts_write_new_descr(void)
 		/* if no description, skip this interface */
 		if (strlen(desc) == 0)
 			continue;
+		
 		/* get interface name */
 		gtk_clist_get_text(GTK_CLIST(cur_clist), i, 0, &ifnm);
-		
+
 		/*
 		 * create/cat interface description to new string
 		 * (leave space for parens, comma and terminator)
 		 */
-		if ((strlen(ifnm) + strlen(desc) + 4 + 
-				strlen(new_descr)) <  MAX_VAL_LEN) {
-			if (first_if == 1)
-				g_snprintf(tmp_descr, IFOPTS_MAX_DESCR_LEN+MAX_WIN_IF_NAME_LEN+4,
-						"%s(%s)", ifnm, desc);
-			else
-				g_snprintf(tmp_descr, IFOPTS_MAX_DESCR_LEN+MAX_WIN_IF_NAME_LEN+4,
-						",%s(%s)", ifnm, desc);
-			strcat(new_descr, tmp_descr);
-			/* set first-in-list flag to false */
-			first_if = 0;
-		}
-		/* interface name + description is too large */
-		else {
-			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-					"Error saving interface "
-					"description for:\n%s\n(too long)",
-					ifnm);
-			continue;
-		}
+		if (first_if == TRUE)
+			tmp_descr = g_strdup_printf("%s(%s)", ifnm, desc);
+		else
+			tmp_descr = g_strdup_printf(",%s(%s)", ifnm, desc);
+		strcat(new_descr, tmp_descr);
+        g_free(tmp_descr);
+		/* set first-in-list flag to false */
+		first_if = FALSE;
 	}
 	
 	/* write new description string to preferences */
@@ -777,12 +788,13 @@ ifopts_write_new_descr(void)
 
 /*
  * Create/write new "hidden" interfaces string based on current CList.
+ * Put it into the preferences value.
  */
 static void
 ifopts_write_new_hide(void)
 {
 	gint	i;
-	gint	first_if = 1;				/* flag to check if 1st in list */
+	gint	first_if = TRUE;				/* flag to check if first in list */
 	gchar	*ifnm;
 	gchar	*hide;
 	gchar	*tmp_hide;
@@ -790,28 +802,6 @@ ifopts_write_new_hide(void)
 	
 	/* new preferences "hidden" interfaces string */
 	new_hide = g_malloc0(MAX_VAL_LEN);
-	if (new_hide == NULL) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-				"Error (1) saving \"hidden\" "
-				"interfaces: malloc failure");
-		return;
-	}
-	
-	/*
-	 * current row's interface name if "hidden"
-	 * (leave space for comma and terminator)
-	 *
-	 * XXX - Currently, MAX_WIN_IF_NAME_LEN is 511. This should be large 
-	 * enough for *nix. ;o)
-	 */
-	tmp_hide = g_malloc0(MAX_WIN_IF_NAME_LEN + 2);
-	if (tmp_hide == NULL) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-				"Error (2) saving \"hidden\" "
-				"interfaces: malloc failure");
-		g_free(new_hide);
-		return;
-	}
 	
 	/* get "hidden" flag text for each row (interface) */
 	for (i = 0; ;i++) {
@@ -821,29 +811,22 @@ ifopts_write_new_hide(void)
 		/* if flag text is "No", skip this interface */
 		if (strcmp("No", hide) == 0)
 			continue;
-		/* get interface name */
+
+        /* get interface name */
 		gtk_clist_get_text(GTK_CLIST(cur_clist), i, 0, &ifnm);
 		
 		/*
 		 * create/cat interface to new string
-		 * (leave space for comma and terminator)
 		 */
-		if ((strlen(ifnm) + 2 + strlen(new_hide)) <  MAX_VAL_LEN) {
-			if (first_if == 1)
-				g_snprintf(tmp_hide, MAX_WIN_IF_NAME_LEN+2, "%s", ifnm);
-			else
-				g_snprintf(tmp_hide, MAX_WIN_IF_NAME_LEN+2, ",%s", ifnm);
-			strcat(new_hide, tmp_hide);
-			/* set first-in-list flag to false */
-			first_if = 0;
-		}
-		/* interface name is too large */
-		else {
-			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-					"Error saving \"hidden\" "
-					"interface for:\n%s\n(too long)", ifnm);
-			continue;
-		}
+		if (first_if == TRUE)
+			tmp_hide = g_strdup_printf("%s", ifnm);
+		else
+			tmp_hide = g_strdup_printf(",%s", ifnm);
+
+		strcat(new_hide, tmp_hide);
+        g_free(tmp_hide);
+		/* set first-in-list flag to false */
+		first_if = FALSE;
 	}
 	
 	/* write new "hidden" string to preferences */
