@@ -1,7 +1,7 @@
 /* color_dlg.c
  * Definitions for dialog boxes for color filters
  *
- * $Id: color_dlg.c,v 1.44 2004/03/13 15:15:23 ulfl Exp $
+ * $Id: color_dlg.c,v 1.52 2004/06/05 10:13:37 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -46,6 +46,8 @@
 #include "compat_macros.h"
 #include "filter_prefs.h"
 #include "file_dlg.h"
+#include "gtkglobals.h"
+#include "prefs.h"
 
 static GtkWidget* colorize_dialog_new(char *filter);
 static void add_filter_to_list(gpointer filter_arg, gpointer list_arg);
@@ -83,7 +85,6 @@ static void edit_color_filter_destroy_cb(GObject *object, gpointer user_data);
 static void edit_color_filter_fg_cb(GtkButton *button, gpointer user_data);
 static void edit_color_filter_bg_cb(GtkButton *button, gpointer user_data);
 static void edit_color_filter_ok_cb(GtkButton *button, gpointer user_data);
-static void edit_color_filter_cancel_cb(GtkObject *object, gpointer user_data);
 
 static GtkWidget* color_sel_win_new(color_filter_t *colorf, gboolean);
 static void color_sel_ok_cb(GtkButton *button, gpointer user_data);
@@ -215,21 +216,15 @@ colorize_dialog_new (char *filter)
    * All other widgets are always packed depending on the current window size. */
   color_win = dlg_window_new ("Ethereal: Coloring Rules");
   OBJECT_SET_DATA(color_win, "color_win", color_win);
-  gtk_window_set_default_size(GTK_WINDOW(color_win), 600, 350);  
+  gtk_window_set_default_size(GTK_WINDOW(color_win), DEF_WIDTH, DEF_HEIGHT * 2/3);
   dlg_vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_ref (dlg_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "dlg_vbox", dlg_vbox, gtk_widget_unref);
   gtk_container_set_border_width  (GTK_CONTAINER (dlg_vbox), 5);
   gtk_container_add (GTK_CONTAINER (color_win), dlg_vbox);
 
   main_hbox = gtk_hbox_new (FALSE, 0);
-  gtk_widget_ref (main_hbox);
-  OBJECT_SET_DATA_FULL(color_win, "main_hbox", main_hbox, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (dlg_vbox), main_hbox, TRUE, TRUE, 0);
 
   ctrl_vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_ref (ctrl_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "ctrl_vbox", ctrl_vbox, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (main_hbox), ctrl_vbox, FALSE, FALSE, 0);
 
 
@@ -239,15 +234,11 @@ colorize_dialog_new (char *filter)
 
   /* edit_vbox is first button column (containing: new, edit and such) */
   edit_vbox = gtk_vbutton_box_new();
-  gtk_widget_ref (edit_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "edit_vbox", edit_vbox, gtk_widget_unref);
   gtk_button_box_set_child_size(GTK_BUTTON_BOX(edit_vbox), 50, 20);
   gtk_container_set_border_width  (GTK_CONTAINER (edit_vbox), 5);
   gtk_container_add(GTK_CONTAINER(edit_fr), edit_vbox);
 
   color_new = BUTTON_NEW_FROM_STOCK(GTK_STOCK_NEW);
-  gtk_widget_ref (color_new);
-  OBJECT_SET_DATA_FULL(color_win, "color_new", color_new, gtk_widget_unref);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_new, 50, 20);
 #endif
@@ -255,8 +246,6 @@ colorize_dialog_new (char *filter)
   gtk_tooltips_set_tip (tooltips, color_new, ("Create a new filter at the end of the list"), NULL);
 
   color_props = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_EDIT);
-  gtk_widget_ref (color_props);
-  OBJECT_SET_DATA_FULL(color_win, "color_props", color_props, gtk_widget_unref);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_props, 50, 20);
 #endif
@@ -265,8 +254,6 @@ colorize_dialog_new (char *filter)
   gtk_widget_set_sensitive (color_props, FALSE);
 
   color_delete = BUTTON_NEW_FROM_STOCK(GTK_STOCK_DELETE);
-  gtk_widget_ref (color_delete);
-  OBJECT_SET_DATA_FULL(color_win, "color_delete", color_delete, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (edit_vbox), color_delete, FALSE, FALSE, 5);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_delete, 50, 20);
@@ -281,13 +268,10 @@ colorize_dialog_new (char *filter)
   gtk_box_pack_start (GTK_BOX (ctrl_vbox), manage_fr, FALSE, FALSE, 0);
   
   manage_vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_ref (manage_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "manage_vbox", manage_vbox, gtk_widget_unref);
   gtk_container_set_border_width  (GTK_CONTAINER (manage_vbox), 5);
   gtk_container_add(GTK_CONTAINER(manage_fr), manage_vbox);
 
   color_export = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_EXPORT);
-  gtk_widget_ref(color_export);
   gtk_box_pack_start (GTK_BOX (manage_vbox), color_export, FALSE, FALSE, 5);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_export, 50, 20);
@@ -295,7 +279,6 @@ colorize_dialog_new (char *filter)
   gtk_tooltips_set_tip(tooltips, color_export, ("Save all/marked filters to a file"), NULL);
 
   color_import = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_IMPORT);
-  gtk_widget_ref(color_import);
   gtk_box_pack_start (GTK_BOX (manage_vbox), color_import, FALSE, FALSE, 5);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_import, 50, 20);
@@ -303,8 +286,6 @@ colorize_dialog_new (char *filter)
   gtk_tooltips_set_tip(tooltips, color_import, ("Load filters from a file"), NULL);
 
   color_clear = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLEAR);
-  gtk_widget_ref(color_clear);
-  OBJECT_SET_DATA_FULL(color_win, "color_clear", color_clear, gtk_widget_unref);
   gtk_box_pack_start(GTK_BOX (manage_vbox), color_clear, FALSE, FALSE, 5);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_clear, 50, 20);
@@ -317,14 +298,10 @@ colorize_dialog_new (char *filter)
   gtk_box_pack_start (GTK_BOX (main_hbox), list_fr, TRUE, TRUE, 0);
 
   list_vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_ref (list_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "list_vbox", list_vbox, gtk_widget_unref);
   gtk_container_set_border_width  (GTK_CONTAINER (list_vbox), 5);
   gtk_container_add(GTK_CONTAINER(list_fr), list_vbox);
 
   list_label = gtk_label_new (("[List is processed in order until match is found]"));
-  gtk_widget_ref (list_label);
-  OBJECT_SET_DATA_FULL(color_win, "list_label", list_label, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (list_vbox), list_label, FALSE, FALSE, 0);
 
   /* create the list of filters */
@@ -333,8 +310,6 @@ colorize_dialog_new (char *filter)
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow1), 
                                    GTK_SHADOW_IN);
 #endif
-  gtk_widget_ref (scrolledwindow1);
-  OBJECT_SET_DATA_FULL(color_win, "scrolledwindow1", scrolledwindow1, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (list_vbox), scrolledwindow1, TRUE, TRUE, 0);
 
 #if GTK_MAJOR_VERSION < 2
@@ -385,14 +360,10 @@ colorize_dialog_new (char *filter)
   gtk_box_pack_start (GTK_BOX (main_hbox), order_fr, FALSE, FALSE, 0);
 
   order_vbox = gtk_vbox_new (TRUE, 0);
-  gtk_widget_ref (order_vbox);
-  OBJECT_SET_DATA_FULL(color_win, "order_vbox", order_vbox, gtk_widget_unref);
   gtk_container_set_border_width  (GTK_CONTAINER (order_vbox), 5);
   gtk_container_add(GTK_CONTAINER(order_fr), order_vbox);
 
   color_filter_up = BUTTON_NEW_FROM_STOCK(GTK_STOCK_GO_UP);
-  gtk_widget_ref (color_filter_up);
-  OBJECT_SET_DATA_FULL(color_win, "color_filter_up", color_filter_up, gtk_widget_unref);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_filter_up, 50, 20);
 #endif
@@ -401,14 +372,9 @@ colorize_dialog_new (char *filter)
   gtk_widget_set_sensitive (color_filter_up, FALSE);
 
   order_move_label = gtk_label_new (("Move\nselected filter\nup or down"));
-  gtk_widget_ref (order_move_label);
-  OBJECT_SET_DATA_FULL(color_win, "order_move_label", order_move_label, gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (order_vbox), order_move_label, FALSE, FALSE, 0);
 
   color_filter_down = BUTTON_NEW_FROM_STOCK(GTK_STOCK_GO_DOWN);
-  gtk_widget_ref (color_filter_down);
-  OBJECT_SET_DATA_FULL(color_win, "color_filter_down", color_filter_down,
-                       gtk_widget_unref);
 #if GTK_MAJOR_VERSION < 2
   WIDGET_SET_SIZE(color_filter_down, 50, 20);
 #endif
@@ -422,7 +388,6 @@ colorize_dialog_new (char *filter)
   gtk_box_pack_start (GTK_BOX (dlg_vbox), button_ok_hbox, FALSE, FALSE, 5);
 
   color_ok = OBJECT_GET_DATA(button_ok_hbox, GTK_STOCK_OK);
-  gtk_widget_grab_default(color_ok);
   gtk_tooltips_set_tip (tooltips, color_ok, ("Apply the color filters to the display and close this dialog"), NULL);
 
   color_apply = OBJECT_GET_DATA(button_ok_hbox, GTK_STOCK_APPLY);
@@ -431,10 +396,12 @@ colorize_dialog_new (char *filter)
   color_save = OBJECT_GET_DATA(button_ok_hbox, GTK_STOCK_SAVE);
   gtk_tooltips_set_tip (tooltips, color_save, ("Save the color filters permanently and keep this dialog open"), NULL);
 
-/*  color_cancel = OBJECT_GET_DATA(button_ok_hbox, GTK_STOCK_CANCEL);*/
   color_cancel = OBJECT_GET_DATA(button_ok_hbox, GTK_STOCK_CLOSE);
+  window_set_cancel_button(color_win, color_cancel, color_cancel_cb);
   gtk_tooltips_set_tip (tooltips, color_cancel, ("Close this dialog but don't apply the color filter changes to the display"), NULL);
 
+  gtk_widget_grab_default(color_ok);
+
   /* signals and such */
   SIGNAL_CONNECT(color_win, "destroy", color_destroy_cb, NULL);
   OBJECT_SET_DATA(color_filter_up, COLOR_FILTERS_CL, color_filters);
@@ -466,13 +433,13 @@ colorize_dialog_new (char *filter)
   SIGNAL_CONNECT(color_clear, "clicked", color_clear_cb, NULL);
   SIGNAL_CONNECT(color_ok, "clicked", color_ok_cb, NULL);
   SIGNAL_CONNECT(color_apply, "clicked", color_apply_cb, NULL);
-  SIGNAL_CONNECT(color_cancel, "clicked", color_cancel_cb, NULL);
+
+  SIGNAL_CONNECT(color_win, "delete_event", window_delete_event_cb, NULL);
 
   gtk_widget_grab_focus(color_filters);
-  OBJECT_SET_DATA(color_win, "tooltips", tooltips);
-  gtk_widget_show_all(color_win);
 
-  dlg_set_cancel(color_win, color_cancel);
+  gtk_widget_show_all(color_win);
+  window_present(color_win);
 
   if(filter){
     /* if we specified a preset filter string, open the new dialog and
@@ -852,7 +819,7 @@ destroy_edit_dialog_cb(gpointer filter_arg, gpointer dummy _U_)
   color_filter_t *colorf = (color_filter_t *)filter_arg;
 
   if (colorf->edit_dialog != NULL)
-    gtk_widget_destroy(colorf->edit_dialog);
+    window_destroy(colorf->edit_dialog);
 }
 
 /* XXX - we don't forbid having more than one "Edit color filter" dialog
@@ -925,6 +892,7 @@ static void
 create_new_color_filter(GtkButton *button, char *filter)
 {
   color_filter_t   *colorf;
+  GtkStyle         *style;
   GtkWidget        *color_filters;
 #if GTK_MAJOR_VERSION >= 2
   GtkTreeSelection *sel;
@@ -939,13 +907,19 @@ create_new_color_filter(GtkButton *button, char *filter)
   gtk_clist_unselect_all (GTK_CLIST(color_filters));
 #endif
 
-  colorf = new_color_filter("name", filter); /* Adds at end! */
+  /* Use the default background and foreground colors as the colors. */
+  style = gtk_widget_get_style(packet_list);
+  colorf = new_color_filter("name", filter, &style->base[GTK_STATE_NORMAL],
+                            &style->text[GTK_STATE_NORMAL]); /* Adds at end! */
 
   color_add_colorf(color_filters, colorf);
 
   edit_color_filter_dialog_new(color_filters, &filt_name_entry,
                                &filt_text_entry);
   
+  /* Show the (in)validity of the default filter string */
+  filter_te_syntax_check_cb(filt_text_entry);
+
 #if GTK_MAJOR_VERSION >= 2
   gtk_widget_grab_focus(color_filters);
 #endif
@@ -993,7 +967,7 @@ color_delete(gint row, GtkWidget  *color_filters)
     
     /* Destroy any "Edit color filter" dialog boxes editing it. */
     if (colorf->edit_dialog != NULL)
-    gtk_widget_destroy(colorf->edit_dialog);
+    window_destroy(colorf->edit_dialog);
     
     /* Remove the color filter from the list of color filters. */
     remove_color_filter(colorf);
@@ -1011,7 +985,7 @@ color_delete(gint row, GtkWidget  *color_filters)
 
     /* Destroy any "Edit color filter" dialog boxes editing it. */
     if (colorf->edit_dialog != NULL)
-        gtk_widget_destroy(colorf->edit_dialog);
+        window_destroy(colorf->edit_dialog);
 
     /* Remove the color filter from the list of color filters. */
     remove_color_filter(colorf);
@@ -1087,7 +1061,7 @@ color_clear_cmd(GtkWidget *widget)
 
     /* Destroy the dialog box. */
     /* XXX: is this useful? user might want to continue with editing new colors */
-    gtk_widget_destroy(colorize_win);
+    window_destroy(colorize_win);
 }
 
 /* clear button: user responded to question */
@@ -1110,7 +1084,7 @@ color_clear_cb(GtkWidget *widget, gpointer data _U_) {
     gpointer  dialog;
 
     /* ask user, if he/she is really sure */
-    dialog = simple_dialog(ESD_TYPE_WARN, ESD_BTN_CLEAR | ESD_BTN_CANCEL, 
+    dialog = simple_dialog(ESD_TYPE_CONFIRMATION, ESD_BTN_CLEAR | ESD_BTN_CANCEL, 
                 PRIMARY_TEXT_START "Remove all your personal color settings?" PRIMARY_TEXT_END "\n\n"
                 "This will revert the color settings to global defaults.\n\n"
                 "Are you really sure?");
@@ -1126,7 +1100,7 @@ color_ok_cb(GtkButton *button _U_, gpointer user_data _U_)
   colorize_packets(&cfile);
 
   /* Destroy the dialog box. */
-  gtk_widget_destroy(colorize_win);
+  window_destroy(colorize_win);
 }
 
 /* Exit dialog without colorizing packets with the new list.
@@ -1135,7 +1109,7 @@ static void
 color_cancel_cb(GtkWidget *widget _U_, gpointer user_data _U_)
 {
   /* Destroy the dialog box. */
-  gtk_widget_destroy(colorize_win);
+  window_destroy(colorize_win);
 }
 
 /* Apply new list of color filters to the capture. */
@@ -1200,13 +1174,10 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     /* dialog window */
     edit_dialog = dlg_window_new ("Ethereal: Edit Color Filter");
     gtk_window_set_default_size(GTK_WINDOW(edit_dialog), 500, -1);  
-    /*  gtk_window_set_position(GTK_WINDOW(edit_dialog), GTK_WIN_POS_MOUSE); */
     OBJECT_SET_DATA(edit_dialog, "edit_dialog", edit_dialog);
     colorf->edit_dialog = edit_dialog;
 
     dialog_vbox = gtk_vbox_new (FALSE, 0);
-    gtk_widget_ref (dialog_vbox);
-    OBJECT_SET_DATA_FULL(edit_dialog, "dialog_vbox", dialog_vbox, gtk_widget_unref);
     gtk_container_set_border_width  (GTK_CONTAINER (dialog_vbox), 5);
     gtk_container_add (GTK_CONTAINER (edit_dialog), dialog_vbox);
 
@@ -1215,28 +1186,17 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     gtk_box_pack_start (GTK_BOX (dialog_vbox), filter_fr, FALSE, FALSE, 0);
 
     filter_fr_vbox = gtk_vbox_new (FALSE, 0);
-    gtk_widget_ref (filter_fr_vbox);
-    OBJECT_SET_DATA_FULL(edit_dialog, "filter_fr_vbox", filter_fr_vbox, gtk_widget_unref);
     gtk_container_set_border_width  (GTK_CONTAINER (filter_fr_vbox), 5);
     gtk_container_add(GTK_CONTAINER(filter_fr), filter_fr_vbox);
 
     /* filter name hbox */
     filter_name_hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_ref (filter_name_hbox);
-    OBJECT_SET_DATA_FULL(edit_dialog, "filter_name_hbox", filter_name_hbox,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (filter_fr_vbox), filter_name_hbox, TRUE, FALSE, 3);
 
     color_filter_name = gtk_label_new (("Name: "));
-    gtk_widget_ref (color_filter_name);
-    OBJECT_SET_DATA_FULL(edit_dialog, "color_filter_name", color_filter_name,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (filter_name_hbox), color_filter_name, FALSE, FALSE, 0);
 
     *colorize_filter_name = gtk_entry_new ();
-    gtk_widget_ref (*colorize_filter_name);
-    OBJECT_SET_DATA_FULL(edit_dialog, "*colorize_filter_name", *colorize_filter_name,
-                         gtk_widget_unref);
     gtk_entry_set_text(GTK_ENTRY(*colorize_filter_name), colorf->filter_name);
 
     style = gtk_style_copy(gtk_widget_get_style(*colorize_filter_name));
@@ -1254,24 +1214,15 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
 
     /* filter string hbox */
     filter_string_hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_ref (filter_string_hbox);
-    OBJECT_SET_DATA_FULL(edit_dialog, "filter_string_hbox", filter_string_hbox,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (filter_fr_vbox), filter_string_hbox, TRUE, FALSE, 3);
 
     color_filter_text = gtk_label_new (("String: "));
-    gtk_widget_ref (color_filter_text);
-    OBJECT_SET_DATA_FULL(edit_dialog, "color_filter_text", color_filter_text,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (filter_string_hbox), color_filter_text, FALSE, FALSE, 0);
 
     *colorize_filter_text = gtk_entry_new ();
-    gtk_widget_ref (*colorize_filter_text);
-    OBJECT_SET_DATA_FULL(edit_dialog, "*colorize_filter_text", *colorize_filter_text,
-                         gtk_widget_unref);
+    SIGNAL_CONNECT(*colorize_filter_text, "changed", filter_te_syntax_check_cb, NULL);
     gtk_entry_set_text(GTK_ENTRY(*colorize_filter_text), colorf->filter_text);
 
-
 #if 0
     style = gtk_style_copy(gtk_widget_get_style(*colorize_filter_text));
     style->base[GTK_STATE_NORMAL] = colorf->bg_color;
@@ -1294,22 +1245,14 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     gtk_box_pack_start (GTK_BOX (dialog_vbox), colorize_fr, FALSE, FALSE, 0);
 
     colorize_hbox = gtk_hbox_new (FALSE, 0);
-    gtk_widget_ref (colorize_hbox);
-    OBJECT_SET_DATA_FULL(edit_dialog, "colorize_hbox", colorize_hbox, gtk_widget_unref);
     gtk_container_set_border_width  (GTK_CONTAINER (colorize_hbox), 5);
     gtk_container_add(GTK_CONTAINER(colorize_fr), colorize_hbox);
 
     colorize_filter_fg = gtk_button_new_with_label (("Foreground Color..."));
-    gtk_widget_ref (colorize_filter_fg);
-    OBJECT_SET_DATA_FULL(edit_dialog, "colorize_filter_fg", colorize_filter_fg,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (colorize_hbox), colorize_filter_fg, TRUE, FALSE, 0);
     gtk_tooltips_set_tip (tooltips, colorize_filter_fg, ("Select foreground color for data display"), NULL);
 
     colorize_filter_bg = gtk_button_new_with_label (("Background Color..."));
-    gtk_widget_ref (colorize_filter_bg);
-    OBJECT_SET_DATA_FULL(edit_dialog, "colorize_filter_bg", colorize_filter_bg,
-                         gtk_widget_unref);
     gtk_box_pack_start (GTK_BOX (colorize_hbox), colorize_filter_bg, TRUE, FALSE, 0);
     gtk_tooltips_set_tip (tooltips, colorize_filter_bg, ("Select background color for data display"), NULL);
 
@@ -1320,12 +1263,13 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     gtk_container_set_border_width  (GTK_CONTAINER (bbox), 0);
 
     edit_color_filter_ok = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
-    gtk_widget_grab_default(edit_color_filter_ok);
     gtk_tooltips_set_tip (tooltips, edit_color_filter_ok, ("Accept filter color change"), NULL);
 
     edit_color_filter_cancel = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
+    window_set_cancel_button(edit_dialog, edit_color_filter_cancel, window_cancel_button_cb);
     gtk_tooltips_set_tip (tooltips, edit_color_filter_cancel, ("Reject filter color change"), NULL);
 
+    gtk_widget_grab_default(edit_color_filter_ok);
 
     /* signals and such */
     OBJECT_SET_DATA(edit_dialog, COLOR_FILTER, colorf);
@@ -1337,14 +1281,11 @@ edit_color_filter_dialog_new(GtkWidget *color_filters,
     OBJECT_SET_DATA(edit_color_filter_ok, COLOR_FILTERS_CL, color_filters);
     OBJECT_SET_DATA(edit_color_filter_ok, COLOR_FILTER, colorf);
     SIGNAL_CONNECT(edit_color_filter_ok, "clicked", edit_color_filter_ok_cb, edit_dialog);
-    SIGNAL_CONNECT(edit_color_filter_cancel, "clicked", edit_color_filter_cancel_cb,
-                   edit_dialog);
 
-    OBJECT_SET_DATA(edit_dialog, "tooltips", tooltips);
-
-    dlg_set_cancel(edit_dialog, edit_color_filter_cancel);
+    SIGNAL_CONNECT(edit_dialog, "delete_event", window_delete_event_cb, NULL);
 
     gtk_widget_show_all(edit_dialog);
+    window_present(edit_dialog);
 }
 
 /* Called when the dialog box is being destroyed; destroy any color
@@ -1367,10 +1308,10 @@ edit_color_filter_destroy_cb(GObject *object, gpointer user_data _U_)
   /* Destroy any color selection dialogs this dialog had open. */
   color_sel = (GtkWidget *)OBJECT_GET_DATA(object, COLOR_SELECTION_FG);
   if (color_sel != NULL)
-    gtk_widget_destroy(color_sel);
+    window_destroy(color_sel);
   color_sel = (GtkWidget *)OBJECT_GET_DATA(object, COLOR_SELECTION_BG);
   if (color_sel != NULL)
-    gtk_widget_destroy(color_sel);
+    window_destroy(color_sel);
 }
 
 /* Pop up a color selection box to choose the foreground color. */
@@ -1504,22 +1445,10 @@ edit_color_filter_ok_cb                (GtkButton       *button,
 #endif
 
         /* Destroy the dialog box. */
-        gtk_widget_destroy(dialog);
+        window_destroy(dialog);
     }
 }
 
-/* Exit dialog and do not process list */
-static void
-edit_color_filter_cancel_cb(GtkObject *object _U_, gpointer user_data)
-{
-  GtkWidget *dialog;
-
-  dialog = (GtkWidget *)user_data;
-
-  /* Destroy the dialog box. */
-  gtk_widget_destroy(dialog);
-}
-
 static GtkWidget*
 color_sel_win_new(color_filter_t *colorf, gboolean is_bg)
 {
@@ -1580,7 +1509,6 @@ color_sel_win_new(color_filter_t *colorf, gboolean is_bg)
 
 
   GTK_WIDGET_SET_FLAGS (color_sel_help, GTK_CAN_DEFAULT);
-  SIGNAL_CONNECT(color_sel_win, "destroy", color_sel_cancel_cb, color_sel_win);
 
   SIGNAL_CONNECT(color_sel_ok, "clicked", color_sel_ok_cb, color_sel_win);
   SIGNAL_CONNECT(color_sel_cancel, "clicked", color_sel_cancel_cb, color_sel_win);
@@ -1613,7 +1541,7 @@ color_sel_win_destroy(GtkWidget *sel_win)
   }
 
   /* Now destroy it. */
-  gtk_widget_destroy(sel_win);
+  window_destroy(sel_win);
 }
 
 /* Retrieve selected color */
