@@ -1,7 +1,7 @@
 /* color_filters.h
  * Definitions for color filters
  *
- * $Id: color_filters.h,v 1.5 2004/03/14 23:55:53 deniel Exp $
+ * $Id: color_filters.h,v 1.7 2004/06/01 20:28:04 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -24,17 +24,56 @@
 #ifndef  __COLOR_FILTERS_H__
 #define  __COLOR_FILTERS_H__
 
-#define CFILTERS_CONTAINS_FILTER(filter) \
-	((filter)->num_of_filters != 0)
+/** @file
+ *  Color filters.
+ */
 
+/** Init the color filters. */
 void colfilter_init(void);
 
+/** Save filters in users filter file.
+ *
+ * @return TRUE if write succeeded
+ */
 gboolean write_filters(void);
+
+/** Delete users filter file and reload global filters.
+ *
+ * @return TRUE if write succeeded
+ */
 gboolean revert_filters(void);
 
-color_filter_t *new_color_filter(gchar *name, gchar *filter_string);
+/** Create a new color filter.
+ *
+ * @param name the name of the filter
+ * @param filter_string the filter string
+ * @param bg_color background color
+ * @param fg_color foreground color
+ * @return the new color filter
+ */
+color_filter_t *new_color_filter(gchar *name, gchar *filter_string,
+    GdkColor *bg_color, GdkColor *fg_color);
+
+/** Remove the color filter.
+ *
+ * @param colorf the color filter to be removed
+ */
 void remove_color_filter(color_filter_t *colorf);
+
+/** Load filters from some other filter file.
+ *
+ * @param path the path to the filter file
+ * @param arg the color filter widget
+ * @return TRUE, if read succeeded
+ */
 gboolean read_other_filters(gchar *path, gpointer arg);
+
+/** Save filters to some other filter file.
+ *
+ * @param path the path to the filter file
+ * @param only_marked TRUE if only the marked filters should be saved
+ * @return TRUE, if write succeeded
+ */
 gboolean write_other_filters(gchar *path, gboolean only_marked);
 
 #endif
