@@ -1,7 +1,7 @@
 /* dcerpc_stat.c
  * dcerpc_stat   2002 Ronnie Sahlberg
  *
- * $Id: dcerpc_stat.c,v 1.53 2004/03/13 15:15:23 ulfl Exp $
+ * $Id: dcerpc_stat.c,v 1.58 2004/06/02 07:12:09 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -51,10 +51,9 @@
 #include "filter_prefs.h"
 #include "compat_macros.h"
 #include "service_response_time_table.h"
+#include "gtkglobals.h"
 
 
-extern GtkWidget   *main_display_filter_widget;
-
 /* used to keep track of the statistics for an entire program interface */
 typedef struct _rpcstat_t {
 	GtkWidget *win;
@@ -133,7 +132,7 @@ dcerpcstat_packet(void *rs_arg, packet_info *pinfo, epan_dissect_t *edt _U_, voi
 	}
 
 	/* we are only interested in reply packets */
-	if(ri->request){
+	if(ri->ptype != PDU_RESP){
 		return 0;
 	}
 
@@ -194,6 +193,8 @@ gtk_dcerpcstat_init(char *optarg)
 	GtkWidget *vbox;
 	GtkWidget *stat_label;
 	GtkWidget *filter_label;
+    GtkWidget *bbox;
+    GtkWidget *close_bt;
 	dcerpc_sub_dissector *procs;
 	e_uuid_t uuid;
 	guint d1,d2,d3,d40,d41,d42,d43,d44,d45,d46,d47;
@@ -259,26 +260,22 @@ gtk_dcerpcstat_init(char *optarg)
 	rs->uuid=uuid;
 	rs->ver=ver;
 
-	rs->win=window_new(GTK_WINDOW_TOPLEVEL, NULL);
+	rs->win=window_new(GTK_WINDOW_TOPLEVEL, "dcerpc-stat");
 	dcerpcstat_set_title(rs);
 	gtk_window_set_default_size(GTK_WINDOW(rs->win), 550, 400);
-	SIGNAL_CONNECT(rs->win, "destroy", win_destroy_cb, rs);
 
-	vbox=gtk_vbox_new(FALSE, 0);
+	vbox=gtk_vbox_new(FALSE, 3);
 	gtk_container_add(GTK_CONTAINER(rs->win), vbox);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-	gtk_widget_show(vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 
 	title_string=dcerpcstat_gen_title(rs);
 	stat_label=gtk_label_new(title_string);
 	g_free(title_string);
 	gtk_box_pack_start(GTK_BOX(vbox), stat_label, FALSE, FALSE, 0);
-	gtk_widget_show(stat_label);
 
 	g_snprintf(filter_string,255,"Filter:%s",filter?filter:"");
 	filter_label=gtk_label_new(filter_string);
 	gtk_box_pack_start(GTK_BOX(vbox), filter_label, FALSE, FALSE, 0);
-	gtk_widget_show(filter_label);
 
 	for(i=0,max_procs=0;procs[i].name;i++){
 		if(procs[i].num>max_procs){
@@ -288,7 +285,7 @@ gtk_dcerpcstat_init(char *optarg)
 	rs->num_procedures=max_procs+1;
 
 	/* We must display TOP LEVEL Widget before calling init_srt_table() */
-	gtk_widget_show(rs->win);
+	gtk_widget_show_all(rs->win);
 
 	if(hf_opnum!=-1){
 		init_srt_table(&rs->srt_table, max_procs+1, vbox, proto_registrar_get_nth(hf_opnum)->abbrev);
@@ -321,9 +318,20 @@ gtk_dcerpcstat_init(char *optarg)
 		return;
 	}
 
+	/* Button row. */
+	bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+	gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
-	gtk_widget_show_all(rs->win);
-	retap_packets(&cfile);
+	close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(rs->win, close_bt, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(rs->win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(rs->win, "destroy", win_destroy_cb, rs);
+
+    gtk_widget_show_all(rs->win);
+    window_present(rs->win);
+
+    retap_packets(&cfile);
 }
 
 
@@ -528,13 +536,6 @@ dlg_destroy_cb(void)
 }
 
 
-static void
-dlg_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
-{
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-
 static void
 gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 {
@@ -558,7 +559,7 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	}
 
 	dlg=dlg_window_new("Ethereal: Compute DCE-RPC SRT statistics");
-	SIGNAL_CONNECT(dlg, "destroy", dlg_destroy_cb, NULL);
+	gtk_window_set_default_size(GTK_WINDOW(dlg), 400, -1);
 
 	dlg_box=gtk_vbox_new(FALSE, 10);
 	gtk_container_border_width(GTK_CONTAINER(dlg_box), 10);
@@ -627,7 +628,6 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 
 	/* Filter entry */
 	filter_entry=gtk_entry_new();
-	WIDGET_SET_SIZE(filter_entry, 300, -1);
     SIGNAL_CONNECT(filter_entry, "changed", filter_te_syntax_check_cb, NULL);
 	gtk_box_pack_start(GTK_BOX(filter_box), filter_entry, TRUE, TRUE, 0);
 	filter=gtk_entry_get_text(GTK_ENTRY(main_display_filter_widget));
@@ -647,12 +647,14 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
     gtk_widget_show(bbox);
 
     start_button = OBJECT_GET_DATA(bbox, ETHEREAL_STOCK_CREATE_STAT);
-    gtk_widget_grab_default(start_button );
 	SIGNAL_CONNECT_OBJECT(start_button, "clicked", 
                               dcerpcstat_start_button_clicked, NULL);
 
     cancel_button = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-	SIGNAL_CONNECT(cancel_button, "clicked", dlg_cancel_cb, dlg);
+    window_set_cancel_button(dlg, cancel_button, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(dlg, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(dlg, "destroy", dlg_destroy_cb, NULL);
 
 	/* Catch the "activate" signal on the filter text entry, so that
 	   if the user types Return there, we act as if the "Create Stat"
@@ -661,15 +663,13 @@ gtk_dcerpcstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	   focus. */
 	dlg_set_activate(filter_entry, start_button);
 
-	/* Catch the "key_press_event" signal in the window, so that we can
-	   catch the ESC key being pressed and act as if the "Cancel" button
-	   had been selected. */
-	dlg_set_cancel(dlg, cancel_button);
+    gtk_widget_grab_default(start_button );
 
 	/* Give the initial focus to the "Filter" entry box. */
 	gtk_widget_grab_focus(filter_entry);
 
 	gtk_widget_show_all(dlg);
+    window_present(dlg);
 }
 
 void
