@@ -1,6 +1,6 @@
 /* decode_as_dlg.c
  *
- * $Id: decode_as_dlg.c,v 1.31 2003/03/01 13:08:59 deniel Exp $
+ * $Id: decode_as_dlg.c,v 1.41 2004/06/10 09:46:26 guy Exp $
  *
  * Routines to modify dissector tables on the fly.
  *
@@ -71,7 +71,7 @@ enum srcdst_type {
     E_DECODE_BPORT
 };
 
-#define E_DECODE_MIN_HEIGHT 100
+#define E_DECODE_MIN_HEIGHT 300
 #define E_NOTEBOOK "notebook"
 
 #define E_MENU_SRCDST "menu_src_dst"
@@ -97,7 +97,7 @@ enum srcdst_type {
  * Columns for a "Display" list
  */
 #define E_LIST_D_TABLE	    0
-#define E_LIST_D_PORT	    1
+#define E_LIST_D_SELECTOR   1
 #define E_LIST_D_INITIAL    2
 #define E_LIST_D_CURRENT    3
 #define E_LIST_D_MAX	    E_LIST_D_CURRENT
@@ -151,8 +151,13 @@ static enum action_type	requested_action = -1;
 struct dissector_delete_item {
     /* The name of the dissector table */
     const gchar *ddi_table_name;
-    /* The port number in the dissector table */
-    guint   ddi_port;
+    /* The type of the selector in that dissector table */
+    ftenum_t ddi_selector_type;
+    /* The selector in the dissector table */
+    union {
+	guint   sel_uint;
+	char    *sel_string;
+    } ddi_selector;
 };
 
 /*
@@ -188,14 +193,32 @@ GSList *dissector_reset_list = NULL;
  * @param user_data Unused.
  */
 static void
-decode_build_reset_list (gchar *table_name, gpointer key,
-			 gpointer value _U_, gpointer user_data _U_)
+decode_build_reset_list (gchar *table_name, ftenum_t selector_type,
+			 gpointer key, gpointer value _U_,
+			 gpointer user_data _U_)
 {
     dissector_delete_item_t *item;
 
     item = g_malloc(sizeof(dissector_delete_item_t));
     item->ddi_table_name = table_name;
-    item->ddi_port = GPOINTER_TO_UINT(key);
+    item->ddi_selector_type = selector_type;
+    switch (selector_type) {
+
+    case FT_UINT8:
+    case FT_UINT16:
+    case FT_UINT24:
+    case FT_UINT32:
+	item->ddi_selector.sel_uint = GPOINTER_TO_UINT(key);
+	break;
+
+    case FT_STRING:
+    case FT_STRINGZ:
+	item->ddi_selector.sel_string = key;
+	break;
+
+    default:
+    	g_assert_not_reached();
+    }
     dissector_reset_list = g_slist_prepend(dissector_reset_list, item);
 }
 
@@ -224,8 +247,8 @@ decode_build_reset_list (gchar *table_name, gpointer key,
  * should be stored.
  */
 static void
-decode_build_show_list (gchar *table_name, gpointer key,
-			gpointer value, gpointer user_data)
+decode_build_show_list (gchar *table_name, ftenum_t selector_type,
+			gpointer key, gpointer value, gpointer user_data)
 {
 #if GTK_MAJOR_VERSION < 2
     GtkCList  *clist;
@@ -258,42 +281,59 @@ decode_build_show_list (gchar *table_name, gpointer key,
 	initial_proto_name = dissector_handle_get_short_name(initial);
 
     text[E_LIST_D_TABLE] = get_dissector_table_ui_name(table_name);
-    switch (get_dissector_table_base(table_name)) {
+    switch (selector_type) {
 
-    case BASE_DEC:
-	sprintf(string1, "%u", GPOINTER_TO_UINT(key));
-	break;
+    case FT_UINT8:
+    case FT_UINT16:
+    case FT_UINT24:
+    case FT_UINT32:
+	switch (get_dissector_table_base(table_name)) {
 
-    case BASE_HEX:
-	switch (get_dissector_table_type(table_name)) {
-
-	case FT_UINT8:
-	    sprintf(string1, "0x%02x", GPOINTER_TO_UINT(key));
+	case BASE_DEC:
+	    g_snprintf(string1, sizeof(string1), "%u", GPOINTER_TO_UINT(key));
 	    break;
 
-	case FT_UINT16:
-	    sprintf(string1, "0x%04x", GPOINTER_TO_UINT(key));
+	case BASE_HEX:
+	    switch (get_dissector_table_selector_type(table_name)) {
+
+	    case FT_UINT8:
+		g_snprintf(string1, sizeof(string1), "0x%02x", GPOINTER_TO_UINT(key));
+		break;
+
+	    case FT_UINT16:
+		g_snprintf(string1, sizeof(string1), "0x%04x", GPOINTER_TO_UINT(key));
+		break;
+
+	    case FT_UINT24:
+		g_snprintf(string1, sizeof(string1), "0x%06x", GPOINTER_TO_UINT(key));
+		break;
+
+	    case FT_UINT32:
+		g_snprintf(string1, sizeof(string1), "0x%08x", GPOINTER_TO_UINT(key));
+		break;
+
+	    default:
+		g_assert_not_reached();
+		break;
+	    }
 	    break;
 
-	case FT_UINT24:
-	    sprintf(string1, "0x%06x", GPOINTER_TO_UINT(key));
-	    break;
-
-	case FT_UINT32:
-	    sprintf(string1, "0x%08x", GPOINTER_TO_UINT(key));
-	    break;
-
-	default:
-	    g_assert_not_reached();
+	case BASE_OCT:
+	    g_snprintf(string1, sizeof(string1), "%#o", GPOINTER_TO_UINT(key));
 	    break;
 	}
+	text[E_LIST_D_SELECTOR] = string1;
 	break;
 
-    case BASE_OCT:
-	sprintf(string1, "%#o", GPOINTER_TO_UINT(key));
+    case FT_STRING:
+    case FT_STRINGZ:
+	text[E_LIST_D_SELECTOR] = key;
+	break;
+
+    default:
+	g_assert_not_reached();
 	break;
     }
-    text[E_LIST_D_PORT] = string1;
     text[E_LIST_D_INITIAL] = initial_proto_name;
     text[E_LIST_D_CURRENT] = current_proto_name;
 #if GTK_MAJOR_VERSION < 2
@@ -301,7 +341,7 @@ decode_build_show_list (gchar *table_name, gpointer key,
 #else
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter, E_LIST_D_TABLE, text[E_LIST_D_TABLE],
-                       E_LIST_D_PORT, text[E_LIST_D_PORT],
+                       E_LIST_D_SELECTOR, text[E_LIST_D_SELECTOR],
                        E_LIST_D_INITIAL, text[E_LIST_D_INITIAL],
                        E_LIST_D_CURRENT, text[E_LIST_D_CURRENT], -1);
 #endif
@@ -320,22 +360,22 @@ decode_build_show_list (gchar *table_name, gpointer key,
 static void
 decode_show_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
 {
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
+    window_destroy(GTK_WIDGET(parent_w));
 }
 
 
 /*
- * This routine is called when the user clicks the "Reset" button in
+ * This routine is called when the user clicks the "Clear" button in
  * the "Decode As:Show..." dialog window.  This routine resets all the
  * dissector values and then destroys the dialog box and performs
  * other housekeeping functions.
  *
- * @param GtkWidget * A pointer to the "Reset" button.
+ * @param GtkWidget * A pointer to the "Clear" button.
  *
  * @param gpointer A pointer to the dialog window.
  */
 static void
-decode_show_reset_cb (GtkWidget *reset_bt _U_, gpointer parent_w)
+decode_show_clear_cb (GtkWidget *clear_bt _U_, gpointer parent_w)
 {
     dissector_delete_item_t *item;
     GSList *tmp;
@@ -344,7 +384,24 @@ decode_show_reset_cb (GtkWidget *reset_bt _U_, gpointer parent_w)
 
     for (tmp = dissector_reset_list; tmp; tmp = g_slist_next(tmp)) {
 	item = tmp->data;
-	dissector_reset(item->ddi_table_name, item->ddi_port);
+	switch (item->ddi_selector_type) {
+
+	case FT_UINT8:
+	case FT_UINT16:
+	case FT_UINT24:
+	case FT_UINT32:
+	    dissector_reset(item->ddi_table_name, item->ddi_selector.sel_uint);
+	    break;
+
+	case FT_STRING:
+	case FT_STRINGZ:
+	    dissector_reset_string(item->ddi_table_name,
+				   item->ddi_selector.sel_string);
+	    break;
+
+	default:
+	    g_assert_not_reached();
+	}
 	g_free(item);
     }
     g_slist_free(dissector_reset_list);
@@ -352,7 +409,7 @@ decode_show_reset_cb (GtkWidget *reset_bt _U_, gpointer parent_w)
 
     redissect_packets(&cfile);
 
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
+    window_destroy(GTK_WIDGET(parent_w));
 }
 
 
@@ -402,7 +459,7 @@ decode_show_destroy_cb (GtkWidget *win _U_, gpointer user_data _U_)
 void
 decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
 {
-    GtkWidget         *main_vb, *bbox, *ok_bt, *button, *scrolled_window;
+    GtkWidget         *main_vb, *bbox, *ok_bt, *clear_bt, *scrolled_window;
     gchar             *titles[E_LIST_D_COLUMNS] = {
         "Table", "Port", "Initial", "Current"
     };
@@ -424,8 +481,8 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
     }
 
     decode_show_w = dlg_window_new("Ethereal: Decode As: Show");
-    SIGNAL_CONNECT(decode_show_w, "delete_event", decode_show_delete_cb, NULL);
-    SIGNAL_CONNECT(decode_show_w, "destroy", decode_show_destroy_cb, NULL);
+	/* Provide a minimum of a couple of rows worth of data */
+    gtk_window_set_default_size(GTK_WINDOW(decode_show_w), -1, E_DECODE_MIN_HEIGHT);
 
     /* Container for each row of widgets */
     main_vb = gtk_vbox_new(FALSE, 2);
@@ -469,43 +526,45 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
 #endif
 
 	/* Put clist into a scrolled window */
-	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+	scrolled_window = scrolled_window_new(NULL, NULL);
+    /* this will result to set the width of the dialog to the required size */
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+#if GTK_MAJOR_VERSION >= 2
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
+                                   GTK_SHADOW_IN);
+#endif
 	gtk_container_add(GTK_CONTAINER(scrolled_window),
                           GTK_WIDGET(list));
 	gtk_box_pack_start(GTK_BOX(main_vb), scrolled_window, TRUE, TRUE, 0);
-	/* Provide a minimum of a couple of rows worth of data */
-	WIDGET_SET_SIZE(scrolled_window, -1, E_DECODE_MIN_HEIGHT);
     }
 
-    /* Button row: OK and reset buttons */
-    bbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
-    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 10);
+    /* Button row */
+    bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_CLEAR, NULL);
+    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 0);
+    gtk_widget_show(bbox);
 
-    button = gtk_button_new_with_label("Reset Changes");
-    SIGNAL_CONNECT(button, "clicked", decode_show_reset_cb, decode_show_w);
-    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-#if GTK_MAJOR_VERSION < 2
-    gtk_widget_set_sensitive(button, (list->rows != 0));
-
-    ok_bt = gtk_button_new_with_label("OK");
-#else
-    gtk_widget_set_sensitive(button,
-                             gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter));
-
-    ok_bt = gtk_button_new_from_stock(GTK_STOCK_OK);
-#endif
+    ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
     SIGNAL_CONNECT(ok_bt, "clicked", decode_show_ok_cb, decode_show_w);
-    GTK_WIDGET_SET_FLAGS(ok_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), ok_bt, FALSE, FALSE, 0);
-    gtk_widget_grab_default(ok_bt);
-    dlg_set_cancel(decode_show_w, ok_bt);
+    
+    clear_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLEAR);
+    SIGNAL_CONNECT(clear_bt, "clicked", decode_show_clear_cb, decode_show_w);
+
+    /* set ok as default, this button won't change anything */
+    window_set_cancel_button(decode_show_w, ok_bt, NULL);
+
+    SIGNAL_CONNECT(decode_show_w, "delete_event", decode_show_delete_cb, NULL);
+    SIGNAL_CONNECT(decode_show_w, "destroy", decode_show_destroy_cb, NULL);
+    
+#if GTK_MAJOR_VERSION < 2
+    gtk_widget_set_sensitive(clear_bt, (list->rows != 0));
+#else
+    gtk_widget_set_sensitive(clear_bt,
+                             gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter));
+#endif
 
     gtk_widget_show_all(decode_show_w);
+    window_present(decode_show_w);
 }
 
 
@@ -538,7 +597,7 @@ decode_show_cb (GtkWidget * w _U_, gpointer data _U_)
  * buffer.
  */
 static void
-decode_change_one_dissector(gchar *table_name, gint selector, GtkWidget *list)
+decode_change_one_dissector(gchar *table_name, guint selector, GtkWidget *list)
 {
     dissector_handle_t handle;
     gchar              *abbrev;
@@ -608,19 +667,18 @@ decode_debug (GtkCList *clist, gchar *leadin)
     dissector_handle_t handle;
     gint row;
 
-    string = g_malloc(1024);
     if (clist->selection) {
 	row = GPOINTER_TO_INT(clist->selection->data);
 	gtk_clist_get_text(clist, row, E_LIST_S_PROTO_NAME, &text[E_LIST_S_PROTO_NAME]);
 	gtk_clist_get_text(clist, row, E_LIST_S_TABLE, &text[E_LIST_S_TABLE]);
 	handle = gtk_clist_get_row_data(clist, row);
-	sprintf(string, "%s clist row %d: <put handle here>, name %s, table %s",
+	string = g_strdup_printf("%s clist row %d: <put handle here>, name %s, table %s",
 		leadin, row, text[E_LIST_S_PROTO_NAME],
 		text[E_LIST_S_TABLE]);
     } else {
-	sprintf(string, "%s clist row (none), aka do not decode", leadin);
+	string = g_strdup_printf("%s clist row (none), aka do not decode", leadin);
     }
-    simple_dialog(ESD_TYPE_INFO, NULL, string);
+    simple_dialog(ESD_TYPE_INFO, ESD_BTN_OK, string);
     g_free(string);
 }
 #endif
@@ -644,7 +702,7 @@ decode_simple (GtkWidget *notebook_pg)
     gchar *string;
 #endif
     gchar *table_name;
-    gint value;
+    guint value;
 
     list = OBJECT_GET_DATA(notebook_pg, E_PAGE_LIST);
     if (requested_action == E_DECODE_NO)
@@ -660,7 +718,7 @@ decode_simple (GtkWidget *notebook_pg)
 #endif
 
     table_name = OBJECT_GET_DATA(notebook_pg, E_PAGE_TABLE);
-    value = GPOINTER_TO_INT(OBJECT_GET_DATA(notebook_pg, E_PAGE_VALUE));
+    value = GPOINTER_TO_UINT(OBJECT_GET_DATA(notebook_pg, E_PAGE_VALUE));
     decode_change_one_dissector(table_name, value, list);
 }
 
@@ -739,7 +797,7 @@ decode_ok_cb (GtkWidget *ok_bt _U_, gpointer parent_w)
     func(notebook_pg);
 
     /* Now destroy the "Decode As" dialog. */
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
+    window_destroy(GTK_WIDGET(parent_w));
     g_slist_free(decode_dimmable);
     decode_dimmable = NULL;
 
@@ -787,7 +845,7 @@ decode_apply_cb (GtkWidget *apply_bt _U_, gpointer parent_w)
 static void
 decode_cancel_cb (GtkWidget *cancel_bt _U_, gpointer parent_w)
 {
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
+    window_destroy(GTK_WIDGET(parent_w));
     g_slist_free(decode_dimmable);
     decode_dimmable = NULL;
 }
@@ -934,13 +992,13 @@ decode_add_srcdst_menu (GtkWidget *page)
 
     optmenu = gtk_option_menu_new();
     menu = gtk_menu_new();
-    sprintf(tmp, "source (%u)", cfile.edt->pi.srcport);
+    g_snprintf(tmp, 100, "source (%u)", cfile.edt->pi.srcport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_SPORT));
     gtk_menu_append(GTK_MENU(menu), menuitem);
     gtk_widget_show(menuitem);	/* gtk_widget_show_all() doesn't show this */
 
-    sprintf(tmp, "destination (%u)", cfile.edt->pi.destport);
+    g_snprintf(tmp, 100, "destination (%u)", cfile.edt->pi.destport);
     menuitem = gtk_menu_item_new_with_label(tmp);
     OBJECT_SET_DATA(menuitem, "user_data", GINT_TO_POINTER(E_DECODE_DPORT));
     gtk_menu_append(GTK_MENU(menu), menuitem);
@@ -1020,6 +1078,7 @@ decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
     GtkTreeIter   iter;
     struct handle_lookup_info hli;
 #endif
+    gint       i;
 
     g_assert(user_data);
     g_assert(value);
@@ -1028,6 +1087,10 @@ decode_add_to_list (gchar *table_name, gpointer value, gpointer user_data)
     handle = value;
     proto_name = dissector_handle_get_short_name(handle);
 
+    i = dissector_handle_get_protocol_index(handle);
+    if (i >= 0 && !proto_is_protocol_enabled(find_protocol_by_id(i)))
+        return;
+  
 #if GTK_MAJOR_VERSION < 2
     row = gtk_clist_find_row_from_data(list, handle);
     /* We already have an entry for this handle.
@@ -1075,7 +1138,6 @@ static void
 decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
                        GtkWidget **scrolled_win_p)
 {
-    GtkWidget *window;
 #if GTK_MAJOR_VERSION < 2
     gchar             *titles[E_LIST_S_COLUMNS] = {"Short Name", "Table Name"};
     GtkCList          *list;
@@ -1115,16 +1177,19 @@ decode_list_menu_start(GtkWidget *page, GtkWidget **list_p,
     g_object_set_data(G_OBJECT(page), E_PAGE_LIST, list);
 #endif
 
-    *scrolled_win_p = window = gtk_scrolled_window_new(NULL, NULL);
-    /* Provide a minimum of a couple of rows worth of data */
-    WIDGET_SET_SIZE(window, -1, E_DECODE_MIN_HEIGHT);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(window),
+    *scrolled_win_p = scrolled_window_new(NULL, NULL);
+    /* this will result to set the width of the dialog to the required size */
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*scrolled_win_p),
 				   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+#if GTK_MAJOR_VERSION >= 2
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(*scrolled_win_p), 
+                                   GTK_SHADOW_IN);
+#endif
 #if GTK_MAJOR_VERSION < 2
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(window),
+    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(*scrolled_win_p),
 					  GTK_WIDGET(list));
 #else
-    gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(list));
+    gtk_container_add(GTK_CONTAINER(*scrolled_win_p), GTK_WIDGET(list));
 #endif
 
     *list_p = GTK_WIDGET(list);
@@ -1222,7 +1287,7 @@ decode_add_simple_menu (GtkWidget *page, gchar *table_name)
  */
 static GtkWidget *
 decode_add_simple_page (gchar *prompt, gchar *title, gchar *table_name,
-			gint value)
+			guint value)
 {
     GtkWidget	*page, *label, *scrolled_window;
 
@@ -1230,7 +1295,7 @@ decode_add_simple_page (gchar *prompt, gchar *title, gchar *table_name,
     OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_simple);
     OBJECT_SET_DATA(page, E_PAGE_TABLE, table_name);
     OBJECT_SET_DATA(page, E_PAGE_TITLE, title);
-    OBJECT_SET_DATA(page, E_PAGE_VALUE, GINT_TO_POINTER(value));
+    OBJECT_SET_DATA(page, E_PAGE_VALUE, GUINT_TO_POINTER(value));
 
     /* Always enabled */
     label = gtk_label_new(prompt);
@@ -1334,7 +1399,7 @@ decode_add_notebook (GtkWidget *format_hb)
 
     /* Add link level selection page */
     if (cfile.edt->pi.ethertype) {
-	sprintf(buffer, "Ethertype 0x%04x", cfile.edt->pi.ethertype);
+	g_snprintf(buffer, 40, "Ethertype 0x%04x", cfile.edt->pi.ethertype);
 	page = decode_add_simple_page(buffer, "Link", "ethertype", cfile.edt->pi.ethertype);
 	label = gtk_label_new("Link");
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
@@ -1345,7 +1410,7 @@ decode_add_notebook (GtkWidget *format_hb)
 	/*
 	 * The network-layer protocol is IP.
 	 */
-	sprintf(buffer, "IP protocol %u", cfile.edt->pi.ipproto);
+	g_snprintf(buffer, 40, "IP protocol %u", cfile.edt->pi.ipproto);
 	page = decode_add_simple_page(buffer, "Network", "ip.proto", cfile.edt->pi.ipproto);
 	OBJECT_SET_DATA(page, E_PAGE_ACTION, decode_simple);
 	label = gtk_label_new("Network");
@@ -1407,8 +1472,8 @@ decode_as_cb (GtkWidget * w _U_, gpointer data _U_)
 
     requested_action = E_DECODE_YES;
     decode_w = dlg_window_new("Ethereal: Decode As");
-    SIGNAL_CONNECT(decode_w, "delete_event", decode_delete_cb, NULL);
-    SIGNAL_CONNECT(decode_w, "destroy", decode_destroy_cb, NULL);
+	/* Provide a minimum of a couple of rows worth of data */
+    gtk_window_set_default_size(GTK_WINDOW(decode_w), -1, E_DECODE_MIN_HEIGHT);
 
     /* Container for each row of widgets */
     main_vb = gtk_vbox_new(FALSE, 2);
@@ -1423,56 +1488,36 @@ decode_as_cb (GtkWidget * w _U_, gpointer data _U_)
 	button_vb = decode_add_yes_no();
 	gtk_box_pack_start(GTK_BOX(format_hb), button_vb, TRUE, TRUE, 10);
 
-	decode_add_notebook(format_hb);
-    }
-
-    /* Button row: OK, Apply and cancel buttons */
-    bbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
-    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 10);
-
-#if GTK_MAJOR_VERSION < 2
-    ok_bt = gtk_button_new_with_label("OK");
-#else
-    ok_bt = gtk_button_new_from_stock(GTK_STOCK_OK);
-#endif
-    SIGNAL_CONNECT(ok_bt, "clicked", decode_ok_cb, decode_w);
-    GTK_WIDGET_SET_FLAGS(ok_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), ok_bt, FALSE, FALSE, 0);
-    gtk_widget_grab_default(ok_bt);
-
-#if GTK_MAJOR_VERSION < 2
-    apply_bt = gtk_button_new_with_label("Apply");
-#else
-    apply_bt = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-#endif
-    SIGNAL_CONNECT(apply_bt, "clicked", decode_apply_cb, decode_w);
-    GTK_WIDGET_SET_FLAGS(apply_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), apply_bt, FALSE, FALSE, 0);
-
     button = gtk_button_new_with_label("Show Current");
     SIGNAL_CONNECT(button, "clicked", decode_show_cb, decode_w);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(button_vb), button, FALSE, FALSE, 0);
 
-#if GTK_MAJOR_VERSION < 2
-    cancel_bt = gtk_button_new_with_label("Cancel");
-#else
-    cancel_bt = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-#endif
+	decode_add_notebook(format_hb);
+    }
+
+    /* Button row */
+    bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_CANCEL, NULL);
+    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 0);
+    gtk_widget_show(bbox);
+
+    ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
+    SIGNAL_CONNECT(ok_bt, "clicked", decode_ok_cb, decode_w);
+
+    apply_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_APPLY);
+    SIGNAL_CONNECT(apply_bt, "clicked", decode_apply_cb, decode_w);
+
+    cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
+    window_set_cancel_button(decode_w, cancel_bt, NULL);
     SIGNAL_CONNECT(cancel_bt, "clicked", decode_cancel_cb, decode_w);
-    GTK_WIDGET_SET_FLAGS(cancel_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), cancel_bt, FALSE, FALSE, 0);
 
-    /*
-     * Catch the "key_press_event" signal in the window, so that
-     * we can catch the ESC key being pressed and act as if the
-     * "Cancel" button had been selected.
-     */
-    dlg_set_cancel(decode_w, cancel_bt);
+    gtk_widget_grab_default(ok_bt);
+
+    SIGNAL_CONNECT(decode_w, "delete_event", decode_delete_cb, NULL);
+    SIGNAL_CONNECT(decode_w, "destroy", decode_destroy_cb, NULL);
 
     gtk_widget_show_all(decode_w);
+    window_present(decode_w);
 }
 
 
