@@ -1,7 +1,7 @@
 /* dlg_utils.c
  * Utilities to use when constructing dialogs
  *
- * $Id: dlg_utils.c,v 1.21 2004/02/13 00:53:35 guy Exp $
+ * $Id: dlg_utils.c,v 1.39 2004/06/20 19:35:04 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -29,20 +29,36 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include <epan/filesystem.h>
+
+#include "globals.h"
+
 #include "gtkglobals.h"
 #include "ui_util.h"
+#include "dlg_utils.h"
+#include "keys.h"
 #include "compat_macros.h"
+#include "main.h"
 
 #include <string.h>
 #include <stdio.h>
 #include <stdarg.h>
 
+
+/* Keys ... */
+#define E_FS_CALLER_PTR_KEY       "fs_caller_ptr"
+
+static gchar *last_open_dir = NULL;
+static gboolean updated_last_open_dir = FALSE;
+
+
 static void
 dlg_activate (GtkWidget *widget, gpointer ok_button);
 
-static gint
-dlg_key_press (GtkWidget *widget, GdkEventKey *event, gpointer cancel_button);
-
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 4) || GTK_MAJOR_VERSION < 2
+static void file_selection_browse_ok_cb(GtkWidget *w, gpointer data);
+#endif
+static void file_selection_browse_destroy_cb(GtkWidget *win, GtkWidget* file_te);
 
 
 /* create a button for the button row (helper for dlg_button_row_new) */
@@ -285,6 +301,19 @@ dlg_button_row_new(gchar *stock_id_first, ...)
 }
 
 
+/* this is called, when a dialog was closed */
+static void dlg_destroy_cb(GtkWidget *dialog _U_, gpointer data	_U_)
+{
+#if GTK_MAJOR_VERSION >= 2
+    if(top_level) {
+        /* bring main window back to front (workaround for a bug in win32 GTK2.x)
+           XXX - do this only on Windows? */
+        gtk_window_present(GTK_WINDOW(top_level));
+    }
+#endif
+}
+
+
 /* Create a dialog box window that belongs to Ethereal's main window. */
 GtkWidget *
 dlg_window_new(const gchar *title)
@@ -296,6 +325,7 @@ dlg_window_new(const gchar *title)
 #else
   win = window_new(GTK_WINDOW_TOPLEVEL, title);
 #endif
+
   /*
    * XXX - if we're running in the capture child process, we can't easily
    * make this window transient for the main process's window.  We just
@@ -313,16 +343,56 @@ dlg_window_new(const gchar *title)
   if (top_level) {
     gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(top_level));
   }
-#if GTK_MAJOR_VERSION >= 2
-  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
-#endif
+
+  SIGNAL_CONNECT(win, "destroy", dlg_destroy_cb, NULL);
+
   return win;
 }
 
+
 /* Create a file selection dialog box window that belongs to Ethereal's
    main window. */
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
 GtkWidget *
-file_selection_new(const gchar *title)
+file_selection_new(const gchar *title, file_selection_action_t action)
+{
+  GtkWidget *win;
+  GtkFileChooserAction gtk_action;
+  const gchar *ok_button_text;
+
+  switch (action) {
+
+  case FILE_SELECTION_OPEN:
+    gtk_action = GTK_FILE_CHOOSER_ACTION_OPEN;
+    ok_button_text = GTK_STOCK_OPEN;
+    break;
+
+  case FILE_SELECTION_SAVE:
+    gtk_action = GTK_FILE_CHOOSER_ACTION_SAVE;
+    ok_button_text = GTK_STOCK_SAVE;
+    break;
+
+  default:
+    g_assert_not_reached();
+    gtk_action = -1;
+    ok_button_text = NULL;
+    break;
+  }
+  win = gtk_file_chooser_dialog_new(title, GTK_WINDOW(top_level), gtk_action,
+                                    ok_button_text, GTK_RESPONSE_ACCEPT,
+                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                    NULL);
+
+  /* If we've opened a file before, start out by showing the files in the directory
+     in which that file resided. */
+  if (last_open_dir)
+    file_selection_set_current_folder(win, last_open_dir);
+
+  return win;
+}
+#else
+GtkWidget *
+file_selection_new(const gchar *title, file_selection_action_t action _U_)
 {
   GtkWidget *win;
 
@@ -331,8 +401,208 @@ file_selection_new(const gchar *title)
   gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
 #endif
   gtk_window_set_transient_for(GTK_WINDOW(win), GTK_WINDOW(top_level));
+
+  /* XXX - why are we doing this?  We don't do it with the GtkFileChooser,
+     as it complains that the file name isn't being set to an absolute
+     path; does this provoke a similar complaint? */
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(win), "");
+
+  /* If we've opened a file before, start out by showing the files in the directory
+     in which that file resided. */
+  if (last_open_dir)
+    file_selection_set_current_folder(win, last_open_dir);
+
   return win;
 }
+#endif
+
+/* Set the current folder for a file selection dialog. */
+gboolean
+file_selection_set_current_folder(GtkWidget *fs, const gchar *filename)
+{
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    gboolean ret;
+    int filename_len = strlen(filename);
+    gchar *new_filename;
+
+    /* trim filename, so gtk_file_chooser_set_current_folder() likes it */
+    if (filename[filename_len -1] == G_DIR_SEPARATOR && filename[1] != ':') {
+        new_filename = g_strdup(filename);
+	    new_filename[filename_len-1] = '\0';
+    } else {
+        new_filename = g_strdup(filename);
+    }
+
+    /* this function is very pedantic about it's filename parameter */
+    /* no trailing '\' allowed, unless a win32 root dir "D:\" */
+    ret = gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fs), new_filename);
+    g_free(new_filename);
+    return ret;
+#else
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), filename);
+    return TRUE;
+#endif
+}
+
+/* Set the "extra" widget for a file selection dialog, with user-supplied
+   options. */
+void
+file_selection_set_extra_widget(GtkWidget *fs, GtkWidget *extra)
+{
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+  gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(fs), extra);
+#else
+  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(fs)->action_area), extra,
+                     FALSE, FALSE, 0);
+#endif
+}
+
+
+/*
+ * A generic select_file routine that is intended to be connected to
+ * a Browse button on other dialog boxes. This allows the user to browse
+ * for a file and select it. We fill in the text_entry that is given to us. 
+ *
+ * We display the window label specified in our args.
+ */
+void
+file_selection_browse(GtkWidget *file_bt, GtkWidget *file_te, const char *label, file_selection_action_t action)
+{
+  GtkWidget *caller = gtk_widget_get_toplevel(file_bt);
+  GtkWidget *fs;
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+  gchar     *f_name;
+#endif
+
+  /* Has a file selection dialog box already been opened for that top-level
+     widget? */
+  fs = OBJECT_GET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY);
+  if (fs != NULL) {
+    /* Yes.  Just re-activate that dialog box. */
+    reactivate_window(fs);
+    return;
+  }
+
+  fs = file_selection_new(label, action);
+
+  OBJECT_SET_DATA(fs, PRINT_FILE_TE_KEY, file_te);
+
+  /* Set the E_FS_CALLER_PTR_KEY for the new dialog to point to our caller. */
+  OBJECT_SET_DATA(fs, E_FS_CALLER_PTR_KEY, caller);
+
+  /* Set the E_FILE_SEL_DIALOG_PTR_KEY for the caller to point to us */
+  OBJECT_SET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY, fs);
+
+  /* Call a handler when the file selection box is destroyed, so we can inform
+     our caller, if any, that it's been destroyed. */
+  SIGNAL_CONNECT(fs, "destroy", GTK_SIGNAL_FUNC(file_selection_browse_destroy_cb), 
+		 file_te);
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+  if (gtk_dialog_run(GTK_DIALOG(fs)) == GTK_RESPONSE_ACCEPT)
+  {
+      f_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
+      gtk_entry_set_text(GTK_ENTRY(file_te), f_name);
+      g_free(f_name);
+  }
+  window_destroy(fs);
+#else
+  SIGNAL_CONNECT(GTK_FILE_SELECTION(fs)->ok_button, "clicked", 
+		 file_selection_browse_ok_cb, fs);
+
+  window_set_cancel_button(fs, GTK_FILE_SELECTION(fs)->cancel_button, NULL);
+
+  SIGNAL_CONNECT(fs, "delete_event", window_delete_event_cb, fs);
+
+  gtk_widget_show(fs);
+  window_present(fs);
+#endif
+}
+
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 4) || GTK_MAJOR_VERSION < 2
+static void
+file_selection_browse_ok_cb(GtkWidget *w _U_, gpointer data)
+{
+  gchar     *f_name;
+  GtkWidget *win = data;
+
+  f_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION (data)));
+
+  /* Perhaps the user specified a directory instead of a file.
+     Check whether they did. */
+  if (test_for_directory(f_name) == EISDIR) {
+        /* It's a directory - set the file selection box to display it. */
+        set_last_open_dir(f_name);
+        g_free(f_name);
+        file_selection_set_current_folder(data, last_open_dir);
+        return;
+  }
+
+  gtk_entry_set_text(GTK_ENTRY(OBJECT_GET_DATA(win, PRINT_FILE_TE_KEY)),
+                     f_name);
+  window_destroy(GTK_WIDGET(win));
+
+  g_free(f_name);
+}
+#endif
+
+static void
+file_selection_browse_destroy_cb(GtkWidget *win, GtkWidget* parent_te)
+{
+  GtkWidget *caller;
+
+  /* Get the widget that requested that we be popped up.
+     (It should arrange to destroy us if it's destroyed, so
+     that we don't get a pointer to a non-existent window here.) */
+  caller = OBJECT_GET_DATA(win, E_FS_CALLER_PTR_KEY);
+
+  /* Tell it we no longer exist. */
+  OBJECT_SET_DATA(caller, E_FILE_SEL_DIALOG_PTR_KEY, NULL);
+
+  /* Give the focus to the file text entry widget so the user can just press
+     Return to print to the file. */
+  gtk_widget_grab_focus(parent_te);
+}
+
+
+void
+set_last_open_dir(char *dirname)
+{
+	int len;
+	gchar *new_last_open_dir;
+
+	if (dirname) {
+		len = strlen(dirname);
+		if (dirname[len-1] == G_DIR_SEPARATOR) {
+			new_last_open_dir = g_strconcat(dirname, NULL);
+		}
+		else {
+			new_last_open_dir = g_strconcat(dirname,
+				G_DIR_SEPARATOR_S, NULL);
+		}
+
+		if (last_open_dir == NULL ||
+		    strcmp(last_open_dir, new_last_open_dir) != 0)
+			updated_last_open_dir = TRUE;
+	}
+	else {
+		new_last_open_dir = NULL;
+		if (last_open_dir != NULL)
+			updated_last_open_dir = TRUE;
+	}
+
+	if (last_open_dir) {
+		g_free(last_open_dir);
+	}
+	last_open_dir = new_last_open_dir;
+}
+
+char *
+get_last_open_dir(void)
+{
+    return last_open_dir;
+}
 
 /* Set the "activate" signal for a widget to call a routine to
    activate the "OK" button for a dialog box.
@@ -355,37 +625,6 @@ dlg_activate (GtkWidget *widget _U_, gpointer ok_button)
   gtk_widget_activate(GTK_WIDGET(ok_button));
 }
 
-/* Set the "key_press_event" signal for a top-level dialog window to
-   call a routine to activate the "Cancel" button for a dialog box if
-   the key being pressed is the <Esc> key.
-
-   XXX - there should be a GTK+ widget that'll do that for you, and
-   let you specify a "Cancel" button.  It should also not impose
-   a requirement that there be a separator in the dialog box, as
-   the GtkDialog widget does; the visual convention that there's
-   such a separator between the rest of the dialog boxes and buttons
-   such as "OK" and "Cancel" is, for better or worse, not universal
-   (not even in GTK+ - look at the GtkFileSelection dialog!). */
-void
-dlg_set_cancel(GtkWidget *widget, GtkWidget *cancel_button)
-{
-  SIGNAL_CONNECT(widget, "key_press_event", dlg_key_press, cancel_button);
-}
-
-static gint
-dlg_key_press (GtkWidget *widget, GdkEventKey *event, gpointer cancel_button)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  if (event->keyval == GDK_Escape) {
-    gtk_widget_activate(GTK_WIDGET(cancel_button));
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
 #if GTK_MAJOR_VERSION < 2
 /* Sigh.  GTK+ appears not to acknowledge that it should be possible
    to attach mnemonics to anything other than menu items; provide
