@@ -3,7 +3,7 @@
  * (This used to be a notebook page under "Preferences", hence the
  * "prefs" in the file name.)
  *
- * $Id: filter_prefs.c,v 1.44 2003/03/02 13:46:01 deniel Exp $
+ * $Id: filter_prefs.c,v 1.67 2004/06/05 10:27:38 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -42,6 +42,8 @@
 #include "simple_dialog.h"
 #include "dfilter_expr_dlg.h"
 #include "compat_macros.h"
+#include "gtkglobals.h"
+#include "help_dlg.h"
 
 #define E_FILT_DIALOG_PTR_KEY       "filter_dialog_ptr"
 #define E_FILT_BUTTON_PTR_KEY       "filter_button_ptr"
@@ -54,7 +56,6 @@
 #define E_FILT_COPY_BT_KEY          "filter_copy_bt"
 #define E_FILT_DEL_BT_KEY           "filter_del_bt"
 #define E_FILT_NAME_TE_KEY          "filter_name_te"
-#define E_FILT_FILTER_TE_KEY        "filter_filter_te"
 #define E_FILT_DBLFUNC_KEY          "filter_dblfunc"
 #define E_FILT_DBLARG_KEY           "filter_dblarg"
 #define E_FILT_DBLACTIVATE_KEY      "filter_dblactivate"
@@ -73,8 +74,7 @@ static void filter_dlg_ok_cb(GtkWidget *ok_bt, gpointer dummy);
 static void filter_dlg_apply_cb(GtkWidget *apply_bt, gpointer dummy);
 static void filter_apply(GtkWidget *main_w, gboolean destroy);
 static void filter_dlg_save_cb(GtkWidget *save_bt, gpointer parent_w);
-static void filter_dlg_close_cb(GtkWidget *close_bt, gpointer parent_w);
-static void filter_dlg_destroy(GtkWidget *win, gpointer data);
+static void filter_dlg_destroy_cb(GtkWidget *win, gpointer data);
 
 static gint filter_sel_list_button_cb(GtkWidget *, GdkEventButton *,
                                       gpointer);
@@ -83,17 +83,9 @@ static void filter_sel_list_cb(GtkWidget *, gpointer);
 #else
 static void filter_sel_list_cb(GtkTreeSelection *, gpointer);
 #endif
-static void filter_list_destroy_cb(GtkWidget *, gpointer);
 static void filter_new_bt_clicked_cb(GtkWidget *, gpointer);
-static void filter_chg_bt_clicked_cb(GtkWidget *, gpointer);
-static void filter_chg_bt_destroy_cb(GtkWidget *, gpointer);
-static void filter_copy_bt_clicked_cb(GtkWidget *, gpointer);
-static void filter_copy_bt_destroy_cb(GtkWidget *, gpointer);
 static void filter_del_bt_clicked_cb(GtkWidget *, gpointer);
-static void filter_del_bt_destroy_cb(GtkWidget *, gpointer);
-static void filter_expr_cb(GtkWidget *, gpointer);
-static void filter_name_te_destroy_cb(GtkWidget *, gpointer);
-static void filter_filter_te_destroy_cb(GtkWidget *, gpointer);
+static void filter_name_te_changed_cb(GtkWidget *, gpointer);
 
 #ifdef HAVE_LIBPCAP
 /* Create a filter dialog for constructing a capture filter.
@@ -174,7 +166,7 @@ display_filter_construct_cb(GtkWidget *w, gpointer construct_args_ptr)
 }
 
 /* Should be called when a button that creates filters is destroyed; it
-   destroys any filter created by that button. */
+   destroys any filter dialog created by that button. */
 void
 filter_button_destroy_cb(GtkWidget *button, gpointer user_data _U_)
 {
@@ -187,7 +179,7 @@ filter_button_destroy_cb(GtkWidget *button, gpointer user_data _U_)
 	if (filter_w != NULL) {
 		/* Yes.  Break the association, and destroy the dialog. */
 		OBJECT_SET_DATA(button, E_FILT_DIALOG_PTR_KEY, NULL);
-		gtk_widget_destroy(filter_w);
+		window_destroy(filter_w);
 	}
 }
 
@@ -202,7 +194,7 @@ cfilter_dialog_cb(GtkWidget *w _U_)
 	/* No Apply button, and there's no text widget to set, much less
 	   activate, on "OK". */
 	static construct_args_t args = {
-		"Ethereal: Edit Capture Filter List",
+		"Ethereal: Capture Filter",
 		FALSE,
 		FALSE
 	};
@@ -224,35 +216,18 @@ cfilter_dialog_cb(GtkWidget *w _U_)
 }
 #endif
 
-static GtkWidget *global_dfilter_w;
-
 /* Create a filter dialog for editing display filters; this is to be used
    as a callback for menu items, toolbars, etc.. */
 void
 dfilter_dialog_cb(GtkWidget *w _U_)
 {
-	/* No Apply button, and there's no text widget to set, much less
-	   activate, on "OK". */
 	static construct_args_t args = {
-		"Ethereal: Edit Display Filter List",
-		FALSE,
-		FALSE
+		"Ethereal: Display Filter",
+		TRUE,
+		TRUE
 	};
 
-	/* Has a filter dialog box already been opened for editing
-	   display filters? */
-	if (global_dfilter_w != NULL) {
-		/* Yes.  Just reactivate it. */
-		reactivate_window(global_dfilter_w);
-		return;
-	}
-
-	/*
-	 * No.  Create one; we didn't pop this up as a result of pressing
-	 * a button next to some text entry field, so don't associate it
-	 * with a text entry field or button.
-	 */
-	global_dfilter_w = filter_dialog_new(NULL, NULL, DFILTER_LIST, &args);
+    display_filter_construct_cb(OBJECT_GET_DATA(top_level, E_FILT_BT_PTR_KEY), &args);
 }
 
 /* List of capture filter dialogs, so that if the list of filters changes
@@ -319,13 +294,13 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                *ok_bt,            /* "OK" button */
                *apply_bt,         /* "Apply" button */
                *save_bt,          /* "Save" button */
-               *close_bt;         /* "Cancel" button */
-    GtkWidget  *filter_pg = NULL; /* filter settings box */
+               *close_bt,         /* "Cancel" button */
+               *help_bt;          /* "Help" button */
+    GtkWidget  *filter_vb,        /* filter settings box */
+               *props_vb;
     GtkWidget  *top_hb,
                *list_bb,
                *new_bt,
-               *chg_bt,
-               *copy_bt,
                *del_bt,
                *filter_sc,
                *filter_l,
@@ -335,27 +310,30 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                *bottom_hb,
                *filter_lb,
                *filter_te,
-	       *add_expression_bt,
-	       *filter_frame;
+               *add_expression_bt,
+               *filter_fr,
+               *edit_fr,
+               *props_fr;
+    GtkTooltips *tooltips;
     GList      *fl_entry;
     filter_def *filt;
-    static filter_list_type_t cfilter_list = CFILTER_LIST;
-    static filter_list_type_t dfilter_list = DFILTER_LIST;
-    filter_list_type_t *filter_list_p;
-    GList     **filter_dialogs;
+    static filter_list_type_t cfilter_list_type = CFILTER_LIST;
+    static filter_list_type_t dfilter_list_type = DFILTER_LIST;
+    filter_list_type_t *filter_list_type_p;
+    GList       **filter_dialogs;
+    const gchar *filter_te_str = NULL;
 #if GTK_MAJOR_VERSION < 2
-    gchar      *filter_te_str = NULL;
     GtkWidget  *nl_item,
                *nl_lb,
                *l_select = NULL;
 #else
-    const gchar       *filter_te_str = NULL;
     gboolean           l_select = FALSE;
     GtkListStore      *store;
     GtkCellRenderer   *renderer;
     GtkTreeViewColumn *column;
     GtkTreeSelection  *sel;
     GtkTreeIter        iter;
+    GtkTreeIter        sel_iter;
 #endif
 
     /* Get a pointer to a static variable holding the type of filter on
@@ -365,124 +343,96 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 
     case CFILTER_LIST:
         filter_dialogs = &cfilter_dialogs;
-        filter_list_p = &cfilter_list;
+        filter_list_type_p = &cfilter_list_type;
         break;
 
     case DFILTER_LIST:
         filter_dialogs = &dfilter_dialogs;
-        filter_list_p = &dfilter_list;
+        filter_list_type_p = &dfilter_list_type;
         break;
 
     default:
         g_assert_not_reached();
         filter_dialogs = NULL;
-        filter_list_p = NULL;
+        filter_list_type_p = NULL;
         break;
     }
 
+    tooltips = gtk_tooltips_new ();
+
     main_w = dlg_window_new(construct_args->title);
+	gtk_window_set_default_size(GTK_WINDOW(main_w), 400, 400);
     OBJECT_SET_DATA(main_w, E_FILT_CONSTRUCT_ARGS_KEY, construct_args);
 
-    /* Call a handler when we're destroyed, so we can detach ourselves
-       from the button. */
-    SIGNAL_CONNECT(main_w, "destroy", filter_dlg_destroy, filter_list_p);
-
     main_vb = gtk_vbox_new(FALSE, 0);
-    gtk_container_border_width(GTK_CONTAINER(main_vb), 1);
+    gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
     gtk_container_add(GTK_CONTAINER(main_w), main_vb);
     gtk_widget_show(main_vb);
 
-    if (list == CFILTER_LIST)
-      filter_frame = gtk_frame_new("Capture Filters");
-    else
-      filter_frame = gtk_frame_new("Display Filters");
-    gtk_box_pack_start(GTK_BOX(main_vb), filter_frame, TRUE, TRUE, 0);
-    gtk_container_border_width(GTK_CONTAINER(filter_frame), 5);
-    gtk_widget_show(filter_frame);
-
     /* Make sure everything is set up */
     if (parent_filter_te)
         filter_te_str = gtk_entry_get_text(GTK_ENTRY(parent_filter_te));
 
     /* Container for each row of widgets */
-    filter_pg = gtk_vbox_new(FALSE, 5);
-    gtk_container_border_width(GTK_CONTAINER(filter_pg), 5);
-    gtk_container_add(GTK_CONTAINER(filter_frame), filter_pg);
-    gtk_widget_show(filter_pg);
+    filter_vb = gtk_vbox_new(FALSE, 0);
+    gtk_container_border_width(GTK_CONTAINER(filter_vb), 0);
+    gtk_container_add(GTK_CONTAINER(main_vb), filter_vb);
+    gtk_widget_show(filter_vb);
 
-    /* Top row: Filter list and buttons */
-    top_hb = gtk_hbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(filter_pg), top_hb);
+    /* Top row: Buttons and filter list */
+    top_hb = gtk_hbox_new(FALSE, 0);
+    gtk_container_add(GTK_CONTAINER(filter_vb), top_hb);
     gtk_widget_show(top_hb);
 
-    list_bb = gtk_vbutton_box_new();
-    gtk_button_box_set_layout (GTK_BUTTON_BOX (list_bb), GTK_BUTTONBOX_START);
-    gtk_container_add(GTK_CONTAINER(top_hb), list_bb);
+    edit_fr = gtk_frame_new("Edit");
+    gtk_box_pack_start(GTK_BOX(top_hb), edit_fr, FALSE, FALSE, 0);
+    gtk_widget_show(edit_fr);
+
+    list_bb = gtk_vbox_new(TRUE, 0);
+    gtk_container_border_width(GTK_CONTAINER(list_bb), 5);
+    gtk_container_add(GTK_CONTAINER(edit_fr), list_bb);
     gtk_widget_show(list_bb);
 
+    new_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_NEW);
+    SIGNAL_CONNECT(new_bt, "clicked", filter_new_bt_clicked_cb, filter_list_type_p);
 #if GTK_MAJOR_VERSION < 2
-    new_bt = gtk_button_new_with_label ("New");
-#else
-    new_bt = gtk_button_new_from_stock(GTK_STOCK_NEW);
+    WIDGET_SET_SIZE(new_bt, 50, 20);
 #endif
-    SIGNAL_CONNECT(new_bt, "clicked", filter_new_bt_clicked_cb, filter_list_p);
-    gtk_container_add(GTK_CONTAINER(list_bb), new_bt);
     gtk_widget_show(new_bt);
+    gtk_box_pack_start (GTK_BOX (list_bb), new_bt, FALSE, FALSE, 0);
+    gtk_tooltips_set_tip (tooltips, new_bt, 
+        "Create a new filter at the end of the list (with the current properties)", NULL);
 
-    chg_bt = gtk_button_new_with_label ("Change");
-    gtk_widget_set_sensitive(chg_bt, FALSE);
-    SIGNAL_CONNECT(chg_bt, "clicked", filter_chg_bt_clicked_cb, filter_list_p);
-    OBJECT_SET_DATA(main_w, E_FILT_CHG_BT_KEY, chg_bt);
-    SIGNAL_CONNECT(chg_bt, "destroy", filter_chg_bt_destroy_cb, NULL);
-    gtk_container_add(GTK_CONTAINER(list_bb), chg_bt);
-    gtk_widget_show(chg_bt);
-
-#if GTK_MAJOR_VERSION < 2
-    copy_bt = gtk_button_new_with_label ("Copy");
-#else
-    copy_bt = gtk_button_new_from_stock(GTK_STOCK_COPY);
-#endif
-    gtk_widget_set_sensitive(copy_bt, FALSE);
-    SIGNAL_CONNECT(copy_bt, "clicked", filter_copy_bt_clicked_cb,
-                   filter_list_p);
-    OBJECT_SET_DATA(main_w, E_FILT_COPY_BT_KEY, copy_bt);
-    SIGNAL_CONNECT(copy_bt, "destroy", filter_copy_bt_destroy_cb, NULL);
-    gtk_container_add(GTK_CONTAINER(list_bb), copy_bt);
-    gtk_widget_show(copy_bt);
-
-#if GTK_MAJOR_VERSION < 2
-    del_bt = gtk_button_new_with_label ("Delete");
-#else
-    del_bt = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-#endif
+    del_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_DELETE);
     gtk_widget_set_sensitive(del_bt, FALSE);
-    SIGNAL_CONNECT(del_bt, "clicked", filter_del_bt_clicked_cb, filter_list_p);
+    SIGNAL_CONNECT(del_bt, "clicked", filter_del_bt_clicked_cb, filter_list_type_p);
     OBJECT_SET_DATA(main_w, E_FILT_DEL_BT_KEY, del_bt);
-    SIGNAL_CONNECT(del_bt, "destroy", filter_del_bt_destroy_cb, NULL);
-    gtk_container_add(GTK_CONTAINER(list_bb), del_bt);
+#if GTK_MAJOR_VERSION < 2
+    WIDGET_SET_SIZE(del_bt, 50, 20);
+#endif
     gtk_widget_show(del_bt);
+    gtk_box_pack_start (GTK_BOX (list_bb), del_bt, FALSE, FALSE, 0);
+    gtk_tooltips_set_tip (tooltips, del_bt, ("Delete the selected filter"), NULL);
 
-    if (list == DFILTER_LIST) {
-        /* Create the "Add Expression..." button, to pop up a dialog
-           for constructing filter comparison expressions. */
-        add_expression_bt = gtk_button_new_with_label("Add Expression...");
-        SIGNAL_CONNECT(add_expression_bt, "clicked", filter_expr_cb, main_w);
-        gtk_container_add(GTK_CONTAINER(list_bb), add_expression_bt);
-        gtk_widget_show(add_expression_bt);
-    }
+    filter_fr = gtk_frame_new("Filter");
+    gtk_box_pack_start(GTK_BOX(top_hb), filter_fr, TRUE, TRUE, 0);
+    gtk_widget_show(filter_fr);
 
-    filter_sc = gtk_scrolled_window_new(NULL, NULL);
-    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(filter_sc),
-                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-    WIDGET_SET_SIZE(filter_sc, 250, 150);
-    gtk_container_add(GTK_CONTAINER(top_hb), filter_sc);
+    filter_sc = scrolled_window_new(NULL, NULL);
+#if GTK_MAJOR_VERSION >= 2
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(filter_sc), 
+                                   GTK_SHADOW_IN);
+#endif
+
+    gtk_container_set_border_width  (GTK_CONTAINER (filter_sc), 5);
+    gtk_container_add(GTK_CONTAINER(filter_fr), filter_sc);
     gtk_widget_show(filter_sc);
 
 #if GTK_MAJOR_VERSION < 2
     filter_l = gtk_list_new();
     gtk_list_set_selection_mode(GTK_LIST(filter_l), GTK_SELECTION_SINGLE);
     SIGNAL_CONNECT(filter_l, "selection_changed", filter_sel_list_cb,
-                   filter_pg);
+                   filter_vb);
 #else
     store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
     filter_l = tree_view_new(GTK_TREE_MODEL(store));
@@ -494,12 +444,11 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     gtk_tree_view_append_column(GTK_TREE_VIEW(filter_l), column);
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(filter_l));
     gtk_tree_selection_set_mode(sel, GTK_SELECTION_SINGLE);
-    SIGNAL_CONNECT(sel, "changed", filter_sel_list_cb, filter_pg);
+    SIGNAL_CONNECT(sel, "changed", filter_sel_list_cb, filter_vb);
     SIGNAL_CONNECT(filter_l, "button_press_event", filter_sel_list_button_cb,
                    NULL);
 #endif
     OBJECT_SET_DATA(main_w, E_FILT_FILTER_L_KEY, filter_l);
-    SIGNAL_CONNECT(filter_l, "destroy", filter_list_destroy_cb, NULL);
 #if GTK_MAJOR_VERSION < 2
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(filter_sc),
                                           filter_l);
@@ -515,6 +464,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     OBJECT_SET_DATA(filter_l, E_FILT_DBLACTIVATE_KEY,
                     construct_args->activate_on_ok ? "" : NULL);
 
+    /* fill in data */
     fl_entry = get_filter_list_first(list);
     while (fl_entry != NULL) {
         filt    = (filter_def *) fl_entry->data;
@@ -543,7 +493,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 #if GTK_MAJOR_VERSION < 2 
                 l_select = nl_item;
 #else
-                gtk_tree_selection_select_iter(sel, &iter);
+                sel_iter = iter;
                 l_select = TRUE;
 #endif
             }
@@ -555,74 +505,90 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     g_object_unref(G_OBJECT(store));
 #endif
 
-    /* Middle row: Filter name entry */
-    middle_hb = gtk_hbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(filter_pg), middle_hb);
+
+    props_fr = gtk_frame_new("Properties");
+    gtk_box_pack_start(GTK_BOX(filter_vb), props_fr, FALSE, FALSE, 0);
+    gtk_widget_show(props_fr);
+
+    props_vb = gtk_vbox_new(FALSE, 3);
+    gtk_container_border_width(GTK_CONTAINER(props_vb), 5);
+    gtk_container_add(GTK_CONTAINER(props_fr), props_vb);
+    gtk_widget_show(props_vb);
+
+    /* row: Filter name entry */
+    middle_hb = gtk_hbox_new(FALSE, 3);
+    gtk_container_add(GTK_CONTAINER(props_vb), middle_hb);
     gtk_widget_show(middle_hb);
 
     name_lb = gtk_label_new("Filter name:");
-    gtk_box_pack_start(GTK_BOX(middle_hb), name_lb, FALSE, FALSE, 3);
+    gtk_box_pack_start(GTK_BOX(middle_hb), name_lb, FALSE, FALSE, 0);
     gtk_widget_show(name_lb);
 
     name_te = gtk_entry_new();
-    gtk_box_pack_start(GTK_BOX(middle_hb), name_te, TRUE, TRUE, 3);
+    gtk_box_pack_start(GTK_BOX(middle_hb), name_te, TRUE, TRUE, 0);
     OBJECT_SET_DATA(main_w, E_FILT_NAME_TE_KEY, name_te);
-    SIGNAL_CONNECT(name_te, "destroy", filter_name_te_destroy_cb, NULL);
+    SIGNAL_CONNECT(name_te, "changed", filter_name_te_changed_cb, filter_list_type_p);
     gtk_widget_show(name_te);
 
-    /* Bottom row: Filter text entry */
-    bottom_hb = gtk_hbox_new(FALSE, 5);
-    gtk_container_add(GTK_CONTAINER(filter_pg), bottom_hb);
+    /* row: Filter text entry */
+    bottom_hb = gtk_hbox_new(FALSE, 3);
+    gtk_container_add(GTK_CONTAINER(props_vb), bottom_hb);
     gtk_widget_show(bottom_hb);
 
     filter_lb = gtk_label_new("Filter string:");
-    gtk_box_pack_start(GTK_BOX(bottom_hb), filter_lb, FALSE, FALSE, 3);
+    gtk_box_pack_start(GTK_BOX(bottom_hb), filter_lb, FALSE, FALSE, 0);
     gtk_widget_show(filter_lb);
 
     filter_te = gtk_entry_new();
-    gtk_box_pack_start(GTK_BOX(bottom_hb), filter_te, TRUE, TRUE, 3);
+    gtk_box_pack_start(GTK_BOX(bottom_hb), filter_te, TRUE, TRUE, 0);
     OBJECT_SET_DATA(main_w, E_FILT_FILTER_TE_KEY, filter_te);
-
-    SIGNAL_CONNECT(filter_te, "destroy", filter_filter_te_destroy_cb, NULL);
+    SIGNAL_CONNECT(filter_te, "changed", filter_name_te_changed_cb, filter_list_type_p);
     gtk_widget_show(filter_te);
 
-#if GTK_MAJOR_VERSION < 2 
-    if (l_select) {
-        gtk_list_select_child(GTK_LIST(filter_l), l_select);
-    } else if (filter_te_str && filter_te_str[0]) {
-        gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
-        gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
-    }
-#else
-    if (!l_select && filter_te_str && filter_te_str[0]) {
-        gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
-        gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
-    }
-#endif
-
     OBJECT_SET_DATA(main_w, E_FILT_PARENT_FILTER_TE_KEY, parent_filter_te);
 
-    bbox = gtk_hbutton_box_new();
-    gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
-    gtk_container_add(GTK_CONTAINER(main_vb), bbox);
+    if (list == DFILTER_LIST) {
+        gtk_tooltips_set_tip(tooltips, filter_te, 
+            "Enter a display filter. "
+            "The background color of this field is changed by a continuous syntax check (green is valid, red is invalid).", 
+            NULL);
+
+        /* Create the "Add Expression..." button, to pop up a dialog
+           for constructing filter comparison expressions. */
+        add_expression_bt = BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_ADD_EXPRESSION);
+        SIGNAL_CONNECT(add_expression_bt, "clicked", filter_add_expr_bt_cb, main_w);
+        gtk_box_pack_start(GTK_BOX(bottom_hb), add_expression_bt, FALSE, FALSE, 0);
+        gtk_widget_show(add_expression_bt);
+        gtk_tooltips_set_tip (tooltips, add_expression_bt, ("Add an expression to the filter string"), NULL);
+    }
+
+
+    /* button row */
+    if (parent_filter_te != NULL) {
+        if (construct_args->wants_apply_button) {
+            bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+        } else {
+            bbox = dlg_button_row_new(GTK_STOCK_OK, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+        }
+    } else {
+        if (construct_args->wants_apply_button) {
+            bbox = dlg_button_row_new(GTK_STOCK_APPLY, GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+        } else {
+            bbox = dlg_button_row_new(GTK_STOCK_SAVE, GTK_STOCK_CLOSE, GTK_STOCK_HELP, NULL);
+        }
+    }
+    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 5);
     gtk_widget_show(bbox);
 
+    ok_bt = NULL;
     if (parent_filter_te != NULL) {
         /*
          * We have a filter text entry that we can fill in if
          * the "OK" button is clicked, so put in an "OK" button.
          */
-#if GTK_MAJOR_VERSION < 2
-        ok_bt = gtk_button_new_with_label ("OK");
-#else
-        ok_bt = gtk_button_new_from_stock(GTK_STOCK_OK);
-#endif
+        ok_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_OK);
         SIGNAL_CONNECT(ok_bt, "clicked", filter_dlg_ok_cb, NULL);
-        GTK_WIDGET_SET_FLAGS(ok_bt, GTK_CAN_DEFAULT);
-        gtk_box_pack_start(GTK_BOX(bbox), ok_bt, TRUE, TRUE, 0);
-        gtk_widget_grab_default(ok_bt);
-        gtk_widget_show(ok_bt);
+        gtk_tooltips_set_tip (tooltips, ok_bt, ("Apply the filters and close this dialog"), NULL);
 
         /* Catch the "activate" signal on the filter name and filter
            expression text entries, so that if the user types Return
@@ -634,47 +600,40 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     }
 
     if (construct_args->wants_apply_button) {
-#if GTK_MAJOR_VERSION < 2
-        apply_bt = gtk_button_new_with_label ("Apply");
-#else
-        apply_bt = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-#endif
+        apply_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_APPLY);
         SIGNAL_CONNECT(apply_bt, "clicked", filter_dlg_apply_cb, NULL);
-        GTK_WIDGET_SET_FLAGS(apply_bt, GTK_CAN_DEFAULT);
-        gtk_box_pack_start(GTK_BOX(bbox), apply_bt, TRUE, TRUE, 0);
-        gtk_widget_show(apply_bt);
+        gtk_tooltips_set_tip (tooltips, apply_bt, ("Apply the filters and keep this dialog open"), NULL);
     }
 
-#if GTK_MAJOR_VERSION < 2
-    save_bt = gtk_button_new_with_label ("Save");
-#else
-    save_bt = gtk_button_new_from_stock(GTK_STOCK_SAVE);
-#endif
-    SIGNAL_CONNECT(save_bt, "clicked", filter_dlg_save_cb, filter_list_p);
-    GTK_WIDGET_SET_FLAGS(save_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), save_bt, TRUE, TRUE, 0);
-    gtk_widget_show(save_bt);
+    save_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_SAVE);
+    SIGNAL_CONNECT(save_bt, "clicked", filter_dlg_save_cb, filter_list_type_p);
+    gtk_tooltips_set_tip (tooltips, save_bt, ("Save the filters permanently and keep this dialog open"), NULL);
 
-#if GTK_MAJOR_VERSION < 2
-    close_bt = gtk_button_new_with_label ("Close");
-#else
-    close_bt = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
-#endif
-    SIGNAL_CONNECT(close_bt, "clicked", filter_dlg_close_cb, main_w);
-    GTK_WIDGET_SET_FLAGS(close_bt, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(bbox), close_bt, TRUE, TRUE, 0);
-    gtk_widget_show(close_bt);
+    close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    gtk_tooltips_set_tip (tooltips, close_bt, ("Close this dialog but don't apply the filter changes"), NULL);
+    if (parent_filter_te == NULL)
+        gtk_widget_grab_default(close_bt);
 
-    /*
-     * Catch the "key_press_event" signal in the window, so that we can
-     * catch the ESC key being pressed and act as if the "Close" button
-     * had been selected.
-     */
-    dlg_set_cancel(main_w, close_bt);
+    help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
+    if (list == CFILTER_LIST) {
+        SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Capture Filters");
+    } else {
+        SIGNAL_CONNECT(help_bt, "clicked", help_topic_cb, "Display Filters");
+    }
+    gtk_tooltips_set_tip (tooltips, help_bt, ("Show topic specific help"), NULL);
+
+    window_set_cancel_button(main_w, close_bt, window_cancel_button_cb);
+
+    if(ok_bt) {
+        gtk_widget_grab_default(ok_bt);
+    }
 
     remember_filter_dialog(main_w, filter_dialogs);
 
-    gtk_widget_show(main_w);
+    SIGNAL_CONNECT(main_w, "delete_event", window_delete_event_cb, NULL);
+    /* Call a handler when we're destroyed, so we can detach ourselves
+       from the button. */
+    SIGNAL_CONNECT(main_w, "destroy", filter_dlg_destroy_cb, filter_list_type_p);
 
     if (button != NULL) {
 	/* This dialog box was created by a "Filter" button.
@@ -686,6 +645,27 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 	OBJECT_SET_DATA(button, E_FILT_DIALOG_PTR_KEY, main_w);
     }
 
+    /* DO SELECTION THINGS *AFTER* SHOWING THE DIALOG! */
+    /* otherwise the updatings can get confused */
+#if GTK_MAJOR_VERSION < 2 
+    if (l_select) {
+        gtk_list_select_child(GTK_LIST(filter_l), l_select);
+    } else if (filter_te_str && filter_te_str[0]) {
+        gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
+        gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
+    }
+#else
+    if (l_select) {
+        gtk_tree_selection_select_iter(sel, &sel_iter);        
+    } else if (filter_te_str && filter_te_str[0]) {
+        gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
+        gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
+    }
+#endif
+
+    gtk_widget_show(main_w);
+    window_present(main_w);
+
     return main_w;
 }
 
@@ -745,13 +725,13 @@ filter_dlg_dclick(GtkWidget *filter_l, gpointer main_w_arg, gpointer activate)
                     /*
                      * Yes - do so.
                      */
-                    SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate");
+                    SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate", NULL);
                 }
             }
         }
     }
 
-    gtk_widget_destroy(main_w);
+    window_destroy(main_w);
 }
 
 static void
@@ -799,7 +779,7 @@ filter_apply(GtkWidget *main_w, gboolean destroy)
 		/*
 		 * Destroy the filter dialog box.
 		 */
-		gtk_widget_destroy(main_w);
+		window_destroy(main_w);
 	}
 
 	if (parent_filter_te != NULL) {
@@ -823,7 +803,7 @@ filter_apply(GtkWidget *main_w, gboolean destroy)
 		 * to destroy it.
 		 */
 		if (construct_args->activate_on_ok) {
-			SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate");
+			SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate", NULL);
 		}
 	}
 }
@@ -840,7 +820,7 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 	/* Create the directory that holds personal configuration files,
 	   if necessary.  */
 	if (create_persconffile_dir(&pf_dir_path) == -1) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Can't create directory\n\"%s\"\nfor filter files: %s.",
 		    pf_dir_path, strerror(errno));
 		g_free(pf_dir_path);
@@ -865,7 +845,7 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 			filter_type = NULL;
 			break;
 		}
-		simple_dialog(ESD_TYPE_CRIT, NULL,
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 		    "Could not save to your %s filter file\n\"%s\": %s.",
 		    filter_type, f_path, strerror(f_save_errno));
 		g_free(f_path);
@@ -873,13 +853,7 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 }
 
 static void
-filter_dlg_close_cb(GtkWidget *close_bt _U_, gpointer parent_w)
-{
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-static void
-filter_dlg_destroy(GtkWidget *win, gpointer data)
+filter_dlg_destroy_cb(GtkWidget *win, gpointer data)
 {
 	filter_list_type_t list = *(filter_list_type_t *)data;
 	GtkWidget *button;
@@ -903,12 +877,6 @@ filter_dlg_destroy(GtkWidget *win, gpointer data)
 			global_cfilter_w = NULL;
 			break;
 #endif
-
-		case DFILTER_LIST:
-			g_assert(win == global_dfilter_w);
-			global_dfilter_w = NULL;
-			break;
-
 		default:
 			g_assert_not_reached();
 			break;
@@ -917,10 +885,6 @@ filter_dlg_destroy(GtkWidget *win, gpointer data)
 
 	/* Remove this from the list of filter dialog windows. */
 	forget_filter_dialog(win, list);
-
-	/* Now nuke this window. */
-	gtk_grab_remove(GTK_WIDGET(win));
-	gtk_widget_destroy(GTK_WIDGET(win));
 }
 
 #if GTK_MAJOR_VERSION < 2
@@ -979,7 +943,7 @@ filter_sel_list_cb(GtkTreeSelection *sel, gpointer data _U_)
     GtkWidget    *copy_bt = OBJECT_GET_DATA(main_w, E_FILT_COPY_BT_KEY);
     GtkWidget    *del_bt = OBJECT_GET_DATA(main_w, E_FILT_DEL_BT_KEY);
     filter_def   *filt;
-    gchar        *name = "", *strval = "";
+    gchar        *name = NULL, *strval = NULL;
     GList        *flp;
     gint          sensitivity = FALSE;
 
@@ -998,8 +962,8 @@ filter_sel_list_cb(GtkTreeSelection *sel, gpointer data _U_)
 #endif
         if (flp) {
             filt   = (filter_def *) flp->data;
-            name   = filt->name;
-            strval = filt->strval;
+            name   = g_strdup(filt->name);
+            strval = g_strdup(filt->strval);
             sensitivity = TRUE;
         }
     }
@@ -1023,23 +987,19 @@ filter_sel_list_cb(GtkTreeSelection *sel, gpointer data _U_)
      * destroyed, we know that we shouldn't do anything to those widgets.
      */
     if (name_te != NULL)
-        gtk_entry_set_text(GTK_ENTRY(name_te), name);
+        gtk_entry_set_text(GTK_ENTRY(name_te), name ? name : "");
     if (filter_te != NULL)
-        gtk_entry_set_text(GTK_ENTRY(filter_te), strval);
+        gtk_entry_set_text(GTK_ENTRY(filter_te), strval ? strval : "");
     if (chg_bt != NULL)
         gtk_widget_set_sensitive(chg_bt, sensitivity);
     if (copy_bt != NULL)
         gtk_widget_set_sensitive(copy_bt, sensitivity);
     if (del_bt != NULL)
         gtk_widget_set_sensitive(del_bt, sensitivity);
-}
-
-static void
-filter_list_destroy_cb(GtkWidget *l, gpointer data _U_)
-{
-  GtkWidget  *main_w = gtk_widget_get_toplevel(l);
-
-  OBJECT_SET_DATA(main_w, E_FILT_FILTER_L_KEY, NULL);
+    if (name != NULL)
+        g_free(name);
+    if (strval != NULL)
+        g_free(strval);
 }
 
 /* To do: add input checking to each of these callbacks */
@@ -1109,13 +1069,22 @@ filter_new_bt_clicked_cb(GtkWidget *w, gpointer data)
   GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
   filter_list_type_t list = *(filter_list_type_t *)data;
   GList      *fl_entry;
-  gchar      *name, *strval;
+  const gchar *name, *strval;
   new_filter_cb_args_t args;
 
   name   = gtk_entry_get_text(GTK_ENTRY(name_te));
   strval = gtk_entry_get_text(GTK_ENTRY(filter_te));
 
-  if (strlen(name) > 0 && strlen(strval) > 0) {
+  /* if the user didn't entered a name, set default one */
+  if (strlen(name) == 0) {
+    name = "new";
+  }
+
+  /* if the user didn't entered a string value, set default one */
+  if (strlen(strval) == 0) {
+    strval = "new";
+  }
+
     /* Add a new entry to the filter list. */
     fl_entry = add_to_filter_list(list, name, strval);
 
@@ -1124,14 +1093,7 @@ filter_new_bt_clicked_cb(GtkWidget *w, gpointer data)
     args.active_filter_l = filter_l;
     args.nflp = fl_entry;
     g_list_foreach(get_filter_dialog_list(list), new_filter_cb, &args);
-  } else {
-    /* Give the user some basic directions on how to use the 'new' button */
-    simple_dialog(ESD_TYPE_WARN, NULL,
-     "You have left either the 'Filter name' or 'Filter string' field empty.\n"
-     "To add a new filter, enter a name for the filter, and enter the expression\n"
-     "for the filter or use the 'Add Expression...' button to construct an\n"
-     "expression, then click 'New'.");
-  }
+
 }
 
 #if GTK_MAJOR_VERSION < 2
@@ -1188,7 +1150,7 @@ chg_filter_cb(gpointer data, gpointer user_data)
 }
 
 static void
-filter_chg_bt_clicked_cb(GtkWidget *w, gpointer data)
+filter_name_te_changed_cb(GtkWidget *w, gpointer data)
 {
     GtkWidget  *main_w = gtk_widget_get_toplevel(w);
     GtkWidget  *name_te = OBJECT_GET_DATA(main_w, E_FILT_NAME_TE_KEY);
@@ -1196,14 +1158,15 @@ filter_chg_bt_clicked_cb(GtkWidget *w, gpointer data)
     GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
     filter_def *filt;
     GList      *fl_entry;
-    filter_list_type_t list = *(filter_list_type_t *)data;
+    filter_list_type_t  list = *(filter_list_type_t *)data;
+    const gchar         *name = "";
+    const gchar         *strval = "";
+
 #if GTK_MAJOR_VERSION < 2
-    gchar      *name = "", *strval = "";
     GList      *sl;
     GtkObject  *l_item;
     GtkLabel   *nl_lb;
 #else
-    const gchar       *name, *strval;
     GtkTreeSelection  *sel;
     GtkTreeModel      *model;
     GtkTreeIter        iter;
@@ -1217,6 +1180,11 @@ filter_chg_bt_clicked_cb(GtkWidget *w, gpointer data)
     name   = gtk_entry_get_text(GTK_ENTRY(name_te));
     strval = gtk_entry_get_text(GTK_ENTRY(filter_te));
 
+    if (DFILTER_LIST == list) {
+        /* colorize filter string entry */
+        filter_te_syntax_check_cb(filter_te);
+    }
+
     /* if something was selected */
 #if GTK_MAJOR_VERSION < 2
     if (sl) {
@@ -1247,71 +1215,6 @@ filter_chg_bt_clicked_cb(GtkWidget *w, gpointer data)
     }
 }
 
-static void
-filter_chg_bt_destroy_cb(GtkWidget *chg_bt, gpointer data _U_)
-{
-  GtkWidget  *main_w = gtk_widget_get_toplevel(chg_bt);
-
-  OBJECT_SET_DATA(main_w, E_FILT_CHG_BT_KEY, NULL);
-}
-
-static void
-filter_copy_bt_clicked_cb(GtkWidget *w, gpointer data)
-{
-    GtkWidget  *main_w = gtk_widget_get_toplevel(w);
-    GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
-    GList      *fl_entry, *nfl_entry;
-    gchar      *prefix = "Copy of ", *name;
-    filter_def *filt;
-    filter_list_type_t list = *(filter_list_type_t *)data;
-    new_filter_cb_args_t args;
-#if GTK_MAJOR_VERSION < 2
-    GList      *sl;
-    GtkObject  *l_item;
-#else
-    GtkTreeSelection    *sel;
-    GtkTreeModel        *model;
-    GtkTreeIter          iter;
-#endif
-
-#if GTK_MAJOR_VERSION < 2
-    sl     = GTK_LIST(filter_l)->selection;
-    /* if something was selected */
-    if (sl) {
-        l_item = GTK_OBJECT(sl->data);
-        fl_entry = (GList *) OBJECT_GET_DATA(l_item, E_FILT_LIST_ITEM_MODEL_KEY);
-#else
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(filter_l));
-    /* if something was selected */
-    if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
-        gtk_tree_model_get(model, &iter, 1, &fl_entry, -1);
-#endif
-        if (fl_entry != NULL) {
-            /* Add a new entry, copying the existing entry, to the
-             * filter list. */
-            filt = (filter_def *) fl_entry->data;
-            name = g_malloc(strlen(prefix) + strlen(filt->name) + 1);
-            sprintf(name, "%s%s", prefix, filt->name);
-            nfl_entry = add_to_filter_list(list, name, filt->strval);
-            g_free(name);
-
-            /* Update all the filter list widgets, not just the one in
-               the dialog box in which we clicked on "Copy". */
-            args.active_filter_l = filter_l;
-            args.nflp = nfl_entry;
-            g_list_foreach(get_filter_dialog_list(list), new_filter_cb, &args);
-        }
-    }
-}
-
-static void
-filter_copy_bt_destroy_cb(GtkWidget *copy_bt, gpointer data _U_)
-{
-  GtkWidget  *main_w = gtk_widget_get_toplevel(copy_bt);
-
-  OBJECT_SET_DATA(main_w, E_FILT_COPY_BT_KEY, NULL);
-}
-
 static void
 delete_filter_cb(gpointer data, gpointer user_data)
 {
@@ -1387,16 +1290,8 @@ filter_del_bt_clicked_cb(GtkWidget *w, gpointer data)
     }
 }
 
-static void
-filter_del_bt_destroy_cb(GtkWidget *del_bt, gpointer data _U_)
-{
-  GtkWidget  *main_w = gtk_widget_get_toplevel(del_bt);
-
-  OBJECT_SET_DATA(main_w, E_FILT_DEL_BT_KEY, NULL);
-}
-
-static void
-filter_expr_cb(GtkWidget *w _U_, gpointer main_w_arg)
+void
+filter_add_expr_bt_cb(GtkWidget *w _U_, gpointer main_w_arg)
 {
 	GtkWidget  *main_w = GTK_WIDGET(main_w_arg);
 	GtkWidget  *filter_te;
@@ -1406,17 +1301,60 @@ filter_expr_cb(GtkWidget *w _U_, gpointer main_w_arg)
 }
 
 static void
-filter_name_te_destroy_cb(GtkWidget *name_te, gpointer data _U_)
+color_filter_te(GtkWidget *w, guint16 red, guint16 green, guint16 blue)
 {
-  GtkWidget  *main_w = gtk_widget_get_toplevel(name_te);
+    GdkColor    bg;
+    GtkStyle    *style;
 
-  OBJECT_SET_DATA(main_w, E_FILT_NAME_TE_KEY, NULL);
+    bg.pixel    = 0;
+    bg.red      = red;
+    bg.green    = green;
+    bg.blue     = blue;
+
+    style = gtk_style_copy(gtk_widget_get_style(w));
+    style->base[GTK_STATE_NORMAL] = bg;
+    gtk_widget_set_style(w, style);
 }
 
-static void
-filter_filter_te_destroy_cb(GtkWidget *filter_te, gpointer data _U_)
+void
+colorize_filter_te_as_empty(GtkWidget *w)
 {
-  GtkWidget  *main_w = gtk_widget_get_toplevel(filter_te);
-
-  OBJECT_SET_DATA(main_w, E_FILT_FILTER_TE_KEY, NULL);
+    /* white */
+    color_filter_te(w, 0xFFFF, 0xFFFF, 0xFFFF);
 }
+
+void
+colorize_filter_te_as_invalid(GtkWidget *w)
+{
+    /* light red */
+    color_filter_te(w, 0xFFFF, 0xAFFF, 0xAFFF);
+}
+
+void
+colorize_filter_te_as_valid(GtkWidget *w)
+{
+    /* light green */
+    color_filter_te(w, 0xAFFF, 0xFFFF, 0xAFFF);
+}
+
+void
+filter_te_syntax_check_cb(GtkWidget *w)
+{
+    const gchar *strval;
+    dfilter_t   *dfp;
+
+    strval = gtk_entry_get_text(GTK_ENTRY(w));
+
+    /* colorize filter string entry */
+    if (strval && dfilter_compile(strval, &dfp)) {
+    	if (dfp != NULL)
+    	  dfilter_free(dfp);
+        if (strlen(strval) == 0)
+            colorize_filter_te_as_empty(w);
+        else
+            colorize_filter_te_as_valid(w);
+    } else
+        colorize_filter_te_as_invalid(w);
+}
+
+
