@@ -1,6 +1,6 @@
 /* follow_dlg.c
  *
- * $Id: follow_dlg.c,v 1.53 2004/03/13 15:15:24 ulfl Exp $
+ * $Id: follow_dlg.c,v 1.68 2004/06/29 03:27:51 jmayer Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -48,7 +48,6 @@
 #include "dlg_utils.h"
 #include "keys.h"
 #include "globals.h"
-#include "gtkglobals.h"
 #include "main.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
@@ -62,6 +61,8 @@
 #include "compat_macros.h"
 #include "ipproto.h"
 #include "tap_menu.h"
+#include "print_mswin.h"
+#include "font_utils.h"
 
 /* Show Stream */
 typedef enum {
@@ -100,15 +101,15 @@ static void follow_load_text(follow_info_t *follow_info);
 static void follow_filter_out_stream(GtkWidget * w, gpointer parent_w);
 static void follow_print_stream(GtkWidget * w, gpointer parent_w);
 static void follow_save_as_cmd_cb(GtkWidget * w, gpointer data);
-static void follow_save_as_ok_cb(GtkWidget * w, GtkFileSelection * fs);
+static void follow_save_as_ok_cb(GtkWidget * w, gpointer fs);
 static void follow_save_as_destroy_cb(GtkWidget * win, gpointer user_data);
 static void follow_stream_om_both(GtkWidget * w, gpointer data);
 static void follow_stream_om_client(GtkWidget * w, gpointer data);
 static void follow_stream_om_server(GtkWidget * w, gpointer data);
 
 
-extern FILE *data_out_file;
-
+/* With MSVC and a libethereal.dll, we need a special declaration. */
+ETH_VAR_IMPORT FILE *data_out_file;
 
 #define E_FOLLOW_INFO_KEY "follow_info_key"
 
@@ -247,21 +248,19 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	fclose(data_out_file);
 
 	/* The data_out_filename file now has all the text that was in the session */
-	streamwindow = window_new(GTK_WINDOW_TOPLEVEL, "Follow TCP stream");
+	streamwindow = dlg_window_new("Follow TCP stream");
 
 	/* needed in follow_filter_out_stream(), is there a better way? */
 	follow_info->streamwindow = streamwindow;
 
 	gtk_widget_set_name(streamwindow, "TCP stream window");
-
-	SIGNAL_CONNECT(streamwindow, "destroy", follow_destroy_cb, NULL);
-	WIDGET_SET_SIZE(streamwindow, DEF_WIDTH, DEF_HEIGHT);
-	gtk_container_border_width(GTK_CONTAINER(streamwindow), 2);
+    gtk_window_set_default_size(GTK_WINDOW(streamwindow), DEF_WIDTH, DEF_HEIGHT);
+	gtk_container_border_width(GTK_CONTAINER(streamwindow), 6);
 
 	/* setup the container */
     tooltips = gtk_tooltips_new ();
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_container_add(GTK_CONTAINER(streamwindow), vbox);
 
     /* content frame */
@@ -273,14 +272,14 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
   	gtk_container_add(GTK_CONTAINER(vbox), stream_fr);
   	gtk_widget_show(stream_fr);
 
-    stream_vb = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(stream_vb) , 5);
+    stream_vb = gtk_vbox_new(FALSE, 6);
+	gtk_container_set_border_width( GTK_CONTAINER(stream_vb) , 6);
 	gtk_container_add(GTK_CONTAINER(stream_fr), stream_vb);
 
 	/* create a scrolled window for the text */
 	txt_scrollw = scrolled_window_new(NULL, NULL);
 #if GTK_MAJOR_VERSION >= 2
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(txt_scrollw), 
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(txt_scrollw),
                                    GTK_SHADOW_IN);
 #endif
 	gtk_box_pack_start(GTK_BOX(stream_vb), txt_scrollw, TRUE, TRUE, 0);
@@ -372,7 +371,7 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(stream_om), stream_menu);
 	/* Set history to 0th item, i.e., the first item. */
 	gtk_option_menu_set_history(GTK_OPTION_MENU(stream_om), 0);
-    gtk_tooltips_set_tip (tooltips, stream_om, 
+    gtk_tooltips_set_tip (tooltips, stream_om,
         "Select the stream direction to display", NULL);
 	gtk_box_pack_start(GTK_BOX(hbox), stream_om, FALSE, FALSE, 0);
 
@@ -424,23 +423,18 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	/* Create exclude stream button */
 	button = gtk_button_new_with_label("Filter out this stream");
 	SIGNAL_CONNECT(button, "clicked", follow_filter_out_stream, follow_info);
-    gtk_tooltips_set_tip (tooltips, button, 
+    gtk_tooltips_set_tip (tooltips, button,
         "Build a display filter which cuts this stream from the capture", NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox), button, FALSE, FALSE, 0);
 
 	/* Create Close Button */
     button = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
-	SIGNAL_CONNECT_OBJECT(button, "clicked", gtk_widget_destroy,
-                              streamwindow);
-    gtk_tooltips_set_tip (tooltips, button, 
+    gtk_tooltips_set_tip (tooltips, button,
         "Close the dialog and keep the current display filter", NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox), button, FALSE, FALSE, 0);
+    GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 
-	/* Catch the "key_press_event" signal in the window, so that we can catch
-	the ESC key being pressed and act as if the "Cancel" button had
-	been selected. */
-	dlg_set_cancel(streamwindow, button);
-
+    window_set_cancel_button(streamwindow, button, window_cancel_button_cb);
 
 	/* Tuck away the follow_info object into the window */
 	OBJECT_SET_DATA(streamwindow, E_FOLLOW_INFO_KEY, follow_info);
@@ -450,11 +444,16 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 
 	data_out_file = NULL;
 
+    SIGNAL_CONNECT(streamwindow, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(streamwindow, "destroy", follow_destroy_cb, NULL);
+
 	/* Make sure this widget gets destroyed if we quit the main loop,
 	   so that if we exit, we clean up any temporary files we have
 	   for "Follow TCP Stream" windows. */
 	gtk_quit_add_destroy(gtk_main_level(), GTK_OBJECT(streamwindow));
+
 	gtk_widget_show_all(streamwindow);
+	window_present(streamwindow);
 }
 
 /* The destroy call back has the responsibility of
@@ -464,11 +463,15 @@ static void
 follow_destroy_cb(GtkWidget *w, gpointer data _U_)
 {
 	follow_info_t	*follow_info;
+    int i;
 
 	follow_info = OBJECT_GET_DATA(w, E_FOLLOW_INFO_KEY);
-	unlink(follow_info->data_out_filename);
+	i = unlink(follow_info->data_out_filename);
+    if(i != 0) {
+        g_warning("Follow: Couldn't remove temporary file: \"%s\", errno: %s (%u)", 
+            follow_info->data_out_filename, strerror(errno), errno);        
+    }
 	g_free(follow_info->filter_out_filter);
-	gtk_widget_destroy(w);
 	forget_follow_info(follow_info);
 	g_free(follow_info);
 }
@@ -504,7 +507,7 @@ follow_stream_om_server(GtkWidget *w _U_, gpointer data)
 static void
 follow_charset_toggle_cb(GtkWidget * w _U_, gpointer data)
 {
-	follow_info_t	*follow_info = data;
+    follow_info_t	*follow_info = data;
 
 	if (GTK_TOGGLE_BUTTON(follow_info->ebcdic_bt)->active)
 		follow_info->show_type = SHOW_EBCDIC;
@@ -531,22 +534,22 @@ typedef enum {
 
 static frs_return_t
 follow_read_stream(follow_info_t *follow_info,
-		   gboolean (*print_line) (char *, int, gboolean, void *),
-		   void *arg)
+		   gboolean (*print_line) (char *, int, gboolean, void *, print_format_e),
+		   void *arg, print_format_e format)
 {
     tcp_stream_chunk	sc;
     int			bcount, iplen;
     guint8		client_addr[MAX_IPADDR_LEN];
     guint16		client_port = 0;
     gboolean		is_server;
-    guint16		current_pos, global_client_pos = 0, global_server_pos = 0;
-    guint16		*global_pos;
+    guint32		current_pos, global_client_pos = 0, global_server_pos = 0;
+    guint32		*global_pos;
     gboolean		skip;
     gchar               initbuf[256];
     guint32             server_packet_count = 0;
     guint32             client_packet_count = 0;
     char                buffer[FLT_BUF_SIZE];
-    int                 nchars;
+    size_t              nchars;
 
     iplen = (follow_info->is_ipv6) ? 16 : 4;
 
@@ -592,7 +595,7 @@ follow_read_stream(follow_info_t *follow_info,
 		case SHOW_EBCDIC:
 		    /* If our native arch is ASCII, call: */
 		    EBCDIC_to_ASCII(buffer, nchars);
-		    if (!(*print_line) (buffer, nchars, is_server, arg))
+		    if (!(*print_line) (buffer, nchars, is_server, arg, format))
 			goto print_error;
 		    break;
 
@@ -600,7 +603,7 @@ follow_read_stream(follow_info_t *follow_info,
 		    /* If our native arch is EBCDIC, call:
 		     * ASCII_TO_EBCDIC(buffer, nchars);
 		     */
-		    if (!(*print_line) (buffer, nchars, is_server, arg))
+		    if (!(*print_line) (buffer, nchars, is_server, arg, format))
 			goto print_error;
 		    break;
 
@@ -657,7 +660,7 @@ follow_read_stream(follow_info_t *follow_info,
 			(*global_pos) += i;
 			hexbuf[cur++] = '\n';
 			hexbuf[cur] = 0;
-			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg))
+			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg, format))
 			    goto print_error;
 		    }
 		    break;
@@ -666,7 +669,7 @@ follow_read_stream(follow_info_t *follow_info,
 		    current_pos = 0;
 		    g_snprintf(initbuf, 256, "char peer%d_%d[] = {\n", is_server ? 1 : 0,
 			    is_server ? server_packet_count++ : client_packet_count++);
-		    if (!(*print_line) (initbuf, strlen(initbuf), is_server, arg))
+		    if (!(*print_line) (initbuf, strlen(initbuf), is_server, arg, format))
 			goto print_error;
 		    while (current_pos < nchars) {
 			gchar hexbuf[256];
@@ -700,7 +703,7 @@ follow_read_stream(follow_info_t *follow_info,
 			(*global_pos) += i;
 			hexbuf[cur++] = '\n';
 			hexbuf[cur] = 0;
-			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg))
+			if (!(*print_line) (hexbuf, strlen(hexbuf), is_server, arg, format))
 			    goto print_error;
 		    }
 		    break;
@@ -717,6 +720,8 @@ follow_read_stream(follow_info_t *follow_info,
 	return FRS_READ_ERROR;
     }
 
+	fclose(data_out_file);
+	data_out_file = NULL;
     return FRS_OK;
 
 print_error:
@@ -727,25 +732,39 @@ print_error:
 
 /*
  * XXX - for text printing, we probably want to wrap lines at 80 characters;
- * for PostScript printing, we probably want to wrap them at the appropriate
- * width, and perhaps put some kind of dingbat (to use the technical term)
- * to indicate a wrapped line, along the lines of what's done when displaying
- * this in a window, as per Warren Young's suggestion.
- *
- * For now, we support only text printing.
+ * (PostScript printing is doing this already), and perhaps put some kind of 
+ * dingbat (to use the technical term) to indicate a wrapped line, along the 
+ * lines of what's done when displaying this in a window, as per Warren Young's 
+ * suggestion.
  */
 static gboolean
-follow_print_text(char *buffer, int nchars, gboolean is_server _U_, void *arg)
+follow_print_text(char *buffer, int nchars, gboolean is_server _U_, void *arg, print_format_e format)
 {
     FILE *fh = arg;
+    int i;
+    char *str;
+
+    /* convert non printable characters */
+    for (i = 0; i < nchars; i++) {
+        if (buffer[i] == '\n' || buffer[i] == '\r')
+            continue;
+        if (! isprint(buffer[i])) {
+            buffer[i] = '.';
+        }
+    }
+
+    /* convert unterminated char array to a zero terminated string */
+    str = g_malloc(nchars + 1);
+    memcpy(str, buffer, nchars);
+    str[nchars] = 0;
+    print_line(fh, /*indent*/ 0, format, str);
+    g_free(str);
 
-    if (fwrite(buffer, nchars, 1, fh) != 1)
-      return FALSE;
     return TRUE;
 }
 
 static void
-follow_filter_out_stream(GtkWidget * w _U_, gpointer data) 
+follow_filter_out_stream(GtkWidget * w _U_, gpointer data)
 {
     follow_info_t	*follow_info = data;
 
@@ -759,8 +778,8 @@ follow_filter_out_stream(GtkWidget * w _U_, gpointer data)
     main_filter_packets(&cfile, follow_info->filter_out_filter, FALSE);
 
     /* we force a subsequent close */
-    gtk_widget_destroy(follow_info->streamwindow);
-  
+    window_destroy(follow_info->streamwindow);
+
     return;
 }
 
@@ -771,79 +790,104 @@ follow_print_stream(GtkWidget * w _U_, gpointer data)
     gboolean		to_file;
     char		*print_dest;
     follow_info_t	*follow_info = data;
+#ifdef _WIN32
+    gboolean win_printer = FALSE;
+#endif
+
 
     switch (prefs.pr_dest) {
     case PR_DEST_CMD:
-	print_dest = prefs.pr_cmd;
-	to_file = FALSE;
-	break;
-
+#ifdef _WIN32
+        win_printer = TRUE;
+        /*XXX should use temp file stuff in util routines */
+        print_dest = g_strdup(tmpnam(NULL));
+        to_file = TRUE;
+#else
+        print_dest = prefs.pr_cmd;
+        to_file = FALSE;
+#endif
+        break;
     case PR_DEST_FILE:
-	print_dest = prefs.pr_file;
-	to_file = TRUE;
-	break;
+        print_dest = prefs.pr_file;
+        to_file = TRUE;
+        break;
     default:			/* "Can't happen" */
-	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		      "Couldn't figure out where to send the print "
-		      "job. Check your preferences.");
-	return;
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+            "Couldn't figure out where to send the print "
+            "job. Check your preferences.");
+        return;
     }
 
     fh = open_print_dest(to_file, print_dest);
     if (fh == NULL) {
-	if (to_file)
-	    open_failure_alert_box(prefs.pr_file, errno, TRUE);
-	else {
-	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-			  "Couldn't run print command %s.", prefs.pr_cmd);
-	}
-	return;
+        if (to_file) {
+            open_failure_alert_box(prefs.pr_file, errno, TRUE);
+        } else {
+            simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+	              "Couldn't run print command %s.", prefs.pr_cmd);
+        }
+        return;
     }
 
-    print_preamble(fh, PR_FMT_TEXT);
+    print_preamble(fh, prefs.pr_format, cfile.filename);
     if (ferror(fh))
-	goto print_error;
-    switch (follow_read_stream(follow_info, follow_print_text, fh)) {
+        goto print_error;
 
+    switch (follow_read_stream(follow_info, follow_print_text, fh, prefs.pr_format)) {
     case FRS_OK:
-	break;
-
+    	break;
     case FRS_OPEN_ERROR:
     case FRS_READ_ERROR:
-	/* XXX - cancel printing? */
-	close_print_dest(to_file, fh);
-	return;
-
+	    /* XXX - cancel printing? */
+	    close_print_dest(to_file, fh);
+	    return;
     case FRS_PRINT_ERROR:
-	goto print_error;
+	    goto print_error;
     }
-    print_finale(fh, PR_FMT_TEXT);
+
+    print_finale(fh, prefs.pr_format);
     if (ferror(fh))
-	goto print_error;
+	    goto print_error;
+
     if (!close_print_dest(to_file, fh)) {
-	if (to_file)
-	    write_failure_alert_box(prefs.pr_file, errno);
-	else {
-	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+        if (to_file) {
+    	    write_failure_alert_box(prefs.pr_file, errno);
+        } else {
+	        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 			  "Error closing print destination.");
-	}
+        }
     }
+#ifdef _WIN32
+    if (win_printer) {
+        print_mswin(print_dest);
+
+        /* trash temp file */
+        remove(print_dest);
+    }
+#endif
     return;
 
 print_error:
-    if (to_file)
-	write_failure_alert_box(prefs.pr_file, errno);
-    else {
-	simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		      "Error writing to print command: %s", strerror(errno));
+    if (to_file) {
+        write_failure_alert_box(prefs.pr_file, errno);
+    } else {
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+          "Error writing to print command: %s", strerror(errno));
     }
     /* XXX - cancel printing? */
     close_print_dest(to_file, fh);
+
+#ifdef _WIN32
+    if (win_printer) {
+        /* trash temp file */
+        remove(print_dest);
+    }
+#endif
 }
 
 static gboolean
 follow_add_to_gtk_text(char *buffer, int nchars, gboolean is_server,
-		       void *arg)
+		       void *arg, print_format_e format _U_)
 {
     GtkWidget *text = arg;
     GdkColor   fg, bg;
@@ -880,12 +924,12 @@ follow_add_to_gtk_text(char *buffer, int nchars, gboolean is_server,
     	color_t_to_gdkcolor(&bg, &prefs.st_client_bg);
     }
 #if GTK_MAJOR_VERSION < 2
-    gtk_text_insert(GTK_TEXT(text), m_r_font, &fg, &bg, buffer, nchars);
+    gtk_text_insert(GTK_TEXT(text), user_font_get_regular(), &fg, &bg, buffer, nchars);
 #else
     gtk_text_buffer_get_end_iter(buf, &iter);
     tag = gtk_text_buffer_create_tag(buf, NULL, "foreground-gdk", &fg,
                                      "background-gdk", &bg, "font-desc",
-                                     m_r_font, NULL);
+                                     user_font_get_regular(), NULL);
     convbuf = g_locale_to_utf8(buffer, nchars, NULL, &outbytes, NULL);
     gtk_text_buffer_insert_with_tags(buf, &iter, convbuf, outbytes, tag,
                                      NULL);
@@ -901,7 +945,7 @@ follow_load_text(follow_info_t *follow_info)
     int bytes_already;
 #else
     GtkTextBuffer *buf;
-    
+
     buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(follow_info->text));
 #endif
 
@@ -918,7 +962,7 @@ follow_load_text(follow_info_t *follow_info)
 #else
     gtk_text_buffer_set_text(buf, "", -1);
 #endif
-    follow_read_stream(follow_info, follow_add_to_gtk_text, follow_info->text);
+    follow_read_stream(follow_info, follow_add_to_gtk_text, follow_info->text, PR_FMT_TEXT);
 #if GTK_MAJOR_VERSION < 2
     gtk_text_thaw(GTK_TEXT(follow_info->text));
 #endif
@@ -934,7 +978,7 @@ follow_load_text(follow_info_t *follow_info)
 static void
 follow_save_as_cmd_cb(GtkWidget *w _U_, gpointer data)
 {
-    GtkWidget		*ok_bt, *new_win;
+    GtkWidget		*new_win;
     follow_info_t	*follow_info = data;
 
     if (follow_info->follow_save_as_w != NULL) {
@@ -943,94 +987,99 @@ follow_save_as_cmd_cb(GtkWidget *w _U_, gpointer data)
 	return;
     }
 
-    new_win = gtk_file_selection_new("Ethereal: Save TCP Follow Stream As");
+    new_win = file_selection_new("Ethereal: Save TCP Follow Stream As",
+                                 FILE_SELECTION_SAVE);
     follow_info->follow_save_as_w = new_win;
-    SIGNAL_CONNECT(new_win, "destroy", follow_save_as_destroy_cb, follow_info);
 
     /* Tuck away the follow_info object into the window */
     OBJECT_SET_DATA(new_win, E_FOLLOW_INFO_KEY, follow_info);
 
-    /* If we've opened a file, start out by showing the files in the directory
-       in which that file resided. */
-    if (last_open_dir)
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION(new_win),
-				    last_open_dir);
+    SIGNAL_CONNECT(new_win, "destroy", follow_save_as_destroy_cb, follow_info);
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    if (gtk_dialog_run(GTK_DIALOG(new_win)) == GTK_RESPONSE_ACCEPT)
+    {
+        follow_save_as_ok_cb(new_win, new_win);
+    } else {
+        window_destroy(new_win);
+    }
+#else
     /* Connect the ok_button to file_save_as_ok_cb function and pass along a
        pointer to the file selection box widget */
-    ok_bt = GTK_FILE_SELECTION(new_win)->ok_button;
-    SIGNAL_CONNECT(ok_bt, "clicked", follow_save_as_ok_cb, new_win);
+    SIGNAL_CONNECT(GTK_FILE_SELECTION(new_win)->ok_button, 
+        "clicked", follow_save_as_ok_cb, new_win);
 
-    /* Connect the cancel_button to destroy the widget */
-    SIGNAL_CONNECT_OBJECT(GTK_FILE_SELECTION(new_win)->cancel_button, "clicked",
-                          gtk_widget_destroy, new_win);
-
-    /* Catch the "key_press_event" signal in the window, so that we can catch
-       the ESC key being pressed and act as if the "Cancel" button had
-       been selected. */
-    dlg_set_cancel(new_win,
-		   GTK_FILE_SELECTION(new_win)->cancel_button);
+    window_set_cancel_button(new_win, 
+        GTK_FILE_SELECTION(new_win)->cancel_button, window_cancel_button_cb);
 
     gtk_file_selection_set_filename(GTK_FILE_SELECTION(new_win), "");
+
+    SIGNAL_CONNECT(new_win, "delete_event", window_delete_event_cb, NULL);
+
     gtk_widget_show_all(new_win);
+    window_present(new_win);
+#endif
 }
 
 
 static void
-follow_save_as_ok_cb(GtkWidget * w _U_, GtkFileSelection * fs)
+follow_save_as_ok_cb(GtkWidget * w _U_, gpointer fs)
 {
-	gchar		*to_name;
-	follow_info_t	*follow_info;
-	FILE		*fh;
-	gchar		*dirname;
+    gchar		*to_name;
+    follow_info_t	*follow_info;
+    FILE		*fh;
+    gchar		*dirname;
 
-	to_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    to_name = g_strdup(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fs)));
+#else
+    to_name = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+#endif
 
-	/* Perhaps the user specified a directory instead of a file.
-	   Check whether they did. */
-	if (test_for_directory(to_name) == EISDIR) {
-		/* It's a directory - set the file selection box to display that
-		   directory, and leave the selection box displayed. */
-		set_last_open_dir(to_name);
-		g_free(to_name);
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs),
-			last_open_dir);
-		return;
-	}
+    /* Perhaps the user specified a directory instead of a file.
+       Check whether they did. */
+    if (test_for_directory(to_name) == EISDIR) {
+        /* It's a directory - set the file selection box to display that
+           directory, and leave the selection box displayed. */
+        set_last_open_dir(to_name);
+        g_free(to_name);
+        file_selection_set_current_folder(fs, get_last_open_dir());
+        return;
+    }
 
-	fh = fopen(to_name, "wb");
-	if (fh == NULL) {
-		open_failure_alert_box(to_name, errno, TRUE);
-		g_free(to_name);
-		return;
-	}
+    fh = fopen(to_name, "wb");
+    if (fh == NULL) {
+        open_failure_alert_box(to_name, errno, TRUE);
+        g_free(to_name);
+        return;
+    }
 
-	gtk_widget_hide(GTK_WIDGET(fs));
-	follow_info = OBJECT_GET_DATA(fs, E_FOLLOW_INFO_KEY);
-	gtk_widget_destroy(GTK_WIDGET(fs));
+    gtk_widget_hide(GTK_WIDGET(fs));
+    follow_info = OBJECT_GET_DATA(fs, E_FOLLOW_INFO_KEY);
+    window_destroy(GTK_WIDGET(fs));
 
-	switch (follow_read_stream(follow_info, follow_print_text, fh)) {
+    switch (follow_read_stream(follow_info, follow_print_text, fh, PR_FMT_TEXT)) {
 
-	case FRS_OK:
-		if (fclose(fh) == EOF)
-			write_failure_alert_box(to_name, errno);
-		break;
+    case FRS_OK:
+        if (fclose(fh) == EOF)
+            write_failure_alert_box(to_name, errno);
+        break;
 
-	case FRS_OPEN_ERROR:
-	case FRS_READ_ERROR:
-		fclose(fh);
-		break;
+    case FRS_OPEN_ERROR:
+    case FRS_READ_ERROR:
+        fclose(fh);
+        break;
 
-	case FRS_PRINT_ERROR:
-		write_failure_alert_box(to_name, errno);
-		fclose(fh);
-		break;
-	}
+    case FRS_PRINT_ERROR:
+        write_failure_alert_box(to_name, errno);
+        fclose(fh);
+        break;
+    }
 
-	/* Save the directory name for future file dialogs. */
-	dirname = get_dirname(to_name);  /* Overwrites to_name */
-	set_last_open_dir(dirname);
-	g_free(to_name);
+    /* Save the directory name for future file dialogs. */
+    dirname = get_dirname(to_name);  /* Overwrites to_name */
+    set_last_open_dir(dirname);
+    g_free(to_name);
 }
 
 static void
