@@ -1,7 +1,7 @@
 /* gui_prefs.c
  * Dialog box for GUI preferences
  *
- * $Id: gui_prefs.c,v 1.66 2004/03/13 15:15:24 ulfl Exp $
+ * $Id: gui_prefs.c,v 1.75 2004/06/29 17:10:53 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -42,10 +42,13 @@
 #include "dlg_utils.h"
 #include "proto_draw.h"
 #include "main.h"
-#include "packet_list.h"
 #include "compat_macros.h"
+#include "font_utils.h"
+#include "packet_list.h"
 #include "toolbar.h"
 #include "recent.h"
+#include "webbrowser.h"
+
 
 static gint fetch_enum_value(gpointer control, const enum_val_t *enumvals);
 static gint fileopen_dir_changed_cb(GtkWidget *myentry _U_, GdkEvent *event, gpointer parent_w);
@@ -62,70 +65,89 @@ static gint recent_files_count_changed_cb(GtkWidget *recent_files_entry _U_,
 #else
 #define ALTERN_COLORS_KEY               "altern_colors"
 #endif
+#define FILTER_TOOLBAR_PLACEMENT_KEY    "filter_toolbar_show_in_statusbar"
 #define HEX_DUMP_HIGHLIGHT_STYLE_KEY	"hex_dump_highlight_style"
 #define GEOMETRY_POSITION_KEY		"geometry_position"
 #define GEOMETRY_SIZE_KEY		"geometry_size"
 #define GEOMETRY_MAXIMIZED_KEY		"geometry_maximized"
 
+#define GUI_CONSOLE_OPEN_KEY "console_open"
 #define GUI_FILEOPEN_KEY	"fileopen_behavior"
 #define GUI_RECENT_FILES_COUNT_KEY "recent_files_count"
 #define GUI_FILEOPEN_DIR_KEY	"fileopen_directory"
+#define GUI_ASK_UNSAVED_KEY     "ask_unsaved"
+#define GUI_WEBBROWSER_KEY	    "webbrowser"
 
 #define GUI_TOOLBAR_STYLE_KEY	"toolbar_style"
 
 static const enum_val_t scrollbar_placement_vals[] = {
-	{ "Left",  FALSE },
-	{ "Right", TRUE },
-	{ NULL,    0 }
+	{ "FALSE", "Left", FALSE },
+	{ "TRUE",  "Right", TRUE },
+	{ NULL,    NULL,    0 }
 };
 
 static const enum_val_t selection_mode_vals[] = {
-	{ "Selects", FALSE },
-	{ "Browses", TRUE },
-	{ NULL,      0 }
+	{ "FALSE", "Selects", FALSE },
+	{ "TRUE",  "Browses", TRUE },
+	{ NULL,    NULL,      0 }
 };
 
 #if GTK_MAJOR_VERSION < 2
 static const enum_val_t line_style_vals[] = {
-	{ "None",   0 },
-	{ "Solid",  1 },
-	{ "Dotted", 2 },
-	{ "Tabbed", 3 },
-	{ NULL,     0 }
+	{ "NONE",   "None",   0 },
+	{ "SOLID",  "Solid",  1 },
+	{ "DOTTED", "Dotted", 2 },
+	{ "TABBED", "Tabbed", 3 },
+	{ NULL,     NULL,     0 }
 };
 
 static const enum_val_t expander_style_vals[] = {
-	{ "None",     0 },
-	{ "Square",   1 },
-	{ "Triangle", 2 },
-	{ "Circular", 3 },
-	{ NULL,       0 }
+	{ "NONE",     "None",     0 },
+	{ "SQUARE",   "Square",   1 },
+	{ "TRIANGLE", "Triangle", 2 },
+	{ "CIRCULAR", "Circular", 3 },
+	{ NULL,       NULL,       0 }
 };
 #else
 static const enum_val_t altern_colors_vals[] = {
-	{ "No",  FALSE },
-	{ "Yes",  TRUE },
-	{ NULL,      0 }
+	{ "FALSE", "No",  FALSE },
+	{ "TRUE",  "Yes", TRUE },
+	{ NULL,    NULL,  0 }
 };
 #endif
 
+static const enum_val_t filter_toolbar_placement_vals[] = {
+	{ "FALSE", "Below the main toolbar", FALSE },
+	{ "TRUE",  "Insert into statusbar",  TRUE },
+	{ NULL,    NULL,                     0 }
+};
+
 static const enum_val_t highlight_style_vals[] = {
-  	{ "Bold",     FALSE },
-  	{ "Inverse",  TRUE },
-	{ NULL,       0 }
+  	{ "FALSE", "Bold",     FALSE },
+  	{ "TRUE",  "Inverse",  TRUE },
+	{ NULL,    NULL,       0 }
 };
 
 static const enum_val_t toolbar_style_vals[] = {
-  	{ "Icons only",     TB_STYLE_ICONS },
-  	{ "Text only",      TB_STYLE_TEXT },
-  	{ "Icons & Text",   TB_STYLE_BOTH },
-	{ NULL,             0 }
+  	{ "ICONS", "Icons only",     TB_STYLE_ICONS },
+  	{ "TEXT",  "Text only",      TB_STYLE_TEXT },
+  	{ "BOTH",  "Icons & Text",   TB_STYLE_BOTH },
+	{ NULL,    NULL,             0 }
 };
 
+#ifdef _WIN32
+static const enum_val_t gui_console_open_vals[] = {
+	{ "NEVER",     "Never",                      console_open_never },
+	{ "AUTOMATIC", "Automatic (advanced user)",  console_open_auto },
+	{ "ALWAYS",    "Always (debugging)",         console_open_always },
+	{ NULL,        NULL,                         0 }
+};
+#endif
+
 static const enum_val_t gui_fileopen_vals[] = {
-	{ "Remember last directory", FO_STYLE_LAST_OPENED },
-	{ "Always start in:", FO_STYLE_SPECIFIED },
-	{ NULL,    0 }
+	{ "LAST_OPENED", "Remember last directory", FO_STYLE_LAST_OPENED },
+	{ "SPECIFIED",   "Always start in:",        FO_STYLE_SPECIFIED },
+	{ NULL,          NULL,                      0 }
 };
 
 /* Set to FALSE initially; set to TRUE if the user ever hits "OK" on
@@ -155,8 +177,13 @@ gui_prefs_show(void)
 	GtkWidget *main_tb, *main_vb, *hbox;
 	GtkWidget *scrollbar_om, *plist_browse_om;
 	GtkWidget *ptree_browse_om, *highlight_style_om;
+#ifdef _WIN32
+	GtkWidget *console_open_om;
+#endif
 	GtkWidget *fileopen_rb, *fileopen_dir_te, *toolbar_style_om;
-	GtkWidget *recent_files_count_max_te;
+    GtkWidget *filter_toolbar_placement_om;
+	GtkWidget *recent_files_count_max_te, *ask_unsaved_cb;
+    GtkWidget *webbrowser_te;
 	GtkWidget *save_position_cb, *save_size_cb, *save_maximized_cb;
 #if GTK_MAJOR_VERSION < 2
 	GtkWidget *expander_style_om, *line_style_om;
@@ -237,6 +264,12 @@ gui_prefs_show(void)
 	OBJECT_SET_DATA(main_vb, GUI_TOOLBAR_STYLE_KEY,
 	    toolbar_style_om);
 
+    /* Placement of Filter toolbar */
+    filter_toolbar_placement_om = create_preference_option_menu(main_tb, pos++,
+       "Filter toolbar placement:", NULL,
+       filter_toolbar_placement_vals, prefs.filter_toolbar_show_in_statusbar);
+    OBJECT_SET_DATA(main_vb, FILTER_TOOLBAR_PLACEMENT_KEY, filter_toolbar_placement_om);
+
 	/* Geometry prefs */
 	save_position_cb = create_preference_check_button(main_tb, pos++,
 	    "Save window position:", NULL, prefs.gui_geometry_save_position);
@@ -250,6 +283,14 @@ gui_prefs_show(void)
 	    "Save maximized state:", NULL, prefs.gui_geometry_save_maximized);
 	OBJECT_SET_DATA(main_vb, GEOMETRY_MAXIMIZED_KEY, save_maximized_cb);
 
+#ifdef _WIN32
+	/* How the console window should be opened */
+    console_open_om = create_preference_option_menu(main_tb, pos++,
+       "Open a console window", NULL,
+       gui_console_open_vals, prefs.gui_console_open);
+	OBJECT_SET_DATA(main_vb, GUI_CONSOLE_OPEN_KEY, console_open_om);
+#endif
+
 	/* Allow user to select where they want the File Open dialog to open to
 	 * by default */
 	fileopen_rb = create_preference_radio_buttons(main_tb, pos++,
@@ -275,6 +316,20 @@ gui_prefs_show(void)
 
 	fileopen_selected_cb(NULL, main_vb);        
 
+    /* ask for unsaved capture files? */
+	ask_unsaved_cb = create_preference_check_button(main_tb, pos++,
+	    "Ask for unsaved capture files:", NULL, prefs.gui_ask_unsaved);
+	OBJECT_SET_DATA(main_vb, GUI_ASK_UNSAVED_KEY, ask_unsaved_cb);
+
+	/* Webbrowser */
+    if(browser_needs_pref()) {
+	    webbrowser_te = create_preference_entry(main_tb, pos++, 
+            "Web browser command:", NULL, prefs.gui_webbrowser);
+	    gtk_entry_set_text(GTK_ENTRY(webbrowser_te), prefs.gui_webbrowser);
+	    OBJECT_SET_DATA(main_vb, GUI_WEBBROWSER_KEY, webbrowser_te);
+    }
+
+
 	/* Show 'em what we got */
 	gtk_widget_show_all(main_vb);
 
@@ -286,38 +341,8 @@ gui_prefs_show(void)
 GtkWidget *
 gui_font_prefs_show(void)
 {
-#if 0
-#if GTK_MAJOR_VERSION < 2
-	static gchar *fixedwidths[] = { "c", "m", NULL };
-#endif
-#endif
-
-	/* Now create a new widget. */
+	/* Create the font selection widget. */
 	font_browse_w = (GtkWidget *) gtk_font_selection_new();
-
-    /*gtk_font_selection_set_preview_text(GTK_FONT_SELECTION(font_browse_w), 
-        "WWWWWWWW llllllll (Tip: use a fixed width font)");*/
-
-#if 0
-    /* GTK (at least version 1.3) has an annoying bug: */
-    /* when using the font selection instead of the dialog, */
-    /* the widget seems to disconnected from the parent, */
-    /* if a filter is used! */
-#if GTK_MAJOR_VERSION < 2
-	/* Set its filter to show only fixed_width fonts. */
-	gtk_font_selection_set_filter(
-	    GTK_FONT_SELECTION(font_browse_w),
-	    GTK_FONT_FILTER_BASE, /* user can't change the filter */
-	    GTK_FONT_ALL,	  /* bitmap or scalable are fine */
-	    NULL,		  /* all foundries are OK */
-	    NULL,		  /* all weights are OK (XXX - normal only?) */
-	    NULL,		  /* all slants are OK (XXX - Roman only?) */
-	    NULL,		  /* all setwidths are OK */
-	    fixedwidths,	  /* ONLY fixed-width fonts */
-	    NULL);	/* all charsets are OK (XXX - ISO 8859/1 only?) */
-#endif
-#endif
-
 	gtk_widget_show(font_browse_w);
 
 	return font_browse_w;
@@ -328,12 +353,6 @@ static gboolean
 font_fetch(void)
 {
 	gchar   *font_name;
-#if GTK_MAJOR_VERSION < 2
-	gchar   *bold_font_name;
-	GdkFont *new_r_font, *new_b_font;
-#else
-	PangoFontDescription *new_r_font, *new_b_font;
-#endif
 
 	font_name = g_strdup(gtk_font_selection_get_font_name(
 	      GTK_FONT_SELECTION(font_browse_w)));
@@ -346,53 +365,12 @@ font_fetch(void)
 		return FALSE;
 	}
 
-#if GTK_MAJOR_VERSION < 2
-	/* Get the name that the boldface version of that font would have. */
-	bold_font_name = font_boldify(font_name);
+    if(user_font_test(font_name)) {
+    	new_font_name = font_name;
+        return TRUE;
+    }
 
-	/* Now load those fonts, just to make sure we can. */
-	new_r_font = gdk_font_load(font_name);
-#else
-	new_r_font = pango_font_description_from_string(font_name);
-#endif
-	if (new_r_font == NULL) {
-		/* Oops, that font didn't work.
-		   Tell the user, but don't tear down the font selection
-		   dialog, so that they can try again. */
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		   "The font you selected cannot be loaded.");
-
-		g_free(font_name);
-#if GTK_MAJOR_VERSION < 2
-		g_free(bold_font_name);
-#endif
-		return FALSE;
-	}
-
-#if GTK_MAJOR_VERSION < 2
-	new_b_font = gdk_font_load(bold_font_name);
-#else
-	new_b_font = pango_font_description_copy(new_r_font);
-	pango_font_description_set_weight(new_b_font, PANGO_WEIGHT_BOLD);
-#endif
-	if (new_b_font == NULL) {
-		/* Oops, that font didn't work.
-		   Tell the user, but don't tear down the font selection
-		   dialog, so that they can try again. */
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-		   "The font you selected doesn't have a boldface version.");
-
-		g_free(font_name);
-#if GTK_MAJOR_VERSION < 2
-		g_free(bold_font_name);
-		gdk_font_unref(new_r_font);
-#else
-		pango_font_description_free(new_r_font);
-#endif
-		return FALSE;
-	}
-
-	new_font_name = font_name;
+    g_free(font_name);
 	return TRUE;
 }
 
@@ -422,6 +400,8 @@ gui_prefs_fetch(GtkWidget *w)
 	prefs.gui_altern_colors = fetch_enum_value(
 	    OBJECT_GET_DATA(w, ALTERN_COLORS_KEY), altern_colors_vals);
 #endif
+    prefs.filter_toolbar_show_in_statusbar = fetch_enum_value(
+        OBJECT_GET_DATA(w, FILTER_TOOLBAR_PLACEMENT_KEY), filter_toolbar_placement_vals);
 	prefs.gui_hex_dump_highlight_style = fetch_enum_value(
 	    OBJECT_GET_DATA(w, HEX_DUMP_HIGHLIGHT_STYLE_KEY),
 	    highlight_style_vals);
@@ -435,6 +415,10 @@ gui_prefs_fetch(GtkWidget *w)
 	    gtk_toggle_button_get_active(OBJECT_GET_DATA(w, GEOMETRY_SIZE_KEY));
 	prefs.gui_geometry_save_maximized =
 	    gtk_toggle_button_get_active(OBJECT_GET_DATA(w, GEOMETRY_MAXIMIZED_KEY));
+#ifdef _WIN32
+	prefs.gui_console_open = fetch_enum_value(
+	    OBJECT_GET_DATA(w, GUI_CONSOLE_OPEN_KEY), gui_console_open_vals);
+#endif
 	prefs.gui_fileopen_style = fetch_preference_radio_buttons_val(
 	    OBJECT_GET_DATA(w, GUI_FILEOPEN_KEY), gui_fileopen_vals);
 	    
@@ -443,6 +427,16 @@ gui_prefs_fetch(GtkWidget *w)
 	prefs.gui_fileopen_dir = g_strdup(gtk_entry_get_text(
 		GTK_ENTRY(OBJECT_GET_DATA(w, GUI_FILEOPEN_DIR_KEY))));
 
+    prefs.gui_ask_unsaved = 
+	    gtk_toggle_button_get_active(OBJECT_GET_DATA(w, GUI_ASK_UNSAVED_KEY));
+
+    if(browser_needs_pref()) {
+		g_free(prefs.gui_webbrowser);
+	    prefs.gui_webbrowser = g_strdup(gtk_entry_get_text(
+		    GTK_ENTRY(OBJECT_GET_DATA(w, GUI_WEBBROWSER_KEY))));
+    }
+
+
 	/*
 	 * XXX - we need to have a way to fetch the preferences into
 	 * local storage and only set the permanent preferences if there
@@ -465,15 +459,22 @@ void
 gui_prefs_apply(GtkWidget *w _U_)
 {
 
+#ifdef _WIN32
+    /* user immediately wants to see a console */
+    if (prefs.gui_console_open == console_open_always) {
+        create_console();
+    }
+#endif
+
 	if (font_changed) {
 		/* This redraws the hex dump windows. */
-		switch (font_apply()) {
+		switch (user_font_apply()) {
 
 		case FA_SUCCESS:
 			break;
 
 		case FA_FONT_NOT_RESIZEABLE:
-			/* "font_apply()" popped up an alert box. */
+			/* "user_font_apply()" popped up an alert box. */
 			/* turn off zooming - font can't be resized */
 			recent.gui_zoom_level = 0;
 			break;
@@ -482,7 +483,7 @@ gui_prefs_apply(GtkWidget *w _U_)
 			/* We assume this means that the specified size
 			   isn't available. */
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
-			    "That font font isn't available at the specified zoom level;\n"
+			    "That font isn't available at the specified zoom level;\n"
 			    "turning zooming off.");
 			recent.gui_zoom_level = 0;
 			break;
@@ -505,6 +506,7 @@ gui_prefs_apply(GtkWidget *w _U_)
 	set_plist_sel_browse(prefs.gui_plist_sel_browse);
 	set_ptree_sel_browse_all(prefs.gui_ptree_sel_browse);
 	set_tree_styles_all();
+    main_widgets_rearrange();
 }
 
 void
