@@ -1,7 +1,7 @@
 /* io_stat.c
  * io_stat   2002 Ronnie Sahlberg
  *
- * $Id: io_stat.c,v 1.73 2004/03/13 15:15:24 ulfl Exp $
+ * $Id: io_stat.c,v 1.76 2004/05/23 23:24:06 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -1519,7 +1519,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 	if(gio->io->count_type==COUNT_TYPE_ADVANCED){
 		/* warn and bail out if there was no field specified */
 		if(field==NULL || field[0]==0){
-			simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK, "You did not specify a field name.");
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "You did not specify a field name.");
 			disable_graph(gio);
 			io_stat_redraw(gio->io);
 			return 0;
@@ -1527,7 +1527,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 		/* warn and bail out if the field could not be found */
 		hfi=proto_registrar_get_byname(field);
 		if(hfi==NULL){
-			simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK, "'%s' is not a valid field name.", field);
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "'%s' is not a valid field name.", field);
 			disable_graph(gio);
 			io_stat_redraw(gio->io);
 			return 0;
@@ -1546,7 +1546,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 			/* these values support all calculations except LOAD */
 			switch(gio->calc_type){
 			case CALC_TYPE_LOAD:
-				simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+				simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				    "LOAD(*) is only supported for relative-time fields.");
 				disable_graph(gio);
 				io_stat_redraw(gio->io);
@@ -1564,7 +1564,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 			case CALC_TYPE_LOAD:
 				break;
 			default:
-				simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+				simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				    "%s is a relative-time field, so %s calculations are not supported on it.",
 				    field,
 				    calc_type_names[gio->calc_type]);
@@ -1580,7 +1580,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 			 * available?
 			 */
 			if(gio->calc_type!=CALC_TYPE_COUNT){
-				simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+				simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				    "%s is a 64-bit integer, so %s calculations are not supported on it.",
 				    field,
 				    calc_type_names[gio->calc_type]);
@@ -1595,7 +1595,7 @@ filter_callback(GtkWidget *widget _U_, io_stat_graph_t *gio)
 			 * numbers?
 			 */
 			if(gio->calc_type!=CALC_TYPE_COUNT){
-				simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
+				simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
 				    "%s doesn't have integral values, so %s calculations are not supported on it.",
 				    field,
 				    calc_type_names[gio->calc_type]);
@@ -1868,16 +1868,6 @@ create_filter_area(io_stat_t *io, GtkWidget *box)
 }
 
 
-static void
-io_stat_close_cb(
-    GtkButton		*button _U_,
-    gpointer		parent_w)
-{
-    gtk_grab_remove(GTK_WIDGET(parent_w));
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-
 static void 
 init_io_stat_window(io_stat_t *io)
 {
@@ -1886,9 +1876,7 @@ init_io_stat_window(io_stat_t *io)
     GtkWidget *bt_close;
 
 	/* create the main window */
-	io->window=window_new(GTK_WINDOW_TOPLEVEL, NULL);
-
-	gtk_widget_set_name(io->window, "I/O Statistics");
+	io->window=window_new(GTK_WINDOW_TOPLEVEL, "I/O Graphs");
 
 	vbox=gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(io->window), vbox);
@@ -1912,15 +1900,12 @@ init_io_stat_window(io_stat_t *io)
     gtk_widget_show(hbox);
 
     bt_close = OBJECT_GET_DATA(hbox, GTK_STOCK_CLOSE);
-    gtk_widget_grab_default(bt_close);
-    SIGNAL_CONNECT(bt_close, "clicked", io_stat_close_cb, io->window);
+    window_set_cancel_button(io->window, bt_close, window_cancel_button_cb);
 
-    /* Catch the "key_press_event" signal in the window, so that we can
-	   catch the ESC key being pressed and act as if the "Cancel" button
-	   had been selected. */
-	dlg_set_cancel(io->window, bt_close);
+    SIGNAL_CONNECT(io->window, "delete_event", window_delete_event_cb, NULL);
 
     gtk_widget_show(io->window);
+    window_present(io->window);
 }
 
 
