@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c,v 1.90 2004/03/20 06:34:09 guy Exp $
+ * $Id: proto_draw.c,v 1.107 2004/06/30 06:58:58 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -67,6 +67,8 @@
 #include "compat_macros.h"
 #include "alert_box.h"
 #include "simple_dialog.h"
+#include "progress_dlg.h"
+#include "font_utils.h"
 
 
 /* Win32 needs the O_BINARY flag for open() */
@@ -85,6 +87,11 @@
 #define E_BYTE_VIEW_END_KEY       "byte_view_end"
 #define E_BYTE_VIEW_ENCODE_KEY    "byte_view_encode"
 
+
+#if GTK_MAJOR_VERSION < 2
+GtkStyle *item_style = NULL;
+#endif
+
 /* gtk_tree_view_expand_to_path doesn't exist in gtk+ v2.0 so we must include it
  * when building with this version (taken from gtk+ v2.2.4) */
 #if GTK_MAJOR_VERSION >= 2 && GTK_MINOR_VERSION == 0
@@ -214,10 +221,20 @@ redraw_hex_dump(GtkWidget *nb, frame_data *fd, field_info *finfo)
 void
 redraw_hex_dump_all(void)
 {
-  if (cfile.current_frame != NULL)
-    redraw_hex_dump( byte_nb_ptr, cfile.current_frame, cfile.finfo_selected);
+    if (cfile.current_frame != NULL)
+            redraw_hex_dump( byte_nb_ptr, cfile.current_frame, cfile.finfo_selected);
 
   redraw_hex_dump_packet_wins();
+
+#if GTK_MAJOR_VERSION >= 2
+  /* XXX - this is a hack, to workaround a bug in GTK2.x!
+     when changing the font size, even refilling of the corresponding 
+     gtk_text_buffer doesn't seem to trigger an update.     
+     The only workaround is to freshly select the frame, which will remove any 
+     existing notebook tabs and "restart" the whole byte view again. */
+  if (cfile.current_frame != NULL)
+    goto_frame(&cfile, cfile.current_frame->num);
+#endif
 }
 
 #if GTK_MAJOR_VERSION < 2
@@ -481,8 +498,8 @@ byte_view_select(GtkWidget *widget, GdkEventButton *event)
      * which text column/row the user selected. This could be off
      * if the bold version of the font is bigger than the
      * regular version of the font. */
-    column = (int) ((bv->hadj->value + event->x) / m_font_width);
-    row = (int) ((bv->vadj->value + event->y) / m_font_height);
+    column = (int) ((bv->hadj->value + event->x) / user_font_get_regular_width());
+    row = (int) ((bv->vadj->value + event->y) / user_font_get_regular_height());
 #else
     /* get the row/column selected */
     gtk_text_view_window_to_buffer_coords(bv,
@@ -630,6 +647,12 @@ byte_view_new(void)
   /* this will only have an effect, if no tabs are shown */
   gtk_notebook_set_show_border(GTK_NOTEBOOK(byte_nb), FALSE);
 
+  /* set the tabs scrollable, if they don't fit into the pane */
+  gtk_notebook_set_scrollable(GTK_NOTEBOOK(byte_nb), TRUE);
+
+  /* enable a popup menu containing the tab labels, will be helpful if tabs don't fit into the pane */
+  gtk_notebook_popup_enable(GTK_NOTEBOOK(byte_nb));
+
   /* Add a placeholder byte view so that there's at least something
      displayed in the byte view notebook. */
   add_byte_tab(byte_nb, "", NULL, NULL, NULL);
@@ -698,13 +721,13 @@ add_byte_tab(GtkWidget *byte_nb, const char *name, tvbuff_t *tvb,
   gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(byte_view), FALSE);
   buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(byte_view));
   style = gtk_widget_get_style(GTK_WIDGET(byte_view));
-  gtk_text_buffer_create_tag(buf, "plain", "font-desc", m_r_font, NULL);
+  gtk_text_buffer_create_tag(buf, "plain", "font-desc", user_font_get_regular(), NULL);
   gtk_text_buffer_create_tag(buf, "reverse",
-                             "font-desc", m_r_font,
+                             "font-desc", user_font_get_regular(),
                              "foreground-gdk", &style->text[GTK_STATE_SELECTED],
                              "background-gdk", &style->base[GTK_STATE_SELECTED],
                              NULL);
-  gtk_text_buffer_create_tag(buf, "bold", "font-desc", m_b_font, NULL);
+  gtk_text_buffer_create_tag(buf, "bold", "font-desc", user_font_get_bold(), NULL);
 #endif
   OBJECT_SET_DATA(byte_view, E_BYTE_VIEW_TVBUFF_KEY, tvb);
   gtk_container_add(GTK_CONTAINER(byte_scrollw), byte_view);
@@ -724,6 +747,10 @@ add_byte_tab(GtkWidget *byte_nb, const char *name, tvbuff_t *tvb,
   else
         gtk_notebook_set_show_tabs(GTK_NOTEBOOK(byte_nb), TRUE);
 
+  /* set this page (this will print the packet data) */
+  gtk_notebook_set_page(GTK_NOTEBOOK(byte_nb), 
+    gtk_notebook_page_num(GTK_NOTEBOOK(byte_nb), byte_nb));
+
   return byte_view;
 }
 
@@ -762,8 +789,9 @@ add_byte_views(epan_dissect_t *edt, GtkWidget *tree_view,
 	gtk_notebook_set_page(GTK_NOTEBOOK(byte_nb_ptr), 0);
 }
 
+
+
 static GtkWidget *savehex_dlg=NULL;
-static GtkWidget *file_entry=NULL;
 
 static void
 savehex_dlg_destroy_cb(void)
@@ -771,144 +799,20 @@ savehex_dlg_destroy_cb(void)
         savehex_dlg = NULL;
 }
 
-static void
-savehex_dlg_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
-{
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-/* Forward declaration */
-static void
-savehex_save_clicked_cb(GtkWidget * w, gpointer data);
-
-/* Launch the dialog box to put up the file selection box etc */
-void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
-{
-	int start, end, len;
-	const guint8 *data_p = NULL;
-    gchar label[200];
-
-    GtkWidget   *bv;
-	GtkWidget   *dlg_box, *dlg_lb;
-	GtkWidget   *file_box, *file_bt = NULL;
-	GtkWidget   *bbox, *save_button, *cancel_button;
-	GtkTooltips *tooltips;
-
-
-    /* don't show up the dialog, if no data has to be saved */
-	bv = get_notebook_bv_ptr(byte_nb_ptr);
-	if (bv == NULL) {
-		/* shouldn't happen */
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Could not find the corresponding text window!");
-		return;
-	}
-	end = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_START_KEY));
-	start = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_END_KEY));
-	data_p = get_byte_view_data_and_length(GTK_WIDGET(bv), &len);
-
-	if (data_p == NULL || start == -1 || start > end) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "No data selected to save!");
-		return;
-	}
-
-    /* if the window is already open, bring it to front */
-	if(savehex_dlg){
-		gdk_window_raise(savehex_dlg->window);
-		return;
-	}
-
-	tooltips = gtk_tooltips_new();
-
-	/*
-	 * Build the dialog box we need ... a text entry field and a
-	 * browse button, along with OK and Cancel
-	 */
-
-	savehex_dlg=dlg_window_new("Ethereal: Export Selected Packet Bytes");
-	SIGNAL_CONNECT(savehex_dlg, "destroy", savehex_dlg_destroy_cb, NULL);
-
-	dlg_box=gtk_vbox_new(FALSE, 10);
-	gtk_container_border_width(GTK_CONTAINER(dlg_box), 10);
-	gtk_container_add(GTK_CONTAINER(savehex_dlg), dlg_box);
-	gtk_widget_show(dlg_box);
-
-    /* label */
-    g_snprintf(label, sizeof(label), "Will save %u %s of raw binary data to file:", 
-        end - start, plurality(end - start, "byte", "bytes"));
-    dlg_lb = gtk_label_new(label);
-	gtk_box_pack_start(GTK_BOX(dlg_box), dlg_lb, FALSE, FALSE, 0);
-	gtk_widget_show(dlg_lb);
-
-	/* File entry box */
-	file_box=gtk_hbox_new(FALSE, 3);
-
-	/* File entry */
-	file_entry=gtk_entry_new();
-	WIDGET_SET_SIZE(file_entry, 300, -1);
-	gtk_tooltips_set_tip(tooltips, file_entry, ("Enter Save Data filename"), NULL);
-	gtk_box_pack_start(GTK_BOX(file_box), file_entry, TRUE, TRUE, 0);
-	gtk_widget_show(file_entry);
-
-	/* File Browse button */
-	file_bt=BUTTON_NEW_FROM_STOCK(ETHEREAL_STOCK_BROWSE);
-	SIGNAL_CONNECT(file_bt, "clicked", select_file_cb,
-		       "Ethereal: Export Selected Packet Bytes");
-
-	/* file entry for print dialog */
-	OBJECT_SET_DATA(file_bt, E_FILE_TE_PTR_KEY, file_entry);
-	/* file entry for print dialog */
-
-	gtk_tooltips_set_tip (tooltips, file_bt, ("Browse output filename in filesystem"), NULL);
-	gtk_box_pack_start(GTK_BOX(file_box), file_bt, FALSE, TRUE, 0);
-	gtk_widget_show(file_bt);
-
-	gtk_box_pack_start(GTK_BOX(dlg_box), file_box, TRUE, TRUE, 0);
-	gtk_widget_show(file_box);
-
-	/* Now, the button box */
-    bbox = dlg_button_row_new(GTK_STOCK_SAVE, GTK_STOCK_CANCEL, NULL);
-    gtk_box_pack_start(GTK_BOX(dlg_box), bbox, FALSE, FALSE, 0);
-    gtk_widget_show(bbox);
-
-	/* the save button */
-    save_button = OBJECT_GET_DATA(bbox, GTK_STOCK_SAVE);
-    SIGNAL_CONNECT_OBJECT(save_button, "clicked",
-                              savehex_save_clicked_cb, NULL);
-	gtk_widget_grab_default(save_button);
-	gtk_tooltips_set_tip (tooltips, save_button, ("Save the data to the specified file"), NULL);
-
-	cancel_button = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-	SIGNAL_CONNECT(cancel_button, "clicked", savehex_dlg_cancel_cb, savehex_dlg);
-	gtk_tooltips_set_tip (tooltips, cancel_button, ("Cancel save and quit dialog"), NULL);
-
-	/* Catch the "activate" signal on the filter text entry, so that
-	   if the user types Return there, we act as if the "Create Stat"
-	   button had been selected, as happens if Return is typed if some
-	   widget that *doesn't* handle the Return key has the input
-	   focus. */
-	dlg_set_activate(file_entry, save_button);
-
-	/* Catch the "key_press_event" signal in the window, so that we can
-	   catch the ESC key being pressed and act as if the "Cancel" button
-	   had been selected. */
-	dlg_set_cancel(savehex_dlg, cancel_button);
-
-	/* Give the initial focus to the "File" entry box. */
-	gtk_widget_grab_focus(file_entry);
-
-	gtk_widget_show_all(savehex_dlg);
-}
-
-/* save the current highlighted hex data as hex_raw.dat */
+/* save the current highlighted hex data */
 static void
 savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 {
         GtkWidget *bv;
 	int fd, start, end, len;
 	const guint8 *data_p = NULL;
-	char *file = NULL;
+	const char *file = NULL;
 
-	file = (char *)gtk_entry_get_text(GTK_ENTRY(file_entry));
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(savehex_dlg));
+#else
+    file = gtk_file_selection_get_filename(GTK_FILE_SELECTION(savehex_dlg));
+#endif
 
 	if (!file ||! *file) {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Please enter a filename!");
@@ -952,10 +856,100 @@ savehex_save_clicked_cb(GtkWidget * w _U_, gpointer data _U_)
 	}
 
 	/* Get rid of the dialog box */
-	gtk_widget_destroy(GTK_WIDGET(savehex_dlg));
+	window_destroy(GTK_WIDGET(savehex_dlg));
+}
+
+/* Launch the dialog box to put up the file selection box etc */
+void savehex_cb(GtkWidget * w _U_, gpointer data _U_)
+{
+	int start, end, len;
+	const guint8 *data_p = NULL;
+    gchar *label;
+
+    GtkWidget   *bv;
+	GtkWidget   *dlg_lb;
+
+
+    /* don't show up the dialog, if no data has to be saved */
+	bv = get_notebook_bv_ptr(byte_nb_ptr);
+	if (bv == NULL) {
+		/* shouldn't happen */
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Could not find the corresponding text window!");
+		return;
+	}
+	end = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_START_KEY));
+	start = GPOINTER_TO_INT(OBJECT_GET_DATA(bv, E_BYTE_VIEW_END_KEY));
+	data_p = get_byte_view_data_and_length(GTK_WIDGET(bv), &len);
+
+	if (data_p == NULL || start == -1 || start > end) {
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "No data selected to save!");
+		return;
+	}
+
+    /* if the window is already open, bring it to front */
+	if(savehex_dlg){
+		reactivate_window(savehex_dlg);
+		return;
+	}
+
+	/*
+	 * Build the dialog box we need.
+	 */
+    savehex_dlg = file_selection_new("Ethereal: Export Selected Packet Bytes", FILE_SELECTION_SAVE);
+
+    /* label */
+    label = g_strdup_printf("Will save %u %s of raw binary data to specified file.", 
+        end - start, plurality(end - start, "byte", "bytes"));
+    dlg_lb = gtk_label_new(label);
+    g_free(label);
+    file_selection_set_extra_widget(savehex_dlg, dlg_lb);
+	gtk_widget_show(dlg_lb);
+
+    SIGNAL_CONNECT(savehex_dlg, "destroy", savehex_dlg_destroy_cb, NULL);
+
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 4) || GTK_MAJOR_VERSION > 2
+    if (gtk_dialog_run(GTK_DIALOG(savehex_dlg)) == GTK_RESPONSE_ACCEPT) {
+        savehex_save_clicked_cb(savehex_dlg, savehex_dlg);
+    } else {
+        window_destroy(savehex_dlg);
+    }
+#else
+    /* Connect the ok_button to file_save_as_ok_cb function and pass along a
+    pointer to the file selection box widget */
+    SIGNAL_CONNECT(GTK_FILE_SELECTION (savehex_dlg)->ok_button, "clicked",
+        savehex_save_clicked_cb, savehex_dlg);
+
+    window_set_cancel_button(savehex_dlg, 
+    GTK_FILE_SELECTION(savehex_dlg)->cancel_button, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(savehex_dlg, "delete_event", window_delete_event_cb, NULL);
+
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(savehex_dlg), "");
+
+    gtk_widget_show_all(savehex_dlg);
+    window_present(savehex_dlg);
+#endif
 }
 
 
+
+/* Update the progress bar this many times when reading a file. */
+#define N_PROGBAR_UPDATES	100
+
+
+/*
+ * XXX - at least in GTK+ 2.x, this is not fast - in one capture with a
+ * 64K-or-so reassembled HTTP reply, it takes about 3 seconds to construct
+ * the hex dump pane on a 1.4 GHz G4 PowerMac on OS X 10.3.3.  (That's
+ * presumably why there's a progress bar for it.)
+ *
+ * Perhaps what's needed is a custom widget (either one that lets you stuff
+ * text into it more quickly, or one that's a "virtual" widget so that the
+ * text for a row is constructed, via a callback, when the row is to be
+ * displayed).  A custom widget might also let us treat the offset, hex
+ * data, and ASCII data as three columns, so you can select purely in
+ * the hex dump column.
+ */
 #if GTK_MAJOR_VERSION < 2
 static void
 packet_hex_print_common(GtkText *bv, const guint8 *pd, int len, int bstart,
@@ -986,6 +980,14 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
   GtkTextMark   *mark = NULL;
 #endif
 
+  progdlg_t  *progbar = NULL;
+  float       progbar_val;
+  gboolean    progbar_stop_flag;
+  GTimeVal    progbar_start_time;
+  gchar       progbar_status_str[100];
+  int         progbar_nextstep;
+  int         progbar_quantum;
+
 #if GTK_MAJOR_VERSION < 2
   /* Freeze the text for faster display */
   gtk_text_freeze(bv);
@@ -1022,7 +1024,51 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
   /* Record the number of digits in this text view. */
   OBJECT_SET_DATA(bv, E_BYTE_VIEW_NDIGITS_KEY, GUINT_TO_POINTER(use_digits));
 
+  /* Update the progress bar when it gets to this value. */
+  progbar_nextstep = 0;
+  /* When we reach the value that triggers a progress bar update,
+     bump that value by this amount. */
+  progbar_quantum = len/N_PROGBAR_UPDATES;
+
+  progbar_stop_flag = FALSE;
+  g_get_current_time(&progbar_start_time);
+
   while (i < len) {
+    /* Update the progress bar, but do it only N_PROGBAR_UPDATES times;
+       when we update it, we have to run the GTK+ main loop to get it
+       to repaint what's pending, and doing so may involve an "ioctl()"
+       to see if there's any pending input from an X server, and doing
+       that for every packet can be costly, especially on a big file. */
+    if (i >= progbar_nextstep) {
+      /* let's not divide by zero. I should never be started
+       * with count == 0, so let's assert that
+       */
+      g_assert(len > 0);
+      progbar_val = (gfloat) i / len;
+
+      if (progbar == NULL)
+        /* Create the progress bar if necessary */
+        progbar = delayed_create_progress_dlg("Processing", "Packet Details",
+                                              &progbar_stop_flag,
+                                              &progbar_start_time,
+                                              progbar_val);
+
+      if (progbar != NULL) {
+        g_snprintf(progbar_status_str, sizeof(progbar_status_str),
+                   "%4u of %u bytes", i, len);
+        update_progress_dlg(progbar, progbar_val, progbar_status_str);
+      }
+
+      progbar_nextstep += progbar_quantum;
+    }
+
+    if (progbar_stop_flag) {
+      /* Well, the user decided to abort the operation.  Just stop,
+         and arrange to return TRUE to our caller, so they know it
+         was stopped explicitly. */
+      break;
+    }
+
     /* Print the line number */
     j = use_digits;
     cur = 0;
@@ -1038,7 +1084,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
     /* Display with inverse video ? */
 #if GTK_MAJOR_VERSION < 2
     if (prefs.gui_hex_dump_highlight_style) {
-      gtk_text_insert(bv, m_r_font, &BLACK, &WHITE, line, -1);
+      gtk_text_insert(bv, user_font_get_regular(), &BLACK, &WHITE, line, -1);
       /* Do we start in reverse? */
       reverse = i >= bstart && i < bend;
       fg = reverse ? &WHITE : &BLACK;
@@ -1058,7 +1104,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	newreverse = i >= bstart && i < bend;
 	/* Have we gone from reverse to plain? */
 	if (reverse && (reverse != newreverse)) {
-	  gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+	  gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
 	  fg = &BLACK;
 	  bg = &WHITE;
 	  cur = 0;
@@ -1073,7 +1119,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	}
 	/* Have we gone from plain to reversed? */
 	if (!reverse && (reverse != newreverse)) {
-	  gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+	  gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
 	  fg = &WHITE;
 	  bg = &BLACK;
 	  cur = 0;
@@ -1081,11 +1127,11 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	reverse = newreverse;
       }
       /* Print remaining part of line */
-      gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+      gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
       cur = 0;
       /* Print some space at the end of the line */
       line[cur++] = ' '; line[cur++] = ' '; line[cur++] = ' ';
-      gtk_text_insert(bv, m_r_font, &BLACK, &WHITE, line, cur);
+      gtk_text_insert(bv, user_font_get_regular(), &BLACK, &WHITE, line, cur);
       cur = 0;
 
       /* Print the ASCII bit */
@@ -1113,7 +1159,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	newreverse = i >= bstart && i < bend;
 	/* Have we gone from reverse to plain? */
 	if (reverse && (reverse != newreverse)) {
-	  gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+	  gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
 	  fg = &BLACK;
 	  bg = &WHITE;
 	  cur = 0;
@@ -1126,7 +1172,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	}
 	/* Have we gone from plain to reversed? */
 	if (!reverse && (reverse != newreverse)) {
-	  gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+	  gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
 	  fg = &WHITE;
 	  bg = &BLACK;
 	  cur = 0;
@@ -1134,16 +1180,16 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	reverse = newreverse;
       }
       /* Print remaining part of line */
-      gtk_text_insert(bv, m_r_font, fg, bg, line, cur);
+      gtk_text_insert(bv, user_font_get_regular(), fg, bg, line, cur);
       cur = 0;
       line[cur++] = '\n';
       line[cur]   = '\0';
-      gtk_text_insert(bv, m_r_font, &BLACK, &WHITE, line, -1);
+      gtk_text_insert(bv, user_font_get_regular(), &BLACK, &WHITE, line, -1);
     }
     else {
-      gtk_text_insert(bv, m_r_font, NULL, NULL, line, -1);
+      gtk_text_insert(bv, user_font_get_regular(), NULL, NULL, line, -1);
       /* Do we start in bold? */
-      cur_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
+      cur_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
       j   = i;
       k   = i + BYTE_VIEW_WIDTH;
       cur = 0;
@@ -1160,7 +1206,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	/* insert a space every BYTE_VIEW_SEP bytes */
 	if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
 	/* Did we cross a bold/plain boundary? */
-	new_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
+	new_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
 	if (cur_font != new_font) {
 	  gtk_text_insert(bv, cur_font, NULL, NULL, line, cur);
 	  cur_font = new_font;
@@ -1173,7 +1219,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
       cur = 0;
       i = j;
       /* Print the ASCII bit */
-      cur_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
+      cur_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
       while (i < k) {
 	if (i < len) {
 	  if (encoding == CHAR_ASCII) {
@@ -1193,7 +1239,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 	/* insert a space every BYTE_VIEW_SEP bytes */
 	if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
 	/* Did we cross a bold/plain boundary? */
-	new_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
+	new_font = (i >= bstart && i < bend) ? user_font_get_bold() : user_font_get_regular();
 	if (cur_font != new_font) {
 	  gtk_text_insert(bv, cur_font, NULL, NULL, line, cur);
 	  cur_font = new_font;
@@ -1319,6 +1365,11 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
 #endif
   }
 
+  /* We're done printing the packets; destroy the progress bar if
+     it was created. */
+  if (progbar != NULL)
+    destroy_progress_dlg(progbar);
+
   /* scroll text into position */
 #if GTK_MAJOR_VERSION < 2
   gtk_text_thaw(bv); /* must thaw before adjusting scroll bars */
@@ -1327,7 +1378,7 @@ packet_hex_print_common(GtkTextView *bv, const guint8 *pd, int len, int bstart,
     float scrollval;
 
     linenum = bstart / BYTE_VIEW_WIDTH;
-    scrollval = MIN(linenum * m_font_height,
+    scrollval = MIN(linenum * user_font_get_regular_height(),
 		    bv->vadj->upper - bv->vadj->page_size);
 
     gtk_adjustment_set_value(bv->vadj, scrollval);
@@ -1483,13 +1534,8 @@ set_ptree_font_cb(gpointer data, gpointer user_data)
 }
 #endif
 
-#if GTK_MAJOR_VERSION < 2
 void
-set_ptree_font_all(GdkFont *font)
-#else
-void
-set_ptree_font_all(PangoFontDescription *font)
-#endif
+set_ptree_font_all(FONT_TYPE *font)
 {
 #if GTK_MAJOR_VERSION < 2
     GtkStyle *style;
@@ -1509,6 +1555,50 @@ set_ptree_font_all(PangoFontDescription *font)
 #endif
 }
 
+
+#if GTK_MAJOR_VERSION >= 2
+void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
+                                             GtkCellRenderer *cell,
+                                             GtkTreeModel *tree_model,
+                                             GtkTreeIter *iter,
+                                             gpointer data _U_)
+{
+    field_info   *fi;
+
+    gtk_tree_model_get(tree_model, iter, 1, &fi, -1);
+
+    /* for each field, we have to reset the renderer attributes */
+    g_object_set (cell, "foreground-set", FALSE, NULL);
+
+    g_object_set (cell, "underline", PANGO_UNDERLINE_NONE, NULL);
+    g_object_set (cell, "underline-set", FALSE, NULL);
+
+    /*g_object_set (cell, "style", PANGO_STYLE_NORMAL, NULL);
+    g_object_set (cell, "style-set", FALSE, NULL);*/
+
+    /*g_object_set (cell, "weight", PANGO_WEIGHT_NORMAL, NULL);
+    g_object_set (cell, "weight-set", FALSE, NULL);*/
+
+    if(FI_GET_FLAG(fi, FI_GENERATED)) {
+        /* as some fonts don't support italic, don't use this */
+        /*g_object_set (cell, "style", PANGO_STYLE_ITALIC, NULL);
+        g_object_set (cell, "style-set", TRUE, NULL);
+        */
+        /*g_object_set (cell, "weight", PANGO_WEIGHT_BOLD, NULL);
+        g_object_set (cell, "weight-set", TRUE, NULL);*/
+    }
+
+    if(fi->hfinfo->type == FT_FRAMENUM) {
+        g_object_set (cell, "foreground", "blue", NULL);
+        g_object_set (cell, "foreground-set", TRUE, NULL);
+
+        g_object_set (cell, "underline", PANGO_UNDERLINE_SINGLE, NULL);
+        g_object_set (cell, "underline-set", TRUE, NULL);
+    }
+}
+#endif
+
+
 GtkWidget *
 main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
 {
@@ -1541,11 +1631,18 @@ main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
   g_object_unref(G_OBJECT(store));
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), FALSE);
   renderer = gtk_cell_renderer_text_new();
+  g_object_set (renderer, "ypad", 0, NULL);
   col_offset = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view),
                                                            -1, "Name", renderer,
                                                            "text", 0, NULL);
   column = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view),
                                     col_offset - 1);
+  gtk_tree_view_column_set_cell_data_func(column,
+                                             renderer,
+                                             tree_cell_renderer,
+                                             NULL,
+                                             NULL);
+
   gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column),
                                   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   SIGNAL_CONNECT(tree_view, "row-expanded", expand_tree, NULL);
@@ -1554,9 +1651,15 @@ main_tree_view_new(e_prefs *prefs, GtkWidget **tree_view_p)
   gtk_container_add( GTK_CONTAINER(tv_scrollw), tree_view );
   set_ptree_sel_browse(tree_view, prefs->gui_ptree_sel_browse);
 #if GTK_MAJOR_VERSION < 2
+  if(item_style == NULL) {
+      item_style = gtk_style_new();
+      gdk_font_unref(item_style->font);
+      item_style->font = user_font_get_regular();
+  }
+
   gtk_widget_set_style(tree_view, item_style);
 #else
-  gtk_widget_modify_font(tree_view, m_r_font);
+  gtk_widget_modify_font(tree_view, user_font_get_regular());
 #endif
   remember_ptree_widget(tree_view);
 
@@ -1617,6 +1720,16 @@ main_proto_tree_draw(proto_tree *protocol_tree)
     proto_tree_draw(protocol_tree, tree_view);
 }
 
+
+void
+tree_view_follow_link(field_info   *fi)
+{
+    if(fi->hfinfo->type == FT_FRAMENUM) {
+        goto_frame(&cfile, fi->value.value.integer);
+    }
+}
+
+
 /* If the user selected a position in the tree view, try to find
  * the item in the GUI proto_tree that corresponds to that byte, and
  * select it. */
@@ -1628,6 +1741,7 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
         GtkCTreeNode *node;
         gint         row;
         gint         column;
+        field_info   *fi;
 
 
         if(gtk_clist_get_selection_info(GTK_CLIST(widget), 
@@ -1641,6 +1755,12 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
             g_assert(node);
 
             gtk_ctree_select(ctree, node);
+
+            /* if that's a doubleclick, try to follow the link */
+            if(event->type == GDK_2BUTTON_PRESS) {
+                fi = gtk_ctree_node_get_row_data(ctree, node);
+                tree_view_follow_link(fi);
+            }
         } else {
             return FALSE;
         }
@@ -1655,6 +1775,18 @@ tree_view_select(GtkWidget *widget, GdkEventButton *event)
         {
             sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
             gtk_tree_selection_select_path(sel, path);
+
+            /* if that's a doubleclick, try to follow the link */
+            if(event->type == GDK_2BUTTON_PRESS) {
+                GtkTreeModel *model;
+                GtkTreeIter iter;
+                field_info   *fi;
+
+                if(gtk_tree_selection_get_selected (sel, &model, &iter)) {
+                    gtk_tree_model_get(model, &iter, 1, &fi, -1);
+                    tree_view_follow_link(fi);
+                }
+            }
         } else {
             return FALSE;
         }
@@ -1718,7 +1850,7 @@ proto_tree_draw_node(proto_node *node, gpointer data)
     GtkTreePath  *path;
 #endif
 
-    if (!fi->visible)
+    if (PROTO_ITEM_IS_HIDDEN(node))
         return;
 
     /* was a free format label produced? */
@@ -1745,6 +1877,10 @@ proto_tree_draw_node(proto_node *node, gpointer data)
         is_expanded = FALSE;
     }
 
+    if(PROTO_ITEM_IS_GENERATED(node)) {
+        label_ptr = g_strdup_printf("[%s]", label_ptr);
+    }
+
 #if GTK_MAJOR_VERSION < 2
     info.ctree = parent_info->ctree;
     parent = gtk_ctree_insert_node ( info.ctree, parent_info->ctree_node, NULL,
@@ -1759,6 +1895,10 @@ proto_tree_draw_node(proto_node *node, gpointer data)
     gtk_tree_store_set(store, &iter, 0, label_ptr, 1, fi, -1);
 #endif
 
+    if(PROTO_ITEM_IS_GENERATED(node)) {
+        g_free(label_ptr);
+    }
+
     if (!is_leaf) {
 #if GTK_MAJOR_VERSION < 2
         info.ctree_node = parent;
