@@ -1,7 +1,7 @@
 /* rpc_progs.c
  * rpc_progs   2002 Ronnie Sahlberg
  *
- * $Id: rpc_progs.c,v 1.22 2004/03/13 12:09:27 ulfl Exp $
+ * $Id: rpc_progs.c,v 1.25 2004/05/23 23:24:06 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -22,7 +22,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-/* This module provides rpc call/reply RTT statistics to tethereal.
+/* This module provides rpc call/reply SRT statistics to tethereal.
  * It is only used by tethereal and not ethereal
  *
  * It serves as an example on how to use the tap api.
@@ -43,6 +43,7 @@
 #include "packet-rpc.h"
 #include "../globals.h"
 #include "ui_util.h"
+#include "dlg_utils.h"
 #include "compat_macros.h"
 
 static GtkWidget *win=NULL;
@@ -325,6 +326,8 @@ gtk_rpcprogs_init(char *optarg _U_)
 	GtkWidget *stat_label;
 	GtkWidget *tmp;
 	GString *error_string;
+    GtkWidget	*bt_close;
+    GtkWidget	*bbox;
 
 	if(win){
 		gdk_window_raise(win->window);
@@ -332,16 +335,13 @@ gtk_rpcprogs_init(char *optarg _U_)
 	}
 
 	win=window_new(GTK_WINDOW_TOPLEVEL, "ONC-RPC Program Statistics");
-	SIGNAL_CONNECT(win, "destroy", win_destroy_cb, win);
 
-	vbox=gtk_vbox_new(FALSE, 0);
+	vbox=gtk_vbox_new(FALSE, 3);
 	gtk_container_add(GTK_CONTAINER(win), vbox);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-	gtk_widget_show(vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 
 	stat_label=gtk_label_new("ONC-RPC Program Statistics");
 	gtk_box_pack_start(GTK_BOX(vbox), stat_label, FALSE, FALSE, 0);
-	gtk_widget_show(stat_label);
 
 
 	table=gtk_table_new(1, 5, TRUE);
@@ -350,35 +350,26 @@ gtk_rpcprogs_init(char *optarg _U_)
 	tmp=gtk_label_new("Program");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 0,1,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_LEFT);
-	gtk_widget_show(tmp);
 
 	tmp=gtk_label_new("Version");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 1,2,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_RIGHT);
-	gtk_widget_show(tmp);
 
 	tmp=gtk_label_new("Calls");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 2,3,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_RIGHT);
-	gtk_widget_show(tmp);
 
-	tmp=gtk_label_new("Min RTT");
+	tmp=gtk_label_new("Min SRT");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 3,4,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_RIGHT);
-	gtk_widget_show(tmp);
 
-	tmp=gtk_label_new("Max RTT");
+	tmp=gtk_label_new("Max SRT");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 4,5,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_RIGHT);
-	gtk_widget_show(tmp);
 
-	tmp=gtk_label_new("Avg RTT");
+	tmp=gtk_label_new("Avg SRT");
 	gtk_table_attach_defaults(GTK_TABLE(table), tmp, 5,6,0,1);
 	gtk_label_set_justify(GTK_LABEL(tmp), GTK_JUSTIFY_RIGHT);
-	gtk_widget_show(tmp);
-
-	
-	gtk_widget_show(table);
 
 	error_string=register_tap_listener("rpc", win, NULL, (void*)rpcprogs_reset, (void*)rpcprogs_packet, (void*)rpcprogs_draw);
 	if(error_string){
@@ -388,9 +379,20 @@ gtk_rpcprogs_init(char *optarg _U_)
 		exit(1);
 	}
 
+	/* Button row. */
+    bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
-	gtk_widget_show_all(win);
-	redissect_packets(&cfile);
+    bt_close = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(win, bt_close, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(win, "destroy", win_destroy_cb, win);
+
+    gtk_widget_show_all(win);
+    window_present(win);
+	
+    redissect_packets(&cfile);
 }
 
 static void
