@@ -3,7 +3,7 @@
  * Helper routines common to all service response time statistics
  * tap.
  *
- * $Id: service_response_time_table.c,v 1.16 2004/03/13 15:15:25 ulfl Exp $
+ * $Id: service_response_time_table.c,v 1.19 2004/06/02 07:12:11 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -45,8 +45,9 @@
 #include "gtk/color_dlg.h"
 #include "main.h"
 #include "ui_util.h"
+#include "gtkglobals.h"
+
 
-extern GtkWidget   *main_display_filter_widget;
 #define GTK_MENU_FUNC(a) ((GtkItemFactoryCallback)(a))
 
 
@@ -253,33 +254,33 @@ srt_show_popup_menu_cb(void *widg _U_, GdkEvent *event, srt_stat_table *rst)
 static GtkItemFactoryEntry srt_list_menu_items[] =
 {
 	/* Match */
-	ITEM_FACTORY_ENTRY("/Match Display Filter", NULL, NULL, 0, "<Branch>", NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter", NULL, NULL, 0, "<Branch>", NULL),
+	ITEM_FACTORY_ENTRY("/Apply as Filter/Selected", NULL,
 		srt_select_filter_cb, 0*256+0, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter/... not Selected", NULL,
 		srt_select_filter_cb, 0*256+1, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/And Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter/.. and Selected", NULL,
 		srt_select_filter_cb, 0*256+2, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/Or Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter/... or Selected", NULL,
 		srt_select_filter_cb, 0*256+3, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/And Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter/... and not Selected", NULL,
 		srt_select_filter_cb, 0*256+4, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Match Display Filter/Or Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Apply as Filter/... or not Selected", NULL,
 		srt_select_filter_cb, 0*256+5, NULL, NULL),
 
 	/* Prepare */
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter", NULL, NULL, 0, "<Branch>", NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter", NULL, NULL, 0, "<Branch>", NULL),
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/Selected", NULL,
 		srt_select_filter_cb, 1*256+0, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/Not Selected", NULL,
 		srt_select_filter_cb, 1*256+1, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/And Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/... and Selected", NULL,
 		srt_select_filter_cb, 1*256+2, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/Or Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/... or Selected", NULL,
 		srt_select_filter_cb, 1*256+3, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/And Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/... and not Selected", NULL,
 		srt_select_filter_cb, 1*256+4, NULL, NULL),
-	ITEM_FACTORY_ENTRY("/Prepare Display Filter/Or Not Selected", NULL,
+	ITEM_FACTORY_ENTRY("/Prepare a Filter/... or not Selected", NULL,
 		srt_select_filter_cb, 1*256+5, NULL, NULL),
 
 	/* Find Frame */
@@ -430,7 +431,25 @@ init_srt_table_row(srt_stat_table *rst, int index, char *procedure)
 {
 	char str[10];
 
-
+	/* we have discovered a new procedure. Extend the table accordingly */
+	if(index>=rst->num_procs){
+		int old_num_procs=rst->num_procs;
+		int i,j;
+		rst->num_procs=index+1;
+		rst->procedures=g_realloc(rst->procedures, sizeof(srt_procedure_t)*(rst->num_procs));
+		for(i=old_num_procs;i<rst->num_procs;i++){
+			rst->procedures[i].num=0;
+			rst->procedures[i].min.secs=0;
+			rst->procedures[i].min.nsecs=0;
+			rst->procedures[i].max.secs=0;
+			rst->procedures[i].max.nsecs=0;
+			rst->procedures[i].tot.secs=0;
+			rst->procedures[i].tot.nsecs=0;
+			for(j=0;j<6;j++){
+				rst->procedures[i].entries[j]=NULL;
+			}
+		}
+	}
 	g_snprintf(str, 10, "%d",index);
 	rst->procedures[index].entries[0]=g_strdup(str);
 
