@@ -1,7 +1,7 @@
 /* sip_stat.c
  * sip_stat   2004 Martin Mathieson
  *
- * $Id: sip_stat.c,v 1.1 2004/03/26 00:28:39 guy Exp $
+ * $Id: sip_stat.c,v 1.6 2004/05/23 23:24:06 ulfl Exp $
  * Copied from http_stat.c
  *
  * Ethereal - Network traffic analyzer
@@ -43,6 +43,8 @@
 #include "../packet-sip.h"
 #include "../globals.h"
 #include "compat_macros.h"
+#include "../tap_dfilter_dlg.h"
+#include "tap_dfilter_dlg.h"
 
 #define SUM_STR_MAX	1024
 
@@ -53,7 +55,9 @@ typedef struct _sip_stats_t {
     GHashTable  *hash_responses;
     GHashTable  *hash_requests;
     guint32	    packets;        /* number of sip packets, including continuations */
+    guint32     resent_packets;
     GtkWidget   *packets_label;
+    GtkWidget   *resent_label;
 
     GtkWidget   *request_box;		/* container for INVITE, ... */
 
@@ -86,9 +90,6 @@ typedef struct _sip_request_method_t {
     sipstat_t	*sp;                /* Pointer back to main struct */
 } sip_request_method_t;
 
-static GtkWidget *dlg=NULL;
-static GtkWidget *filter_entry;
-
 /* TODO: extra codes to be added from SIP extensions? */
 static const value_string vals_status_code[] = {
     { 100, "Trying"},
@@ -326,10 +327,12 @@ static void
 sipstat_reset(void *psp)
 {
     sipstat_t *sp = psp;
-    if (!sp)
+    if (sp)
     {
+    	sp->packets = 0;
+        sp->resent_packets = 0;
         g_hash_table_foreach(sp->hash_responses, (GHFunc)sip_reset_hash_responses, NULL);
-        g_hash_table_foreach(sp->hash_responses, (GHFunc)sip_reset_hash_requests, NULL);
+        g_hash_table_foreach(sp->hash_requests, (GHFunc)sip_reset_hash_requests, NULL);
     }
 }
 
@@ -343,6 +346,13 @@ sipstat_packet(void *psp, packet_info *pinfo _U_, epan_dissect_t *edt _U_, void
     /* Total number of packets, including continuation packets */
     sp->packets++;
 
+    /* Update resent count if flag set */
+    if (value->resend)
+    {
+        sp->resent_packets++;
+    }
+
+
     /* Looking at both requests and responses */
     if (value->response_code != 0)
     {
@@ -445,9 +455,16 @@ sipstat_draw(void *psp)
                 "SIP stats (%d packets)", sp->packets);
     gtk_label_set(GTK_LABEL(sp->packets_label), string_buff);
 
+    /* Set resend count label */
+    g_snprintf(string_buff, sizeof(string_buff),
+                "(%d resent packets)", sp->resent_packets);
+    gtk_label_set(GTK_LABEL(sp->resent_label), string_buff);
+
     /* Draw responses and requests from their tables */
     g_hash_table_foreach(sp->hash_responses, (GHFunc)sip_draw_hash_responses, NULL);
     g_hash_table_foreach(sp->hash_requests,  (GHFunc)sip_draw_hash_requests, NULL);
+    
+    gtk_widget_show_all(sp->win);
 }
 
 
@@ -478,6 +495,7 @@ win_destroy_cb(GtkWindow *win _U_, gpointer data)
     g_free(sp);
 }
 
+
 /* Create a new instance of gtk_sipstat. */
 static void
 gtk_sipstat_init(char *optarg)
@@ -490,9 +508,11 @@ gtk_sipstat_init(char *optarg)
                *informational_fr, *success_fr, *redirection_fr,
                *client_errors_fr, *server_errors_fr, *global_failures_fr,
                *request_fr;
+    GtkWidget	*bt_close;
+    GtkWidget	*bbox;
 
 
-    if (!strncmp (optarg, "sip,stat,", 9))
+    if (strncmp (optarg, "sip,stat,", 9) == 0)
     {
         /* Skip those characters from filter to display */
         filter=optarg + 9;
@@ -505,11 +525,12 @@ gtk_sipstat_init(char *optarg)
 
     /* Create sip stats window structure */
     sp = g_malloc(sizeof(sipstat_t));
+    sp->win = window_new(GTK_WINDOW_TOPLEVEL, "sip-stat");
 
     /* Set title to include any filter given */
     if (filter)
     {
-        sp->filter = strdup(filter);
+        sp->filter = g_strdup(filter);
         title = g_strdup_printf("SIP statistics with filter: %s", filter);
     }
     else
@@ -518,102 +539,81 @@ gtk_sipstat_init(char *optarg)
         title = g_strdup("SIP statistics");
     }
 
-    /* Create underlying window */
-    sp->win = window_new(GTK_WINDOW_TOPLEVEL, title);
+    gtk_window_set_title(GTK_WINDOW(sp->win), title);
     g_free(title);
 
-    /* Set destroy callback for underlying window */
-    SIGNAL_CONNECT(sp->win, "destroy", win_destroy_cb, sp);
-
 
     /* Create container for all widgets */
-    main_vb = gtk_vbox_new(FALSE, 10);
-    gtk_container_border_width(GTK_CONTAINER(main_vb), 10);
+    main_vb = gtk_vbox_new(FALSE, 12);
+    gtk_container_border_width(GTK_CONTAINER(main_vb), 12);
     gtk_container_add(GTK_CONTAINER(sp->win), main_vb);
-    gtk_widget_show(main_vb);
 
     /* Initialise & show number of packets */
     sp->packets = 0;
     sp->packets_label = gtk_label_new("SIP stats (0 SIP packets)");
     gtk_container_add(GTK_CONTAINER(main_vb), sp->packets_label);
-    gtk_widget_show(sp->packets_label);
+
+    sp->resent_packets = 0;
+    sp->resent_label = gtk_label_new("(0 resent packets)");
+    gtk_container_add(GTK_CONTAINER(main_vb), sp->resent_label);
+    gtk_widget_show(sp->resent_label);
 
 
     /* Informational response frame */
     informational_fr = gtk_frame_new("Informational  SIP 1xx");
     gtk_container_add(GTK_CONTAINER(main_vb), informational_fr);
-    gtk_widget_show(informational_fr);
 
     /* Information table (within that frame) */
     sp->informational_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(informational_fr), sp->informational_table);
-    gtk_widget_show(sp->informational_table);
-
 
     /* Success table and frame */
     success_fr = gtk_frame_new	("Success         SIP 2xx");
     gtk_container_add(GTK_CONTAINER(main_vb), success_fr);
-    gtk_widget_show(success_fr);
 
     sp->success_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(success_fr), sp->success_table);
-    gtk_widget_show(sp->success_table);
-
 
     /* Redirection table and frame */
     redirection_fr = gtk_frame_new	("Redirection     SIP 3xx");
     gtk_container_add(GTK_CONTAINER(main_vb), redirection_fr);
-    gtk_widget_show(redirection_fr);
 
     sp->redirection_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(redirection_fr), sp->redirection_table);
-    gtk_widget_show(sp->redirection_table);
-
 
     /* Client Errors table and frame */
     client_errors_fr = gtk_frame_new("Client errors  SIP 4xx");
     gtk_container_add(GTK_CONTAINER(main_vb), client_errors_fr);
-    gtk_widget_show(client_errors_fr);
 
     sp->client_error_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(client_errors_fr), sp->client_error_table);
-    gtk_widget_show(sp->client_error_table);
-
 
     /* Server Errors table and frame */
     server_errors_fr = gtk_frame_new("Server errors  SIP 5xx");
     gtk_container_add(GTK_CONTAINER(main_vb), server_errors_fr);
-    gtk_widget_show(server_errors_fr);
 
     sp->server_errors_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(server_errors_fr), sp->server_errors_table);
-    gtk_widget_show(sp->server_errors_table);
-
 
     /* Global Failures table and frame */
     global_failures_fr = gtk_frame_new("Global failures  SIP 6xx");
     gtk_container_add(GTK_CONTAINER(main_vb), global_failures_fr);
-    gtk_widget_show(global_failures_fr);
 
     sp->global_failures_table = gtk_table_new(0, 2, FALSE);
     gtk_container_add(GTK_CONTAINER(global_failures_fr), sp->global_failures_table);
-    gtk_widget_show(sp->global_failures_table);
 
 
     /* Separator between requests and responses */
     separator = gtk_hseparator_new();
     gtk_container_add(GTK_CONTAINER(main_vb), separator);
 
-
     /* Request table and frame */
     request_fr = gtk_frame_new("List of request methods");
     gtk_container_add(GTK_CONTAINER(main_vb), request_fr);
     gtk_container_border_width(GTK_CONTAINER(request_fr), 0);
-    gtk_widget_show(request_fr);
 
     sp->request_box = gtk_vbox_new(FALSE, 10);
     gtk_container_add(GTK_CONTAINER(request_fr), sp->request_box);
-    gtk_widget_show(sp->request_box);
 
 
     /* Register this tap listener now */
@@ -633,131 +633,37 @@ gtk_sipstat_init(char *optarg)
         return;
     }
 
-    /* If there was an existing dialog, destroy it */
-    if (dlg)
-    {
-        gtk_widget_destroy(dlg);
-    }
+	/* Button row. */
+    bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+    gtk_box_pack_start(GTK_BOX(main_vb), bbox, FALSE, FALSE, 0);
+
+    bt_close = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(sp->win, bt_close, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(sp->win, "delete_event", window_delete_event_cb, NULL);
+    SIGNAL_CONNECT(sp->win, "destroy", win_destroy_cb, sp);
 
     /* Display up-to-date contents */
     gtk_widget_show_all(sp->win);
+    window_present(sp->win);
+
     sip_init_hash(sp);
     retap_packets(&cfile);
 }
 
-
-/* Callback function for start button (filter dialog) */
-static void
-sipstat_start_button_clicked(GtkWidget *item _U_, gpointer data _U_)
-{
-    GString *str;
-    char *filter;
-
-    /* Will append "sip,stat," onto the front of every filter */
-    str = g_string_new("sip,stat,");
-    filter = (char *)gtk_entry_get_text(GTK_ENTRY(filter_entry));
-    str = g_string_append(str, filter);
-    gtk_sipstat_init(str->str);
-    g_string_free(str, TRUE);
-}
-
-/* Callback function for (filter) dialog being destroyed */
-static void
-dlg_destroy_cb(void)
-{
-    dlg = NULL;
-}
-
-/* Callback function for cancel button (filter dialog) */
-static void
-dlg_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
-{
-    gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
-/* Show the SIP stats dialog */
-static void
-gtk_sipstat_cb(GtkWidget *w _U_, gpointer d _U_)
-{
-    GtkWidget *dlg_box;
-    GtkWidget *filter_box, *filter_label;
-    GtkWidget *bbox, *start_button, *cancel_button;
-
-    /* If the window is already open, bring it to front */
-    if (dlg)
-    {
-        gdk_window_raise(dlg->window);
-        return;
-    }
-
-    /* Otherwise, create a new SIP stats dialog */
-    dlg = dlg_window_new("Ethereal: Compute SIP statistics");
-
-    /* Set callback for destroy */
-    SIGNAL_CONNECT(dlg, "destroy", dlg_destroy_cb, NULL);
-
-    /* Create and show the empty filter dialog */
-    dlg_box = gtk_vbox_new(FALSE, 10);
-    gtk_container_border_width(GTK_CONTAINER(dlg_box), 10);
-    gtk_container_add(GTK_CONTAINER(dlg), dlg_box);
-    gtk_widget_show(dlg_box);
-
-    /* Filter box */
-    filter_box = gtk_hbox_new(FALSE, 3);
-
-    /* Filter label */
-    filter_label = gtk_label_new("Filter:");
-    gtk_box_pack_start(GTK_BOX(filter_box), filter_label, FALSE, FALSE, 0);
-    gtk_widget_show(filter_label);
-
-    /* Filter entry */
-    filter_entry = gtk_entry_new();
-    WIDGET_SET_SIZE(filter_entry, 300, -1);
-    gtk_box_pack_start(GTK_BOX(filter_box), filter_entry, TRUE, TRUE, 0);
-    gtk_widget_show(filter_entry);
-
-    gtk_box_pack_start(GTK_BOX(dlg_box), filter_box, TRUE, TRUE, 0);
-    gtk_widget_show(filter_box);
-
-    /* 'Create stat' button */
-    bbox = dlg_button_row_new(ETHEREAL_STOCK_CREATE_STAT, GTK_STOCK_CANCEL, NULL);
-    gtk_box_pack_start(GTK_BOX(dlg_box), bbox, FALSE, FALSE, 0);
-    gtk_widget_show(bbox);
-
-    start_button = OBJECT_GET_DATA(bbox, ETHEREAL_STOCK_CREATE_STAT);
-    gtk_widget_grab_default(start_button);
-    SIGNAL_CONNECT_OBJECT(start_button, "clicked",
-                          sipstat_start_button_clicked, NULL);
-
-    /* Set callback for cancel button */
-    cancel_button = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-    SIGNAL_CONNECT(cancel_button, "clicked", dlg_cancel_cb, dlg);
-
-    /* Catch the "activate" signal on the filter text entry, so that
-       if the user types Return there, we act as if the "Create Stat"
-       button had been selected, as happens if Return is typed if
-       some widget that *doesn't* handle the Return key has the input
-       focus. */
-    dlg_set_activate(filter_entry, start_button);
-
-    /* Catch the "key_press_event" signal in the window, so that we can
-       catch the ESC key being pressed and act as if the "Cancel" button
-       had been selected. */
-    dlg_set_cancel(dlg, cancel_button);
-
-    /* Give the initial focus to the "Filter" entry box. */
-    gtk_widget_grab_focus(filter_entry);
-
-    /* Show everything now. */
-    gtk_widget_show_all(dlg);
-}
+static tap_dfilter_dlg sip_stat_dlg = {
+	"SIP Packet Counter",
+	"sip,stat",
+	gtk_sipstat_init,
+	-1
+};
 
 /* Register this tap listener and add menu item. */
 void
 register_tap_listener_gtksipstat(void)
 {
-    register_ethereal_tap("sip,stat,", gtk_sipstat_init);
+    register_ethereal_tap("sip,stat", gtk_sipstat_init);
 
     register_tap_menu_item("SIP", REGISTER_TAP_GROUP_NONE,
-                           gtk_sipstat_cb, NULL, NULL, NULL);
+                           gtk_tap_dfilter_dlg_cb, NULL, NULL, &(sip_stat_dlg));
 }
