@@ -1,7 +1,7 @@
 /* smb_stat.c
  * smb_stat   2003 Ronnie Sahlberg
  *
- * $Id: smb_stat.c,v 1.41 2004/03/13 15:15:25 ulfl Exp $
+ * $Id: smb_stat.c,v 1.45 2004/06/02 07:12:11 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -51,8 +51,8 @@
 #include "../globals.h"
 #include "filter_prefs.h"
 #include "service_response_time_table.h"
+#include "gtkglobals.h"
 
-extern GtkWidget   *main_display_filter_widget;
 
 /* used to keep track of the statistics for an entire program interface */
 typedef struct _smbstat_t {
@@ -159,6 +159,8 @@ gtk_smbstat_init(char *optarg)
 	GString *error_string;
 	int i;
 	GtkWidget *vbox;
+    GtkWidget *bbox;
+    GtkWidget *close_bt;
 
 	if(!strncmp(optarg,"smb,srt,",8)){
 		filter=optarg+8;
@@ -168,32 +170,26 @@ gtk_smbstat_init(char *optarg)
 
 	ss=g_malloc(sizeof(smbstat_t));
 
-	ss->win=window_new(GTK_WINDOW_TOPLEVEL, NULL);
+	ss->win=window_new(GTK_WINDOW_TOPLEVEL, "smb-stat");
 	gtk_window_set_default_size(GTK_WINDOW(ss->win), 550, 600);
 	smbstat_set_title(ss);
-	SIGNAL_CONNECT(ss->win, "destroy", win_destroy_cb, ss);
 
-	vbox=gtk_vbox_new(FALSE, 0);
+	vbox=gtk_vbox_new(FALSE, 3);
 	gtk_container_add(GTK_CONTAINER(ss->win), vbox);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
-	gtk_widget_show(vbox);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 
 	label=gtk_label_new("SMB Service Response Time statistics");
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 
 	g_snprintf(filter_string,255,"Filter:%s",filter?filter:"");
 	label=gtk_label_new(filter_string);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-
 
 	label=gtk_label_new("SMB Commands");
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 
 	/* We must display TOP LEVEL Widget before calling init_srt_table() */
-	gtk_widget_show(ss->win);
+	gtk_widget_show_all(ss->win);
 
 	init_srt_table(&ss->smb_srt_table, 256, vbox, "smb.cmd");
 	for(i=0;i<256;i++){
@@ -203,7 +199,6 @@ gtk_smbstat_init(char *optarg)
 
 	label=gtk_label_new("Transaction2 Sub-Commands");
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	init_srt_table(&ss->trans2_srt_table, 256, vbox, "smb.trans2.cmd");
 	for(i=0;i<256;i++){
 		init_srt_table_row(&ss->trans2_srt_table, i, val_to_str(i, trans2_cmd_vals, "Unknown(0x%02x)"));
@@ -212,7 +207,6 @@ gtk_smbstat_init(char *optarg)
 
 	label=gtk_label_new("NT Transaction Sub-Commands");
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
 	init_srt_table(&ss->nt_trans_srt_table, 256, vbox, "smb.nt.function");
 	for(i=0;i<256;i++){
 		init_srt_table_row(&ss->nt_trans_srt_table, i, val_to_str(i, nt_cmd_vals, "Unknown(0x%02x)"));
@@ -227,8 +221,20 @@ gtk_smbstat_init(char *optarg)
 		return;
 	}
 
-	gtk_widget_show_all(ss->win);
-	retap_packets(&cfile);
+	/* Button row. */
+	bbox = dlg_button_row_new(GTK_STOCK_CLOSE, NULL);
+	gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
+
+	close_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CLOSE);
+    window_set_cancel_button(ss->win, close_bt, window_cancel_button_cb);
+
+    SIGNAL_CONNECT(ss->win, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(ss->win, "destroy", win_destroy_cb, ss);
+
+    gtk_widget_show_all(ss->win);
+    window_present(ss->win);
+	
+    retap_packets(&cfile);
 }
 
 
@@ -242,12 +248,6 @@ dlg_destroy_cb(void)
 	dlg=NULL;
 }
 
-static void
-dlg_cancel_cb(GtkWidget *cancel_bt _U_, gpointer parent_w)
-{
-	gtk_widget_destroy(GTK_WIDGET(parent_w));
-}
-
 static void
 smbstat_start_button_clicked(GtkWidget *item _U_, gpointer data _U_)
 {
@@ -283,7 +283,7 @@ gtk_smbstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	}
 
 	dlg=dlg_window_new("Ethereal: Compute SMB SRT statistics");
-	SIGNAL_CONNECT(dlg, "destroy", dlg_destroy_cb, NULL);
+	gtk_window_set_default_size(GTK_WINDOW(dlg), 300, -1);
 
 	dlg_box=gtk_vbox_new(FALSE, 10);
 	gtk_container_border_width(GTK_CONTAINER(dlg_box), 10);
@@ -301,7 +301,6 @@ gtk_smbstat_cb(GtkWidget *w _U_, gpointer d _U_)
 
 	/* Filter entry */
 	filter_entry=gtk_entry_new();
-	WIDGET_SET_SIZE(filter_entry, 300, -1);
     SIGNAL_CONNECT(filter_entry, "changed", filter_te_syntax_check_cb, NULL);
 
 	/* filter prefs dialog */
@@ -324,12 +323,11 @@ gtk_smbstat_cb(GtkWidget *w _U_, gpointer d _U_)
     gtk_widget_show(bbox);
 
     start_button = OBJECT_GET_DATA(bbox, ETHEREAL_STOCK_CREATE_STAT);
-    gtk_widget_grab_default(start_button );
     SIGNAL_CONNECT_OBJECT(start_button, "clicked",
                               smbstat_start_button_clicked, NULL);
 
     cancel_button = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
-	SIGNAL_CONNECT(cancel_button, "clicked", dlg_cancel_cb, dlg);    
+    window_set_cancel_button(dlg, cancel_button, window_cancel_button_cb);
 
 	/* Catch the "activate" signal on the filter text entry, so that
 	   if the user types Return there, we act as if the "Create Stat"
@@ -338,15 +336,16 @@ gtk_smbstat_cb(GtkWidget *w _U_, gpointer d _U_)
 	   focus. */
 	dlg_set_activate(filter_entry, start_button);
 
-	/* Catch the "key_press_event" signal in the window, so that we can
-	   catch the ESC key being pressed and act as if the "Cancel" button
-	   had been selected. */
-	dlg_set_cancel(dlg, cancel_button);
+    gtk_widget_grab_default(start_button );
 
-	/* Give the initial focus to the "Filter" entry box. */
+    /* Give the initial focus to the "Filter" entry box. */
 	gtk_widget_grab_focus(filter_entry);
 
-	gtk_widget_show_all(dlg);
+    SIGNAL_CONNECT(dlg, "delete_event", window_delete_event_cb, NULL);
+	SIGNAL_CONNECT(dlg, "destroy", dlg_destroy_cb, NULL);
+
+    gtk_widget_show_all(dlg);
+    window_present(dlg);
 }
 
 void
