@@ -2,7 +2,7 @@
  * Routines for AIM Instant Messenger (OSCAR) dissection, SNAC Family Generic
  * Copyright 2004, Jelmer Vernooij <jelmer@samba.org>
  *
- * $Id: packet-aim-generic.c,v 1.1 2004/03/23 06:21:16 guy Exp $
+ * $Id: packet-aim-generic.c,v 1.3 2004/04/26 18:21:09 obiot Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -54,7 +54,6 @@
 #define FAMILY_GENERIC_RATEINFOREQ    0x0006
 #define FAMILY_GENERIC_RATEINFO       0x0007
 #define FAMILY_GENERIC_RATEINFOACK    0x0008
-#define FAMILY_GENERIC_UNKNOWNx09     0x0009
 #define FAMILY_GENERIC_RATECHANGE     0x000a
 #define FAMILY_GENERIC_SERVERPAUSE    0x000b
 #define FAMILY_GENERIC_CLIENTPAUSEACK 0x000c
@@ -84,7 +83,6 @@ static const value_string aim_fnac_family_generic[] = {
   { FAMILY_GENERIC_RATEINFOREQ, "Rate Info Request" },
   { FAMILY_GENERIC_RATEINFO, "Rate Info" },
   { FAMILY_GENERIC_RATEINFOACK, "Rate Info Ack" },
-  { FAMILY_GENERIC_UNKNOWNx09, "Unknown" },
   { FAMILY_GENERIC_RATECHANGE, "Rate Change" },
   { FAMILY_GENERIC_SERVERPAUSE, "Server Pause" },
   { FAMILY_GENERIC_CLIENTPAUSEACK, "Client Pause Ack" },
@@ -98,8 +96,8 @@ static const value_string aim_fnac_family_generic[] = {
   { FAMILY_GENERIC_SETPRIVFLAGS, "Set Privilege Flags" },
   { FAMILY_GENERIC_WELLKNOWNURL, "Well Known URL" },
   { FAMILY_GENERIC_NOP, "noop" },
-  { FAMILY_GENERIC_CAPABILITIES, "Capabilities (ICQ specific)" },
-  { FAMILY_GENERIC_CAPACK, "Capabilities Ack (ICQ specific)" },
+  { FAMILY_GENERIC_CAPABILITIES, "Capabilities" },
+  { FAMILY_GENERIC_CAPACK, "Capabilities Ack" },
   { FAMILY_GENERIC_SETSTATUS, "Set Status (ICQ specific)" },
   { FAMILY_GENERIC_CLIENTVERREQ, "Client Verification Requst" },
   { FAMILY_GENERIC_CLIENTVERREPL, "Client Verification Reply" },
@@ -171,6 +169,12 @@ static int hf_generic_migration_numfams  = -1;
 static int hf_generic_priv_flags = -1;
 static int hf_generic_allow_idle_see = -1;
 static int hf_generic_allow_member_see = -1;
+static int hf_generic_selfinfo_warninglevel = -1;
+static int hf_generic_evil_new_warn_level = -1;
+static int hf_generic_idle_time = -1;
+static int hf_generic_client_ver_req_offset = -1;
+static int hf_generic_client_ver_req_length = -1;
+static int hf_generic_client_ver_req_hash = -1;
 
 /* Initialize the subtree pointers */
 static gint ett_generic_clientready = -1;
@@ -205,12 +209,14 @@ static int dissect_generic_rateinfo(tvbuff_t *tvb, packet_info *pinfo _U_,
 	guint16 i;
 	proto_item *ti;
 	guint16 numclasses = tvb_get_ntohs(tvb, 0);	
-	proto_tree *classes_tree, *groups_tree, *group_tree;
+	proto_tree *classes_tree = NULL, *groups_tree, *group_tree;
     proto_tree_add_uint(tree, hf_generic_rateinfo_numclasses, tvb, 0, 2, numclasses );
 	offset+=2;
 
-	ti = proto_tree_add_text(tree, tvb, offset, 33*numclasses, "Available Rate Classes");
-	classes_tree = proto_item_add_subtree(ti, ett_generic_rateinfo_classes);
+	if(tree) {
+		ti = proto_tree_add_text(tree, tvb, offset, 33*numclasses, "Available Rate Classes");
+		classes_tree = proto_item_add_subtree(ti, ett_generic_rateinfo_classes);
+	}
 
 	for(i = 0; i < numclasses; i++) {
 		guint16 myid = tvb_get_ntohs(tvb, offset); 
@@ -252,35 +258,33 @@ static int dissect_generic_rateinfo(tvbuff_t *tvb, packet_info *pinfo _U_,
 static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo, 
 				    proto_tree *tree)
 {
-  int offset = 0;
-  const char *name;
-  struct aiminfo *aiminfo = pinfo->private_data;
-  guint16 n, i;
-  proto_item *ti = NULL;
-  proto_tree *gen_tree = NULL;
-  proto_tree *entry = NULL;
+	int offset = 0;
+	const char *name;
+	struct aiminfo *aiminfo = pinfo->private_data;
+	guint16 n, i;
+  	proto_item *ti = NULL;
+	proto_tree *gen_tree = NULL;
+	proto_tree *entry = NULL;
 
-  if(tree) {
-    ti = proto_tree_add_text(tree, tvb, 0, -1,"AIM Generic Service");
-    gen_tree = proto_item_add_subtree(ti, ett_generic);
-  }
+	if(tree) {
+		ti = proto_tree_add_text(tree, tvb, 0, -1,"AIM Generic Service");
+		gen_tree = proto_item_add_subtree(ti, ett_generic);
+	}
 	
-  if ((name = match_strval(aiminfo->subtype, aim_fnac_family_generic)) != NULL) {
-    if (ti)
-      proto_item_append_text(ti, ", %s", name);
+	if ((name = match_strval(aiminfo->subtype, aim_fnac_family_generic)) != NULL) {
+	  if (ti) proto_item_append_text(ti, ", %s", name);
 
-    if (check_col(pinfo->cinfo, COL_INFO))
-      col_add_fstr(pinfo->cinfo, COL_INFO, name);
-  }
+      if (check_col(pinfo->cinfo, COL_INFO)) col_add_fstr(pinfo->cinfo, COL_INFO, name);
+	}
 
   switch(aiminfo->subtype)
     {
 	case FAMILY_GENERIC_ERROR:
-	   return dissect_aim_snac_error(tvb, pinfo, 0, gen_tree);
+      return dissect_aim_snac_error(tvb, pinfo, 0, gen_tree);
 	case FAMILY_GENERIC_CLIENTREADY:
-	   ti = proto_tree_add_text(gen_tree, tvb, 0, -1, "Supported services");
+	   ti = proto_tree_add_text(gen_tree, tvb, 0, tvb_length_remaining(tvb, 0), "Supported services");
 	   entry = proto_item_add_subtree(ti, ett_generic_clientready);
-	   while(tvb_reported_length_remaining(tvb, offset) > 0) {
+	   while(tvb_length_remaining(tvb, offset) > 0) {
 			guint16 famnum = tvb_get_ntohs(tvb, offset);
 			const char *famname = aim_get_familyname(famnum);
 			proto_tree *subentry;
@@ -289,7 +293,7 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 			
 			subentry = proto_item_add_subtree(ti, ett_generic_clientready_item);
 
-			proto_tree_add_text(subentry, tvb, offset, 2, "Version: %u", tvb_get_ntohs(tvb, offset) ); offset += 2;
+			proto_tree_add_text(subentry, tvb, offset, 2, "Version: %d", tvb_get_ntohs(tvb, offset) ); offset += 2;
 			proto_tree_add_text(subentry, tvb, offset, 4, "DLL Version: %u", tvb_get_ntoh24(tvb, offset) ); offset += 4;
 	  }
 	  return offset;
@@ -305,12 +309,12 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 	  return offset;
 	case FAMILY_GENERIC_SERVICEREQ:
 	  name = aim_get_familyname( tvb_get_ntohs(tvb, offset) );
-	  proto_tree_add_uint_format(gen_tree, hf_generic_servicereq_service, tvb, offset, 2, tvb_get_ntohs(tvb, offset), "%s (0x%04x)", name?name:"Unknown", tvb_get_ntohs(tvb, offset) );
+      proto_tree_add_uint_format(gen_tree, hf_generic_servicereq_service, tvb, offset, 2, tvb_get_ntohs(tvb, offset), "%s (0x%04x)", name?name:"Unknown", tvb_get_ntohs(tvb, offset) );
 	  offset+=2;
 	  return offset;
 	case FAMILY_GENERIC_REDIRECT:
 	  while(tvb_length_remaining(tvb, offset) > 0) {
-		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree);
+		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree, client_tlvs);
 	  }
 	  return offset;
 	case FAMILY_GENERIC_CAPABILITIES:
@@ -336,11 +340,11 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 
 
     case FAMILY_GENERIC_MOTD: 
-	  proto_tree_add_item(gen_tree, hf_generic_motd_motdtype, tvb, offset, 
+      proto_tree_add_item(gen_tree, hf_generic_motd_motdtype, tvb, offset, 
 			  2, tvb_get_ntohs(tvb, offset));
 	  offset+=2;
 	  while(tvb_length_remaining(tvb, offset) > 0) {
-		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree);
+		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree, motd_tlvs);
 	  }
 	  return offset;
 
@@ -352,9 +356,6 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 		  offset+=2;
 	  }
 	  return offset;
-	case FAMILY_GENERIC_UNKNOWNx09:
-	  /* Unknown: FIXME: */
-	  return offset;
 	case FAMILY_GENERIC_RATECHANGE:
 	  proto_tree_add_uint(gen_tree, hf_generic_ratechange_msg, tvb, offset, 2, tvb_get_ntohs(tvb, offset));
 	  offset+=2;
@@ -363,7 +364,7 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 	  
 
 	case FAMILY_GENERIC_CLIENTPAUSEACK:
-	  while(tvb_length_remaining(tvb, offset) > 0) {
+	   while(tvb_length_remaining(tvb, offset) > 0) {
 			guint16 famnum = tvb_get_ntohs(tvb, offset);
 			const char *famname = aim_get_familyname(famnum);
 			proto_tree_add_text(gen_tree, tvb, offset, 4, "Family: %s (0x%x)", famname?famname:"Unknown Family", famnum);
@@ -390,7 +391,7 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 	  }
 
 	  while(tvb_length_remaining(tvb, offset) > 0) {
-		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree);
+		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree, client_tlvs);
 	  }
 
 	  return offset;
@@ -405,12 +406,33 @@ static int dissect_aim_snac_generic(tvbuff_t *tvb, packet_info *pinfo,
 	  }
 	  return offset;
 	case FAMILY_GENERIC_SELFINFO:
+	  offset = dissect_aim_buddyname(tvb, pinfo, offset, gen_tree);
+	  proto_tree_add_item(gen_tree, hf_generic_selfinfo_warninglevel, tvb, offset, 2, FALSE);
+	  offset += 2;
+	  return dissect_aim_tlv_list(tvb, pinfo, offset, gen_tree, onlinebuddy_tlvs);
 	case FAMILY_GENERIC_EVIL:
+	  proto_tree_add_item(gen_tree, hf_generic_evil_new_warn_level, tvb, offset, 2, FALSE);
+	  while(tvb_length_remaining(tvb, offset)) {
+		offset = dissect_aim_userinfo(tvb, pinfo, offset, gen_tree);
+	  }
+	  return offset;
 	case FAMILY_GENERIC_SETIDLE:
+	  proto_tree_add_item(gen_tree, hf_generic_idle_time, tvb, offset, 2, FALSE);
+	  return 4;
 	case FAMILY_GENERIC_SETSTATUS:
-	case FAMILY_GENERIC_WELLKNOWNURL:
+	  while(tvb_length_remaining(tvb, offset) > 0) {
+		offset = dissect_aim_tlv(tvb, pinfo, offset, gen_tree, onlinebuddy_tlvs);
+	  }
+	  return offset;
 	case FAMILY_GENERIC_CLIENTVERREQ:
+	  proto_tree_add_item(gen_tree, hf_generic_client_ver_req_offset, tvb, offset, 4, FALSE);
+	  offset+=4;
+	  proto_tree_add_item(gen_tree, hf_generic_client_ver_req_length, tvb, offset, 4, FALSE);
+	  return offset+4;
 	case FAMILY_GENERIC_CLIENTVERREPL:
+	  proto_tree_add_item(gen_tree, hf_generic_client_ver_req_hash, tvb, offset, 16, FALSE);
+	  return 16;
+	case FAMILY_GENERIC_WELLKNOWNURL:
 	  /* FIXME */
 	  return 0;
 	default: return 0;
@@ -487,6 +509,24 @@ proto_register_aim_generic(void)
 	{ &hf_generic_allow_member_see,
 	  { "Allow other users to see how long account has been a member", "aim.privilege_flags.allow_member", FT_BOOLEAN, 32, TFS(&flags_set_truth), 0x0002, "", HFILL },
 	},
+	{ &hf_generic_selfinfo_warninglevel,
+		{ "Warning level", "aim.selfinfo.warn_level", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
+	{ &hf_generic_evil_new_warn_level,
+		{ "New warning level", "aim.evil.new_warn_level", FT_UINT16, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
+	{ &hf_generic_idle_time,
+		{ "Idle time (seconds)", "aim.idle_time", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
+	{ &hf_generic_client_ver_req_offset, 
+		{ "Client Verification Request Offset", "aim.client_verification.offset", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
+	{ &hf_generic_client_ver_req_length, 
+		{ "Client Verification Request Length", "aim.client_verification.length", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
+	{ &hf_generic_client_ver_req_hash,
+		{ "Client Verification MD5 Hash", "aim.client_verification.hash", FT_BYTES, BASE_DEC, NULL, 0x0, "", HFILL },
+	},
   };
 
 /* Setup protocol subtree array */
