@@ -1,7 +1,7 @@
 /* packet-beep.c
  * Routines for BEEP packet disassembly
  *
- * $Id: packet-beep.c,v 1.12 2002/11/28 03:57:49 guy Exp $
+ * $Id: packet-beep.c,v 1.14 2003/12/23 21:18:57 guy Exp $
  *
  * Copyright (c) 2000 by Richard Sharpe <rsharpe@ns.aus.com>
  * Modified 2001 Darren New <dnew@invisible.net> for BEEP.
@@ -271,6 +271,7 @@ dissect_beep_more(tvbuff_t *tvb, int offset,
 
 }
 
+#if 0
 static void dissect_beep_status(tvbuff_t *tvb, int offset,
 				proto_tree *tree)
 {
@@ -304,6 +305,7 @@ static void dissect_beep_status(tvbuff_t *tvb, int offset,
   }
 
 }
+#endif
 
 static int num_len(tvbuff_t *tvb, int offset)
 {
@@ -387,9 +389,6 @@ static int header_len(tvbuff_t *tvb, int offset)
 
   while (1) {
 
-    if (tvb_ensure_length_remaining(tvb, offset + i) < 1)
-      return i;   /* Not enough characters left ... */
-
     if ((sc = tvb_get_guint8(tvb, offset + i)) == 0x0d
 	&& tvb_get_guint8(tvb, offset + i + 1) == 0x0a)
       return i;   /* Done here ... */
