@@ -2,7 +2,7 @@
  * Helpers for ASN.1/BER dissection
  * Ronnie Sahlberg (C) 2004
  *
- * $Id: packet-ber.c,v 1.6 2004/03/26 00:21:53 guy Exp $
+ * $Id: packet-ber.c,v 1.13 2004/06/25 09:24:17 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,6 +43,7 @@
 #include <epan/packet.h>
 
 #include <epan/strutil.h>
+#include <epan/int-64bit.h>
 #include "prefs.h"
 #include "packet-ber.h"
 
@@ -96,7 +97,7 @@ static const value_string ber_uni_tag_codes[] = {
 	{ BER_UNI_TAG_TeletextString	, "TeletextString, T61String" },
 	{ BER_UNI_TAG_VideotexString	, "VideotexString" },
 	{ BER_UNI_TAG_IA5String		, "IA5String" },
-	{ BER_UNI_TAG_UCTTime			, "UCTTime" },
+	{ BER_UNI_TAG_UTCTime		, "UTCTime" },
 	{ BER_UNI_TAG_GeneralizedTime	, "GeneralizedTime" },
 	{ BER_UNI_TAG_GraphicString	, "GraphicString" },
 	{ BER_UNI_TAG_VisibleString	, "VisibleString, ISO64String" },
@@ -315,6 +316,21 @@ dissect_ber_integer(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int off
 	offset=dissect_ber_length(pinfo, tree, tvb, offset, &len, NULL);
 
 /*	if(class!=BER_CLASS_UNI)*/
+
+	/* ok,  we cant handle >4 byte integers so lets fake them */
+	if(len>8){
+		proto_tree_add_text(tree, tvb, offset, len, "BER: Can not parse %d byte integers", len);
+		return 0xdeadbeef;
+	}
+	if(len>4){
+		header_field_info *hfinfo;
+		char buf[8]={0,0,0,0,0,0,0,0};
+
+		tvb_memcpy(tvb, buf+8-len, offset, len);
+		hfinfo = proto_registrar_get_nth(hf_id);
+		proto_tree_add_text(tree, tvb, offset, len, "%s: %s", hfinfo->name, u64toa(buf));
+		return 0xdeadbeef;
+	}
 	
 	val=0;
 	if (len > 0) {
@@ -330,6 +346,7 @@ dissect_ber_integer(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int off
 	ber_last_created_item=NULL;
 
 	if(hf_id!=-1){	
+		/* XXX - what if "len" is not 1, 2, 3, or 4? */
 		ber_last_created_item=proto_tree_add_item(tree, hf_id, tvb, offset-len, len, FALSE);
 	}
 	if(value){
@@ -487,12 +504,34 @@ dissect_ber_choice(packet_info *pinfo, proto_tree *parent_tree, tvbuff_t *tvb, i
 	proto_item *item=NULL;
 	int end_offset;
 	int hoffset = offset;
+	header_field_info	*hfinfo;
+
 
 	/* read header and len for choice field */
 	offset=get_ber_identifier(tvb, offset, &class, &pc, &tag);
 	offset=get_ber_length(tvb, offset, &len, NULL);
 	end_offset=offset+len;
 
+	/* Some sanity checks.  
+	 * The hf field passed to us MUST be an integer type 
+	 */
+	if(hf_id!=-1){
+		hfinfo=proto_registrar_get_nth(hf_id);
+		switch(hfinfo->type) {
+			case FT_UINT8:
+			case FT_UINT16:
+			case FT_UINT24:
+			case FT_UINT32:
+				break;
+		default:
+			proto_tree_add_text(tree, tvb, offset, len,"dissect_ber_choice(): Was passed a HF field that was not integer type : %s",hfinfo->abbrev);
+			fprintf(stderr,"dissect_ber_choice(): frame:%d offset:%d Was passed a HF field that was not integer type : %s\n",pinfo->fd->num,offset,hfinfo->abbrev);
+			return end_offset;
+		}
+	}
+
+	
+
 	/* loop over all entries until we find the right choice or 
 	   run out of entries */
 	ch = choice;
@@ -725,9 +764,10 @@ static int dissect_ber_sq_of(gboolean implicit_tag, guint32 type, packet_info *p
 	if (hf_id != -1) {
 		hfi = proto_registrar_get_nth(hf_id);
 		if(parent_tree){
-			if (hfi->type == FT_NONE)
+			if (hfi->type == FT_NONE) {
 				item = proto_tree_add_item(parent_tree, hf_id, tvb, offset, len, FALSE);
-			else {
+				proto_item_append_text(item, ":");
+			} else {
 				item = proto_tree_add_uint(parent_tree, hf_id, tvb, offset, len, cnt);
 				proto_item_append_text(item, (cnt==1)?" item":" items");
 			}
@@ -833,11 +873,13 @@ int dissect_ber_bitstring(gboolean implicit_tag, packet_info *pinfo, proto_tree
 	gboolean pc, ind;
 	guint32 tag;
 	guint32 len;
-	guint8 pad, b0, b1, val;
+	guint8 pad=0, b0, b1, val;
 	int end_offset;
 	proto_item *item = NULL;
 	proto_tree *tree = NULL;
 	asn_namedbit *nb;
+	char *sep;
+	gboolean term;
 
 	/* read header and len for the octet string */
 	offset = dissect_ber_identifier(pinfo, parent_tree, tvb, offset, &class, &pc, &tag);
@@ -878,13 +920,17 @@ int dissect_ber_bitstring(gboolean implicit_tag, packet_info *pinfo, proto_tree
 	}
 
 	if (named_bits) {
+		sep = " (";
+		term = FALSE;
 		nb = named_bits;
 		while (nb->p_id) {
 			if (nb->bit < (8*len-pad)) {
 				val = tvb_get_guint8(tvb, offset + nb->bit/8);
 				val &= 0x80 >> (nb->bit%8);
-				b0 = (nb->gb0 == -1) ? nb->bit/8 : nb->gb0/8;
-				b1 = (nb->gb1 == -1) ? nb->bit/8 : nb->gb1/8;
+				b0 = (nb->gb0 == -1) ? nb->bit/8 :
+						       ((guint32)nb->gb0)/8;
+				b1 = (nb->gb1 == -1) ? nb->bit/8 :
+						       ((guint32)nb->gb1)/8;
 				proto_tree_add_item(tree, *(nb->p_id), tvb, offset + b0, b1 - b0 + 1, FALSE);
 			} else {  /* 8.6.2.4 */
 				val = 0;
@@ -892,13 +938,17 @@ int dissect_ber_bitstring(gboolean implicit_tag, packet_info *pinfo, proto_tree
 			}
 			if (val) {
 				if (item && nb->tstr)
-					proto_item_append_text(item, " %s", nb->tstr);
+					proto_item_append_text(item, "%s%s", sep, nb->tstr);
 			} else {
 				if (item && nb->fstr)
-					proto_item_append_text(item, " %s", nb->fstr);
+					proto_item_append_text(item, "%s%s", sep, nb->fstr);
 			}
 			nb++;
+			sep = ", ";
+			term = TRUE;
 		}
+		if (term)
+			proto_item_append_text(item, ")");
 	}
 
 	return end_offset;
@@ -911,20 +961,41 @@ int dissect_ber_bitstring32(gboolean implicit_tag, packet_info *pinfo, proto_tre
 	guint32 val;
 	int **bf;
 	header_field_info *hfi;
+	char *sep;
+	gboolean term;
+	unsigned int i, tvb_len;
 
 	offset = dissect_ber_bitstring(implicit_tag, pinfo, parent_tree, tvb, offset, NULL, hf_id, ett_id, &tmp_tvb);
 	
 	tree = proto_item_get_subtree(ber_last_created_item);
 	if (bit_fields && tree) {
-		val = tvb_get_ntohl(tmp_tvb, 0);
+		/* tmp_tvb points to the actual bitstring (including any pad bits at the end.
+		 * note that this bitstring is not neccessarily always encoded as 4 bytes
+		 * so we have to read it byte by byte.
+		 */
+		val=0;
+		tvb_len=tvb_length(tmp_tvb);
+		for(i=0;i<4;i++){
+			val<<=8;
+			if(i<tvb_len){
+				val|=tvb_get_guint8(tmp_tvb,i);
+			}
+		}
 		bf = bit_fields;
+		sep = " (";
+		term = FALSE;
 		while (*bf) {
-			proto_tree_add_item(tree, **bf, tmp_tvb, 0, 4, FALSE);
+			proto_tree_add_boolean(tree, **bf, tmp_tvb, 0, tvb_len, val);
 			hfi = proto_registrar_get_nth(**bf);
-			if (val & hfi->bitmask)
-				proto_item_append_text(ber_last_created_item, " %s", hfi->name);
+			if (val & hfi->bitmask) {
+				proto_item_append_text(ber_last_created_item, "%s%s", sep, hfi->name);
+				sep = ", ";
+				term = TRUE;
+			}
 			bf++;
 		}
+		if (term)
+			proto_item_append_text(ber_last_created_item, ")");
 	}
 
 	if (out_tvb)
