@@ -2,7 +2,7 @@
  * Copyright 2001, Todd Sabin <tas@webspan.net>
  * Copyright 2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-dcerpc.h,v 1.40 2004/03/05 23:09:32 sahlberg Exp $
+ * $Id: packet-dcerpc.h,v 1.43 2004/06/09 09:24:07 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -226,18 +226,13 @@ value_string *value_string_from_subdissectors(dcerpc_sub_dissector *sd);
 
 /* Private data structure to pass to DCERPC dissector. This is used to
    pass transport specific information down to the dissector from the
-   dissector that parsed this encapsulated calls. */
-
-#define DCERPC_TRANSPORT_SMB  1
+   dissector that parsed this encapsulated calls. 
+   When it comes to DCERPC over SMB the only thing we really want to pass
+   on is the FID.
+*/
 
 typedef struct _dcerpc_private_info {
-    int transport_type;		/* Tag */
-
-    union {
-	struct {		/* DCERPC_TRANSPORT_SMB */
-	    guint16 fid;
-	} smb;
-    } data;
+	guint16 fid;
 } dcerpc_private_info;
 
 /* Private data passed to subdissectors from the main DCERPC dissector. */
@@ -256,7 +251,7 @@ typedef struct _dcerpc_info {
 	conversation_t *conv;	/* Which TCP stream we are in */
 	guint32 call_id;	/* Context id for this call */
 	guint16 smb_fid;	/* FID for DCERPC over SMB */
-	gboolean request;
+    guint8 ptype;       /* packet type: PDU_REQ, PDU_RESP, ... */
 	gboolean conformant_run;
 	gint32 conformant_eaten; /* how many bytes did the conformant run eat?*/
 	guint32 array_max_count;	/* max_count for conformant arrays */
@@ -271,8 +266,10 @@ typedef struct _dcerpc_info {
 } dcerpc_info;
 
 
-/* the registered subdissectors */
-extern GHashTable *dcerpc_uuids;
+/* the registered subdissectors. With MSVC and a 
+ * libethereal.dll, we need a special declaration.
+ */
+ETH_VAR_IMPORT GHashTable *dcerpc_uuids;
 
 typedef struct _dcerpc_uuid_key {
     e_uuid_t uuid;
