@@ -7,7 +7,7 @@
  * Changed to run on new version of TCAP, many changes for
  * EOC matching, and parameter separation.  (2003)
  *
- * $Id: packet-gsm_map.c,v 1.9 2004/03/19 07:54:57 guy Exp $
+ * $Id: packet-gsm_map.c,v 1.11 2004/04/21 05:53:56 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -313,6 +313,7 @@ static gint ett_param = -1;
 static gint ett_params = -1;
 static gint ett_problem = -1;
 static gint ett_opr_code = -1;
+static gint ett_err_code = -1;
 
 typedef struct dgt_set_t
 {
@@ -407,23 +408,6 @@ my_match_strval(guint32 val, const value_string *vs, gint *idx)
     return(NULL);
 }
 
-static gboolean
-check_map_tag(ASN1_SCK *asn1, guint tag)
-{
-    guint	saved_offset, real_tag;
-
-    if (tvb_length_remaining(asn1->tvb, asn1->offset) <= 0)
-    {
-	return(FALSE);
-    }
-
-    saved_offset = asn1->offset;
-    asn1_id_decode1(asn1, &real_tag);
-    asn1->offset = saved_offset;
-
-    return(tag == real_tag);
-}
-
 #define	GSM_MAP_START_SUBTREE(_Gtree, _Gsaved_offset, _Gtag, _Gstr1, _Gett, _Gdef_len_p, _Glen_p, _Gsubtree_p) \
     { \
 	guint		_len_offset; \
@@ -469,7 +453,7 @@ dissect_map_params(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
     orig_offset = asn1->offset;
 
     while ((tvb_length_remaining(asn1->tvb, asn1->offset) > 0) &&
-	(!check_map_tag(asn1, 0)))
+	(!tcap_check_tag(asn1, 0)))
     {
 	if ((exp_len != 0) &&
 	    ((asn1->offset - orig_offset) >= exp_len))
@@ -642,84 +626,6 @@ param_alertReason(ASN1_SCK *asn1, proto_tree *tree, guint len, int hf_field)
 	saved_offset, len, str);
 }
 
-static void
-param_AddressString(ASN1_SCK *asn1, proto_tree *tree, guint len, int hf_field)
-{
-    guint	saved_offset;
-    gint32	value;
-    guchar	*poctets;
-    gchar	*str = NULL;
-    char	bigbuf[1024];
-
-    saved_offset = asn1->offset;
-    asn1_int32_value_decode(asn1, 1, &value);
-
-    other_decode_bitfield_value(bigbuf, value, 0x80, 8);
-    proto_tree_add_text(tree, asn1->tvb,
-	saved_offset, 1,
-	"%s :  %sxtension",
-	bigbuf, (value & 0x80) ? "No E" : "E");
-
-    switch ((value & 0x70) >> 4)
-    {
-    case 0x00: str = "unknown"; break;
-    case 0x01: str = "International Number"; break;
-    case 0x02: str = "National Significant Number"; break;
-    case 0x03: str = "Network Specific Number"; break;
-    case 0x04: str = "Subscriber Number"; break;
-    case 0x05: str = "Reserved"; break;
-    case 0x06: str = "Abbreviated Number"; break;
-    case 0x07: str = "Reserved for extension"; break;
-    }
-
-    other_decode_bitfield_value(bigbuf, value, 0x70, 8);
-    proto_tree_add_text(tree, asn1->tvb,
-	saved_offset, asn1->offset - saved_offset,
-	"%s :  %s",
-	bigbuf, str);
-
-    switch (value & 0x0f)
-    {
-    case 0x00: str = "unknown"; break;
-    case 0x01: str = "ISDN/Telephony Numbering (Rec ITU-T E.164)"; break;
-    case 0x02: str = "spare"; break;
-    case 0x03: str = "Data Numbering (ITU-T Rec. X.121)"; break;
-    case 0x04: str = "Telex Numbering (ITU-T Rec. F.69)"; break;
-    case 0x05: str = "spare"; break;
-    case 0x06: str = "Land Mobile Numbering (ITU-T Rec. E.212)"; break;
-    case 0x07: str = "spare"; break;
-    case 0x08: str = "National Numbering"; break;
-    case 0x09: str = "Private Numbering"; break;
-    case 0x0f: str = "Reserved for extension"; break;
-    default:
-	str = "Reserved";
-	break;
-    }
-
-    other_decode_bitfield_value(bigbuf, value, 0x0f, 8);
-    proto_tree_add_text(tree, asn1->tvb,
-	saved_offset, asn1->offset - saved_offset,
-	"%s :  %s",
-	bigbuf, str);
-
-    saved_offset = asn1->offset;
-    asn1_string_value_decode(asn1, len - 1, &poctets);
-
-    my_dgt_tbcd_unpack(bigbuf, poctets, len - 1, &Dgt_msid);
-    g_free(poctets);
-
-    if (hf_field == -1)
-    {
-	proto_tree_add_text(tree, asn1->tvb,
-	    saved_offset, len - 1, "BCD Digits %s", bigbuf);
-    }
-    else
-    {
-	proto_tree_add_string_format(tree, hf_field, asn1->tvb,
-	    saved_offset, len - 1, bigbuf, "BCD Digits %s", bigbuf);
-    }
-}
-
 
 typedef enum
 {
@@ -913,7 +819,7 @@ param_TripletList(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
     orig_offset = asn1->offset;
 
     while ((tvb_length_remaining(asn1->tvb, asn1->offset) > 0) &&
-	(!check_map_tag(asn1, 0)))
+	(!tcap_check_tag(asn1, 0)))
     {
 	if ((exp_len != 0) &&
 	    ((asn1->offset - orig_offset) >= exp_len))
@@ -961,7 +867,7 @@ param_QuintupletList(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
     orig_offset = asn1->offset;
 
     while ((tvb_length_remaining(asn1->tvb, asn1->offset) > 0) &&
-	(!check_map_tag(asn1, 0)))
+	(!tcap_check_tag(asn1, 0)))
     {
 	if ((exp_len != 0) &&
 	    ((asn1->offset - orig_offset) >= exp_len))
@@ -1526,7 +1432,10 @@ op_send_rti_rr(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 
     start_offset = asn1->offset;
 
-    if (check_map_tag(asn1, 0x89))
+    /*
+     * spec says [9] but 'real data' show '04' not '89' !
+     */
+    if (tcap_check_tag(asn1, 0x04))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1534,6 +1443,14 @@ op_send_rti_rr(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_IMSI, "IMSI");
     }
 
+    if (tcap_check_tag(asn1, 0x04))
+    {
+	saved_offset = asn1->offset;
+	asn1_id_decode1(asn1, &tag);
+
+	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_ROAMING_NUM, "Roaming Number");
+    }
+
     dissect_map_params(asn1, subtree, len - (asn1->offset - start_offset));
 
     if (!def_len)
@@ -1581,7 +1498,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 
     GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_MSC_NUMBER, "MSC Number");
 
-    if (check_map_tag(asn1, 0x82))
+    if (tcap_check_tag(asn1, 0x82))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1589,7 +1506,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_MSISDN, "MSISDN");
     }
 
-    if (check_map_tag(asn1, 0x84))
+    if (tcap_check_tag(asn1, 0x84))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1597,7 +1514,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_LMSI, "LMSI");
     }
 
-    if (check_map_tag(asn1, 0x85))
+    if (tcap_check_tag(asn1, 0x85))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1605,7 +1522,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "GSM Bearer Capability");
     }
 
-    if (check_map_tag(asn1, 0xa6))
+    if (tcap_check_tag(asn1, 0xa6))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1613,7 +1530,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Network Signal Info");
     }
 
-    if (check_map_tag(asn1, 0x87))
+    if (tcap_check_tag(asn1, 0x87))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1621,7 +1538,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Suppression Of Announcement");
     }
 
-    if (check_map_tag(asn1, 0x88))
+    if (tcap_check_tag(asn1, 0x88))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1629,7 +1546,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_GMSC_ADDR, "GMSC Address");
     }
 
-    if (check_map_tag(asn1, 0x89))
+    if (tcap_check_tag(asn1, 0x89))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1637,7 +1554,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Call Reference Number");
     }
 
-    if (check_map_tag(asn1, 0x8a))
+    if (tcap_check_tag(asn1, 0x8a))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1645,7 +1562,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "OR Interrogation");
     }
 
-    if (check_map_tag(asn1, 0x8b))
+    if (tcap_check_tag(asn1, 0x8b))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1653,7 +1570,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Extension Container");
     }
 
-    if (check_map_tag(asn1, 0x8c))
+    if (tcap_check_tag(asn1, 0x8c))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1661,7 +1578,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Alerting Pattern");
     }
 
-    if (check_map_tag(asn1, 0x8d))
+    if (tcap_check_tag(asn1, 0x8d))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1669,7 +1586,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "CCBS Call");
     }
 
-    if (check_map_tag(asn1, 0x8f))
+    if (tcap_check_tag(asn1, 0x8f))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1677,7 +1594,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Supported Camel Phases In GMSC");
     }
 
-    if (check_map_tag(asn1, 0x8e))
+    if (tcap_check_tag(asn1, 0x8e))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1685,7 +1602,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Additional Signal Info");
     }
 
-    if (check_map_tag(asn1, 0x90))
+    if (tcap_check_tag(asn1, 0x90))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1693,7 +1610,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "OR Not Supported In GMSC");
     }
 
-    if (check_map_tag(asn1, 0x91))
+    if (tcap_check_tag(asn1, 0x91))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -1701,7 +1618,7 @@ op_provide_rn(ASN1_SCK *asn1, proto_tree *tree, guint exp_len)
 	GSM_MAP_PARAM_DISPLAY(subtree, saved_offset, tag, GSM_MAP_P_NONE, "Pre-paging Supported");
     }
 
-    if (check_map_tag(asn1, 0x92))
+    if (tcap_check_tag(asn1, 0x92))
     {
 	saved_offset = asn1->offset;
 	asn1_id_decode1(asn1, &tag);
@@ -2406,7 +2323,7 @@ dissect_map_invokeId(ASN1_SCK *asn1, proto_tree *tree)
     proto_tree	*subtree;
     gboolean	def_len;
 
-    if (check_map_tag(asn1, TCAP_INVOKE_ID_TAG))
+    if (tcap_check_tag(asn1, TCAP_INVOKE_ID_TAG))
     {
 	saved_offset = asn1->offset;
 	item =
@@ -2574,7 +2491,7 @@ dissect_map_lnkId(ASN1_SCK *asn1, proto_tree *tree)
     proto_tree	*subtree;
     gboolean	def_len;
 
-    if (check_map_tag(asn1, TCAP_LINKED_ID_TAG))
+    if (tcap_check_tag(asn1, TCAP_LINKED_ID_TAG))
     {
 	saved_offset = asn1->offset;
 
@@ -2607,21 +2524,8 @@ dissect_map_opr_code(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gint
     proto_item			*item;
     proto_tree			*subtree;
     gboolean			def_len;
-    static gsm_map_tap_rec_t	tap_rec[4];
-    static gsm_map_tap_rec_t	*tap_p;
-    static int			tap_current=0;
 
-    /*
-     * set tap record pointer
-     */
-    tap_current++;
-    if (tap_current == 4)
-    {
-	tap_current = 0;
-    }
-    tap_p = &tap_rec[tap_current];
-
-    if (check_map_tag(asn1, MAP_OPR_CODE_TAG))
+    if (tcap_check_tag(asn1, MAP_OPR_CODE_TAG))
     {
 	opr_offset = asn1->offset;
 
@@ -2657,10 +2561,6 @@ dissect_map_opr_code(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gint
 	}
 
 	*opr_code_p = val;
-
-	tap_p->opr_code_idx = *op_idx_p;
-
-	tap_queue_packet(gsm_map_tap, pinfo, tap_p);
     }
 
     return(MAP_OK);
@@ -2696,14 +2596,16 @@ dissect_map_eoc(ASN1_SCK *asn1, proto_tree *tree)
 static void
 dissect_map_invoke(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree)
 {
-    proto_tree	*subtree;
-    guint	orig_offset, saved_offset;
-    guint	len;
-    guint	tag;
-    proto_item	*item;
-    gint	op_idx;
-    gboolean	def_len;
-    int		ret;
+    proto_tree			*subtree;
+    guint			orig_offset, saved_offset;
+    guint			len;
+    guint			tag;
+    proto_item			*item;
+    gint			op_idx;
+    gboolean			def_len;
+    int				ret;
+    int				opr_code_sts;
+    static gsm_map_tap_rec_t	tap_rec;
 
     orig_offset = asn1->offset;
     saved_offset = asn1->offset;
@@ -2727,7 +2629,9 @@ dissect_map_invoke(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree)
 
     dissect_map_lnkId(asn1, subtree);
 
-    if (dissect_map_opr_code(asn1, pinfo, subtree, &op_idx, &g_opr_code) == MAP_OK)
+    opr_code_sts = dissect_map_opr_code(asn1, pinfo, subtree, &op_idx, &g_opr_code);
+
+    if (opr_code_sts == MAP_OK)
     {
 	if (def_len)
 	{
@@ -2757,19 +2661,30 @@ dissect_map_invoke(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree)
     }
 
     proto_item_set_len(item, asn1->offset - orig_offset);
+
+    if (opr_code_sts == MAP_OK)
+    {
+	tap_rec.invoke = TRUE;
+	tap_rec.opr_code_idx = op_idx;
+	tap_rec.size = asn1->offset - orig_offset;
+
+	tap_queue_packet(gsm_map_tap, pinfo, &tap_rec);
+    }
 }
 
 
 static void
 dissect_map_rr(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gchar *str)
 {
-    guint	tag, len, comp_len;
-    gint	op_idx;
-    guint	orig_offset, saved_offset;
-    proto_item	*item;
-    proto_tree	*seq_subtree, *subtree;
-    gboolean	def_len;
-    gboolean	comp_def_len;
+    guint			tag, len, comp_len;
+    gint			op_idx;
+    guint			orig_offset, saved_offset;
+    proto_item			*item;
+    proto_tree			*seq_subtree, *subtree;
+    gboolean			def_len;
+    gboolean			comp_def_len;
+    int				opr_code_sts;
+    static gsm_map_tap_rec_t	tap_rec;
 
     tag = -1;
     orig_offset = asn1->offset;
@@ -2804,6 +2719,8 @@ dissect_map_rr(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gchar *str)
     tag = -1;
     asn1_id_decode1(asn1, &tag);
 
+    opr_code_sts = MAP_FAIL;
+
     if (TCAP_CONSTRUCTOR(tag))
     {
 	GSM_MAP_START_SUBTREE(subtree, saved_offset, tag, "Sequence",
@@ -2812,7 +2729,9 @@ dissect_map_rr(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gchar *str)
 
 	saved_offset = asn1->offset;
 
-	if (dissect_map_opr_code(asn1, pinfo, seq_subtree, &op_idx, &g_opr_code) == MAP_OK)
+	opr_code_sts = dissect_map_opr_code(asn1, pinfo, seq_subtree, &op_idx, &g_opr_code);
+
+	if (opr_code_sts == MAP_OK)
 	{
 	    len -= asn1->offset - saved_offset;
 
@@ -2841,6 +2760,15 @@ dissect_map_rr(ASN1_SCK *asn1, packet_info *pinfo, proto_tree *tree, gchar *str)
     }
 
     proto_item_set_len(item, asn1->offset - orig_offset);
+
+    if (opr_code_sts == MAP_OK)
+    {
+	tap_rec.invoke = FALSE;
+	tap_rec.opr_code_idx = op_idx;
+	tap_rec.size = asn1->offset - orig_offset;
+
+	tap_queue_packet(gsm_map_tap, pinfo, &tap_rec);
+    }
 }
 
 
@@ -2849,9 +2777,11 @@ dissect_map_re(ASN1_SCK *asn1, proto_tree *tree)
 {
     guint	tag, len, comp_len;
     guint	orig_offset, saved_offset;
-    proto_item	*item, *null_item;
-    proto_tree	*subtree;
+    proto_item	*item;
+    proto_tree	*subtree, *temp_subtree;
     gboolean	comp_def_len, def_len;
+    gchar	*str;
+    gint32	int_val;
 
     tag = -1;
     orig_offset = asn1->offset;
@@ -2877,38 +2807,66 @@ dissect_map_re(ASN1_SCK *asn1, proto_tree *tree)
 
     dissect_map_invokeId(asn1, subtree);
 
-#define MAP_LOCAL_ERR_CODE_TAG 0x2
-#define MAP_GBL_ERR_CODE_TAG 0x6
-    if (check_map_tag(asn1, MAP_LOCAL_ERR_CODE_TAG))
-    {
-	tag = -1;
-	dissect_map_tag(asn1, subtree, &tag, "Local Error Code Tag", &null_item);
-    }
-    else if (check_map_tag(asn1, MAP_GBL_ERR_CODE_TAG))
-    {
-	tag = -1;
-	dissect_map_tag(asn1, subtree, &tag, "Global Error Code Tag", &null_item);
-    }
-    else
-    {
-	proto_tree_add_text(subtree, asn1->tvb, asn1->offset, comp_len,
-	    "Unknown Error Code");
+    saved_offset = asn1->offset;
+    asn1_id_decode1(asn1, &tag);
 
-	asn1->offset += comp_len;
+#define MAP_LOCAL_ERR_CODE_TAG	0x2
+#define MAP_GBL_ERR_CODE_TAG	0x6
 
-	if (!comp_def_len)
+    switch (tag)
+    {
+    case MAP_LOCAL_ERR_CODE_TAG:
+	GSM_MAP_START_SUBTREE(subtree, saved_offset, tag, "Local Error Code",
+	    ett_err_code,
+	    &def_len, &len, temp_subtree);
+
+	if (len > 0)
 	{
-	    dissect_map_eoc(asn1, subtree);
+	    saved_offset = asn1->offset;
+	    asn1_int32_value_decode(asn1, len, &int_val);
+
+	    str = match_strval(int_val, gsm_ss_err_code_strings);
+
+	    proto_tree_add_text(temp_subtree, asn1->tvb,
+		saved_offset, len, "Error Code: %s (%d)",
+		(str == NULL) ? "Unknown Error Code" : str,
+		int_val);
 	}
+	break;
 
-	proto_item_set_len(item, asn1->offset - orig_offset);
+    case MAP_GBL_ERR_CODE_TAG:
+	GSM_MAP_START_SUBTREE(subtree, saved_offset, tag, "Global Error Code",
+	    ett_err_code,
+	    &def_len, &len, temp_subtree);
 
-	return(MAP_OK);
+	if (len > 0)
+	{
+	    saved_offset = asn1->offset;
+	    asn1_int32_value_decode(asn1, len, &int_val);
+
+	    proto_tree_add_text(temp_subtree, asn1->tvb,
+		saved_offset, len, "Error Code: %d",
+		int_val);
+	}
+	break;
+
+    default:
+	GSM_MAP_START_SUBTREE(subtree, saved_offset, tag, "Unknown Error Code",
+	    ett_err_code,
+	    &def_len, &len, temp_subtree);
+
+	if (len > 0)
+	{
+	    saved_offset = asn1->offset;
+	    asn1_int32_value_decode(asn1, len, &int_val);
+
+	    proto_tree_add_text(temp_subtree, asn1->tvb,
+		saved_offset, len, "Error Code: %d",
+		int_val);
+	}
+	break;
     }
 
-    dissect_map_len(asn1, subtree, &def_len, &len);
-    dissect_map_integer(asn1, subtree, len, "Error Code:");
-
     dissect_map_params(asn1, subtree, comp_len - (asn1->offset - saved_offset));
 
     if (!def_len)
@@ -3128,7 +3086,7 @@ proto_register_map(void)
     };
 
     /* Setup protocol subtree array */
-#define	NUM_INDIVIDUAL_PARAMS	8
+#define	NUM_INDIVIDUAL_PARAMS	9
     static gint *ett[NUM_INDIVIDUAL_PARAMS+(GSM_MAP_NUM_OP*2)+NUM_PARAM_1];
 
     memset((void *) ett, 0, sizeof(ett));
@@ -3141,6 +3099,7 @@ proto_register_map(void)
     ett[5] = &ett_param;
     ett[6] = &ett_params;
     ett[7] = &ett_problem;
+    ett[8] = &ett_err_code;
 
     last_offset = NUM_INDIVIDUAL_PARAMS;
 
