@@ -3,10 +3,11 @@
  *
  * Guy Harris <guy@alum.mit.edu>
  *
+ * Copyright 2004, Jerry Talkington <jtalkington@users.sourceforge.net>
  * Copyright 2002, Tim Potter <tpot@samba.org>
  * Copyright 1999, Andrew Tridgell <tridge@samba.org>
  *
- * $Id: packet-http.c,v 1.95 2004/02/21 01:31:56 guy Exp $
+ * $Id: packet-http.c,v 1.110 2004/06/02 06:35:57 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -73,8 +74,12 @@ static int hf_http_transfer_encoding = -1;
 static gint ett_http = -1;
 static gint ett_http_ntlmssp = -1;
 static gint ett_http_request = -1;
+static gint ett_http_chunked_response = -1;
+static gint ett_http_chunk_data = -1;
+static gint ett_http_encoded_entity = -1;
 
 static dissector_handle_t data_handle;
+static dissector_handle_t media_handle;
 static dissector_handle_t http_handle;
 
 /*
@@ -90,11 +95,26 @@ static gboolean http_desegment_headers = FALSE;
  */
 static gboolean http_desegment_body = FALSE;
 
+/*
+ * De-chunking of content-encoding: chunk entity bodies.
+ */
+static gboolean http_dechunk_body = TRUE;
+
+/*
+ * Decompression of zlib encoded entities.
+ */
+#ifdef HAVE_LIBZ
+static gboolean http_decompress_body = TRUE;
+#else
+static gboolean http_decompress_body = FALSE;
+#endif
+
+
 #define TCP_PORT_HTTP			80
 #define TCP_PORT_PROXY_HTTP		3128
 #define TCP_PORT_PROXY_ADMIN_HTTP	3132
 #define TCP_ALT_PORT_HTTP		8080
-
+#define TCP_PORT_HKP			11371
 /*
  * SSDP is implemented atop HTTP (yes, it really *does* run over UDP).
  */
@@ -125,6 +145,8 @@ typedef struct {
 
 static int is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
 		RequestDissector *req_dissector, int *req_strlen);
+static int chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
+		proto_tree *tree, int offset);
 static void process_header(tvbuff_t *tvb, int offset, int next_offset,
     const guchar *line, int linelen, int colon_offset, packet_info *pinfo,
     proto_tree *tree, headers_t *eh_ptr);
@@ -151,7 +173,7 @@ base64_to_tvb(const char *base64)
 	char *data = g_strdup(base64);
 	size_t len;
 
-	len = base64_decode(data);
+	len = epan_base64_decode(data);
 	tvb = tvb_new_real_data((const guint8 *)data, len, len);
 
 	tvb_set_free_cb(tvb, g_free);
@@ -476,7 +498,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		/*
 		 * Not a blank line - either a request, a reply, or a header
 		 * line.
-		 */ 
+		 */
 		saw_req_resp_or_header = TRUE;
 		if (is_request_or_reply) {
 			if (tree) {
@@ -589,6 +611,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 */
 		tvbuff_t *next_tvb;
 		void *save_private_data = NULL;
+		gint chunks_decoded = 0;
 
 		/*
 		 * Create a tvbuff for the payload.
@@ -606,38 +629,129 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 */
 		next_tvb = tvb_new_subset(tvb, offset, datalen,
 		    reported_datalen);
+		/*
+		 * BEWARE - next_tvb is a subset of another tvb,
+		 * so we MUST NOT attempt tvb_free(next_tvb);
+		 */
 
 		/*
-		 * Handle content encodings other than "identity" (which
+		 * Handle *transfer* encodings other than "identity".
+		 */
+		if (headers.transfer_encoding != NULL &&
+		    strcasecmp(headers.transfer_encoding, "identity") != 0) {
+			if (http_dechunk_body &&
+			    (strcasecmp(headers.transfer_encoding, "chunked")
+			    == 0)) {
+
+				chunks_decoded = chunked_encoding_dissector(
+				    &next_tvb, pinfo, http_tree, 0);
+
+				if (chunks_decoded <= 0) {
+					/*
+					 * The chunks weren't reassembled,
+					 * or there was a single zero
+					 * length chunk.
+					 */
+					goto body_dissected;
+				} else {
+					/*
+					 * Add a new data source for the
+					 * de-chunked data.
+					 */
+					tvb_set_child_real_data_tvbuff(tvb,
+						next_tvb);
+					add_new_data_source(pinfo, next_tvb,
+						"De-chunked entity body");
+				}
+			} else {
+				/*
+				 * We currently can't handle, for example,
+				 * "gzip", "compress", or "deflate" as
+				 * *transfer* encodings; just handle them
+				 * as data for now.
+				 */
+				call_dissector(data_handle, next_tvb, pinfo,
+				    http_tree);
+				goto body_dissected;
+			}
+		}
+		/*
+		 * At this point, any chunked *transfer* coding has been removed
+		 * (the entity body has been dechunked) so it can be presented
+		 * for the following operation (*content* encoding), or it has
+		 * been been handed off to the data dissector.
+		 *
+		 * Handle *content* encodings other than "identity" (which
 		 * shouldn't appear in a Content-Encoding header, but
 		 * we handle it in any case).
 		 */
 		if (headers.content_encoding != NULL &&
 		    strcasecmp(headers.content_encoding, "identity") != 0) {
 			/*
-			 * We currently can't handle, for example, "gzip",
-			 * "compress", or "deflate"; just handle them as
-			 * data for now.
+			 * We currently can't handle, for example, "compress";
+			 * just handle them as data for now.
+			 *
+			 * After July 7, 2004 the LZW patent expires, so support
+			 * might be added then.  However, I don't think that
+			 * anybody ever really implemented "compress", due to
+			 * the aforementioned patent.
 			 */
-			call_dissector(data_handle, next_tvb, pinfo,
-			    http_tree);
-			goto body_dissected;
-		}
+			tvbuff_t *uncomp_tvb = NULL;
+			proto_item *e_ti = NULL;
+			proto_tree *e_tree = NULL;
+
+			if (http_decompress_body &&
+			    (strcasecmp(headers.content_encoding, "gzip") == 0 ||
+			    strcasecmp(headers.content_encoding, "deflate")
+			    == 0)) {
+
+				uncomp_tvb = tvb_uncompress(next_tvb, 0,
+				    tvb_length(next_tvb));
+			}
 
-		/*
-		 * Handle transfer encodings other than "identity".
-		 */
-		if (headers.transfer_encoding != NULL &&
-		    strcasecmp(headers.transfer_encoding, "identity") != 0) {
 			/*
-			 * We currently can't handle, for example, "chunked",
-			 * "gzip", "compress", or "deflate"; just handle them
-			 * as data for now.
+			 * Add the encoded entity to the protocol tree
 			 */
-			call_dissector(data_handle, next_tvb, pinfo,
-			    http_tree);
-			goto body_dissected;
+			e_ti = proto_tree_add_text(http_tree, next_tvb,
+					0, tvb_length(next_tvb),
+					"Content-encoded entity body (%s)",
+					headers.content_encoding);
+			e_tree = proto_item_add_subtree(e_ti,
+					ett_http_encoded_entity);
+
+			if (uncomp_tvb != NULL) {
+				/*
+				 * Decompression worked
+				 */
+
+				/* XXX - Don't free this, since it's possible
+				 * that the data was only partially
+				 * decompressed, such as when desegmentation
+				 * isn't enabled.
+				 *
+ 				tvb_free(next_tvb);
+				*/
+				next_tvb = uncomp_tvb;
+				tvb_set_child_real_data_tvbuff(tvb, next_tvb);
+				add_new_data_source(pinfo, next_tvb,
+				    "Uncompressed entity body");
+			} else {
+				if (chunks_decoded > 1) {
+					tvb_set_child_real_data_tvbuff(tvb,
+					    next_tvb);
+					add_new_data_source(pinfo, next_tvb,
+					    "Compressed entity body");
+				}
+				call_dissector(data_handle, next_tvb, pinfo,
+				    e_tree);
+
+				goto body_dissected;
+			}
 		}
+		/*
+		 * Note that a new data source is added for the entity body
+		 * only if it was content-encoded and/or transfer-encoded.
+		 */
 
 		/*
 		 * Do subdissector checks.
@@ -655,6 +769,11 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			 * for that content type?
 			 */
 			save_private_data = pinfo->private_data;
+			/*
+			 * XXX - this won't get freed if the subdissector
+			 * throws an exception.  Do we really need to
+			 * strdup it?
+			 */
 			if (headers.content_type_parameters)
 				pinfo->private_data = g_strdup(headers.content_type_parameters);
 			else
@@ -668,6 +787,12 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			handle = dissector_get_string_handle(
 			    media_type_subdissector_table,
 			    headers.content_type);
+			/*
+			 * Calling the default media handle otherwise
+			 */
+			if (handle == NULL) {
+			    handle = media_handle;
+			}
 		}
 		if (handle != NULL) {
 			/*
@@ -733,18 +858,208 @@ basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int req_strlen)
 }
 
 static void
-basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int req_strlen _U_)
+basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int resp_strlen)
 {
-	const guchar *data;
+	gchar *data;
 	int minor, major, status_code;
 
-	data = tvb_get_ptr(tvb, 5, 12);
+	/* BEWARE - sscanf() only operates on C strings.
+	 * The pointer returned by tvb_get_ptr points into the real data,
+	 * which is not necessarily NULL terminated. For this reason,
+	 * the sscanf() call is only applied to a buffer guaranteed to
+	 * only contain a NULL terminated string. */
+	data = g_strndup((const gchar *)tvb_get_ptr(tvb, 5, resp_strlen), resp_strlen);
 	if (sscanf((const gchar *)data, "%d.%d %d", &minor, &major, &status_code) == 3) {
 		proto_tree_add_uint(tree, hf_http_response_code, tvb, 9, 3, status_code);
 		stat_info->response_code = status_code;
 	}
+	g_free(data);
 }
 
+/*
+ * Dissect the http data chunks and add them to the tree.
+ */
+static int
+chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
+    proto_tree *tree, int offset)
+{
+	guint8 *chunk_string = NULL;
+	gint chunk_size = 0;
+	gint chunk_offset = 0;
+	gint datalen = 0;
+	gint linelen = 0;
+	gint chunks_decoded = 0;
+	tvbuff_t *tvb = NULL;
+	tvbuff_t *new_tvb = NULL;
+	gint chunked_data_size = 0;
+	proto_tree *subtree = NULL;
+	proto_item *ti = NULL;
+
+	if (tvb_ptr == NULL || *tvb_ptr == NULL) {
+		return 0;
+	}
+
+	tvb = *tvb_ptr;
+
+	datalen = tvb_reported_length_remaining(tvb, offset);
+
+	if (tree) {
+		ti = proto_tree_add_text(tree, tvb, offset, datalen,
+		    "HTTP chunked response");
+		subtree = proto_item_add_subtree(ti, ett_http_chunked_response);
+	}
+
+
+	while (datalen != 0) {
+		proto_item *chunk_ti = NULL;
+		proto_tree *chunk_subtree = NULL;
+		tvbuff_t *data_tvb = NULL;
+		gchar *c = NULL;
+
+		linelen = tvb_find_line_end(tvb, offset, -1, &chunk_offset, TRUE);
+
+		if (linelen <= 0) {
+			/* Can't get the chunk size line */
+			break;
+		}
+
+		chunk_string = tvb_get_string(tvb, offset, linelen);
+
+		if (chunk_string == NULL) {
+			/* Can't get the chunk size line */
+			break;
+		}
+
+		c = chunk_string;
+
+		/*
+		 * We don't care about the extensions.
+		 */
+		if ((c = strchr(c, ';'))) {
+			*c = '\0';
+		}
+
+		if (sscanf(chunk_string, "%x", &chunk_size) != 1) {
+			g_free(chunk_string);
+			break;
+		}
+
+		g_free(chunk_string);
+
+
+		if (chunk_size > datalen) {
+			/*
+			 * The chunk size is more than what's in the tvbuff,
+			 * so either the user hasn't enabled decoding, or all
+			 * of the segments weren't captured.
+			 */
+			chunk_size = datalen;
+		}/* else if (new_tvb == NULL) {
+			new_tvb = tvb_new_composite();
+		}
+
+
+
+		if (new_tvb != NULL && chunk_size != 0) {
+			tvbuff_t *chunk_tvb = NULL;
+
+			chunk_tvb = tvb_new_subset(tvb, chunk_offset,
+			    chunk_size, datalen);
+
+			tvb_composite_append(new_tvb, chunk_tvb);
+
+		}
+		*/
+
+		chunked_data_size += chunk_size;
+
+		if (chunk_size != 0) {
+			guint8 *raw_data = g_malloc(chunked_data_size);
+			gint raw_len = 0;
+
+			if (new_tvb != NULL) {
+				raw_len = tvb_length_remaining(new_tvb, 0);
+				tvb_memcpy(new_tvb, raw_data, 0, raw_len);
+
+				tvb_free(new_tvb);
+			}
+
+			tvb_memcpy(tvb, (guint8 *)(raw_data + raw_len),
+			    chunk_offset, chunk_size);
+
+			new_tvb = tvb_new_real_data(raw_data,
+			    chunked_data_size, chunked_data_size);
+			tvb_set_free_cb(new_tvb, g_free);
+
+		}
+
+		if (subtree) {
+			if (chunk_size == 0) {
+				chunk_ti = proto_tree_add_text(subtree, tvb,
+				    offset,
+				    chunk_offset - offset + chunk_size + 2,
+				    "Data chunk (last chunk)");
+			} else {
+				chunk_ti = proto_tree_add_text(subtree, tvb,
+				    offset,
+				    chunk_offset - offset + chunk_size + 2,
+				    "Data chunk (%u octets)", chunk_size);
+			}
+
+			chunk_subtree = proto_item_add_subtree(chunk_ti,
+			    ett_http_chunk_data);
+
+			proto_tree_add_text(chunk_subtree, tvb, offset,
+			    chunk_offset - offset, "Chunk size: %u octets",
+			    chunk_size);
+
+			data_tvb = tvb_new_subset(tvb, chunk_offset, chunk_size,
+			    datalen);
+
+
+			if (chunk_size > 0) {
+				call_dissector(data_handle, data_tvb, pinfo,
+				    chunk_subtree);
+			}
+
+			proto_tree_add_text(chunk_subtree, tvb, chunk_offset +
+			    chunk_size, 2, "Chunk boundary");
+		}
+
+		chunks_decoded++;
+		offset = chunk_offset + chunk_size + 2;
+		datalen = tvb_reported_length_remaining(tvb, offset);
+	}
+
+	if (new_tvb != NULL) {
+
+		/* Placeholder for the day that composite tvbuffer's will work.
+		tvb_composite_finalize(new_tvb);
+		/ * tvb_set_reported_length(new_tvb, chunked_data_size); * /
+		*/
+
+		/*
+		 * XXX - Don't free this, since the tvbuffer that was passed
+		 * may be used if the data spans multiple frames and reassembly
+		 * isn't enabled.
+		 *
+		tvb_free(*tvb_ptr);
+		 */
+		*tvb_ptr = new_tvb;
+
+	} else {
+		/*
+		 * We didn't create a new tvb, so don't allow sub dissectors
+		 * try to decode the non-existant entity body.
+		 */
+		chunks_decoded = -1;
+	}
+
+	return chunks_decoded;
+
+}
+
+
 /*
  * XXX - this won't handle HTTP 0.9 replies, but they're all data
  * anyway.
@@ -1164,7 +1479,7 @@ check_auth_basic(proto_item *hdr_item, tvbuff_t *tvb, gchar *value)
 				hdr_tree = NULL;
 			value += hdrlen;
 
-			len = base64_decode(value);
+			len = epan_base64_decode(value);
 			value[len] = '\0';
 			proto_tree_add_string(hdr_tree, hf_http_basic, tvb,
 			    0, 0, value);
@@ -1266,6 +1581,9 @@ proto_register_http(void)
 		&ett_http,
 		&ett_http_ntlmssp,
 		&ett_http_request,
+		&ett_http_chunked_response,
+		&ett_http_chunk_data,
+		&ett_http_encoded_entity,
 	};
 	module_t *http_module;
 
@@ -1280,12 +1598,24 @@ proto_register_http(void)
 	    "of a request spanning multiple TCP segments",
 	    &http_desegment_headers);
 	prefs_register_bool_preference(http_module, "desegment_body",
-	    "Trust the \"Content-length:\" header and desegment HTTP "
-	    "bodies\nspanning multiple TCP segments",
+	    "Desegment HTTP bodies spanning multiple TCP segments",
 	    "Whether the HTTP dissector should use the "
-	    "\"Content-length:\" value to desegment the body "
-	    "of a request spanning multiple TCP segments",
+	    "\"Content-length:\" value, if present, to desegment "
+	    "the body of a request spanning multiple TCP segments, "
+	    "and desegment chunked data spanning multiple TCP segments",
 	    &http_desegment_body);
+	prefs_register_bool_preference(http_module, "dechunk_body",
+	    "Reassemble chunked transfer-coded bodies",
+	    "Whether to reassemble bodies of entities that are transfered "
+	    "using the \"Transfer-Encoding: chunked\" method",
+	    &http_dechunk_body);
+#ifdef HAVE_LIBZ
+	prefs_register_bool_preference(http_module, "decompress_body",
+	    "Uncompress entity bodies",
+	    "Whether to uncompress entity bodies that are compressed "
+	    "using \"Content-Encoding: \"",
+	    &http_decompress_body);
+#endif
 
 	http_handle = create_dissector_handle(dissect_http, proto_http);
 
@@ -1347,11 +1677,13 @@ proto_reg_handoff_http(void)
 	dissector_handle_t http_udp_handle;
 
 	data_handle = find_dissector("data");
+	media_handle = find_dissector("media");
 
 	dissector_add("tcp.port", TCP_PORT_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_PORT_PROXY_HTTP, http_handle);
 	dissector_add("tcp.port", TCP_PORT_PROXY_ADMIN_HTTP, http_handle);
+	dissector_add("tcp.port", TCP_PORT_HKP, http_handle);
 
 	/*
 	 * XXX - is there anything to dissect in the body of an SSDP
