@@ -1,7 +1,7 @@
 /* packet-ipv6.h
  * Definitions for IPv6 packet disassembly
  *
- * $Id: packet-ipv6.h,v 1.32 2003/12/19 23:20:53 guy Exp $
+ * $Id: packet-ipv6.h,v 1.33 2004/04/22 08:22:07 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -208,17 +208,15 @@ struct icmp6_hdr {
 #define ICMP6_FQDN_REPLY		140	/* FQDN reply */
 #define ICMP6_NI_QUERY			139	/* node information request */
 #define ICMP6_NI_REPLY			140	/* node information reply */
-
-/* The definitions below are experimental. TBA */
-#define MLD6_MTRACE_RESP		141	/* mtrace response(to sender) */
-#define MLD6_MTRACE			142	/* mtrace messages */
+#define ICMP6_IND_SOLICIT		141	/* Inverse ND Solicitation */
+#define ICMP6_IND_ADVERT		142	/* Inverse ND advertisement */
+#define ICMP6_MLDV2_REPORT		143	/* MLD v2 report message : draft-vida-mld-v2-08.txt */
 
 #define ICMP6_MIP6_DHAAD_REQUEST	144	/* Mobile IPv6 DHAAD */
 #define ICMP6_MIP6_DHAAD_REPLY		145	/* Mobile IPv6 DHAAD */
 #define ICMP6_MIP6_MPS			146	/* Mobile IPv6 MPS */
 #define ICMP6_MIP6_MPA			147	/* Mobile IPv6 MPA */
 
-#define ICMP6_MLDV2_REPORT              206     /* MLD v2 report message : draft-vida-mld-v2-03.txt */
 #define ICMP6_MAXTYPE			153
 
 #define ICMP6_DST_UNREACH_NOROUTE	0	/* no route to destination */
