@@ -2,7 +2,7 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
- * $Id: packet-llc.c,v 1.108 2003/03/02 21:52:16 guy Exp $
+ * $Id: packet-llc.c,v 1.118 2004/01/18 08:32:45 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,6 +40,7 @@
 #include "packet-ip.h"
 #include "packet-ipx.h"
 #include "packet-netbios.h"
+#include "packet-vines.h"
 #include <epan/sna-utils.h>
 
 #include "packet-llc.h"
@@ -56,6 +57,18 @@ static int hf_llc_ssap = -1;
 static int hf_llc_dsap_ig = -1;
 static int hf_llc_ssap_cr = -1;
 static int hf_llc_ctrl = -1;
+static int hf_llc_n_r = -1;
+static int hf_llc_n_s = -1;
+static int hf_llc_p = -1;
+static int hf_llc_p_ext = -1;
+static int hf_llc_f = -1;
+static int hf_llc_f_ext = -1;
+static int hf_llc_s_ftype = -1;
+static int hf_llc_u_modifier_cmd = -1;
+static int hf_llc_u_modifier_resp = -1;
+static int hf_llc_ftype_i = -1;
+static int hf_llc_ftype_s_u = -1;
+static int hf_llc_ftype_s_u_ext = -1;
 static int hf_llc_type = -1;
 static int hf_llc_oui = -1;
 static int hf_llc_pid = -1;
@@ -64,7 +77,6 @@ static gint ett_llc = -1;
 static gint ett_llc_ctrl = -1;
 
 static dissector_table_t subdissector_table;
-static dissector_table_t cisco_subdissector_table;
 static dissector_table_t xid_subdissector_table;
 
 static dissector_handle_t bpdu_handle;
@@ -113,9 +125,14 @@ const value_string sap_vals[] = {
 	{ SAP_SNA1,           "SNA" },
 	{ SAP_SNA2,           "SNA" },
 	{ SAP_PROWAY_NM_INIT, "PROWAY (IEC955) Network Management and Initialization" },
+	{ SAP_NETWARE1,       "NetWare (unofficial?)" },
+	{ SAP_OSINL1,         "ISO Network Layer (OSLAN 1)" },
 	{ SAP_TI,             "Texas Instruments" },
+	{ SAP_OSINL2,         "ISO Network Layer (unofficial?)" },
+	{ SAP_OSINL3,         "ISO Network Layer (unofficial?)" },
 	{ SAP_BPDU,           "Spanning Tree BPDU" },
 	{ SAP_RS511,          "EIA RS-511 Manufacturing Message Service" },
+	{ SAP_OSINL4,         "ISO Network Layer (OSLAN 2)" },
 	{ SAP_X25,            "ISO 8208 (X.25 over 802.2)" },
 	/*
 	 * XXX - setting the group bit of SAP_X25 make 0x7F; is this just
@@ -130,13 +147,13 @@ const value_string sap_vals[] = {
 	{ SAP_SNAP,           "SNAP" },
 	{ SAP_VINES1,         "Banyan Vines" },
 	{ SAP_VINES2,         "Banyan Vines" },
-	{ SAP_NETWARE,        "NetWare" },
+	{ SAP_NETWARE2,       "NetWare" },
 	{ SAP_NETBIOS,        "NetBIOS" },
 	{ SAP_IBMNM,          "IBM Net Management" },
 	{ SAP_HPEXT,          "HP Extended LLC" },
 	{ SAP_UB,             "Ungermann-Bass" },
 	{ SAP_RPL,            "Remote Program Load" },
-	{ SAP_OSINL,          "ISO Network Layer" },
+	{ SAP_OSINL5,         "ISO Network Layer" },
 	{ SAP_GLOBAL,         "Global LSAP" },
 	{ 0x00,               NULL }
 };
@@ -160,9 +177,47 @@ http://www.cisco.com/univercd/cc/td/doc/product/software/ios113ed/113ed_cr/ibm_r
 	{ OUI_ATM_FORUM,   "ATM Forum" },
 	{ OUI_CABLE_BPDU,  "DOCSIS Spanning Tree" }, /* DOCSIS spanning tree BPDU */
 	{ OUI_APPLE_ATALK, "Apple (AppleTalk)" },
+	{ OUI_NORTEL,      "Nortel Networks SONMP" },
 	{ 0,               NULL }
 };
 
+/*
+ * Hash table for translating OUIs to a dissector table/field info pair;
+ * the dissector table maps PID values to dissectors, and the field
+ * corresponds to the PID for that OUI.
+ */
+typedef struct {
+	dissector_table_t table;
+	hf_register_info *field_info;
+} oui_info_t;
+
+static GHashTable *oui_info_table = NULL;
+
+/*
+ * Add an entry for a new OUI.
+ */
+void
+llc_add_oui(guint32 oui, const char *table_name, char *table_ui_name,
+    hf_register_info *hf_item)
+{
+	oui_info_t *new_info;
+
+	new_info = g_malloc(sizeof (oui_info_t));
+	new_info->table = register_dissector_table(table_name,
+	    table_ui_name, FT_UINT16, BASE_HEX);
+	new_info->field_info = hf_item;
+
+	/*
+	 * Create the hash table for OUI information, if it doesn't
+	 * already exist.
+	 */
+	if (oui_info_table == NULL) {
+		oui_info_table = g_hash_table_new(g_direct_hash,
+		    g_direct_equal);
+	}
+	g_hash_table_insert(oui_info_table, (gpointer)oui, new_info);
+}
+
 void
 capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 
@@ -233,7 +288,8 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 				    ld);
 				break;
 
-			case SAP_NETWARE:
+			case SAP_NETWARE1:
+			case SAP_NETWARE2:
 				capture_ipx(ld);
 				break;
 
@@ -241,6 +297,11 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 				capture_netbios(ld);
 				break;
 
+			case SAP_VINES1:
+			case SAP_VINES2:
+				capture_vines(ld);
+				break;
+
 			default:
 				ld->other++;
 				break;
@@ -249,6 +310,32 @@ capture_llc(const guchar *pd, int offset, int len, packet_counts *ld) {
 	}
 }
 
+/* Used only for U frames */
+static const xdlc_cf_items llc_cf_items = {
+	NULL,
+	NULL,
+	&hf_llc_p,
+	&hf_llc_f,
+	NULL,
+	&hf_llc_u_modifier_cmd,
+	&hf_llc_u_modifier_resp,
+	NULL,
+	&hf_llc_ftype_s_u
+};
+
+/* Used only for I and S frames */
+static const xdlc_cf_items llc_cf_items_ext = {
+	&hf_llc_n_r,
+	&hf_llc_n_s,
+	&hf_llc_p_ext,
+	&hf_llc_f_ext,
+	&hf_llc_s_ftype,
+	NULL,
+	NULL,
+	&hf_llc_ftype_i,
+	&hf_llc_ftype_s_u_ext
+};
+
 static void
 dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
@@ -298,7 +385,8 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	 */
 	control = dissect_xdlc_control(tvb, 2, pinfo, llc_tree,
 				hf_llc_ctrl, ett_llc_ctrl,
-				ssap & SSAP_CR_BIT, TRUE);
+				&llc_cf_items, &llc_cf_items_ext,
+				NULL, NULL, ssap & SSAP_CR_BIT, TRUE, FALSE);
 	llc_header_len += XDLC_CONTROL_LEN(control, TRUE);
 	if (is_snap)
 		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of protocol ID */
@@ -367,6 +455,9 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 	guint32		oui;
 	guint16		etype;
 	tvbuff_t	*next_tvb;
+	oui_info_t	*oui_info;
+	dissector_table_t subdissector_table;
+	int		hf;
 
 	oui =	tvb_get_ntoh24(tvb, offset);
 	etype = tvb_get_ntohs(tvb, offset+3);
@@ -394,7 +485,7 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 		   packet type for AARP packets. */
 		if (XDLC_IS_INFORMATION(control)) {
 			ethertype(etype, tvb, offset+5,
-			    pinfo, tree, snap_tree, hf_type, -1);
+			    pinfo, tree, snap_tree, hf_type, -1, 0);
 		} else {
 			next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
 			call_dissector(data_handle,next_tvb, pinfo, tree);
@@ -458,27 +549,6 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 		}
 		break;
 
-	case OUI_CISCO:
-		/* So are all CDP packets LLC packets
-		   with an OUI of OUI_CISCO and a
-		   protocol ID of 0x2000, or
-		   are some of them raw or encapsulated
-		   Ethernet? */
-		if (tree) {
-			proto_tree_add_uint(snap_tree, hf_pid, tvb, offset+3, 2,
-			    etype);
-		}
-		next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
-		if (XDLC_IS_INFORMATION(control)) {
-			/* do lookup with the subdissector table */
-			/* for future reference, 0x0102 is Cisco DRIP */
-			if (!dissector_try_port(cisco_subdissector_table,
-			    etype, next_tvb, pinfo, tree))
-				call_dissector(data_handle,next_tvb, pinfo, tree);
-		} else
-			call_dissector(data_handle,next_tvb, pinfo, tree);
-		break;
-
 	case OUI_CABLE_BPDU:    /* DOCSIS cable modem spanning tree BPDU */
 		if (tree) {
 			proto_tree_add_uint(snap_tree, hf_pid, tvb, offset+3, 2,
@@ -489,12 +559,38 @@ dissect_snap(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree,
 		break;
 
 	default:
+		/*
+		 * Do we have information for this OUI?
+		 */
+		oui_info = g_hash_table_lookup(oui_info_table, (gpointer)oui);
+		if (oui_info != NULL) {
+			/*
+			 * Yes - use it.
+			 */
+			hf = *oui_info->field_info->p_id;
+			subdissector_table = oui_info->table;
+		} else {
+			/*
+			 * No, use hf_pid for the PID and just dissect
+			 * the payload as data.
+			 */
+			hf = hf_pid;
+			subdissector_table = NULL;
+		}
 		if (tree) {
-			proto_tree_add_uint(snap_tree, hf_pid, tvb, offset+3, 2,
+			proto_tree_add_uint(snap_tree, hf, tvb, offset+3, 2,
 			    etype);
 		}
 		next_tvb = tvb_new_subset(tvb, offset+5, -1, -1);
-		call_dissector(data_handle,next_tvb, pinfo, tree);
+		if (XDLC_IS_INFORMATION(control)) {
+			if (subdissector_table != NULL) {
+				/* do lookup with the subdissector table */
+				if (dissector_try_port(subdissector_table,
+				    etype, next_tvb, pinfo, tree))
+					break;
+			}
+		}
+		call_dissector(data_handle, next_tvb, pinfo, tree);
 		break;
 	}
 }
@@ -526,6 +622,54 @@ proto_register_llc(void)
 		{ "Control", "llc.control", FT_UINT16, BASE_HEX,
 			NULL, 0x0, "", HFILL }},
 
+		{ &hf_llc_n_r,
+		{ "N(R)", "llc.control.n_r", FT_UINT16, BASE_DEC,
+			NULL, XDLC_N_R_EXT_MASK, "", HFILL }},
+
+		{ &hf_llc_n_s,
+		{ "N(S)", "llc.control.n_s", FT_UINT16, BASE_DEC,
+			NULL, XDLC_N_S_EXT_MASK, "", HFILL }},
+
+		{ &hf_llc_p,
+		{ "Poll", "llc.control.p", FT_BOOLEAN, 8,
+			TFS(&flags_set_truth), XDLC_P_F, "", HFILL }},
+
+		{ &hf_llc_p_ext,
+		{ "Poll", "llc.control.p", FT_BOOLEAN, 16,
+			TFS(&flags_set_truth), XDLC_P_F_EXT, "", HFILL }},
+
+		{ &hf_llc_f,
+		{ "Final", "llc.control.f", FT_BOOLEAN, 8,
+			TFS(&flags_set_truth), XDLC_P_F, "", HFILL }},
+
+		{ &hf_llc_f_ext,
+		{ "Final", "llc.control.f", FT_BOOLEAN, 16,
+			TFS(&flags_set_truth), XDLC_P_F_EXT, "", HFILL }},
+
+		{ &hf_llc_s_ftype,
+		{ "Supervisory frame type", "llc.control.s_ftype", FT_UINT16, BASE_HEX,
+			VALS(stype_vals), XDLC_S_FTYPE_MASK, "", HFILL }},
+
+		{ &hf_llc_u_modifier_cmd,
+		{ "Command", "llc.control.u_modifier_cmd", FT_UINT8, BASE_HEX,
+			VALS(modifier_vals_cmd), XDLC_U_MODIFIER_MASK, "", HFILL }},
+
+		{ &hf_llc_u_modifier_resp,
+		{ "Response", "llc.control.u_modifier_resp", FT_UINT8, BASE_HEX,
+			VALS(modifier_vals_resp), XDLC_U_MODIFIER_MASK, "", HFILL }},
+
+		{ &hf_llc_ftype_i,
+		{ "Frame type", "llc.control.ftype", FT_UINT16, BASE_HEX,
+			VALS(ftype_vals), XDLC_I_MASK, "", HFILL }},
+
+		{ &hf_llc_ftype_s_u,
+		{ "Frame type", "llc.control.ftype", FT_UINT8, BASE_HEX,
+			VALS(ftype_vals), XDLC_S_U_MASK, "", HFILL }},
+
+		{ &hf_llc_ftype_s_u_ext,
+		{ "Frame type", "llc.control.ftype", FT_UINT16, BASE_HEX,
+			VALS(ftype_vals), XDLC_S_U_MASK, "", HFILL }},
+
 		/* registered here but handled in ethertype.c */
 		{ &hf_llc_type,
 		{ "Type", "llc.type", FT_UINT16, BASE_HEX,
@@ -551,14 +695,20 @@ proto_register_llc(void)
 /* subdissector code */
 	subdissector_table = register_dissector_table("llc.dsap",
 	  "LLC SAP", FT_UINT8, BASE_HEX);
-	cisco_subdissector_table = register_dissector_table("llc.cisco_pid",
-	  "Cisco OUI PID", FT_UINT16, BASE_HEX);
 	xid_subdissector_table = register_dissector_table("llc.xid_dsap",
 	  "LLC XID SAP", FT_UINT8, BASE_HEX);
 
 	register_dissector("llc", dissect_llc, proto_llc);
 }
 
+static void
+register_hf(gpointer key _U_, gpointer value, gpointer user_data _U_)
+{
+	oui_info_t *info = value;
+
+	proto_register_field_array(proto_llc, info->field_info, 1);
+}
+
 void
 proto_reg_handoff_llc(void)
 {
@@ -592,4 +742,9 @@ proto_reg_handoff_llc(void)
 	 * apparently.
 	 */
 	dissector_add("arcnet.protocol_id", ARCNET_PROTO_BACNET, llc_handle);
+
+	/*
+	 * Register all the fields for PIDs for various OUIs.
+	 */
+	g_hash_table_foreach(oui_info_table, register_hf, NULL);
 }
