@@ -2,12 +2,12 @@
  * Routines for MTP2 Peer Adaptation Layer dissection
  * It is hopefully (needs testing) compliant to
  * http://www.ietf.org/internet-drafts/draft-ietf-sigtran-m2pa-02.txt
- * http://www.ietf.org/internet-drafts/draft-ietf-sigtran-m2pa-06.txt
+ * http://www.ietf.org/internet-drafts/draft-ietf-sigtran-m2pa-11.txt
  *
  * Copyright 2001, 2002, Jeff Morriss <jeff.morriss[AT]ulticom.com>,
- * updated by Michael Tuexen <michael.tuexen[AT]siemens.com>
+ * updated by Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-m2pa.c,v 1.16 2003/01/14 23:53:32 guy Exp $
+ * $Id: packet-m2pa.c,v 1.24 2004/05/24 02:25:18 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -53,18 +53,18 @@ static module_t *m2pa_module;
 static int hf_version      = -1;
 static int hf_spare        = -1;
 static int hf_v2_type      = -1;
-static int hf_v6_type      = -1;
+static int hf_v8_type      = -1;
 static int hf_class        = -1;
 static int hf_length       = -1;
 static int hf_unused       = -1;
 static int hf_bsn          = -1;
 static int hf_fsn          = -1;
 static int hf_v2_status    = -1;
-static int hf_v6_status    = -1;
+static int hf_v8_status    = -1;
 static int hf_v2_li_spare  = -1;
-static int hf_v6_li_spare  = -1;
+static int hf_v8_li_spare  = -1;
 static int hf_v2_li_prio   = -1;
-static int hf_v6_li_prio   = -1;
+static int hf_v8_li_prio   = -1;
 static int hf_filler       = -1;
 static int hf_unknown_data = -1;
 
@@ -76,16 +76,16 @@ static dissector_handle_t mtp3_handle;
 
 typedef enum {
   M2PA_V2 = 1,
-  M2PA_V6 = 2
+  M2PA_V11 = 2
 } Version_Type;
 
-static Version_Type m2pa_version = M2PA_V6;
+static Version_Type m2pa_version = M2PA_V11;
 
 #define VERSION_LENGTH         1
 #define SPARE_LENGTH           1
 #define CLASS_LENGTH           1
 #define V2_TYPE_LENGTH         2
-#define V6_TYPE_LENGTH         1
+#define V8_TYPE_LENGTH         1
 #define LENGTH_LENGTH          4
 #define UNUSED_LENGTH          1
 #define BSN_LENGTH             3
@@ -94,8 +94,8 @@ static Version_Type m2pa_version = M2PA_V6;
 #define V2_HEADER_LENGTH        (VERSION_LENGTH + SPARE_LENGTH + \
                                 V2_TYPE_LENGTH + LENGTH_LENGTH)
                                 
-#define V6_HEADER_LENGTH        (VERSION_LENGTH + SPARE_LENGTH + \
-                                CLASS_LENGTH + V6_TYPE_LENGTH + LENGTH_LENGTH + \
+#define V8_HEADER_LENGTH        (VERSION_LENGTH + SPARE_LENGTH + \
+                                CLASS_LENGTH + V8_TYPE_LENGTH + LENGTH_LENGTH + \
                                 UNUSED_LENGTH + BSN_LENGTH + UNUSED_LENGTH + \
                                 FSN_LENGTH)
 
@@ -104,10 +104,10 @@ static Version_Type m2pa_version = M2PA_V6;
 #define SPARE_OFFSET           (VERSION_OFFSET + VERSION_LENGTH)
 #define CLASS_OFFSET           (SPARE_OFFSET + SPARE_LENGTH)
 #define V2_TYPE_OFFSET         (SPARE_OFFSET + SPARE_LENGTH)
-#define V6_TYPE_OFFSET         (CLASS_OFFSET + CLASS_LENGTH)
-#define V6_LENGTH_OFFSET       (V6_TYPE_OFFSET + V6_TYPE_LENGTH)
+#define V8_TYPE_OFFSET         (CLASS_OFFSET + CLASS_LENGTH)
+#define V8_LENGTH_OFFSET       (V8_TYPE_OFFSET + V8_TYPE_LENGTH)
 #define V2_LENGTH_OFFSET       (V2_TYPE_OFFSET + V2_TYPE_LENGTH)
-#define FIRST_UNUSED_OFFSET    (V6_LENGTH_OFFSET + LENGTH_LENGTH)
+#define FIRST_UNUSED_OFFSET    (V8_LENGTH_OFFSET + LENGTH_LENGTH)
 #define BSN_OFFSET             (FIRST_UNUSED_OFFSET + UNUSED_LENGTH)
 #define SECOND_UNUSED_OFFSET   (BSN_OFFSET + BSN_LENGTH)
 #define FSN_OFFSET             (SECOND_UNUSED_OFFSET + UNUSED_LENGTH)
@@ -128,17 +128,24 @@ static const value_string v2_message_type_values[] = {
   { V2_LINK_STATUS_TYPE, "Link Status" },
   { 0,                   NULL } };
 
-#define V6_USER_DATA_TYPE   0x0001
-#define V6_LINK_STATUS_TYPE 0x0002
+#define V8_USER_DATA_TYPE   0x0001
+#define V8_LINK_STATUS_TYPE 0x0002
 
-static const value_string v6_message_type_values[] = {
-  { V6_USER_DATA_TYPE,   "User Data" },
-  { V6_LINK_STATUS_TYPE, "Link Status" },
+static const value_string v8_message_type_values[] = {
+  { V8_USER_DATA_TYPE,   "User Data" },
+  { V8_LINK_STATUS_TYPE, "Link Status" },
   { 0,                   NULL } };
 
 static void
-dissect_v2_header(tvbuff_t *header_tvb, proto_tree *m2pa_tree)
+dissect_v2_header(tvbuff_t *header_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
 {
+  guint16 message_type;
+  
+  message_type  = tvb_get_ntohs(header_tvb, V2_TYPE_OFFSET);
+
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "%s ", val_to_str(message_type, v2_message_type_values, "reserved"));
+
   if (m2pa_tree) {
     proto_tree_add_item(m2pa_tree, hf_version, header_tvb, VERSION_OFFSET,       VERSION_LENGTH, NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_spare,   header_tvb, SPARE_OFFSET,         SPARE_LENGTH,   NETWORK_BYTE_ORDER);
@@ -148,14 +155,21 @@ dissect_v2_header(tvbuff_t *header_tvb, proto_tree *m2pa_tree)
 }
 
 static void
-dissect_v6_header(tvbuff_t *header_tvb, proto_tree *m2pa_tree)
+dissect_v8_header(tvbuff_t *header_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
 {
+  guint8 message_type;
+  
+  message_type  = tvb_get_guint8(header_tvb, V8_TYPE_OFFSET);
+
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_add_fstr(pinfo->cinfo, COL_INFO, "%s ", val_to_str(message_type, v8_message_type_values, "Unknown"));
+
   if (m2pa_tree) {
     proto_tree_add_item(m2pa_tree, hf_version, header_tvb, VERSION_OFFSET,       VERSION_LENGTH, NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_spare,   header_tvb, SPARE_OFFSET,         SPARE_LENGTH,   NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_class,   header_tvb, CLASS_OFFSET,         CLASS_LENGTH,   NETWORK_BYTE_ORDER);
-    proto_tree_add_item(m2pa_tree, hf_v6_type, header_tvb, V6_TYPE_OFFSET,       V6_TYPE_LENGTH, NETWORK_BYTE_ORDER);
-    proto_tree_add_item(m2pa_tree, hf_length,  header_tvb, V6_LENGTH_OFFSET,     LENGTH_LENGTH,  NETWORK_BYTE_ORDER);
+    proto_tree_add_item(m2pa_tree, hf_v8_type, header_tvb, V8_TYPE_OFFSET,       V8_TYPE_LENGTH, NETWORK_BYTE_ORDER);
+    proto_tree_add_item(m2pa_tree, hf_length,  header_tvb, V8_LENGTH_OFFSET,     LENGTH_LENGTH,  NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_unused,  header_tvb, FIRST_UNUSED_OFFSET,  UNUSED_LENGTH,  NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_bsn,     header_tvb, BSN_OFFSET,           BSN_LENGTH,     NETWORK_BYTE_ORDER);
     proto_tree_add_item(m2pa_tree, hf_unused,  header_tvb, SECOND_UNUSED_OFFSET, UNUSED_LENGTH,  NETWORK_BYTE_ORDER);
@@ -192,18 +206,13 @@ dissect_v2_user_data_message(tvbuff_t *message_data_tvb, packet_info *pinfo, pro
 
   payload_tvb = tvb_new_subset(message_data_tvb, MTP3_OFFSET, -1, -1);
   call_dissector(mtp3_handle, payload_tvb, pinfo, tree);
-    
-  if ((!(proto_is_protocol_enabled (mtp3_proto_id))) && (check_col(pinfo->cinfo, COL_INFO)))
-    col_append_str(pinfo->cinfo, COL_INFO, "User Data ");
 }
 
-#define V6_LI_SPARE_MASK         0x3f
-#define V6_LI_PRIORITY_MASK      0xc0
+#define V8_LI_SPARE_MASK         0x3f
+#define V8_LI_PRIORITY_MASK      0xc0
 
 static void
-dissect_v6_user_data_message(tvbuff_t *message_data_tvb, packet_info *pinfo,
-                   proto_item *m2pa_item, proto_tree *m2pa_tree,
-                   proto_tree *tree)
+dissect_v8_user_data_message(tvbuff_t *message_data_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
 {
   proto_item *m2pa_li_item;
   proto_tree *m2pa_li_tree;
@@ -213,21 +222,15 @@ dissect_v6_user_data_message(tvbuff_t *message_data_tvb, packet_info *pinfo,
     if (m2pa_tree) {
       m2pa_li_item = proto_tree_add_text(m2pa_tree, message_data_tvb, LI_OFFSET, LI_LENGTH, "Length Indicator");
       m2pa_li_tree = proto_item_add_subtree(m2pa_li_item, ett_m2pa_li);
-      proto_tree_add_item(m2pa_li_tree, hf_v6_li_prio,  message_data_tvb, LI_OFFSET, LI_LENGTH, NETWORK_BYTE_ORDER);
-      proto_tree_add_item(m2pa_li_tree, hf_v6_li_spare, message_data_tvb, LI_OFFSET, LI_LENGTH, NETWORK_BYTE_ORDER);
+      proto_tree_add_item(m2pa_li_tree, hf_v8_li_prio,  message_data_tvb, LI_OFFSET, LI_LENGTH, NETWORK_BYTE_ORDER);
+      proto_tree_add_item(m2pa_li_tree, hf_v8_li_spare, message_data_tvb, LI_OFFSET, LI_LENGTH, NETWORK_BYTE_ORDER);
 
         /* Re-adjust length of M2PA item since it will be dissected as MTP3 */
-      proto_item_set_len(m2pa_item, V6_HEADER_LENGTH + LI_LENGTH);
+      proto_item_set_len(m2pa_item, V8_HEADER_LENGTH + LI_LENGTH);
     }
 
     payload_tvb = tvb_new_subset(message_data_tvb, MTP3_OFFSET, -1, -1);
     call_dissector(mtp3_handle, payload_tvb, pinfo, tree);
-
-    if ((!(proto_is_protocol_enabled (mtp3_proto_id))) && (check_col(pinfo->cinfo, COL_INFO)))
-      col_append_str(pinfo->cinfo, COL_INFO, "User Data ");
-    } else {
-      if (check_col(pinfo->cinfo, COL_INFO))
-        col_append_str(pinfo->cinfo, COL_INFO, "User Data ");
   }
 }
 
@@ -247,13 +250,12 @@ static void
 dissect_v2_link_status_message(tvbuff_t *message_data_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
 {
   if (check_col(pinfo->cinfo, COL_INFO))
-    col_append_str(pinfo->cinfo, COL_INFO, "Link status ");
-
- if (m2pa_tree)
+    col_append_fstr(pinfo->cinfo, COL_INFO, "(%s) ", val_to_str(tvb_get_ntohl(message_data_tvb, STATUS_OFFSET), v2_link_status_values, "Unknown"));
+  if (m2pa_tree)
     proto_tree_add_item(m2pa_tree, hf_v2_status, message_data_tvb, STATUS_OFFSET, STATUS_LENGTH, NETWORK_BYTE_ORDER);
 }
 
-static const value_string v6_link_status_values[] = {
+static const value_string v8_link_status_values[] = {
   { 1, "Alignment" },
   { 2, "Proving Normal" },
   { 3, "Proving Emergency" },
@@ -266,28 +268,25 @@ static const value_string v6_link_status_values[] = {
   { 0, NULL } };
 
 static void
-dissect_v6_link_status_message(tvbuff_t *message_data_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
+dissect_v8_link_status_message(tvbuff_t *message_data_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
 {
   guint16 filler_length;
+  
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, "(%s) ", val_to_str(tvb_get_ntohl(message_data_tvb, STATUS_OFFSET), v8_link_status_values, "Unknown"));
 
   filler_length = tvb_length(message_data_tvb) - STATUS_LENGTH;
   
-  if (check_col(pinfo->cinfo, COL_INFO))
-    col_append_str(pinfo->cinfo, COL_INFO, "Link status ");
-  
-  proto_tree_add_item(m2pa_tree, hf_v6_status, message_data_tvb, STATUS_OFFSET, STATUS_LENGTH, NETWORK_BYTE_ORDER);
+  proto_tree_add_item(m2pa_tree, hf_v8_status, message_data_tvb, STATUS_OFFSET, STATUS_LENGTH, NETWORK_BYTE_ORDER);
   if (filler_length > 0)
       proto_tree_add_item(m2pa_tree, hf_filler, message_data_tvb, FILLER_OFFSET, filler_length, NETWORK_BYTE_ORDER);
 }
 
 static void
-dissect_unknown_message(tvbuff_t *message_data_tvb, packet_info *pinfo, proto_tree *m2pa_tree)
+dissect_unknown_message(tvbuff_t *message_data_tvb, proto_tree *m2pa_tree)
 {
   guint length;
    
-  if (check_col(pinfo->cinfo, COL_INFO))
-    col_append_str(pinfo->cinfo, COL_INFO, "Unknown ");
-
   length = tvb_length(message_data_tvb);
   if ((m2pa_tree) && (length > 0))
     proto_tree_add_item(m2pa_tree, hf_unknown_data, message_data_tvb, 0, length, NETWORK_BYTE_ORDER);
@@ -302,7 +301,7 @@ dissect_v2_message_data(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m
   guint16 type;
   tvbuff_t *message_data_tvb;
 
-  message_data_length = tvb_get_ntohl(message_tvb,  V2_LENGTH_OFFSET);
+  message_data_length = tvb_get_ntohl(message_tvb, V2_LENGTH_OFFSET);
   message_data_tvb    = tvb_new_subset(message_tvb, V2_MESSAGE_DATA_OFFSET, message_data_length, message_data_length);
   type                = tvb_get_ntohs(message_tvb, V2_TYPE_OFFSET);
 
@@ -314,48 +313,48 @@ dissect_v2_message_data(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m
     dissect_v2_link_status_message(message_data_tvb, pinfo, m2pa_tree);
     break;
   default:
-    dissect_unknown_message(message_data_tvb, pinfo, m2pa_tree);
+    dissect_unknown_message(message_data_tvb, m2pa_tree);
   }
 }
 
-#define V6_MESSAGE_DATA_OFFSET (HEADER_OFFSET + V6_HEADER_LENGTH)
+#define V8_MESSAGE_DATA_OFFSET (HEADER_OFFSET + V8_HEADER_LENGTH)
 
 static void
-dissect_v6_message_data(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
+dissect_v8_message_data(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
 {
   guint32 message_data_length;
-  guint16 type;
+  guint8 type;
   tvbuff_t *message_data_tvb;
 
-  message_data_length = tvb_get_ntohl(message_tvb, V6_LENGTH_OFFSET) - V6_HEADER_LENGTH;
-  message_data_tvb    = tvb_new_subset(message_tvb, V6_MESSAGE_DATA_OFFSET, message_data_length, message_data_length);
-  type                = tvb_get_guint8(message_tvb, V6_TYPE_OFFSET);
+  message_data_length = tvb_get_ntohl(message_tvb, V8_LENGTH_OFFSET) - V8_HEADER_LENGTH;
+  message_data_tvb    = tvb_new_subset(message_tvb, V8_MESSAGE_DATA_OFFSET, message_data_length, message_data_length);
+  type                = tvb_get_guint8(message_tvb, V8_TYPE_OFFSET);
 
 
   switch(type) {
-  case V6_USER_DATA_TYPE:
-    dissect_v6_user_data_message(message_data_tvb, pinfo, m2pa_item, m2pa_tree, tree);
+  case V8_USER_DATA_TYPE:
+    dissect_v8_user_data_message(message_data_tvb, pinfo, m2pa_item, m2pa_tree, tree);
     break;
-  case V6_LINK_STATUS_TYPE:
-    dissect_v6_link_status_message(message_data_tvb, pinfo, m2pa_tree);
+  case V8_LINK_STATUS_TYPE:
+    dissect_v8_link_status_message(message_data_tvb, pinfo, m2pa_tree);
     break;
   default:
-    dissect_unknown_message(message_data_tvb, pinfo, m2pa_tree);
+    dissect_unknown_message(message_data_tvb, m2pa_tree);
   }
 }
 
 static void
 dissect_v2_message(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
 {
-  dissect_v2_header(message_tvb, m2pa_tree);
+  dissect_v2_header(message_tvb, pinfo, m2pa_tree);
   dissect_v2_message_data(message_tvb, pinfo, m2pa_item, m2pa_tree, tree);
 }
 
 static void
-dissect_v6_message(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
+dissect_v8_message(tvbuff_t *message_tvb, packet_info *pinfo, proto_item *m2pa_item, proto_tree *m2pa_tree, proto_tree *tree)
 {
-  dissect_v6_header(message_tvb, m2pa_tree);
-  dissect_v6_message_data(message_tvb, pinfo, m2pa_item, m2pa_tree, tree);
+  dissect_v8_header(message_tvb, pinfo, m2pa_tree);
+  dissect_v8_message_data(message_tvb, pinfo, m2pa_item, m2pa_tree, tree);
 }
 
 static void
@@ -367,10 +366,10 @@ dissect_m2pa(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   if (check_col(pinfo->cinfo, COL_PROTOCOL))
     switch(m2pa_version) {
     case M2PA_V2:
-      col_set_str(pinfo->cinfo, COL_PROTOCOL, "M2PA (02)");
+      col_set_str(pinfo->cinfo, COL_PROTOCOL, "M2PA (ID 02)");
       break;
-    case M2PA_V6:
-      col_set_str(pinfo->cinfo, COL_PROTOCOL, "M2PA (06)");
+    case M2PA_V11:
+      col_set_str(pinfo->cinfo, COL_PROTOCOL, "M2PA (ID 11)");
       break;
     };      
 
@@ -386,8 +385,8 @@ dissect_m2pa(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     case M2PA_V2:
       dissect_v2_message(tvb, pinfo, m2pa_item, m2pa_tree, tree);
       break;
-    case M2PA_V6:
-      dissect_v6_message(tvb, pinfo, m2pa_item, m2pa_tree, tree);
+    case M2PA_V11:
+      dissect_v8_message(tvb, pinfo, m2pa_item, m2pa_tree, tree);
       break;
   };      
 }
@@ -399,18 +398,18 @@ proto_register_m2pa(void)
   { { &hf_version,      { "Version",        "m2pa.version",      FT_UINT8,  BASE_DEC,  VALS(protocol_version_values), 0x0,                 "", HFILL} },
     { &hf_spare,        { "Spare",          "m2pa.spare",        FT_UINT8,  BASE_HEX,  NULL,                          0x0,                 "", HFILL} },
     { &hf_v2_type,      { "Message Type",   "m2pa.type",         FT_UINT16, BASE_HEX,  VALS(v2_message_type_values),  0x0,                 "", HFILL} },
-    { &hf_v6_type,      { "Message Type",   "m2pa.type",         FT_UINT8,  BASE_DEC,  VALS(v6_message_type_values),  0x0,                 "", HFILL} },
+    { &hf_v8_type,      { "Message Type",   "m2pa.type",         FT_UINT8,  BASE_DEC,  VALS(v8_message_type_values),  0x0,                 "", HFILL} },
     { &hf_class,        { "Message Class",  "m2pa.class",        FT_UINT8,  BASE_DEC,  VALS(message_class_values),    0x0,                 "", HFILL} },
     { &hf_length,       { "Message length", "m2pa.length",       FT_UINT32, BASE_DEC,  NULL,                          0x0,                 "", HFILL} },
     { &hf_unused,       { "Unused",         "m2pa.unused",       FT_UINT8,  BASE_DEC,  NULL,                          0x0,                 "", HFILL} },
     { &hf_bsn,          { "BSN",            "m2pa.bsn",          FT_UINT24, BASE_DEC,  NULL,                          0x0,                 "", HFILL} },
     { &hf_fsn,          { "FSN",            "m2pa.fsn",          FT_UINT24, BASE_DEC,  NULL,                          0x0,                 "", HFILL} },
     { &hf_v2_li_spare,  { "Spare",          "m2pa.li_spare",     FT_UINT8,  BASE_DEC,  NULL,                          V2_LI_SPARE_MASK,    "", HFILL} },
-    { &hf_v6_li_spare,  { "Spare",          "m2pa.li_spare",     FT_UINT8,  BASE_HEX,  NULL,                          V6_LI_SPARE_MASK,    "", HFILL} },
+    { &hf_v8_li_spare,  { "Spare",          "m2pa.li_spare",     FT_UINT8,  BASE_HEX,  NULL,                          V8_LI_SPARE_MASK,    "", HFILL} },
     { &hf_v2_li_prio,   { "Priority",       "m2pa.li_priority",  FT_UINT8,  BASE_DEC,  NULL,                          V2_LI_PRIORITY_MASK, "", HFILL} },
-    { &hf_v6_li_prio,   { "Priority",       "m2pa.li_priority",  FT_UINT8,  BASE_HEX,  NULL,                          V6_LI_PRIORITY_MASK, "", HFILL} },
+    { &hf_v8_li_prio,   { "Priority",       "m2pa.li_priority",  FT_UINT8,  BASE_HEX,  NULL,                          V8_LI_PRIORITY_MASK, "", HFILL} },
     { &hf_v2_status,    { "Link Status",    "m2pa.status",       FT_UINT32, BASE_DEC,  VALS(v2_link_status_values),   0x0,                 "", HFILL} },
-    { &hf_v6_status,    { "Link Status",    "m2pa.status",       FT_UINT32, BASE_DEC,  VALS(v6_link_status_values),   0x0,                 "", HFILL} },
+    { &hf_v8_status,    { "Link Status",    "m2pa.status",       FT_UINT32, BASE_DEC,  VALS(v8_link_status_values),   0x0,                 "", HFILL} },
     { &hf_filler,       { "Filler",         "m2pa.filler",       FT_BYTES,  BASE_NONE, NULL,                          0x0,                 "", HFILL} },
     { &hf_unknown_data, { "Unknown Data",   "m2pa.unknown_data", FT_BYTES,  BASE_NONE, NULL,                          0x0,                 "", HFILL} }
   };
@@ -421,9 +420,9 @@ proto_register_m2pa(void)
   };
 
   static enum_val_t m2pa_version_options[] = {
-    { "Internet Draft version 2", M2PA_V2 },
-    { "Internet Draft version 6", M2PA_V6 },
-    { NULL, 0 }
+    { "draft-2", "Internet Draft version 2", M2PA_V2 },
+    { "draft-11", "Internet Draft version 11", M2PA_V11 },
+    { NULL, NULL, 0 }
   };
 
   proto_m2pa = proto_register_protocol("MTP2 Peer Adaptation Layer", "M2PA", "m2pa");
