@@ -7,7 +7,7 @@ proper helper routines
  * Routines for dissection of ASN.1 Aligned PER
  * 2003  Ronnie Sahlberg
  *
- * $Id: packet-per.c,v 1.26 2004/03/25 23:58:06 guy Exp $
+ * $Id: packet-per.c,v 1.29 2004/06/24 21:50:04 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -130,7 +130,7 @@ dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, packet_info *pinfo
 		}
 		return offset;
 	}
-	NOT_DECODED_YET("10.9.3.8.1");
+	PER_NOT_DECODED_YET("10.9.3.8.1");
 	return offset;
 }
 
@@ -303,7 +303,6 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 	}
 
 
-
 	/* xx.x if length is fixed or constrained to be less than or equal to
 	   two bytes, then it will not be byte aligned. */
 	byte_aligned=TRUE;
@@ -313,6 +312,10 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 	if(max_len<2){
 		byte_aligned=FALSE;
 	}
+	if(!length){  
+		/* there is no string at all, so dont do any byte alignment */
+		byte_aligned=FALSE;
+	}
 
 	if(byte_aligned){
 		if(offset&0x07){
@@ -322,7 +325,7 @@ DEBUG_ENTRY("dissect_per_restricted_character_string");
 
 
 	if(length>=1024){
-		NOT_DECODED_YET("restricted char string too long");
+		PER_NOT_DECODED_YET("restricted char string too long");
 		length=1024;
 	}
 
@@ -428,7 +431,7 @@ dissect_per_BMPString(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_t
 	}
 
 	if(length>=1024){
-		NOT_DECODED_YET("BMPString too long");
+		PER_NOT_DECODED_YET("BMPString too long");
 		length=1024;
 	}
 
@@ -548,7 +551,7 @@ DEBUG_ENTRY("dissect_per_object_identifier");
 		offset+=8;
 
 		if((strp-str)>200){
-NOT_DECODED_YET("too long octet_string");
+PER_NOT_DECODED_YET("too long octet_string");
 			/*XXX assert here */
 			return offset;
 		}
@@ -647,7 +650,7 @@ dissect_per_integer(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tre
 	offset=dissect_per_length_determinant(tvb, offset, pinfo, tree, -1, &length);
 	/* gassert here? */
 	if(length>4){
-NOT_DECODED_YET("too long integer");
+PER_NOT_DECODED_YET("too long integer");
 		length=4;
 	}
 
@@ -704,6 +707,8 @@ dissect_per_constrained_integer(tvbuff_t *tvb, guint32 offset, packet_info *pinf
 {
 	proto_item *it=NULL;
 	guint32 range, val;
+	gint val_start, val_length;
+	nstime_t timeval;
 	header_field_info *hfi;
 	int num_bits;
 	int pad;
@@ -735,20 +740,12 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 	num_bits=0;
 	pad=0;
 	val=0;
+	timeval.secs=val; timeval.nsecs=0;
 	/* 10.5.4 */
 	if(range==1){
-		it=proto_tree_add_uint_format(tree, hf_index, tvb, offset>>3, 0, min, "%s: %d", hfi->name, min);
-		if(item){
-			*item=it;
-		}
-		if(value){
-			*value=val;
-		}
-		return offset;
-	}
-
-	/* 10.5.7 */
-	if(range<=255){
+		val_start = offset>>3; val_length = 0;
+		val = min; 
+	} else if(range<=255) {
 		/* 10.5.7.1 */
 		char str[256];
 		int i, bit, length;
@@ -804,19 +801,8 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 			}
 			strcat(str,".");
 		}
+		val_start = (offset-num_bits)>>3; val_length = length;
 		val+=min;
-		if(hfi->strings){
-			it=proto_tree_add_uint_format(tree, hf_index, tvb, (offset-num_bits)>>3, length, val, "%s : %s (%d)", str, val_to_str(val, hfi->strings, "Unknown(%d)"),val);
-		} else {
-			it=proto_tree_add_uint(tree, hf_index, tvb, (offset-num_bits)>>3, length, val);
-		}
-		if(item){
-			*item=it;
-		}
-		if(value){
-			*value=val;
-		}
-		return offset;
 	} else if(range==256){
 		/* 10.5.7.2 */
 		num_bits=8;
@@ -829,15 +815,8 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 		val=tvb_get_guint8(tvb, offset>>3);
 		offset+=8;
 
+		val_start = (offset>>3)-1; val_length = 1;
 		val+=min;
-		it=proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-1, 1, val);
-		if(item){
-			*item=it;
-		}
-		if(value){
-			*value=val;
-		}
-		return offset;
 	} else if(range<=65536){
 		/* 10.5.7.3 */
 		num_bits=16;
@@ -853,15 +832,8 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 		val|=tvb_get_guint8(tvb, offset>>3);
 		offset+=8;
 
+		val_start = (offset>>3)-2; val_length = 2;
 		val+=min;
-		it=proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-2, 2, val);
-		if(item){
-			*item=it;
-		}
-		if(value){
-			*value=val;
-		}
-		return offset;
 	} else {
 		int i,num_bytes;
 		gboolean bit;
@@ -884,20 +856,23 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 			val=(val<<8)|tvb_get_guint8(tvb,offset>>3);
 			offset+=8;
 		}
+		val_start = (offset>>3)-(num_bytes+1); val_length = num_bytes+1;
 		val+=min;
-		it=proto_tree_add_uint(tree, hf_index, tvb, (offset>>3)-(num_bytes+1), num_bytes+1, val);
-		if(item){
-			*item=it;
-		}
-		if(value){
-			*value=val;
-		}
-		return offset;
 	}
 
-	NOT_DECODED_YET("10.5");
-	return offset;
-}
+	timeval.secs = val;
+	if (IS_FT_UINT(hfi->type)) {
+		it = proto_tree_add_uint(tree, hf_index, tvb, val_start, val_length, val);
+	} else if (IS_FT_INT(hfi->type)) {
+		it = proto_tree_add_int(tree, hf_index, tvb, val_start, val_length, val);
+	} else if (IS_FT_TIME(hfi->type)) {
+		it = proto_tree_add_time(tree, hf_index, tvb, val_start, val_length, &timeval);
+	} else {
+		g_assert_not_reached();
+	}
+	if (item) *item = it;
+	if (value) *value = val;
+	return offset;}
 
 /* this functions decodes a CHOICE
    it can only handle CHOICE INDEX values that fits inside a 32 bit integer.
@@ -992,7 +967,7 @@ DEBUG_ENTRY("dissect_per_choice");
 					offset=choice[i].func(tvb, offset, pinfo, choicetree);
 					break;
 				} else {
-					NOT_DECODED_YET(choice[i].name);
+					PER_NOT_DECODED_YET(choice[i].name);
 					break;
 				}
 			}
@@ -1039,7 +1014,7 @@ DEBUG_ENTRY("dissect_per_choice");
 		if(index==-1){
 			/* if we dont know how to decode this one, just step offset to the next structure */
 			offset+=length*8;
-			NOT_DECODED_YET("unknown choice extension");
+			PER_NOT_DECODED_YET("unknown choice extension");
 		} else {
 			guint32 new_offset;
 
@@ -1182,7 +1157,7 @@ DEBUG_ENTRY("dissect_per_sequence");
 			if(sequence[i].func){
 				offset=sequence[i].func(tvb, offset, pinfo, tree);
 			} else {
-				NOT_DECODED_YET(sequence[i].name);
+				PER_NOT_DECODED_YET(sequence[i].name);
 			}
 		}
 	}
@@ -1257,7 +1232,7 @@ DEBUG_ENTRY("dissect_per_sequence");
 			if(i>=num_known_extensions){
 				/* we dont know how to decode this extension */
 				offset+=length*8;
-				NOT_DECODED_YET("unknown sequence extension");
+				PER_NOT_DECODED_YET("unknown sequence extension");
 				continue;
 			}
 
@@ -1275,7 +1250,7 @@ DEBUG_ENTRY("dissect_per_sequence");
 			if(sequence[extension_index].func){
 				new_offset=sequence[extension_index].func(tvb, offset, pinfo, tree);
 			} else {
-				NOT_DECODED_YET(sequence[extension_index].name);
+				PER_NOT_DECODED_YET(sequence[extension_index].name);
 			}
 			offset+=length*8;
 
@@ -1399,25 +1374,30 @@ DEBUG_ENTRY("dissect_per_bit_string");
 guint32
 dissect_per_octet_string(tvbuff_t *tvb, guint32 offset, packet_info *pinfo, proto_tree *tree, int hf_index, int min_len, int max_len, guint32 *value_offset, guint32 *value_len)
 {
+	proto_tree *etr = NULL;
+	proto_item *it = NULL;
+	gint val_start, val_length;
 	guint32 length;
 	header_field_info *hfi;
+	static char bytes[4];
+	char *pbytes = NULL;
 
 	hfi = (hf_index==-1) ? NULL : proto_registrar_get_nth(hf_index);
-
-DEBUG_ENTRY("dissect_per_octet_string");
-	/* 16.5 if the length is 0 bytes there will be no encoding */
-	if(max_len==0){
-		return offset;
+	if (display_internal_per_fields) {
+		etr = tree;
 	}
 
+DEBUG_ENTRY("dissect_per_octet_string");
 
 	if(min_len==-1){
 		min_len=0;
 	}
+	
+	if (max_len==0) {  /* 16.5 if the length is 0 bytes there will be no encoding */
+		val_start = offset>>3; 
+		val_length = 0;
 
-	/* 16.6 if length is fixed and less than or equal to two bytes*/
-	if((min_len==max_len)&&(max_len<=2)){
-		static char bytes[4];
+	} else if((min_len==max_len)&&(max_len<=2)) {  /* 16.6 if length is fixed and less than or equal to two bytes*/
 		guint32 i, old_offset=offset;
 		gboolean bit;
 
@@ -1432,72 +1412,63 @@ DEBUG_ENTRY("dissect_per_octet_string");
 			}
 		}
 		bytes[min_len]=0;
-		if (hfi) {
-			if(hfi->type==FT_STRING){
-				proto_tree_add_string(tree, hf_index, tvb, old_offset>>3, min_len+(offset&0x07)?1:0, bytes);
-			} else {
-				proto_tree_add_bytes(tree, hf_index, tvb, old_offset>>3, min_len+(offset&0x07)?1:0, bytes);
+		pbytes = bytes;
+		val_start = old_offset>>3; 
+		val_length = min_len+(offset&0x07)?1:0;
+
+	} else if ((min_len==max_len)&&(min_len<65536)) {  /* 16.7 if length is fixed and less than to 64k*/
+		/* align to byte */
+		if(offset&0x07){
+			offset=(offset&0xfffffff8)+8;
+		}
+		val_start = offset>>3; 
+		val_length = min_len;
+		offset+=min_len*8;
+
+	} else {  /* 16.8 */
+		if(max_len>0) {  
+			offset = dissect_per_constrained_integer(tvb, offset, pinfo, etr,
+				hf_per_octet_string_length, min_len, max_len,
+				&length, NULL, FALSE);
+		} else {
+			offset = dissect_per_length_determinant(tvb, offset, pinfo, etr, 
+				hf_per_octet_string_length, &length);
+		}
+
+		if(length){
+			/* align to byte */
+			if(offset&0x07){
+				offset=(offset&0xfffffff8)+8;
 			}
 		}
-		if (value_offset) {
-			*value_offset = old_offset>>3;
-		}
-		if (value_len) {
-			*value_len = min_len+(offset&0x07)?1:0;
-		}
-		return offset;
+		val_start = offset>>3; 
+		val_length = length;
+		offset+=length*8;
 	}
 
-
-	/* 16.7 if length is fixed and less than to 64k*/
-	if((min_len==max_len)&&(min_len<65536)){
-		/* align to byte */
-		if(offset&0x07){
-			offset=(offset&0xfffffff8)+8;
-		}
-		if (hfi) {
-			proto_tree_add_item(tree, hf_index, tvb, offset>>3, min_len, FALSE);
-		}
-		if (value_offset) {
-			*value_offset = offset>>3;
-		}
-		if (value_len) {
-			*value_len = min_len;
-		}
-		offset+=min_len*8;
-		return offset;
-	}
-
-	/* 16.8 */
-	if(max_len>0){
-		proto_tree *etr = NULL;
-
-		if(display_internal_per_fields){
-			etr=tree;
-		}
-		offset=dissect_per_constrained_integer(tvb, offset, pinfo,
-			etr, hf_per_octet_string_length, min_len, max_len,
-			&length, NULL, FALSE);
-	} else {
-		offset=dissect_per_length_determinant(tvb, offset, pinfo, tree, hf_per_octet_string_length, &length);
-	}
-	if(length){
-		/* align to byte */
-		if(offset&0x07){
-			offset=(offset&0xfffffff8)+8;
-		}
-		if (hfi) {
-			proto_tree_add_item(tree, hf_index, tvb, offset>>3, length, FALSE);
+	if (hfi) {
+		if (IS_FT_UINT(hfi->type)||IS_FT_INT(hfi->type)) {
+			if (IS_FT_UINT(hfi->type))
+				it = proto_tree_add_uint(tree, hf_index, tvb, val_start, val_length, val_length);
+			else
+				it = proto_tree_add_int(tree, hf_index, tvb, val_start, val_length, val_length);
+			proto_item_append_text(it, (val_length == 1) ? " octet" : " octets");
+		} else {
+			if (pbytes) {
+				if(IS_FT_STRING(hfi->type)){
+					proto_tree_add_string(tree, hf_index, tvb, val_start, val_length, pbytes);
+				} else if (hfi->type==FT_BYTES) {
+					proto_tree_add_bytes(tree, hf_index, tvb, val_start, val_length, pbytes);
+				} else {
+					g_assert_not_reached();
+				}
+			} else {
+				proto_tree_add_item(tree, hf_index, tvb, val_start, val_length, FALSE);
+			}
 		}
 	}
-	if (value_offset) {
-		*value_offset = offset>>3;
-	}
-	if (value_len) {
-		*value_len = length;
-	}
-	offset+=length*8;
-
+	if (value_offset) *value_offset = val_start;
+	if (value_len) *value_len = val_length;
 	return offset;
 }
 
