@@ -4,9 +4,9 @@
  * RTP = Real time Transport Protocol
  *
  * Copyright 2000, Philips Electronics N.V.
- * Written by Andreas Sikkema <andreas.sikkema@philips.com>
+ * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c,v 1.38 2003/03/06 20:35:09 sahlberg Exp $
+ * $Id: packet-rtp.c,v 1.52 2004/07/01 06:59:38 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -62,11 +62,18 @@
 #include <string.h>
 
 #include "packet-rtp.h"
+#include "rtp_pt.h"
 #include <epan/conversation.h>
 #include "tap.h"
 
+#include "prefs.h"
+
+static dissector_handle_t rtp_handle;
+
 static int rtp_tap = -1;
 
+static dissector_table_t rtp_pt_dissector_table;
+
 /* RTP header fields             */
 static int proto_rtp           = -1;
 static int hf_rtp_version      = -1;
@@ -88,19 +95,33 @@ static int hf_rtp_prof_define  = -1;
 static int hf_rtp_length       = -1;
 static int hf_rtp_hdr_ext      = -1;
 
+/* RTP setup fields */
+static int hf_rtp_setup        = -1;
+static int hf_rtp_setup_frame  = -1;
+static int hf_rtp_setup_method = -1;
+
 /* RTP fields defining a sub tree */
 static gint ett_rtp       = -1;
 static gint ett_csrc_list = -1;
 static gint ett_hdr_ext   = -1;
+static gint ett_rtp_setup = -1;
 
-static dissector_handle_t h261_handle;
-static dissector_handle_t mpeg1_handle;
 static dissector_handle_t data_handle;
 
 static gboolean dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo,
     proto_tree *tree );
 static void dissect_rtp( tvbuff_t *tvb, packet_info *pinfo,
     proto_tree *tree );
+static void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+
+/* Preferences bool to control whether or not setup info should be shown */
+static gboolean global_rtp_show_setup_info = TRUE;
+
+/* Try heuristic RTP decode */
+static gboolean global_rtp_heur = FALSE;
+
+/* Memory chunk for storing conversation and per-packet info */
+static GMemChunk *rtp_conversations = NULL;
 
 /*
  * Fields in the first octet of the RTP header.
@@ -137,41 +158,7 @@ static const value_string rtp_version_vals[] =
 /* Payload type is the last 7 bits */
 #define RTP_PAYLOAD_TYPE(octet)	((octet) & 0x7F)
 
-/*
- * RTP Payload types
- * Table B.2 / H.225.0
- * Also RFC 1890, and
- *
- *	http://www.iana.org/assignments/rtp-parameters
- */
-#define PT_PCMU		0	/* RFC 1890 */
-#define PT_1016		1	/* RFC 1890 */
-#define PT_G721		2	/* RFC 1890 */
-#define PT_GSM		3	/* RFC 1890 */
-#define PT_G723		4	/* From Vineet Kumar of Intel; see the Web page */
-#define PT_DVI4_8000	5	/* RFC 1890 */
-#define PT_DVI4_16000	6	/* RFC 1890 */
-#define PT_LPC		7	/* RFC 1890 */
-#define PT_PCMA		8	/* RFC 1890 */
-#define PT_G722		9	/* RFC 1890 */
-#define PT_L16_STEREO	10	/* RFC 1890 */
-#define PT_L16_MONO	11	/* RFC 1890 */
-#define PT_QCELP	12	/* Qualcomm Code Excited Linear Predictive coding? */
-#define PT_CN		13	/* RFC 3389 */
-#define PT_MPA		14	/* RFC 1890, RFC 2250 */
-#define PT_G728		15	/* RFC 1890 */
-#define PT_DVI4_11025	16	/* from Joseph Di Pol of Sun; see the Web page */
-#define PT_DVI4_22050	17	/* from Joseph Di Pol of Sun; see the Web page */
-#define PT_G729		18
-#define PT_CELB		25	/* RFC 2029 */
-#define PT_JPEG		26	/* RFC 2435 */
-#define PT_NV		28	/* RFC 1890 */
-#define PT_H261		31	/* RFC 2032 */
-#define PT_MPV		32	/* RFC 2250 */
-#define PT_MP2T		33	/* RFC 2250 */
-#define PT_H263		34	/* from Chunrong Zhu of Intel; see the Web page */
-
-static const value_string rtp_payload_type_vals[] =
+const value_string rtp_payload_type_vals[] =
 {
 	{ PT_PCMU,	"ITU-T G.711 PCMU" },
 	{ PT_1016,	"USA Federal Standard FS-1016" },
@@ -192,6 +179,7 @@ static const value_string rtp_payload_type_vals[] =
 	{ PT_DVI4_11025, "DVI4 11025 samples/s" },
 	{ PT_DVI4_22050, "DVI4 22050 samples/s" },
 	{ PT_G729,	"ITU-T G.729" },
+	{ PT_CN_OLD,	"Comfort noise (old)" },
 	{ PT_CELB,	"Sun CellB video encoding" },
 	{ PT_JPEG,	"JPEG-compressed video" },
 	{ PT_NV,	"'nv' program" },
@@ -203,13 +191,16 @@ static const value_string rtp_payload_type_vals[] =
 };
 
 static address fake_addr;
-static int heur_init = FALSE;
 
-void rtp_add_address( packet_info *pinfo, const unsigned char* ip_addr,
-    int prt )
+/* Set up an RTP conversation */
+void rtp_add_address(packet_info *pinfo,
+                     const unsigned char* ip_addr, int port,
+                     int other_port,
+                     gchar *setup_method, guint32 setup_frame_number)
 {
 	address src_addr;
-	conversation_t* pconv;
+	conversation_t* p_conv = NULL;
+	struct _rtp_conversation_info *p_conv_data = NULL;
 
 	/*
 	 * If this isn't the first time this packet has been processed,
@@ -217,45 +208,69 @@ void rtp_add_address( packet_info *pinfo, const unsigned char* ip_addr,
 	 * again.
 	 */
 	if (pinfo->fd->flags.visited)
+	{
 		return;
+	}
 
-	src_addr.type = AT_IPv4;
-	src_addr.len = 4;
+	src_addr.type = pinfo->net_src.type;
+	src_addr.len = pinfo->net_src.len;
 	src_addr.data = ip_addr;
 
 	/*
-	 * The first time the function is called let the tcp dissector
-	 * know that we're interested in traffic
-	 */
-	if ( ! heur_init ) {
-		heur_dissector_add( "udp", dissect_rtp_heur, proto_rtp );
-		heur_init = TRUE;
-	}
-
-	/*
-	 * Check if the ip address an dport combination is not
+	 * Check if the ip address and port combination is not
 	 * already registered
 	 */
-	pconv = find_conversation( &src_addr, &fake_addr, PT_UDP, prt, 0, 0 );
+	p_conv = find_conversation( &src_addr, &src_addr, PT_UDP, port, other_port,
+                                NO_ADDR_B | (!other_port ? NO_PORT_B : 0));
 
 	/*
 	 * If not, add
-	 * XXX - use wildcard address and port B?
 	 */
-	if ( ! pconv ) {
-		pconv = conversation_new( &src_addr, &fake_addr, PT_UDP,
-		    (guint32) prt, (guint32) 0, 0 );
-		conversation_add_proto_data(pconv, proto_rtp, NULL);
-	}
+	if ( ! p_conv ) {
+		/* Create conversation data */
+		p_conv_data = g_mem_chunk_alloc(rtp_conversations);
 
+		/* Check length first time we look at method string */
+		strncpy(p_conv_data->method, setup_method,
+		        (strlen(setup_method)+1 <= MAX_RTP_SETUP_METHOD_SIZE) ?
+		             strlen(setup_method)+1 :
+		             MAX_RTP_SETUP_METHOD_SIZE);
+		p_conv_data->method[MAX_RTP_SETUP_METHOD_SIZE] = '\0';
+		p_conv_data->frame_number = setup_frame_number;
+
+		/* Create conversation with this data */
+		p_conv = conversation_new( &src_addr, &src_addr, PT_UDP,
+		                           (guint32)port, (guint32)other_port,
+								   NO_ADDR2 | (!other_port ? NO_PORT2 : 0));
+		conversation_add_proto_data(p_conv, proto_rtp, p_conv_data);
+
+		/* Set dissector */
+		conversation_set_dissector(p_conv, rtp_handle);
+	}
+	else
+	{
+		/* Update existing conversation data */
+		p_conv_data = conversation_get_proto_data(p_conv, proto_rtp);
+		strcpy(p_conv_data->method, setup_method);
+		p_conv_data->frame_number = setup_frame_number;
+	}
 }
 
-#if 0
 static void rtp_init( void )
 {
 	unsigned char* tmp_data;
 	int i;
 
+	/* (Re)allocate mem chunk for conversations */
+	if (rtp_conversations)
+	{
+		g_mem_chunk_destroy(rtp_conversations);
+	}
+	rtp_conversations = g_mem_chunk_new("rtp_conversations",
+	                                    sizeof(struct _rtp_conversation_info),
+	                                    20 * sizeof(struct _rtp_conversation_info),
+	                                    G_ALLOC_ONLY);
+
 	/* Create a fake adddress... */
 	fake_addr.type = AT_IPv4;
 	fake_addr.len = 4;
@@ -266,41 +281,46 @@ static void rtp_init( void )
 	}
 	fake_addr.data = tmp_data;
 }
-#endif
 
 static gboolean
 dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
-	conversation_t* pconv;
-
-	/* This is a heuristic dissector, which means we get all the TCP
+	guint8      octet1, octet2;
+ 	unsigned int version;
+	unsigned int payload_type;
+ 	unsigned int offset = 0;
+   
+	/* This is a heuristic dissector, which means we get all the UDP
 	 * traffic not sent to a known dissector and not claimed by
 	 * a heuristic dissector called before us!
-	 * So we first check if the frame is really meant for us.
 	 */
-	if ( ( pconv = find_conversation( &pinfo->src, &fake_addr, pinfo->ptype,
-	    pinfo->srcport, 0, 0 ) ) == NULL ) {
-		/*
-		 * The source ip:port combination was not what we were
-		 * looking for, check the destination
-		 */
-		if ( ( pconv = find_conversation( &pinfo->dst, &fake_addr,
-		    pinfo->ptype, pinfo->destport, 0, 0 ) ) == NULL ) {
-			return FALSE;
-		}
-	}
 
-	/*
-	 * An RTP conversation always has a data item for RTP.
-	 * (Its existence is sufficient to indicate that this is an RTP
-	 * conversation.)
-	 */
-	if (conversation_get_proto_data(pconv, proto_rtp) == NULL)
+	if (! global_rtp_heur)
 		return FALSE;
 
-	dissect_rtp( tvb, pinfo, tree );
+	/* Get the fields in the first octet */
+	octet1 = tvb_get_guint8( tvb, offset );
+	version = RTP_VERSION( octet1 );
 
-	return TRUE;
+	if (version != 2) {
+		/* Unknown or unsupported version */
+		return FALSE;
+	}
+
+	/* Get the fields in the second octet */
+	octet2 = tvb_get_guint8( tvb, offset + 1 );
+	payload_type = RTP_PAYLOAD_TYPE( octet2 );
+	/*      if (payload_type == PT_PCMU ||
+	 *		     payload_type == PT_PCMA)
+	 *	     payload_type == PT_G729)
+	 *	 */
+	if (payload_type <= PT_H263) {
+ 		dissect_rtp( tvb, pinfo, tree );
+		return TRUE;
+	}
+	else {
+ 		return FALSE;
+	}
 }
 
 static void
@@ -310,23 +330,10 @@ dissect_rtp_data( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 {
 	tvbuff_t *newtvb;
 
-	switch( payload_type ) {
-		case PT_H261:
-			newtvb = tvb_new_subset( tvb, offset, data_len,
-			    data_reported_len );
-			call_dissector(h261_handle, newtvb, pinfo, tree);
-			break;
-
-		case PT_MPV:
-			newtvb = tvb_new_subset( tvb, offset, data_len,
-			    data_reported_len );
-			call_dissector(mpeg1_handle, newtvb, pinfo, tree);
-			break;
-
-		default:
-			proto_tree_add_item( rtp_tree, hf_rtp_data, tvb, offset, data_len, FALSE );
-			break;
-	}
+	newtvb = tvb_new_subset( tvb, offset, data_len, data_reported_len );
+	if (!dissector_try_port(rtp_pt_dissector_table, payload_type, newtvb,
+	    pinfo, tree))
+		proto_tree_add_item( rtp_tree, hf_rtp_data, newtvb, 0, -1, FALSE );
 }
 
 static void
@@ -335,7 +342,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	proto_item *ti            = NULL;
 	proto_tree *rtp_tree      = NULL;
 	proto_tree *rtp_csrc_tree = NULL;
-	guint8      octet;
+	guint8      octet1, octet2;
 	unsigned int version;
 	gboolean    padding_set;
 	gboolean    extension_set;
@@ -345,6 +352,7 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	unsigned int i            = 0;
 	unsigned int hdr_extension= 0;
 	unsigned int padding_count;
+	gint        length, reported_length;
 	int         data_len;
 	unsigned int offset = 0;
 	guint16     seq_num;
@@ -355,8 +363,8 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	static struct _rtp_info rtp_info;
 
 	/* Get the fields in the first octet */
-	octet = tvb_get_guint8( tvb, offset );
-	version = RTP_VERSION( octet );
+	octet1 = tvb_get_guint8( tvb, offset );
+	version = RTP_VERSION( octet1 );
 
 	if (version != 2) {
 		/*
@@ -376,26 +384,26 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 			rtp_tree = proto_item_add_subtree( ti, ett_rtp );
 
 			proto_tree_add_uint( rtp_tree, hf_rtp_version, tvb,
-			    offset, 1, version );
+			    offset, 1, octet1);
 		}
 		return;
 	}
 
-	padding_set = RTP_PADDING( octet );
-	extension_set = RTP_EXTENSION( octet );
-	csrc_count = RTP_CSRC_COUNT( octet );
+	padding_set = RTP_PADDING( octet1 );
+	extension_set = RTP_EXTENSION( octet1 );
+	csrc_count = RTP_CSRC_COUNT( octet1 );
 
 	/* Get the fields in the second octet */
-	octet = tvb_get_guint8( tvb, offset + 1 );
-	marker_set = RTP_MARKER( octet );
-	payload_type = RTP_PAYLOAD_TYPE( octet );
+	octet2 = tvb_get_guint8( tvb, offset + 1 );
+	marker_set = RTP_MARKER( octet2 );
+	payload_type = RTP_PAYLOAD_TYPE( octet2 );
 
 	/* Get the subsequent fields */
 	seq_num = tvb_get_ntohs( tvb, offset + 2 );
 	timestamp = tvb_get_ntohl( tvb, offset + 4 );
 	sync_src = tvb_get_ntohl( tvb, offset + 8 );
 
-	/* fill in the rtp_info structure */ 
+	/* fill in the rtp_info structure */
 	rtp_info.info_padding_set = padding_set;
 	rtp_info.info_padding_count = 0;
 	rtp_info.info_marker_set = marker_set;
@@ -403,7 +411,39 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	rtp_info.info_seq_num = seq_num;
 	rtp_info.info_timestamp = timestamp;
 	rtp_info.info_sync_src = sync_src;
-	rtp_info.info_data_len = tvb_reported_length_remaining( tvb, offset );
+
+	/*
+	 * Do we have all the data?
+	 */
+	length = tvb_length_remaining(tvb, offset);
+	reported_length = tvb_reported_length_remaining(tvb, offset);
+	if (reported_length >= 0 && length >= reported_length) {
+		/*
+		 * Yes.
+		 */
+		rtp_info.info_all_data_present = TRUE;
+		rtp_info.info_data_len = reported_length;
+
+		/*
+		 * Save the pointer to raw rtp data (header + payload incl.
+		 * padding).
+		 * That should be safe because the "epan_dissect_t"
+		 * constructed for the packet has not yet been freed when
+		 * the taps are called.
+		 * (Destroying the "epan_dissect_t" will end up freeing
+		 * all the tvbuffs and hence invalidating pointers to
+		 * their data.)
+		 * See "add_packet_to_packet_list()" for details.
+		 */
+		rtp_info.info_data = tvb_get_ptr(tvb, 0, -1);
+	} else {
+		/*
+		 * No - packet was cut short at capture time.
+		 */
+		rtp_info.info_all_data_present = FALSE;
+		rtp_info.info_data_len = 0;
+		rtp_info.info_data = NULL;
+	}
 
 	if ( check_col( pinfo->cinfo, COL_PROTOCOL ) )   {
 		col_set_str( pinfo->cinfo, COL_PROTOCOL, "RTP" );
@@ -420,23 +460,30 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		    marker_set ? ", Mark" : "");
 	}
 	if ( tree ) {
-		ti = proto_tree_add_item( tree, proto_rtp, tvb, offset, -1, FALSE );
-		rtp_tree = proto_item_add_subtree( ti, ett_rtp );
+		/* Create RTP protocol tree */
+		ti = proto_tree_add_item(tree, proto_rtp, tvb, offset, -1, FALSE );
+		rtp_tree = proto_item_add_subtree(ti, ett_rtp );
+
+		/* Conversation setup info */
+		if (global_rtp_show_setup_info)
+		{
+			show_setup_info(tvb, pinfo, rtp_tree);
+		}
 
 		proto_tree_add_uint( rtp_tree, hf_rtp_version, tvb,
-		    offset, 1, version );
+		    offset, 1, octet1 );
 		proto_tree_add_boolean( rtp_tree, hf_rtp_padding, tvb,
-		    offset, 1, padding_set );
+		    offset, 1, octet1 );
 		proto_tree_add_boolean( rtp_tree, hf_rtp_extension, tvb,
-		    offset, 1, extension_set );
+		    offset, 1, octet1 );
 		proto_tree_add_uint( rtp_tree, hf_rtp_csrc_count, tvb,
-		    offset, 1, csrc_count );
+		    offset, 1, octet1 );
 		offset++;
 
 		proto_tree_add_boolean( rtp_tree, hf_rtp_marker, tvb, offset,
-		    1, marker_set );
+		    1, octet2 );
 		proto_tree_add_uint( rtp_tree, hf_rtp_payload_type, tvb,
-		    offset, 1, payload_type );
+		    offset, 1, octet2 );
 		offset++;
 
 		/* Sequence number 16 bits (2 octets) */
@@ -450,128 +497,197 @@ dissect_rtp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 		/* Synchronization source identifier 32 bits (4 octets) */
 		proto_tree_add_uint( rtp_tree, hf_rtp_ssrc, tvb, offset, 4, sync_src );
 		offset += 4;
-
-		/* CSRC list*/
-		if ( csrc_count > 0 ) {
+	} else {
+		offset += 12;
+	}
+	/* CSRC list*/
+	if ( csrc_count > 0 ) {
+        	if ( tree ) {
 			ti = proto_tree_add_text(rtp_tree, tvb, offset, csrc_count * 4, "Contributing Source identifiers");
 			rtp_csrc_tree = proto_item_add_subtree( ti, ett_csrc_list );
-			for (i = 0; i < csrc_count; i++ ) {
-				csrc_item = tvb_get_ntohl( tvb, offset );
-				proto_tree_add_uint_format( rtp_csrc_tree,
-				    hf_rtp_csrc_item, tvb, offset, 4,
-				    csrc_item,
-				    "CSRC item %d: %u",
-				    i, csrc_item );
-				offset += 4;
-			}
 		}
+		for (i = 0; i < csrc_count; i++ ) {
+			csrc_item = tvb_get_ntohl( tvb, offset );
+			if ( tree ) proto_tree_add_uint_format( rtp_csrc_tree,
+			    hf_rtp_csrc_item, tvb, offset, 4,
+			    csrc_item,
+			    "CSRC item %d: %u",
+			    i, csrc_item );
+			offset += 4;
+		}
+	}
 
-		/* Optional RTP header extension */
-		if ( extension_set ) {
-			/* Defined by profile field is 16 bits (2 octets) */
-			proto_tree_add_uint( rtp_tree, hf_rtp_prof_define, tvb, offset, 2, tvb_get_ntohs( tvb, offset ) );
-			offset += 2;
+	/* Optional RTP header extension */
+	if ( extension_set ) {
+		/* Defined by profile field is 16 bits (2 octets) */
+		if ( tree ) proto_tree_add_uint( rtp_tree, hf_rtp_prof_define, tvb, offset, 2, tvb_get_ntohs( tvb, offset ) );
+		offset += 2;
 
-			hdr_extension = tvb_get_ntohs( tvb, offset );
-			proto_tree_add_uint( rtp_tree, hf_rtp_length, tvb,
-			    offset, 2, hdr_extension);
-			offset += 2;
-			if ( hdr_extension > 0 ) {
+		hdr_extension = tvb_get_ntohs( tvb, offset );
+		if ( tree ) proto_tree_add_uint( rtp_tree, hf_rtp_length, tvb,
+		    offset, 2, hdr_extension);
+		offset += 2;
+		if ( hdr_extension > 0 ) {
+			if ( tree ) {
 				ti = proto_tree_add_text(rtp_tree, tvb, offset, csrc_count * 4, "Header extensions");
 				/* I'm re-using the old tree variable here
 				   from the CSRC list!*/
 				rtp_csrc_tree = proto_item_add_subtree( ti,
 				    ett_hdr_ext );
-				for (i = 0; i < hdr_extension; i++ ) {
-					proto_tree_add_uint( rtp_csrc_tree, hf_rtp_hdr_ext, tvb, offset, 4, tvb_get_ntohl( tvb, offset ) );
-					offset += 4;
-				}
 			}
-		}
-
-		if ( padding_set ) {
-			/*
-			 * This RTP frame has padding - find it.
-			 *
-			 * The padding count is found in the LAST octet of
-			 * the packet; it contains the number of octets
-			 * that can be ignored at the end of the packet.
-			 */
-			if (tvb_length(tvb) < tvb_reported_length(tvb)) {
-				/*
-				 * We don't *have* the last octet of the
-				 * packet, so we can't get the padding
-				 * count.
-				 *
-				 * Put an indication of that into the
-				 * tree, and just put in a raw data
-				 * item.
-				 */
-				proto_tree_add_text(rtp_tree, tvb, 0, 0,
-				    "Frame has padding, but not all the frame data was captured");
-				call_dissector(data_handle,
-				    tvb_new_subset(tvb, offset, -1, -1),
-				    pinfo, rtp_tree);
-				return;
+			for (i = 0; i < hdr_extension; i++ ) {
+				if ( tree ) proto_tree_add_uint( rtp_csrc_tree, hf_rtp_hdr_ext, tvb, offset, 4, tvb_get_ntohl( tvb, offset ) );
+				offset += 4;
 			}
-
-			padding_count = tvb_get_guint8( tvb,
-			    tvb_reported_length( tvb ) - 1 );
-			data_len =
-			    tvb_reported_length_remaining( tvb, offset ) - padding_count;
-			if (data_len > 0) {
-				/*
-				 * There's data left over when you take out
-				 * the padding; dissect it.
-				 */
-				dissect_rtp_data( tvb, pinfo, tree, rtp_tree,
-				    offset,
-				    data_len,
-				    data_len,
-				    payload_type );
-				offset += data_len;
-			} else if (data_len < 0) {
-				/*
-				 * The padding count is bigger than the
-				 * amount of RTP payload in the packet!
-				 * Clip the padding count.
-				 *
-				 * XXX - put an item in the tree to indicate
-				 * that the padding count is bogus?
-				 */
-				padding_count =
-				    tvb_reported_length_remaining(tvb, offset);
-			}
-			if (padding_count > 1) {
-				/*
-				 * There's more than one byte of padding;
-				 * show all but the last byte as padding
-				 * data.
-				 */
-				proto_tree_add_item( rtp_tree, hf_rtp_padding_data,
-				    tvb, offset, padding_count - 1, FALSE );
-				offset += padding_count - 1;
-			}
-			/*
-			 * Show the last byte in the PDU as the padding
-			 * count.
-			 */
-			proto_tree_add_item( rtp_tree, hf_rtp_padding_count,
-			    tvb, offset, 1, FALSE );
-		}
-		else {
-			/*
-			 * No padding.
-			 */
-			dissect_rtp_data( tvb, pinfo, tree, rtp_tree, offset,
-			    tvb_length_remaining( tvb, offset ),
-			    tvb_reported_length_remaining( tvb, offset ),
-			    payload_type );
 		}
 	}
-	tap_queue_packet(rtp_tap, pinfo, &rtp_info);
+
+	if ( padding_set ) {
+		/*
+		 * This RTP frame has padding - find it.
+		 *
+		 * The padding count is found in the LAST octet of
+		 * the packet; it contains the number of octets
+		 * that can be ignored at the end of the packet.
+		 */
+		if (tvb_length(tvb) < tvb_reported_length(tvb)) {
+			/*
+			 * We don't *have* the last octet of the
+			 * packet, so we can't get the padding
+			 * count.
+			 *
+			 * Put an indication of that into the
+			 * tree, and just put in a raw data
+			 * item.
+			 */
+			if ( tree ) proto_tree_add_text(rtp_tree, tvb, 0, 0,
+			    "Frame has padding, but not all the frame data was captured");
+			call_dissector(data_handle,
+			    tvb_new_subset(tvb, offset, -1, -1),
+			    pinfo, rtp_tree);
+			return;
+		}
+
+		padding_count = tvb_get_guint8( tvb,
+		    tvb_reported_length( tvb ) - 1 );
+		data_len =
+		    tvb_reported_length_remaining( tvb, offset ) - padding_count;
+
+		rtp_info.info_payload_offset = offset;
+		rtp_info.info_payload_len = tvb_length_remaining(tvb, offset);
+		rtp_info.info_padding_count = padding_count;
+
+		if (data_len > 0) {
+			/*
+			 * There's data left over when you take out
+			 * the padding; dissect it.
+			 */
+			dissect_rtp_data( tvb, pinfo, tree, rtp_tree,
+			    offset,
+			    data_len,
+			    data_len,
+			    payload_type );
+			offset += data_len;
+		} else if (data_len < 0) {
+			/*
+			 * The padding count is bigger than the
+			 * amount of RTP payload in the packet!
+			 * Clip the padding count.
+			 *
+			 * XXX - put an item in the tree to indicate
+			 * that the padding count is bogus?
+			 */
+			padding_count =
+			    tvb_reported_length_remaining(tvb, offset);
+		}
+		if (padding_count > 1) {
+			/*
+			 * There's more than one byte of padding;
+			 * show all but the last byte as padding
+			 * data.
+			 */
+			if ( tree ) proto_tree_add_item( rtp_tree, hf_rtp_padding_data,
+			    tvb, offset, padding_count - 1, FALSE );
+			offset += padding_count - 1;
+		}
+		/*
+		 * Show the last byte in the PDU as the padding
+		 * count.
+		 */
+		if ( tree ) proto_tree_add_item( rtp_tree, hf_rtp_padding_count,
+		    tvb, offset, 1, FALSE );
+	}
+	else {
+		/*
+		 * No padding.
+		 */
+		dissect_rtp_data( tvb, pinfo, tree, rtp_tree, offset,
+		    tvb_length_remaining( tvb, offset ),
+		    tvb_reported_length_remaining( tvb, offset ),
+		    payload_type );
+		rtp_info.info_payload_offset = offset;
+		rtp_info.info_payload_len = tvb_length_remaining(tvb, offset);
+	}
+	if (!pinfo->in_error_pkt)
+		tap_queue_packet(rtp_tap, pinfo, &rtp_info);
 }
 
+
+/* Look for conversation info and display any setup info found */
+void show_setup_info(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	/* Conversation and current data */
+	conversation_t *p_conv = NULL;
+	struct _rtp_conversation_info *p_conv_data = NULL;
+
+	/* Use existing packet info if available */
+	p_conv_data = p_get_proto_data(pinfo->fd, proto_rtp);
+
+	if (!p_conv_data)
+	{
+		/* First time, get info from conversation */
+		p_conv = find_conversation(&pinfo->net_dst, &pinfo->net_src,
+                                   pinfo->ptype,
+                                   pinfo->destport, pinfo->srcport, NO_ADDR_B);
+		if (p_conv)
+		{
+			/* Create space for packet info */
+			struct _rtp_conversation_info *p_conv_packet_data;
+			p_conv_data = conversation_get_proto_data(p_conv, proto_rtp);
+
+			/* Save this conversation info into packet info */
+			p_conv_packet_data = g_mem_chunk_alloc(rtp_conversations);
+			strcpy(p_conv_packet_data->method, p_conv_data->method);
+			p_conv_packet_data->frame_number = p_conv_data->frame_number;
+			p_add_proto_data(pinfo->fd, proto_rtp, p_conv_packet_data);
+		}
+	}
+
+	/* Create setup info subtree with summary info. */
+	if (p_conv_data)
+	{
+		proto_tree *rtp_setup_tree;
+		proto_item *ti =  proto_tree_add_string_format(tree, hf_rtp_setup, tvb, 0, 0,
+		                                               "",
+		                                               "Stream setup by %s (frame %d)",
+		                                               p_conv_data->method,
+		                                               p_conv_data->frame_number);
+		PROTO_ITEM_SET_GENERATED(ti);
+		rtp_setup_tree = proto_item_add_subtree(ti, ett_rtp_setup);
+		if (rtp_setup_tree)
+		{
+			/* Add details into subtree */
+			proto_item* item = proto_tree_add_uint(rtp_setup_tree, hf_rtp_setup_frame,
+			                                       tvb, 0, 0, p_conv_data->frame_number);
+			PROTO_ITEM_SET_GENERATED(item);
+			item = proto_tree_add_string(rtp_setup_tree, hf_rtp_setup_method,
+			                             tvb, 0, 0, p_conv_data->method);
+			PROTO_ITEM_SET_GENERATED(item);
+		}
+	}
+}
+
+
 void
 proto_register_rtp(void)
 {
@@ -585,7 +701,7 @@ proto_register_rtp(void)
 				FT_UINT8,
 				BASE_DEC,
 				VALS(rtp_version_vals),
-				0x0,
+				0xC0,
 				"", HFILL
 			}
 		},
@@ -595,9 +711,9 @@ proto_register_rtp(void)
 				"Padding",
 				"rtp.padding",
 				FT_BOOLEAN,
-				BASE_NONE,
+				8,
 				NULL,
-				0x0,
+				0x20,
 				"", HFILL
 			}
 		},
@@ -607,9 +723,9 @@ proto_register_rtp(void)
 				"Extension",
 				"rtp.ext",
 				FT_BOOLEAN,
-				BASE_NONE,
+				8,
 				NULL,
-				0x0,
+				0x10,
 				"", HFILL
 			}
 		},
@@ -621,7 +737,7 @@ proto_register_rtp(void)
 				FT_UINT8,
 				BASE_DEC,
 				NULL,
-				0x0,
+				0x0F,
 				"", HFILL
 			}
 		},
@@ -631,9 +747,9 @@ proto_register_rtp(void)
 				"Marker",
 				"rtp.marker",
 				FT_BOOLEAN,
-				BASE_NONE,
+				8,
 				NULL,
-				0x0,
+				0x80,
 				"", HFILL
 			}
 		},
@@ -645,7 +761,7 @@ proto_register_rtp(void)
 				FT_UINT8,
 				BASE_DEC,
 				VALS(rtp_payload_type_vals),
-				0x0,
+				0x7F,
 				"", HFILL
 			}
 		},
@@ -769,15 +885,55 @@ proto_register_rtp(void)
 				"", HFILL
 			}
 		},
-};
+		{
+			&hf_rtp_setup,
+			{
+				"Stream setup",
+				"rtp.setup",
+				FT_STRING,
+				BASE_NONE,
+				NULL,
+				0x0,
+				"Stream setup, method and frame number", HFILL
+			}
+		},
+		{
+			&hf_rtp_setup_frame,
+			{
+				"Setup frame",
+				"rtp.setup-frame",
+				FT_FRAMENUM,
+				BASE_NONE,
+				NULL,
+				0x0,
+				"Frame that set up this stream", HFILL
+			}
+		},
+		{
+			&hf_rtp_setup_method,
+			{
+				"Setup Method",
+				"rtp.setup-method",
+				FT_STRING,
+				BASE_NONE,
+				NULL,
+				0x0,
+				"Method used to set up this stream", HFILL
+			}
+		}
+
+	};
 
 	static gint *ett[] =
 	{
 		&ett_rtp,
 		&ett_csrc_list,
 		&ett_hdr_ext,
+		&ett_rtp_setup
 	};
 
+	module_t *rtp_module;
+
 
 	proto_rtp = proto_register_protocol("Real-Time Transport Protocol",
 	    "RTP", "rtp");
@@ -787,21 +943,29 @@ proto_register_rtp(void)
 	register_dissector("rtp", dissect_rtp, proto_rtp);
 	rtp_tap = register_tap("rtp");
 
-#if 0
+	rtp_pt_dissector_table = register_dissector_table("rtp.pt",
+	    "RTP payload type", FT_UINT8, BASE_DEC);
+
+	rtp_module = prefs_register_protocol(proto_rtp, NULL);
+
+	prefs_register_bool_preference(rtp_module, "show_setup_info",
+		"Show stream setup information",
+		"Where available, show which protocol and frame caused "
+		"this RTP stream to be created",
+		&global_rtp_show_setup_info);
+
+	prefs_register_bool_preference(rtp_module, "heuristic_rtp",
+		"Try to decode RTP outside of conversations ",
+                "If call control SIP/H323/RTSP/.. messages are missing in the trace, "
+                "RTP isn't decoded without this",
+		&global_rtp_heur);
+   
 	register_init_routine( &rtp_init );
-#endif
 }
 
 void
 proto_reg_handoff_rtp(void)
 {
-	dissector_handle_t rtp_handle;
-
-	/*
-	 * Get handles for the H.261 and MPEG-1 dissectors.
-	 */
-	h261_handle = find_dissector("h261");
-	mpeg1_handle = find_dissector("mpeg1");
 	data_handle = find_dissector("data");
 
 	/*
@@ -810,4 +974,6 @@ proto_reg_handoff_rtp(void)
 	 */
 	rtp_handle = find_dissector("rtp");
 	dissector_add_handle("udp.port", rtp_handle);
+
+	heur_dissector_add( "udp", dissect_rtp_heur, proto_rtp);
 }
