@@ -2,7 +2,7 @@
  * Routines for socks versions 4 &5  packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-socks.c,v 1.44 2003/02/26 01:35:07 gerald Exp $
+ * $Id: packet-socks.c,v 1.59 2004/06/03 08:04:07 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -27,10 +27,24 @@
  * The Version 5 decoder is based upon rfc-1928
  * The Version 5 User/Password authentication is based on rfc-1929.
  *
- * See http://www.socks.nec.com/socksprot.html for these and other documents
+ * See
+ *
+ *	http://www.socks.permeo.com/TechnicalResources/ProtocolDocuments.asp
+ *
+ * for these and other documents.  See
+ *
+ *	http://www.socks.nec.com/protocol/socks4a.protocol
+ *
+ * for information on SOCKS version 4a.
  *
  * Revisions:
  *
+ * 2003-09-18 JCFoster Fixed problem with socks tunnel in socks tunnel
+ *			causing heap overflow because of an infinite loop
+ *			where the socks dissect was call over and over.
+ *
+ *			Also remove some old code marked with __JUNK__
+ *
  * 2001-01-08 JCFoster Fixed problem with NULL pointer for hash data.
  *			Now test and exit if hash_info is null.
  */
@@ -40,9 +54,9 @@
  * Add GSS-API authentication per rfc-1961
  * Add CHAP authentication
  * Decode FLAG bits per
- * 	 http://www.socks.nec.com/draft/draft-ietf-aft-socks-pro-v-04.txt
- * In call_next_dissector, could load the destination address into the
- * 	pi structure before calling next dissector.
+ * 	http://archive.socks.permeo.com/draft/draft-ietf-aft-socks-pro-v5-04.txt
+ * In call_next_dissector, could load the destination address into
+ * 	pinfo->src or pinfo->dst structure before calling next dissector.
  * remove display_string or at least make it use protocol identifiers
  * socks_hash_entry_t needs to handle V5 address type and domain names
 */
@@ -61,7 +75,6 @@
 
 #include <epan/packet.h>
 #include <epan/resolv.h>
-#include "alignment.h"
 #include <epan/conversation.h>
 
 #include "packet-tcp.h"
@@ -93,6 +106,9 @@
 #define CHAP_AUTHENTICATION 	3
 #define AUTHENTICATION_FAILED 	0xff
 
+/* 2003-09-18 JCFoster Fixed problem with socks tunnel in socks tunnel */
+
+static int in_socks_dissector_flag = 0;		/* set to 1 to avoid recursive overflow */
 
 /*********** Header field identifiers *************/
 
@@ -106,6 +122,7 @@ static int hf_socks_ver = -1;
 static int hf_socks_ip_dst = -1;
 static int hf_socks_ip6_dst = -1;
 static int hf_user_name = -1;
+static int hf_v4a_dns_name = -1;
 static int hf_socks_dstport = -1;
 static int hf_socks_cmd = -1;
 static int hf_socks_results = -1;
@@ -129,6 +146,7 @@ enum SockState {
 	V5Reply,
 	V5BindReply,
 	UserNameAuth,
+	UserNameAuthReply,
 	GssApiAuth,
 	AuthReply,
 	Done
@@ -138,9 +156,9 @@ enum SockState {
 
 typedef struct {
 	int		state;
-	int	 	version;
-	int	 	command;
-	int	 	grant;
+	int		version;
+	int		command;
+	int		grant;
 	guint32 	port;
 	guint32 	udp_port;
 	guint32 	udp_remote_port;
@@ -154,6 +172,7 @@ typedef struct {
 	row_pointer_type	command_row;
 	row_pointer_type 	auth_method_row;
 	row_pointer_type	user_name_auth_row;
+	row_pointer_type	auth_version;
 	guint32 start_done_row;
 
 	guint32	dst_addr;	/* this needs to handle IPv6 */
@@ -174,16 +193,6 @@ static char *address_type_table[] = {
 
 /* String table for the V4 reply status messages */
 
-#ifdef __JUNK__
-static char *reply_table_v4[] = {
-	"Granted",
-	"Rejected or Failed",
-	"Rejected because SOCKS server cannot connect to identd on the client",
-	"Rejected because the client program and identd report different user-ids",
-	"Unknown"
-};
-#endif
-
 static const value_string reply_table_v4[] = {
 	{90, "Granted"},
 	{91, "Rejected or Failed"},
@@ -194,21 +203,6 @@ static const value_string reply_table_v4[] = {
 
 /* String table for the V5 reply status messages */
 
-#ifdef __JUNK__
-static char *reply_table_v5[] = {
-	"Succeeded",
-	"General SOCKS server failure",
-	"Connection not allowed by ruleset",
-	"Network unreachable",
-	"Host unreachable",
-	"Connection refused",
-	"TTL expired",
-	"Command not supported",
-	"Address type not supported",
-	"Unknown"
-};
-#endif
-
 static const value_string reply_table_v5[] = {
 	{0, "Succeeded"},
 	{1, "General SOCKS server failure"},
@@ -218,7 +212,8 @@ static const value_string reply_table_v5[] = {
 	{5, "Connection refused"},
 	{6, "TTL expired"},
 	{7, "Command not supported"},
-	{8, "Address type not supported"}
+	{8, "Address type not supported"},
+	{0, NULL},
 };
 
 static const value_string cmd_strings[] = {
@@ -258,7 +253,7 @@ static int display_string(tvbuff_t *tvb, int offset,
 	temp[ length ] = 0;
 
    	ti = proto_tree_add_text(tree, tvb, offset, length + 1,
-   	 	"%s: %s" , label, temp);
+   		"%s: %s" , label, temp);
 
 
 	name_tree = proto_item_add_subtree(ti, ett_socks_name);
@@ -403,7 +398,7 @@ socks_udp_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
        		proto_tree_add_text( socks_tree, tvb, offset, 2, "Reserved");
 		offset += 2;
 
-       		proto_tree_add_text( socks_tree, tvb, offset, 1, "Fragment Number: %d", tvb_get_guint8(tvb, offset));
+       		proto_tree_add_text( socks_tree, tvb, offset, 1, "Fragment Number: %u", tvb_get_guint8(tvb, offset));
 		++offset;
 
 
@@ -430,7 +425,7 @@ socks_udp_dissector(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
         *ptr = hash_info->udp_remote_port;
 
-	decode_udp_ports( tvb, offset, pinfo, tree, pinfo->srcport, pinfo->destport);
+	decode_udp_ports( tvb, offset, pinfo, tree, pinfo->srcport, pinfo->destport, -1);
 
         *ptr = hash_info->udp_port;
 
@@ -467,12 +462,14 @@ display_socks_v4(tvbuff_t *tvb, int offset, packet_info *pinfo,
 
 
 	guint command;
+	unsigned char ipaddr[4];
+	guint username_len, domainname_len;
 
 					/* Display command from client */
 	if (compare_packet( hash_info->connect_row)){
 
 		proto_tree_add_text( tree, tvb, offset, 1,
-				"Version: %u ", hash_info->version);
+				"Version: %u", hash_info->version);
 		++offset;
 		command = tvb_get_guint8(tvb, offset);
 
@@ -487,6 +484,7 @@ display_socks_v4(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		offset += 2;
 
 						/* Do destination address */
+		tvb_memcpy(tvb, ipaddr, offset, 4);
 		proto_tree_add_item( tree, hf_socks_ip_dst, tvb, offset,
 				4, FALSE);
 
@@ -496,9 +494,18 @@ display_socks_v4(tvbuff_t *tvb, int offset, packet_info *pinfo,
 /* Should perhaps do TCP reassembly as well */
 		if ( tvb_offset_exists(tvb, offset)) {
 						/* display user name 	*/
-			proto_tree_add_string( tree, hf_user_name, tvb, offset,
-				tvb_strsize(tvb, offset),
-				tvb_get_ptr(tvb, offset, -1));
+			username_len = tvb_strsize(tvb, offset);
+			proto_tree_add_item( tree, hf_user_name, tvb, offset,
+				username_len, FALSE);
+			offset += username_len;
+			if ( ipaddr[0] == 0 && ipaddr[1] == 0 &&
+			     ipaddr[2] == 0 && ipaddr[3] != 0) {
+			     	/* 0.0.0.x , where x!=0 means v4a support */
+			     	domainname_len = tvb_strsize(tvb, offset);
+				proto_tree_add_item( tree, hf_v4a_dns_name,
+					tvb, offset, domainname_len,
+					FALSE);
+			}
 		}
 
 	}
@@ -550,43 +557,41 @@ display_socks_v5(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	unsigned int i, command;
 	guint temp;
 	char *AuthMethodStr;
+	guint8 auth_status;
 
+	proto_tree_add_item( tree, hf_socks_ver, tvb, offset, 1, FALSE);
+	++offset;
 
 	if (compare_packet( hash_info->connect_row)){
 
 		proto_tree      *AuthTree;
 		proto_item      *ti;
 
-						/* Do version 	*/
-		proto_tree_add_item( tree, hf_socks_ver, tvb, offset, 1,
-				hash_info->version);
-		++offset;
-
 		temp = tvb_get_guint8(tvb, offset);	/* Get Auth method count */
 							/* build auth tree */
-		ti = proto_tree_add_text( tree, tvb, offset, 1,
+		ti = proto_tree_add_text( tree, tvb, offset, -1,
 				"Client Authentication Methods");
 
 		AuthTree = proto_item_add_subtree(ti, ett_socks_auth);
 
 		proto_tree_add_text( AuthTree, tvb, offset, 1,
-				"Count: %u ", temp);
+				"Count: %u", temp);
 		++offset;
 
 		for( i = 0; i  < temp; ++i) {
 
 			AuthMethodStr = get_auth_method_name(
-				tvb_get_guint8( tvb, offset + i));
-			proto_tree_add_text( AuthTree, tvb, offset + i, 1,
-				"Method[%d]: %u (%s)", i,
-				tvb_get_guint8( tvb, offset + i), AuthMethodStr);
+				tvb_get_guint8( tvb, offset));
+			proto_tree_add_text( AuthTree, tvb, offset, 1,
+				"Method[%u]: %u (%s)", i,
+				tvb_get_guint8( tvb, offset), AuthMethodStr);
+			++offset;
 		}
+		proto_item_set_end( ti, tvb, offset);
 		return;
 	}					/* Get accepted auth method */
 	else if (compare_packet( hash_info->auth_method_row)) {
 
-		++offset;
-
 		proto_tree_add_text( tree, tvb, offset, 1,
 			"Accepted Auth Method: 0x%0x (%s)", tvb_get_guint8( tvb, offset),
 				get_auth_method_name( tvb_get_guint8( tvb, offset)));
@@ -595,8 +600,6 @@ display_socks_v5(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	}					/* handle user/password auth */
 	else if (compare_packet( hash_info->user_name_auth_row)) {
 
-		proto_tree_add_item( tree, hf_socks_ver, tvb, offset, 1, FALSE);
-		++offset;
 						/* process user name	*/
 		offset += display_string( tvb, offset, tree,
 				"User name");
@@ -606,14 +609,18 @@ display_socks_v5(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	}
 					/* command to the server */
 					/* command response from server */
+	else if (compare_packet( hash_info->auth_version)) {
+		auth_status = tvb_get_guint8(tvb, offset);
+		if(auth_status != 0)
+			proto_tree_add_text( tree, tvb, offset, 1, "Status: %u (failure)", auth_status);
+		else
+			proto_tree_add_text( tree, tvb, offset, 1, "Status: success");
+		offset ++;
+	}
 	else if ((compare_packet( hash_info->command_row)) ||
 	         (compare_packet( hash_info->cmd_reply_row)) ||
 	         (compare_packet( hash_info->bind_reply_row))){
 
-		proto_tree_add_item( tree, hf_socks_ver, tvb, offset, 1, FALSE);
-
-		++offset;
-
 		command = tvb_get_guint8(tvb, offset);
 
 		if (compare_packet( hash_info->command_row))
@@ -635,7 +642,7 @@ display_socks_v5(tvbuff_t *tvb, int offset, packet_info *pinfo,
 /*XXX Add remote port for search somehow */
 						/* Do remote port	*/
 		proto_tree_add_text( tree, tvb, offset, 2,
-				"%sPort: %d",
+				"%sPort: %u",
 				(compare_packet( hash_info->bind_reply_row) ?
 					"Remote Host " : ""),
 				 tvb_get_ntohs(tvb, offset));
@@ -659,7 +666,7 @@ state_machine_v4( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 	if ( hash_info->state == None) {		/* new connection */
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " Connect to server request");
+			col_append_str(pinfo->cinfo, COL_INFO, " Connect to server request");
 
 		hash_info->state = Connecting;	/* change state		*/
 
@@ -709,7 +716,7 @@ state_machine_v4( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 	}else if ( hash_info->state == V4UserNameWait){
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " Connect Request (User name)");
+			col_append_str(pinfo->cinfo, COL_INFO, " Connect Request (User name)");
 
 		hash_info->v4_user_name_row = get_packet_ptr;
 /*XXX may need to check for domain name here */
@@ -725,7 +732,7 @@ state_machine_v4( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 	else if ( hash_info->state == Connecting){
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " Connect Response");
+			col_append_str(pinfo->cinfo, COL_INFO, " Connect Response");
 
 						/* save packet pointer 	*/
 		hash_info->cmd_reply_row = get_packet_ptr;
@@ -767,7 +774,7 @@ state_machine_v5( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 		guint AuthMethod = tvb_get_guint8(tvb, offset + 1);
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " Connect to server response");
+			col_append_str(pinfo->cinfo, COL_INFO, " Connect to server response");
 
 		hash_info->auth_method_row = get_packet_ptr;
 
@@ -792,8 +799,8 @@ state_machine_v5( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 		hash_info->command = tvb_get_guint8(tvb, offset + 1); /* get command */
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_fstr(pinfo->cinfo, COL_INFO, " Command Request - %s",
-	 			get_command_name(hash_info->command));
+			col_append_fstr(pinfo->cinfo, COL_INFO, " Command Request - %s",
+				get_command_name(hash_info->command));
 
 		hash_info->state = V5Reply;
 		hash_info->command_row = get_packet_ptr;
@@ -814,8 +821,8 @@ state_machine_v5( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_fstr(pinfo->cinfo, COL_INFO, " Command Response - %s",
-	 			get_command_name(hash_info->command));
+			col_append_fstr(pinfo->cinfo, COL_INFO, " Command Response - %s",
+				get_command_name(hash_info->command));
 
 		hash_info->cmd_reply_row = get_packet_ptr;
 
@@ -844,24 +851,25 @@ state_machine_v5( socks_hash_entry_t *hash_info, tvbuff_t *tvb,
 	else if ( hash_info->state == V5BindReply) {	/* V5 Bind Second Reply */
 
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " Command Response: Bind remote host info");
+			col_append_str(pinfo->cinfo, COL_INFO, " Command Response: Bind remote host info");
 
 		hash_info->bind_reply_row = get_packet_ptr;
 		hash_info->state = Done;
 	}
 	else if ( hash_info->state == UserNameAuth) {	/* Handle V5 User Auth*/
+		hash_info->auth_version = get_packet_ptr;
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO,
-	 			" User authentication response");
+			col_append_str(pinfo->cinfo, COL_INFO,
+				" User authentication request");
 
 		hash_info->user_name_auth_row = get_packet_ptr;
 		hash_info->state = AuthReply;
 
 	}
 	else if ( hash_info->state == AuthReply){	/* V5 User Auth reply */
-		hash_info->cmd_reply_row = get_packet_ptr;
+		hash_info->auth_version = get_packet_ptr;
 		if (check_col(pinfo->cinfo, COL_INFO))
-	 		col_append_str(pinfo->cinfo, COL_INFO, " User authentication reply");
+			col_append_str(pinfo->cinfo, COL_INFO, " User authentication reply");
 		hash_info->state = V5Command;
 	}
 }
@@ -886,8 +894,8 @@ display_ping_and_tracert(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 		if ( tree)
   			proto_tree_add_text(tree, tvb, offset, 1,
    				(hash_info->command  == PING_COMMAND) ?
-	 			"Ping: End command" :
-	   	 		"Traceroute: End command");
+				"Ping: End command" :
+	   			"Traceroute: End command");
 	}
        	else{ 		/* display the PING or Traceroute results */
 		if (check_col(pinfo->cinfo, COL_INFO))
@@ -895,9 +903,9 @@ display_ping_and_tracert(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 
 		if ( tree){
 			proto_tree_add_text(tree, tvb, offset, -1,
-   		 		(hash_info->command  == PING_COMMAND) ?
-   		 		"Ping Results:" :
-   	 			"Traceroute Results");
+   				(hash_info->command  == PING_COMMAND) ?
+   				"Ping Results:" :
+   				"Traceroute Results");
 
   	      		data = tvb_get_ptr(tvb, offset, -1);
         		dataend = data + tvb_length_remaining(tvb, offset);
@@ -918,8 +926,15 @@ display_ping_and_tracert(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tr
 
 
 
+static void clear_in_socks_dissector_flag(void *dummy _U_)
+{
+	in_socks_dissector_flag = 0; /* avoid recursive overflow */
+}
+
 static void call_next_dissector(tvbuff_t *tvb, int offset, packet_info *pinfo,
-	proto_tree *tree, socks_hash_entry_t *hash_info) {
+	proto_tree *tree, proto_tree *socks_tree,
+	socks_hash_entry_t *hash_info)
+{
 
 /* Display the results for PING and TRACERT extensions or		*/
 /* Call TCP dissector for the port that was passed during the	    	*/
@@ -929,6 +944,8 @@ static void call_next_dissector(tvbuff_t *tvb, int offset, packet_info *pinfo,
 /* the payload, and restore the pinfo port after that is done.		*/
 
 	guint32 *ptr;
+	struct tcpinfo *tcpinfo = pinfo->private_data;
+	guint16 save_can_desegment;
 
  	if (( hash_info->command  == PING_COMMAND) ||
  	    ( hash_info->command  == TRACERT_COMMAND))
@@ -945,7 +962,21 @@ static void call_next_dissector(tvbuff_t *tvb, int offset, packet_info *pinfo,
         		ptr = &pinfo->srcport;
 
 	        *ptr = hash_info->port;
-		decode_tcp_ports( tvb, offset, pinfo, tree, pinfo->srcport, pinfo->destport);
+
+/* 2003-09-18 JCFoster Fixed problem with socks tunnel in socks tunnel */
+
+		in_socks_dissector_flag = 1; /* avoid recursive overflow */
+		CLEANUP_PUSH(clear_in_socks_dissector_flag, NULL);
+
+		save_can_desegment = pinfo->can_desegment;
+		pinfo->can_desegment = pinfo->saved_can_desegment;
+		dissect_tcp_payload(tvb, pinfo, offset, tcpinfo->seq,
+		    tcpinfo->nxtseq, pinfo->srcport, pinfo->destport,
+		    tree, socks_tree);
+		pinfo->can_desegment = save_can_desegment;
+
+		CLEANUP_CALL_AND_POP;
+
 	        *ptr = TCP_PORT_SOCKS;
 	}
 }
@@ -956,11 +987,19 @@ static void
 dissect_socks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
 	int 		offset = 0;
-	proto_tree      *socks_tree;
+	proto_tree      *socks_tree = NULL;
 	proto_item      *ti;
 	socks_hash_entry_t *hash_info;
 	conversation_t *conversation;
 
+/* 2003-09-18 JCFoster Fixed problem with socks tunnel in socks tunnel */
+
+	/* avoid recursive overflow */
+
+	if ( in_socks_dissector_flag) {
+		return;
+	}
+
 	conversation = find_conversation( &pinfo->src, &pinfo->dst, pinfo->ptype,
 		pinfo->srcport, pinfo->destport, 0);
 
@@ -1008,7 +1047,7 @@ dissect_socks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 
 /*XXX		if ( hash_info->port != -1) */
 		if ( hash_info->port != 0)
-			col_append_fstr(pinfo->cinfo, COL_INFO, ", Remote Port: %d",
+			col_append_fstr(pinfo->cinfo, COL_INFO, ", Remote Port: %u",
 				hash_info->port);
 	}
 
@@ -1069,7 +1108,8 @@ dissect_socks(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree) {
 /* call next dissector if ready */
 
 	if ( pinfo->fd->num > hash_info->start_done_row){
-		call_next_dissector(tvb, offset, pinfo, tree, hash_info);
+		call_next_dissector(tvb, offset, pinfo, tree, socks_tree,
+		    hash_info);
 	}
 }
 
@@ -1109,25 +1149,30 @@ proto_register_socks( void){
 
 		{ &hf_socks_ver,
 			{ "Version", "socks.version", FT_UINT8, BASE_DEC, NULL,
-			 	0x0, "", HFILL
+				0x0, "", HFILL
 			}
 		},
 		{ &hf_socks_ip_dst,
 			{ "Remote Address", "socks.dst", FT_IPv4, BASE_NONE, NULL,
-			 	0x0, "", HFILL
+				0x0, "", HFILL
 			}
 		},
 		{ &hf_socks_ip6_dst,
 			{ "Remote Address(ipv6)", "socks.dstV6", FT_IPv6, BASE_NONE, NULL,
-			 	0x0, "", HFILL
+				0x0, "", HFILL
 			}
 		},
 
                 { &hf_user_name,
-                	{ "User Name", "socks.username", FT_STRING, BASE_NONE,
+                	{ "User Name", "socks.username", FT_STRINGZ, BASE_NONE,
                 		 NULL, 0x0, "", HFILL
                 	}
                 },
+		{ &hf_v4a_dns_name,
+			{ "SOCKS v4a Remote Domain Name", "socks.v4a_dns_name", FT_STRINGZ, BASE_NONE,
+				NULL, 0x0, "", HFILL
+			}
+		},
 		{ &hf_socks_dstport,
 			{ "Remote Port", "socks.dstport", FT_UINT16,
 				BASE_DEC, NULL, 0x0, "", HFILL
