@@ -5,7 +5,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@ns.aus.com>
  * Copyright 2003, Richard Sharpe <rsharpe@richardsharpe.com>
  *
- * $Id: packet-spnego.c,v 1.51 2003/07/17 22:17:01 sharpe Exp $
+ * $Id: packet-spnego.c,v 1.56 2004/06/04 01:56:25 sahlberg Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -145,6 +145,14 @@ static const true_false_string tfs_reqflags_integ = {
 
 static dissector_handle_t data_handle;
 
+static dissector_handle_t
+gssapi_dissector_handle(gssapi_oid_value *next_level_value) {
+	if (next_level_value == NULL) {
+		return NULL;
+	}
+	return next_level_value->handle;
+}
+
 static void
 dissect_parse_error(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		    proto_tree *tree, const char *field_name, int ret)
@@ -340,7 +348,7 @@ dissect_spnego_krb5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		 * return.
 		 */
 		krb5_tvb = tvb_new_subset(tvb, offset, -1, -1); 
-		offset = dissect_kerberos_main(krb5_tvb, pinfo, subtree, FALSE);
+		offset = dissect_kerberos_main(krb5_tvb, pinfo, subtree, FALSE, NULL);
 		return;
 
 	    default:
@@ -367,7 +375,7 @@ dissect_spnego_krb5(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	case KRB_TOKEN_AP_REP:
 	case KRB_TOKEN_AP_ERR:
 	  krb5_tvb = tvb_new_subset(tvb, offset, -1, -1); 
-	  offset = dissect_kerberos_main(krb5_tvb, pinfo, subtree, FALSE);
+	  offset = dissect_kerberos_main(krb5_tvb, pinfo, subtree, FALSE, NULL);
 	  break;
 
 	case KRB_TOKEN_GETMIC:
@@ -728,6 +736,7 @@ dissect_spnego_mechToken(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	gboolean def;
 	int ret;
 	guint cls, con, tag, nbytes;
+	gint length_remaining, reported_length_remaining;
 	tvbuff_t *token_tvb;
 
 	/*
@@ -752,24 +761,37 @@ dissect_spnego_mechToken(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 
 	offset = hnd->offset;
 
+
+	/* Dont try to create an item with more bytes than remains in the
+	 * frame or we will not even attempt to dissect those bytes we
+	 * do have. (since there will be an exception)
+	 *
+	 * We use "tvb_ensure_length_remaining()" so that we throw
+	 * an exception if there's nothing to dissect.
+	 */
+	length_remaining = tvb_ensure_length_remaining(tvb,offset);
+	reported_length_remaining = tvb_reported_length_remaining(tvb,offset);
+	if ((guint)length_remaining > nbytes)
+		length_remaining = nbytes;
+	if ((guint)reported_length_remaining > nbytes)
+		reported_length_remaining = nbytes;
 	item = proto_tree_add_item(tree, hf_spnego_mechtoken, tvb, offset, 
-				   nbytes, FALSE);
+				   length_remaining, FALSE);
 	subtree = proto_item_add_subtree(item, ett_spnego_mechtoken);
 
 	/*
 	 * Now, we should be able to dispatch after creating a new TVB.
 	 */
 
-	token_tvb = tvb_new_subset(tvb, offset, nbytes, -1);
+	token_tvb = tvb_new_subset(tvb, offset, length_remaining,
+	    reported_length_remaining);
 	if (next_level_dissector)
 	  call_dissector(next_level_dissector, token_tvb, pinfo, subtree);
 
 	hnd->offset += nbytes; /* Update this ... */
 
- done:
-
-  return offset + nbytes;
-
+done:
+	return offset + nbytes;
 }
 
 static int
@@ -936,13 +958,13 @@ dissect_spnego_negTokenInit(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	  case SPNEGO_mechToken:
 
 	    offset = dissect_spnego_mechToken(tvb, offset, pinfo, subtree, 
-					      hnd, (*next_level_value_p)->handle);
+					      hnd, gssapi_dissector_handle(*next_level_value_p));
 	    break;
 
 	  case SPNEGO_mechListMIC:
 
 	    offset = dissect_spnego_mechListMIC(tvb, offset, pinfo, subtree,
-						hnd, (*next_level_value_p)->handle);
+						hnd, gssapi_dissector_handle(*next_level_value_p));
 	    break;
 
 	  default:
@@ -1223,19 +1245,13 @@ dissect_spnego_negTokenTarg(tvbuff_t *tvb, int offset, packet_info *pinfo _U_,
 	  case SPNEGO_responseToken:
 
 	    offset = dissect_spnego_responseToken(tvb, offset, pinfo, subtree,
-						  hnd,
-						  (*next_level_value_p != NULL) ?
-						      (*next_level_value_p)->handle :
-						      NULL);
+						  hnd, gssapi_dissector_handle(*next_level_value_p));
 	    break;
 
 	  case SPNEGO_mechListMIC:
 
 	    offset = dissect_spnego_mechListMIC(tvb, offset, pinfo, subtree, 
-						hnd,
-						(*next_level_value_p != NULL) ?
-						    (*next_level_value_p)->handle :
-						    NULL);
+						hnd, gssapi_dissector_handle(*next_level_value_p));
 	    break;
 
 	  default:
