@@ -1,8 +1,8 @@
 /* packet-bpkmattr.c
  * Routines for Baseline Privacy Key Management Attributes dissection
- * Copyright 2002, Anand V. Narwani <anarwani@cisco.com>
+ * Copyright 2002, Anand V. Narwani <anand[AT]narwani.org>
  *
- * $Id: packet-bpkmattr.c,v 1.4 2002/09/10 19:07:37 guy Exp $
+ * $Id: packet-bpkmattr.c,v 1.7 2003/12/13 03:18:37 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -40,7 +40,7 @@
 #include <epan/packet.h>
 
 /* BPKM Attributes defined in:
- * http://www.cablemodem.com/Specs/SP-BPI+-I08-020301.pdf
+ * http://www.cablemodem.com/downloads/specs/SP-BPI+_I10-030730.pdf
  */
 #define BPKM_RESERVED 0
 #define BPKM_SERIAL_NUM 1
@@ -131,18 +131,6 @@ static const value_string error_code_vals[] = {
   {0, NULL},
 };
 
-static const value_string data_encr_algo_vals[] = {
-  {0, "Reserved"},
-  {1, "CBC Mode, 56 Bit DES"},
-  {2, "CBC Mode, 40 Bit DES"},
-  {0, NULL},
-};
-
-static const value_string data_auth_algo_vals[] = {
-  {0, "No Data Authentication"},
-  {0, NULL},
-};
-
 static const value_string crypto_suite_attr_vals[] = {
   {0x0100, "CBC Mode, 56 Bit DES & no Data Authentication"},
   {0x0200, "CBC Mode, 40 Bit DES & no Data Authentication"},
@@ -165,8 +153,8 @@ dissect_attrs (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 {
   guint8 type;
   guint16 length;
-  guint16 pos = 0;
-  guint16 total_len;
+  int pos = 0;
+  gint total_len;
   proto_item *cmid_it, *tekp_it, *scap_it;
   proto_item *saqry_it, *dnld_it, *sadsc_it;
   proto_tree *cmid_tree, *tekp_tree, *scap_tree;
@@ -174,7 +162,7 @@ dissect_attrs (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
   tvbuff_t *cmid_tvb, *tekp_tvb, *scap_tvb;
   tvbuff_t *saqry_tvb, *dnld_tvb, *sadsc_tvb;
 
-  total_len = tvb_length_remaining (tvb, 0);
+  total_len = tvb_reported_length_remaining (tvb, 0);
   while (pos < total_len)
     {
       type = tvb_get_guint8 (tvb, pos++);
@@ -388,8 +376,7 @@ dissect_bpkmattr (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
   if (tree)
     {
       it =
-	proto_tree_add_protocol_format (tree, proto_docsis_bpkmattr, tvb, 0,
-					tvb_length_remaining (tvb, 0),
+	proto_tree_add_protocol_format (tree, proto_docsis_bpkmattr, tvb, 0, -1,
 					"BPKM Attributes");
       bpkmattr_tree = proto_item_add_subtree (it, ett_docsis_bpkmattr);
       dissect_attrs (tvb, pinfo, bpkmattr_tree);
@@ -514,7 +501,7 @@ proto_register_docsis_bpkmattr (void)
      },
     {&hf_docsis_bpkmattr_crypto_suite,
      {"20 Cryptographic Suite", "docsis.bpkmattr.cryptosuite",
-      FT_UINT16, BASE_HEX, NULL, 0x0,
+      FT_UINT16, BASE_HEX, VALS(crypto_suite_attr_vals), 0x0,
       "Cryptographic Suite", HFILL}
      },
     {&hf_docsis_bpkmattr_crypto_suite_list,
