@@ -1,7 +1,7 @@
 /* prefs.c
  * Routines for handling preferences
  *
- * $Id: prefs.c,v 1.127 2004/02/28 04:39:12 guy Exp $
+ * $Id: prefs.c,v 1.139 2004/06/29 17:10:52 ulfl Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -87,6 +87,9 @@ static gchar	*gui_ptree_expander_style_text[] =
 static gchar	*gui_hex_dump_highlight_style_text[] =
 	{ "BOLD", "INVERSE", NULL };
 
+static gchar	*gui_console_open_text[] =
+	{ "NEVER", "AUTOMATIC", "ALWAYS", NULL };
+
 static gchar	*gui_fileopen_style_text[] =
 	{ "LAST_OPENED", "SPECIFIED", NULL };
 
@@ -95,6 +98,9 @@ static gchar	*gui_fileopen_style_text[] =
 static gchar	*gui_toolbar_style_text[] =
 	{ "ICONS", "TEXT", "BOTH", NULL };
 
+static gchar	*gui_layout_content_text[] =
+	{ "NONE", "PLIST", "PDETAILS", "PBYTES", NULL };
+
 /*
  * List of all modules with preference settings.
  */
@@ -798,21 +804,35 @@ clear_string_list(GList *sl)
  * Takes a string, a pointer to an array of "enum_val_t"s, and a default gint
  * value.
  * The array must be terminated by an entry with a null "name" string.
- * If the string matches a "name" strings in an entry, the value from that
- * entry is returned. Otherwise, the default value that was passed as the
- * third argument is returned.
+ *
+ * If the string matches a "name" string in an entry, the value from that
+ * entry is returned.
+ *
+ * Otherwise, if a string matches a "desctiption" string in an entry, the
+ * value from that entry is returned; we do that for backwards compatibility,
+ * as we used to have only a "name" string that was used both for command-line
+ * and configuration-file values and in the GUI (which meant either that
+ * the GUI had what might be somewhat cryptic values to select from or that
+ * the "-o" flag took long strings, often with spaces in them).
+ *
+ * Otherwise, the default value that was passed as the third argument is
+ * returned.
  */
 gint
 find_val_for_string(const char *needle, const enum_val_t *haystack,
     gint default_value)
 {
-	int i = 0;
+	int i;
 
-	while (haystack[i].name != NULL) {
+	for (i = 0; haystack[i].name != NULL; i++) {
 		if (strcasecmp(needle, haystack[i].name) == 0) {
 			return haystack[i].value;
 		}
-		i++;
+	}
+	for (i = 0; haystack[i].name != NULL; i++) {
+		if (strcasecmp(needle, haystack[i].description) == 0) {
+			return haystack[i].value;
+		}
 	}
 	return default_value;
 }
@@ -921,6 +941,7 @@ read_prefs(int *gpf_errno_return, int *gpf_read_errno_return,
     prefs.gui_ptree_line_style = 0;
     prefs.gui_ptree_expander_style = 1;
     prefs.gui_hex_dump_highlight_style = 1;
+    prefs.filter_toolbar_show_in_statusbar = FALSE;
     prefs.gui_toolbar_main_style = TB_STYLE_ICONS;
 #ifdef WIN32
     prefs.gui_font_name1 = g_strdup("-*-lucida console-medium-r-*-*-*-100-*-*-*-*-*-*");
@@ -986,9 +1007,16 @@ read_prefs(int *gpf_errno_return, int *gpf_read_errno_return,
     prefs.gui_geometry_save_position =         0;
     prefs.gui_geometry_save_size     =         1;
     prefs.gui_geometry_save_maximized=         1;
+    prefs.gui_console_open           = console_open_never;
     prefs.gui_fileopen_style         = FO_STYLE_LAST_OPENED;
     prefs.gui_recent_files_count_max = 10;
     prefs.gui_fileopen_dir           = g_strdup("");
+    prefs.gui_ask_unsaved            = TRUE;
+    prefs.gui_webbrowser             = g_strdup("mozilla %s");
+    prefs.gui_layout_type            = layout_type_5;
+    prefs.gui_layout_content_1       = layout_pane_content_plist;
+    prefs.gui_layout_content_2       = layout_pane_content_pdetails;
+    prefs.gui_layout_content_3       = layout_pane_content_pbytes;
 
 /* set the default values for the capture dialog box */
     prefs.capture_device           = NULL;
@@ -997,6 +1025,7 @@ read_prefs(int *gpf_errno_return, int *gpf_read_errno_return,
     prefs.capture_prom_mode        = TRUE;
     prefs.capture_real_time        = FALSE;
     prefs.capture_auto_scroll      = FALSE;
+    prefs.capture_show_info        = TRUE;
     prefs.name_resolve             = RESOLV_ALL ^ RESOLV_NETWORK;
     prefs.name_resolve_concurrency = 500;
   }
@@ -1084,6 +1113,7 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
   int       got_c, state = START;
   gboolean  got_val = FALSE;
   gint      var_len = 0, val_len = 0, fline = 1, pline = 1;
+  gchar     hint[] = "(saving your preferences once should remove this warning)";
 
 
   while ((got_c = getc(pf)) != EOF) {
@@ -1093,13 +1123,13 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
       continue;
     }
     if (var_len >= MAX_VAR_LEN) {
-      g_warning ("%s line %d: Variable too long", pf_path, fline);
+      g_warning ("%s line %d: Variable too long %s", pf_path, fline, hint);
       state = IN_SKIP;
       var_len = 0;
       continue;
     }
     if (val_len >= MAX_VAL_LEN) {
-      g_warning ("%s line %d: Value too long", pf_path, fline);
+      g_warning ("%s line %d: Value too long %s", pf_path, fline, hint);
       state = IN_SKIP;
       var_len = 0;
       continue;
@@ -1115,12 +1145,12 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
               switch (pref_set_pair_fct(cur_var, cur_val)) {
 
 	      case PREFS_SET_SYNTAX_ERR:
-                g_warning ("%s line %d: Syntax error", pf_path, pline);
+                g_warning ("%s line %d: Syntax error %s", pf_path, pline, hint);
                 break;
 
 	      case PREFS_SET_NO_SUCH_PREF:
-                g_warning ("%s line %d: No such preference \"%s\"", pf_path,
-				pline, cur_var);
+                g_warning ("%s line %d: No such preference \"%s\" %s", pf_path,
+				pline, cur_var, hint);
                 break;
 
 	      case PREFS_SET_OBSOLETE:
@@ -1131,7 +1161,7 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
                 break;
               }
             } else {
-              g_warning ("%s line %d: Incomplete preference", pf_path, pline);
+              g_warning ("%s line %d: Incomplete preference %s", pf_path, pline, hint);
             }
           }
           state      = IN_VAR;
@@ -1144,7 +1174,7 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
         } else if (got_c == '#') {
           state = IN_SKIP;
         } else {
-          g_warning ("%s line %d: Malformed line", pf_path, fline);
+          g_warning ("%s line %d: Malformed line %s", pf_path, fline, hint);
         }
         break;
       case IN_VAR:
@@ -1183,12 +1213,12 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
       switch (pref_set_pair_fct(cur_var, cur_val)) {
 
       case PREFS_SET_SYNTAX_ERR:
-        g_warning ("%s line %d: Syntax error", pf_path, pline);
+        g_warning ("%s line %d: Syntax error %s", pf_path, pline, hint);
         break;
 
       case PREFS_SET_NO_SUCH_PREF:
-        g_warning ("%s line %d: No such preference \"%s\"", pf_path,
-			pline, cur_var);
+        g_warning ("%s line %d: No such preference \"%s\" %s", pf_path,
+			pline, cur_var, hint);
         break;
 
       case PREFS_SET_OBSOLETE:
@@ -1198,7 +1228,7 @@ read_prefs_file(const char *pf_path, FILE *pf, pref_set_pair_cb pref_set_pair_fc
         break;
       }
     } else {
-      g_warning ("%s line %d: Incomplete preference", pf_path, pline);
+      g_warning ("%s line %d: Incomplete preference %s", pf_path, pline, hint);
     }
   }
   if (ferror(pf))
@@ -1271,6 +1301,7 @@ prefs_set_pref(char *prefarg)
 #define PRS_GUI_PLIST_SEL_BROWSE         "gui.packet_list_sel_browse"
 #define PRS_GUI_PTREE_SEL_BROWSE         "gui.protocol_tree_sel_browse"
 #define PRS_GUI_ALTERN_COLORS            "gui.tree_view_altern_colors"
+#define PRS_GUI_FILTER_TOOLBAR_IN_STATUSBAR "gui.filter_toolbar_show_in_statusbar"
 #define PRS_GUI_PTREE_LINE_STYLE         "gui.protocol_tree_line_style"
 #define PRS_GUI_PTREE_EXPANDER_STYLE     "gui.protocol_tree_expander_style"
 #define PRS_GUI_HEX_DUMP_HIGHLIGHT_STYLE "gui.hex_dump_highlight_style"
@@ -1278,10 +1309,12 @@ prefs_set_pref(char *prefarg)
 #define PRS_GUI_FONT_NAME_2              "gui.gtk2.font_name"
 #define PRS_GUI_MARKED_FG                "gui.marked_frame.fg"
 #define PRS_GUI_MARKED_BG                "gui.marked_frame.bg"
+#define PRS_GUI_CONSOLE_OPEN             "gui.console_open"
 #define PRS_GUI_FILEOPEN_STYLE           "gui.fileopen.style"
 #define PRS_GUI_RECENT_COUNT_MAX         "gui.recent_files_count.max"
 #define PRS_GUI_FILEOPEN_DIR             "gui.fileopen.dir"
 #define PRS_GUI_FILEOPEN_REMEMBERED_DIR  "gui.fileopen.remembered_dir"
+#define PRS_GUI_ASK_UNSAVED              "gui.ask_unsaved"
 #define PRS_GUI_GEOMETRY_SAVE_POSITION   "gui.geometry.save.position"
 #define PRS_GUI_GEOMETRY_SAVE_SIZE       "gui.geometry.save.size"
 #define PRS_GUI_GEOMETRY_SAVE_MAXIMIZED  "gui.geometry.save.maximized"
@@ -1291,6 +1324,11 @@ prefs_set_pref(char *prefarg)
 #define PRS_GUI_GEOMETRY_MAIN_HEIGHT     "gui.geometry.main.height"
 #define PRS_GUI_TOOLBAR_MAIN_SHOW        "gui.toolbar_main_show"
 #define PRS_GUI_TOOLBAR_MAIN_STYLE       "gui.toolbar_main_style"
+#define PRS_GUI_WEBBROWSER               "gui.webbrowser"
+#define PRS_GUI_LAYOUT_TYPE              "gui.layout_type"
+#define PRS_GUI_LAYOUT_CONTENT_1         "gui.layout_content_1"
+#define PRS_GUI_LAYOUT_CONTENT_2         "gui.layout_content_2"
+#define PRS_GUI_LAYOUT_CONTENT_3         "gui.layout_content_3"
 
 /*
  * This applies to more than just captures, so it's not "capture.name_resolve";
@@ -1310,6 +1348,7 @@ prefs_set_pref(char *prefarg)
 #define PRS_CAP_PROM_MODE     "capture.prom_mode"
 #define PRS_CAP_REAL_TIME     "capture.real_time_update"
 #define PRS_CAP_AUTO_SCROLL   "capture.auto_scroll"
+#define PRS_CAP_SHOW_INFO     "capture.show_info"
 
 #define RED_COMPONENT(x)   (guint16) (((((x) >> 16) & 0xff) * 65535 / 255))
 #define GREEN_COMPONENT(x) (guint16) (((((x) >>  8) & 0xff) * 65535 / 255))
@@ -1524,6 +1563,13 @@ set_pref(gchar *pref_name, gchar *value)
   } else if (strcmp(pref_name, PRS_GUI_HEX_DUMP_HIGHLIGHT_STYLE) == 0) {
     prefs.gui_hex_dump_highlight_style =
 	find_index_from_string_array(value, gui_hex_dump_highlight_style_text, 1);
+  } else if (strcmp(pref_name, PRS_GUI_FILTER_TOOLBAR_IN_STATUSBAR) == 0) {
+    if (strcasecmp(value, "true") == 0) {
+            prefs.filter_toolbar_show_in_statusbar = TRUE;
+    }
+    else {
+            prefs.filter_toolbar_show_in_statusbar = FALSE;
+    }
   } else if (strcmp(pref_name, PRS_GUI_TOOLBAR_MAIN_SHOW) == 0) {
     /* obsoleted by recent setting */
   } else if (strcmp(pref_name, PRS_GUI_TOOLBAR_MAIN_STYLE) == 0) {
@@ -1576,6 +1622,10 @@ set_pref(gchar *pref_name, gchar *value)
   } else if (strcmp(pref_name, PRS_GUI_GEOMETRY_MAIN_Y) == 0) {         /* deprecated */
   } else if (strcmp(pref_name, PRS_GUI_GEOMETRY_MAIN_WIDTH) == 0) {     /* deprecated */
   } else if (strcmp(pref_name, PRS_GUI_GEOMETRY_MAIN_HEIGHT) == 0) {    /* deprecated */
+  } else if (strcmp(pref_name, PRS_GUI_CONSOLE_OPEN) == 0) {
+    prefs.gui_console_open =
+	find_index_from_string_array(value, gui_console_open_text,
+				     console_open_never);
   } else if (strcmp(pref_name, PRS_GUI_FILEOPEN_STYLE) == 0) {
     prefs.gui_fileopen_style =
 	find_index_from_string_array(value, gui_fileopen_style_text,
@@ -1592,6 +1642,34 @@ set_pref(gchar *pref_name, gchar *value)
     prefs.gui_fileopen_dir = g_strdup(value);
   } else if (strcmp(pref_name, PRS_GUI_FILEOPEN_REMEMBERED_DIR) == 0) { /* deprecated */
 
+  } else if (strcmp(pref_name, PRS_GUI_ASK_UNSAVED) == 0) {
+    if (strcasecmp(value, "true") == 0) {
+	    prefs.gui_ask_unsaved = TRUE;
+    }
+    else {
+	    prefs.gui_ask_unsaved = FALSE;
+    }
+  } else if (strcmp(pref_name, PRS_GUI_WEBBROWSER) == 0) {
+    g_free(prefs.gui_webbrowser);
+    prefs.gui_webbrowser = g_strdup(value);
+  } else if (strcmp(pref_name, PRS_GUI_LAYOUT_TYPE) == 0) {
+    prefs.gui_layout_type = strtoul(value, NULL, 10);
+    if (prefs.gui_layout_type == layout_unused ||
+        prefs.gui_layout_type >= layout_type_max) {
+      /* XXX - report an error?  It's not a syntax error - we'd need to
+         add a way of reporting a *semantic* error. */
+      prefs.gui_layout_type = layout_type_5;
+    }
+  } else if (strcmp(pref_name, PRS_GUI_LAYOUT_CONTENT_1) == 0) {
+    prefs.gui_layout_content_1 =
+	find_index_from_string_array(value, gui_layout_content_text, 0);
+  } else if (strcmp(pref_name, PRS_GUI_LAYOUT_CONTENT_2) == 0) {
+    prefs.gui_layout_content_2 =
+	find_index_from_string_array(value, gui_layout_content_text, 0);
+  } else if (strcmp(pref_name, PRS_GUI_LAYOUT_CONTENT_3) == 0) {
+    prefs.gui_layout_content_3 =
+	find_index_from_string_array(value, gui_layout_content_text, 0);
+
 /* handle the capture options */
   } else if (strcmp(pref_name, PRS_CAP_DEVICE) == 0) {
     if (prefs.capture_device != NULL)
@@ -1611,6 +1689,8 @@ set_pref(gchar *pref_name, gchar *value)
     prefs.capture_real_time = ((strcasecmp(value, "true") == 0)?TRUE:FALSE);
   } else if (strcmp(pref_name, PRS_CAP_AUTO_SCROLL) == 0) {
     prefs.capture_auto_scroll = ((strcasecmp(value, "true") == 0)?TRUE:FALSE);
+  } else if (strcmp(pref_name, PRS_CAP_SHOW_INFO) == 0) {
+    prefs.capture_show_info = ((strcasecmp(value, "true") == 0)?TRUE:FALSE);
 
 /* handle the global options */
   } else if (strcmp(pref_name, PRS_NAME_RESOLVE) == 0 ||
@@ -1964,13 +2044,20 @@ write_pref(gpointer data, gpointer user_data)
 		break;
 
 	case PREF_ENUM:
+		/*
+		 * For now, we save the "description" value, so that if we
+		 * save the preferences older versions of Ethereal can at
+		 * least read preferences that they supported; we support
+		 * either the short name or the description when reading
+		 * the preferences file or a "-o" option.
+		 */
 		fprintf(arg->pf, "# One of: ");
 		enum_valp = pref->info.enum_info.enumvals;
 		val_string = NULL;
 		while (enum_valp->name != NULL) {
 			if (enum_valp->value == *pref->varp.enump)
-				val_string = enum_valp->name;
-			fprintf(arg->pf, "%s", enum_valp->name);
+				val_string = enum_valp->description;
+			fprintf(arg->pf, "%s", enum_valp->description);
 			enum_valp++;
 			if (enum_valp->name == NULL)
 				fprintf(arg->pf, "\n");
@@ -1978,8 +2065,8 @@ write_pref(gpointer data, gpointer user_data)
 				fprintf(arg->pf, ", ");
 		}
 		fprintf(arg->pf, "# (case-insensitive).\n");
-		fprintf(arg->pf, "%s.%s: %s\n", arg->module->name, pref->name,
-		    val_string);
+		fprintf(arg->pf, "%s.%s: %s\n", arg->module->name,
+		    pref->name, val_string);
 		break;
 
 	case PREF_STRING:
@@ -2031,62 +2118,7 @@ write_prefs(char **pf_path_return)
   fputs("# Configuration file for Ethereal " VERSION ".\n"
     "#\n"
     "# This file is regenerated each time preferences are saved within\n"
-    "# Ethereal.  Making manual changes should be safe, however.\n"
-    "\n"
-    "######## Printing ########\n", pf);
-
-  fprintf (pf, "\n# Can be one of \"text\" or \"postscript\".\n"
-    "print.format: %s\n", pr_formats[prefs.pr_format]);
-
-  fprintf (pf, "\n# Can be one of \"command\" or \"file\".\n"
-    "print.destination: %s\n", pr_dests[prefs.pr_dest]);
-
-  fprintf (pf, "\n# This is the file that gets written to when the "
-    "destination is set to \"file\"\n"
-    "%s: %s\n", PRS_PRINT_FILE, prefs.pr_file);
-
-  fprintf (pf, "\n# Output gets piped to this command when the destination "
-    "is set to \"command\"\n"
-    "%s: %s\n", PRS_PRINT_CMD, prefs.pr_cmd);
-
-  fprintf (pf, "\n######## Columns ########\n");
-  
-  clp = prefs.col_list;
-  col_l = NULL;
-  while (clp) {
-    cfmt = (fmt_data *) clp->data;
-    col_l = g_list_append(col_l, cfmt->title);
-    col_l = g_list_append(col_l, cfmt->fmt);
-    clp = clp->next;
-  }
-  fprintf (pf, "\n# Packet list column format.\n");
-  fprintf (pf, "# Each pair of strings consists of a column title and its format.\n");
-  fprintf (pf, "%s: %s\n", PRS_COL_FMT, put_string_list(col_l));
-  /* This frees the list of strings, but not the strings to which it
-     refers; that's what we want, as we haven't copied those strings,
-     we just referred to them.  */
-  g_list_free(col_l);
-
-  fprintf (pf, "\n######## TCP Stream Window ########\n");
-
-  fprintf (pf, "\n# TCP stream window color preferences.\n");
-  fprintf (pf, "# Each value is a six digit hexadecimal color value in the form rrggbb.\n");
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_CL_FG,
-    (prefs.st_client_fg.red * 255 / 65535),
-    (prefs.st_client_fg.green * 255 / 65535),
-    (prefs.st_client_fg.blue * 255 / 65535));
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_CL_BG,
-    (prefs.st_client_bg.red * 255 / 65535),
-    (prefs.st_client_bg.green * 255 / 65535),
-    (prefs.st_client_bg.blue * 255 / 65535));
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_SR_FG,
-    (prefs.st_server_fg.red * 255 / 65535),
-    (prefs.st_server_fg.green * 255 / 65535),
-    (prefs.st_server_fg.blue * 255 / 65535));
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_SR_BG,
-    (prefs.st_server_bg.red * 255 / 65535),
-    (prefs.st_server_bg.green * 255 / 65535),
-    (prefs.st_server_bg.blue * 255 / 65535));
+    "# Ethereal.  Making manual changes should be safe, however.\n", pf);
 
   fprintf (pf, "\n######## User Interface ########\n");
   
@@ -2110,6 +2142,11 @@ write_prefs(char **pf_path_return)
   fprintf(pf, PRS_GUI_ALTERN_COLORS ": %s\n",
 		  prefs.gui_altern_colors == TRUE ? "TRUE" : "FALSE");
 
+  fprintf(pf, "\n# Place filter toolbar inside the statusbar?\n");
+  fprintf(pf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(pf, PRS_GUI_FILTER_TOOLBAR_IN_STATUSBAR ": %s\n",
+                 prefs.filter_toolbar_show_in_statusbar == TRUE ? "TRUE" : "FALSE");
+
   fprintf(pf, "\n# Protocol-tree line style.\n");
   fprintf(pf, "# One of: NONE, SOLID, DOTTED, TABBED\n");
   fprintf(pf, PRS_GUI_PTREE_LINE_STYLE ": %s\n",
@@ -2130,23 +2167,6 @@ write_prefs(char **pf_path_return)
   fprintf(pf, PRS_GUI_TOOLBAR_MAIN_STYLE ": %s\n",
 		  gui_toolbar_style_text[prefs.gui_toolbar_main_style]);
 
-  fprintf(pf, "\n# Font name for packet list, protocol tree, and hex dump panes (GTK version 1).\n");
-  fprintf(pf, PRS_GUI_FONT_NAME_1 ": %s\n", prefs.gui_font_name1);
-
-  fprintf(pf, "\n# Font name for packet list, protocol tree, and hex dump panes (GTK version 2).\n");
-  fprintf(pf, PRS_GUI_FONT_NAME_2 ": %s\n", prefs.gui_font_name2);
-
-  fprintf (pf, "\n# Color preferences for a marked frame.\n");
-  fprintf (pf, "# Each value is a six digit hexadecimal color value in the form rrggbb.\n");
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_GUI_MARKED_FG,
-    (prefs.gui_marked_fg.red * 255 / 65535),
-    (prefs.gui_marked_fg.green * 255 / 65535),
-    (prefs.gui_marked_fg.blue * 255 / 65535));
-  fprintf (pf, "%s: %02x%02x%02x\n", PRS_GUI_MARKED_BG,
-    (prefs.gui_marked_bg.red * 255 / 65535),
-    (prefs.gui_marked_bg.green * 255 / 65535),
-    (prefs.gui_marked_bg.blue * 255 / 65535));
-
   fprintf(pf, "\n# Save window position at exit?\n");
   fprintf(pf, "# TRUE or FALSE (case-insensitive).\n");
   fprintf(pf, PRS_GUI_GEOMETRY_SAVE_POSITION ": %s\n",
@@ -2162,6 +2182,11 @@ write_prefs(char **pf_path_return)
   fprintf(pf, PRS_GUI_GEOMETRY_SAVE_MAXIMIZED ": %s\n",
 		  prefs.gui_geometry_save_maximized == TRUE ? "TRUE" : "FALSE");
                   
+  fprintf(pf, "\n# Open a console window (WIN32 only)?\n");
+  fprintf(pf, "# One of: NEVER, AUTOMATIC, ALWAYS\n");
+  fprintf(pf, PRS_GUI_CONSOLE_OPEN ": %s\n",
+		  gui_console_open_text[prefs.gui_console_open]);
+
   fprintf(pf, "\n# Where to start the File Open dialog box.\n");
   fprintf(pf, "# One of: LAST_OPENED, SPECIFIED\n");
   fprintf(pf, PRS_GUI_FILEOPEN_STYLE ": %s\n",
@@ -2176,20 +2201,89 @@ write_prefs(char **pf_path_return)
                   prefs.gui_fileopen_dir);
   }
                   
-  fprintf(pf, "\n####### Name Resolution ########\n");
+  fprintf(pf, "\n# Ask to save unsaved capture files?\n");
+  fprintf(pf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(pf, PRS_GUI_ASK_UNSAVED ": %s\n",
+		  prefs.gui_ask_unsaved == TRUE ? "TRUE" : "FALSE");                  
+
+  fprintf(pf, "\n# The path to the webbrowser.\n");
+  fprintf(pf, "# Ex: mozilla %%s\n");
+  fprintf(pf, PRS_GUI_WEBBROWSER ": %s\n", prefs.gui_webbrowser);
+
+  fprintf (pf, "\n######## User Interface: Layout ########\n");
+
+  fprintf(pf, "\n# Layout type (1-6).\n");
+  fprintf(pf, PRS_GUI_LAYOUT_TYPE ": %d\n",
+	          prefs.gui_layout_type);
+
+  fprintf(pf, "\n# Layout content of the panes (1-3).\n");
+  fprintf(pf, "# One of: NONE, PLIST, PDETAILS, PBYTES\n");
+  fprintf(pf, PRS_GUI_LAYOUT_CONTENT_1 ": %s\n",
+	          gui_layout_content_text[prefs.gui_layout_content_1]);
+  fprintf(pf, PRS_GUI_LAYOUT_CONTENT_2 ": %s\n",
+	          gui_layout_content_text[prefs.gui_layout_content_2]);
+  fprintf(pf, PRS_GUI_LAYOUT_CONTENT_3 ": %s\n",
+	          gui_layout_content_text[prefs.gui_layout_content_3]);
+
+  fprintf (pf, "\n######## User Interface: Columns ########\n");
   
-  fprintf(pf, "\n# Resolve addresses to names?\n");
-  fprintf(pf, "# TRUE or FALSE (case-insensitive), or a list of address types to resolve.\n");
-  fprintf(pf, PRS_NAME_RESOLVE ": %s\n",
-		  name_resolve_to_string(prefs.name_resolve));
+  clp = prefs.col_list;
+  col_l = NULL;
+  while (clp) {
+    cfmt = (fmt_data *) clp->data;
+    col_l = g_list_append(col_l, cfmt->title);
+    col_l = g_list_append(col_l, cfmt->fmt);
+    clp = clp->next;
+  }
+  fprintf (pf, "\n# Packet list column format.\n");
+  fprintf (pf, "# Each pair of strings consists of a column title and its format.\n");
+  fprintf (pf, "%s: %s\n", PRS_COL_FMT, put_string_list(col_l));
+  /* This frees the list of strings, but not the strings to which it
+     refers; that's what we want, as we haven't copied those strings,
+     we just referred to them.  */
+  g_list_free(col_l);
 
-  fprintf(pf, "\n# Name resolution concurrency.\n");
-  fprintf(pf, "# A decimal number.\n");
-  fprintf(pf, PRS_NAME_RESOLVE_CONCURRENCY ": %d\n",
-		  prefs.name_resolve_concurrency);
+  fprintf (pf, "\n######## User Interface: Font ########\n");
 
-/* write the capture options */
-  fprintf(pf, "\n####### Capture Options ########\n");
+  fprintf(pf, "\n# Font name for packet list, protocol tree, and hex dump panes (GTK version 1).\n");
+  fprintf(pf, PRS_GUI_FONT_NAME_1 ": %s\n", prefs.gui_font_name1);
+
+  fprintf(pf, "\n# Font name for packet list, protocol tree, and hex dump panes (GTK version 2).\n");
+  fprintf(pf, PRS_GUI_FONT_NAME_2 ": %s\n", prefs.gui_font_name2);
+
+  fprintf (pf, "\n######## User Interface: Colors ########\n");
+
+  fprintf (pf, "\n# Color preferences for a marked frame.\n");
+  fprintf (pf, "# Each value is a six digit hexadecimal color value in the form rrggbb.\n");
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_GUI_MARKED_FG,
+    (prefs.gui_marked_fg.red * 255 / 65535),
+    (prefs.gui_marked_fg.green * 255 / 65535),
+    (prefs.gui_marked_fg.blue * 255 / 65535));
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_GUI_MARKED_BG,
+    (prefs.gui_marked_bg.red * 255 / 65535),
+    (prefs.gui_marked_bg.green * 255 / 65535),
+    (prefs.gui_marked_bg.blue * 255 / 65535));
+
+  fprintf (pf, "\n# TCP stream window color preferences.\n");
+  fprintf (pf, "# Each value is a six digit hexadecimal color value in the form rrggbb.\n");
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_CL_FG,
+    (prefs.st_client_fg.red * 255 / 65535),
+    (prefs.st_client_fg.green * 255 / 65535),
+    (prefs.st_client_fg.blue * 255 / 65535));
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_CL_BG,
+    (prefs.st_client_bg.red * 255 / 65535),
+    (prefs.st_client_bg.green * 255 / 65535),
+    (prefs.st_client_bg.blue * 255 / 65535));
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_SR_FG,
+    (prefs.st_server_fg.red * 255 / 65535),
+    (prefs.st_server_fg.green * 255 / 65535),
+    (prefs.st_server_fg.blue * 255 / 65535));
+  fprintf (pf, "%s: %02x%02x%02x\n", PRS_STREAM_SR_BG,
+    (prefs.st_server_bg.red * 255 / 65535),
+    (prefs.st_server_bg.green * 255 / 65535),
+    (prefs.st_server_bg.blue * 255 / 65535));
+
+  fprintf(pf, "\n####### Capture ########\n");
   
   if (prefs.capture_device != NULL) {
     fprintf(pf, "\n# Default capture device\n");
@@ -2223,6 +2317,41 @@ write_prefs(char **pf_path_return)
   fprintf(pf, PRS_CAP_AUTO_SCROLL ": %s\n",
 		  prefs.capture_auto_scroll == TRUE ? "TRUE" : "FALSE");
 
+  fprintf(pf, "\n# Show capture info dialog while capturing?\n");
+  fprintf(pf, "# TRUE or FALSE (case-insensitive).\n");
+  fprintf(pf, PRS_CAP_SHOW_INFO ": %s\n",
+		  prefs.capture_show_info == TRUE ? "TRUE" : "FALSE");
+
+  fprintf (pf, "\n######## Printing ########\n");
+
+  fprintf (pf, "\n# Can be one of \"text\" or \"postscript\".\n"
+    "print.format: %s\n", pr_formats[prefs.pr_format]);
+
+  fprintf (pf, "\n# Can be one of \"command\" or \"file\".\n"
+    "print.destination: %s\n", pr_dests[prefs.pr_dest]);
+
+  fprintf (pf, "\n# This is the file that gets written to when the "
+    "destination is set to \"file\"\n"
+    "%s: %s\n", PRS_PRINT_FILE, prefs.pr_file);
+
+  fprintf (pf, "\n# Output gets piped to this command when the destination "
+    "is set to \"command\"\n"
+    "%s: %s\n", PRS_PRINT_CMD, prefs.pr_cmd);
+
+  fprintf(pf, "\n####### Name Resolution ########\n");
+  
+  fprintf(pf, "\n# Resolve addresses to names?\n");
+  fprintf(pf, "# TRUE or FALSE (case-insensitive), or a list of address types to resolve.\n");
+  fprintf(pf, PRS_NAME_RESOLVE ": %s\n",
+		  name_resolve_to_string(prefs.name_resolve));
+
+  fprintf(pf, "\n# Name resolution concurrency.\n");
+  fprintf(pf, "# A decimal number.\n");
+  fprintf(pf, PRS_NAME_RESOLVE_CONCURRENCY ": %d\n",
+		  prefs.name_resolve_concurrency);
+
+  fprintf(pf, "\n####### Protocols ########\n");
+
   g_list_foreach(modules, write_module_prefs, pf);
 
   fclose(pf);
@@ -2262,12 +2391,19 @@ copy_prefs(e_prefs *dest, e_prefs *src)
   dest->gui_plist_sel_browse = src->gui_plist_sel_browse;
   dest->gui_ptree_sel_browse = src->gui_ptree_sel_browse;
   dest->gui_altern_colors = src->gui_altern_colors;
+  dest->filter_toolbar_show_in_statusbar = src->filter_toolbar_show_in_statusbar;
   dest->gui_ptree_line_style = src->gui_ptree_line_style;
   dest->gui_ptree_expander_style = src->gui_ptree_expander_style;
   dest->gui_hex_dump_highlight_style = src->gui_hex_dump_highlight_style;
   dest->gui_toolbar_main_style = src->gui_toolbar_main_style;
   dest->gui_fileopen_dir = g_strdup(src->gui_fileopen_dir);
+  dest->gui_console_open = src->gui_console_open;
   dest->gui_fileopen_style = src->gui_fileopen_style;
+  dest->gui_ask_unsaved = src->gui_ask_unsaved;
+  dest->gui_layout_type = src->gui_layout_type;
+  dest->gui_layout_content_1 = src->gui_layout_content_1;
+  dest->gui_layout_content_2 = src->gui_layout_content_2;
+  dest->gui_layout_content_3 = src->gui_layout_content_3;
   dest->gui_font_name1 = g_strdup(src->gui_font_name1);
   dest->gui_font_name2 = g_strdup(src->gui_font_name2);
   dest->gui_marked_fg = src->gui_marked_fg;
@@ -2275,6 +2411,7 @@ copy_prefs(e_prefs *dest, e_prefs *src)
   dest->gui_geometry_save_position = src->gui_geometry_save_position;
   dest->gui_geometry_save_size = src->gui_geometry_save_size;
   dest->gui_geometry_save_maximized = src->gui_geometry_save_maximized;
+  dest->gui_webbrowser = g_strdup(src->gui_webbrowser);
 /*  values for the capture dialog box */
   dest->capture_device = g_strdup(src->capture_device);
   dest->capture_devices_descr = g_strdup(src->capture_devices_descr);
@@ -2282,6 +2419,7 @@ copy_prefs(e_prefs *dest, e_prefs *src)
   dest->capture_prom_mode = src->capture_prom_mode;
   dest->capture_real_time = src->capture_real_time;
   dest->capture_auto_scroll = src->capture_auto_scroll;
+  dest->capture_show_info = src->capture_show_info;
   dest->name_resolve = src->name_resolve;
   dest->name_resolve_concurrency = src->name_resolve_concurrency;
 
@@ -2312,6 +2450,8 @@ free_prefs(e_prefs *pr)
     g_free(pr->gui_fileopen_dir);
     pr->gui_fileopen_dir = NULL;
   }
+  g_free(pr->gui_webbrowser);
+  pr->gui_webbrowser = NULL;
   if (pr->capture_device != NULL) {
     g_free(pr->capture_device);
     pr->capture_device = NULL;
