@@ -1,7 +1,7 @@
 /* print.c
  * Routines for printing packet analysis trees.
  *
- * $Id: print.c,v 1.71 2004/01/25 00:58:12 guy Exp $
+ * $Id: print.c,v 1.83 2004/07/05 16:39:20 ulfl Exp $
  *
  * Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -44,13 +44,15 @@
 #include "packet-frame.h"
 
 #define PDML_VERSION "0"
+#define PSML_VERSION "0"
 
 static void proto_tree_print_node(proto_node *node, gpointer data);
 static void proto_tree_print_node_pdml(proto_node *node, gpointer data);
 static void print_hex_data_buffer(FILE *fh, register const guchar *cp,
-    register guint length, char_enc encoding, gint format);
+    register guint length, char_enc encoding, print_format_e format);
 static void ps_clean_string(unsigned char *out, const unsigned char *in,
 			int outbuf_size);
+static void print_escaped_xml(FILE *fh, char *unescaped_string);
 
 typedef struct {
 	int		            level;
@@ -93,6 +95,7 @@ proto_tree_print(print_args_t *print_args, epan_dissect_t *edt,
     FILE *fh)
 {
 	print_data data;
+	gint	i;
 
 	/* Create the output */
 	data.level = 0;
@@ -106,8 +109,12 @@ proto_tree_print(print_args_t *print_args, epan_dissect_t *edt,
 	data.format = print_args->format;
 	data.edt = edt;
 
-	if (data.format == PR_FMT_PDML) {
-
+    switch(data.format) {
+    case(PR_FMT_TEXT):  /* fall through */
+    case(PR_FMT_PS):
+		proto_tree_children_foreach(edt->tree, proto_tree_print_node, &data);
+        break;
+    case(PR_FMT_PDML):
 		fprintf(fh, "<packet>\n");
 
 		/* Print a "geninfo" protocol as required by PDML */
@@ -116,15 +123,39 @@ proto_tree_print(print_args_t *print_args, epan_dissect_t *edt,
 		proto_tree_children_foreach(edt->tree, proto_tree_print_node_pdml, &data);
 
 		fprintf(fh, "</packet>\n\n");
-	}
-	else {
-		proto_tree_children_foreach(edt->tree, proto_tree_print_node, &data);
+        break;
+    case(PR_FMT_PSML):
+        /* if this is the first packet, we have to create the PSML structure output */
+        if(edt->pi.fd->num == 1) {
+            fprintf(fh, "<structure>\n");
+
+            for(i=0; i < edt->pi.cinfo->num_cols; i++) {
+                fprintf(fh, "<section>");
+                print_escaped_xml(fh, edt->pi.cinfo->col_title[i]);
+                fprintf(fh, "</section>\n");
+            }
+
+            fprintf(fh, "</structure>\n\n");
+        }
+
+		fprintf(fh, "<packet>\n");
+
+        for(i=0; i < edt->pi.cinfo->num_cols; i++) {
+            fprintf(fh, "<section>");
+            print_escaped_xml(fh, edt->pi.cinfo->col_data[i]);
+            fprintf(fh, "</section>\n");
+        }
+
+		fprintf(fh, "</packet>\n\n");
+        break;
+    default:
+        g_assert_not_reached();
 	}
 }
 
 /*
  * Find the data source for a specified field, and return a pointer
- * to the data in it.
+ * to the data in it. Returns NULL if the data is out of bounds.
  */
 static const guint8 *
 get_field_data(GSList *src_list, field_info *fi)
@@ -148,9 +179,12 @@ get_field_data(GSList *src_list, field_info *fi)
 			 * that could be expensive.  Until we fix that,
 			 * we'll do the check here.
 			 */
-			length = fi->length;
 			tvbuff_length = tvb_length_remaining(src_tvb,
 			    fi->start);
+			if (tvbuff_length < 0) {
+				return NULL;
+			}
+			length = fi->length;
 			if (length > tvbuff_length)
 				length = tvbuff_length;
 			return tvb_get_ptr(src_tvb, fi->start, length);
@@ -175,10 +209,10 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 	gchar		*label_ptr;
 
 	/* Don't print invisible entries. */
-	if (!fi->visible)
+	if (PROTO_ITEM_IS_HIDDEN(node))
 		return;
 
-	/* was a free format label produced? */
+    /* was a free format label produced? */
 	if (fi->rep) {
 		label_ptr = fi->rep->representation;
 	}
@@ -196,8 +230,10 @@ void proto_tree_print_node(proto_node *node, gpointer data)
 		 * Find the data for this field.
 		 */
 		pd = get_field_data(pdata->src_list, fi);
-		print_hex_data_buffer(pdata->fh, pd, fi->length,
-		    pdata->encoding, pdata->format);
+		if (pd) {
+			print_hex_data_buffer(pdata->fh, pd, fi->length,
+			    pdata->encoding, pdata->format);
+		}
 	}
 
 	/* If we're printing all levels, or if this node is one with a
@@ -237,6 +273,9 @@ print_escaped_xml(FILE *fh, char *unescaped_string)
 			case '"':
 				fputs("&quot;", fh);
 				break;
+			case '\'':
+				fputs("&apos;", fh);
+				break;
 			default:
 				fputc(*p, fh);
 		}
@@ -249,12 +288,20 @@ print_field_hex_value(print_data *pdata, field_info *fi)
 	int i;
 	const guint8 *pd;
 
+
+    if(fi->length > tvb_length_remaining(fi->ds_tvb, fi->start)) {
+        fprintf(pdata->fh, "field length invalid!");
+        return;
+    }
+
 	/* Find the data for this field. */
 	pd = get_field_data(pdata->src_list, fi);
 
-	/* Print a simple hex dump */
-	for (i = 0 ; i < fi->length; i++) {
-		fprintf(pdata->fh, "%02x", pd[i]);
+	if (pd) {
+		/* Print a simple hex dump */
+		for (i = 0 ; i < fi->length; i++) {
+			fprintf(pdata->fh, "%02x", pd[i]);
+		}
 	}
 }
 
@@ -322,6 +369,24 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 		}
 		print_escaped_xml(pdata->fh, fi->hfinfo->abbrev);
 
+#if 0
+        /* PDML spec, see: 
+         * http://analyzer.polito.it/30alpha/docs/dissectors/PDMLSpec.htm
+         *
+         * the show fields contains things in 'human readable' format
+         * showname: contains only the name of the field
+         * show: contains only the data of the field
+         * showdtl: contains additional details of the field data
+         * showmap: contains mappings of the field data (e.g. the hostname to an IP address)
+         *
+         * XXX - the showname shouldn't contain the field data itself 
+         * (like it's contained in the fi->rep->representation). 
+         * Unfortunately, we don't have the field data representation for 
+         * all fields, so this isn't currently possible */
+		fputs("\" showname=\"", pdata->fh);
+		print_escaped_xml(pdata->fh, fi->hfinfo->name);
+#endif
+
 		if (fi->rep) {
 			fputs("\" showname=\"", pdata->fh);
 			print_escaped_xml(pdata->fh, fi->rep->representation);
@@ -333,6 +398,10 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 			print_escaped_xml(pdata->fh, label_ptr);
 		}
 
+        if(PROTO_ITEM_IS_HIDDEN(node)) {
+    		fprintf(pdata->fh, "\" hide=\"yes");
+        }
+
 		fprintf(pdata->fh, "\" size=\"%d", fi->length);
 		fprintf(pdata->fh, "\" pos=\"%d", fi->start);
 /*		fprintf(pdata->fh, "\" id=\"%d", fi->hfinfo->id);*/
@@ -340,7 +409,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 		if (fi->hfinfo->type != FT_PROTOCOL) {
 			/* Field */
 
-			/* XXX - this is a hack until we can juse call
+			/* XXX - this is a hack until we can just call
 			 * fvalue_to_string_repr() for *all* FT_* types. */
 			dfilter_string = proto_construct_dfilter_string(fi,
 					pdata->edt);
@@ -377,7 +446,7 @@ proto_tree_print_node_pdml(proto_node *node, gpointer data)
 		}
 	}
 
-	/* We always pring all levels for PDML. Recurse here. */
+	/* We always print all levels for PDML. Recurse here. */
 	if (node->first_child != NULL) {
 		pdata->level++;
 		proto_tree_children_foreach(node,
@@ -481,7 +550,7 @@ print_pdml_geninfo(proto_tree *tree, print_data *pdata)
 }
 
 
-void print_hex_data(FILE *fh, gint format, epan_dissect_t *edt)
+void print_hex_data(FILE *fh, print_format_e format, epan_dissect_t *edt)
 {
 	gboolean multiple_sources;
 	GSList *src_le;
@@ -544,7 +613,7 @@ void print_hex_data(FILE *fh, gint format, epan_dissect_t *edt)
 
 static void
 print_hex_data_buffer(FILE *fh, register const guchar *cp,
-    register guint length, char_enc encoding, gint format)
+    register guint length, char_enc encoding, print_format_e format)
 {
 	register unsigned int ad, i, j, k, l;
 	guchar c;
@@ -651,41 +720,120 @@ void ps_clean_string(unsigned char *out, const unsigned char *in,
 
 /* Some formats need stuff at the beginning of the output */
 void
-print_preamble(FILE *fh, gint format)
+print_preamble(FILE *fh, print_format_e format, gchar *filename)
 {
-	if (format == PR_FMT_PS)
+	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
+
+    
+    switch(format) {
+    case(PR_FMT_TEXT):
+        /* do nothing */
+        break;
+    case(PR_FMT_PS):
 		print_ps_preamble(fh);
-	else if (format == PR_FMT_PDML) {
+
+		fputs("%% Set the font to 10 point\n", fh);
+		fputs("/Courier findfont 10 scalefont setfont\n", fh);
+		fputs("\n", fh);
+		fputs("%% the page title\n", fh);
+		ps_clean_string(psbuffer, filename, MAX_PS_LINE_LENGTH);
+		fprintf(fh, "/eth_pagetitle (%s - Ethereal) def\n", psbuffer);
+		fputs("\n", fh);
+        break;
+    case(PR_FMT_PDML):
 		fputs("<?xml version=\"1.0\"?>\n", fh);
 		fputs("<pdml version=\"" PDML_VERSION "\" ", fh);
 		fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
+        break;
+    case(PR_FMT_PSML):
+		fputs("<?xml version=\"1.0\"?>\n", fh);
+		fputs("<psml version=\"" PSML_VERSION "\" ", fh);
+		fprintf(fh, "creator=\"%s/%s\">\n", PACKAGE, VERSION);
+        break;
+    default:
+		g_assert_not_reached();
+	}
+}
+
+void
+print_packet_header(FILE *fh, print_format_e format, guint32 number, gchar *summary) {
+	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
+
+    
+    switch(format) {
+    case(PR_FMT_TEXT):
+        /* do nothing */
+        break;
+    case(PR_FMT_PS):
+		ps_clean_string(psbuffer, summary, MAX_PS_LINE_LENGTH);
+        fprintf(fh, "[/Dest /__frame%u__ /Title (%s)   /OUT pdfmark\n", number, psbuffer);
+        fputs("[/View [/XYZ -4 currentpoint matrix currentmatrix matrix defaultmatrix\n", fh);
+        fputs("matrix invertmatrix matrix concatmatrix transform exch pop 20 add null]\n", fh);
+        fprintf(fh, "/Dest /__frame%u__ /DEST pdfmark\n", number);
+        break;
+    case(PR_FMT_PDML):
+        /* do nothing */
+        break;
+    case(PR_FMT_PSML):
+        /* do nothing */
+        break;
+    default:
+		g_assert_not_reached();
+	}
+}
+
+void
+print_formfeed(FILE *fh, print_format_e format) {
+    switch(format) {
+    case(PR_FMT_TEXT):
+		fputs("\f", fh);
+        break;
+    case(PR_FMT_PS):
+		fputs("formfeed\n", fh);
+        break;
+    case(PR_FMT_PDML):
+        /* do nothing */
+        break;
+    case(PR_FMT_PSML):
+        /* do nothing */
+        break;
+    default:
+		g_assert_not_reached();
 	}
 }
 
 /* Some formats need stuff at the end of the output */
 void
-print_finale(FILE *fh, gint format)
+print_finale(FILE *fh, print_format_e format)
 {
-	if (format == PR_FMT_PS)
+    switch(format) {
+    case(PR_FMT_TEXT):
+        /* do nothing */
+        break;
+    case(PR_FMT_PS):
 		print_ps_finale(fh);
-	else if (format == PR_FMT_PDML) {
+        break;
+    case(PR_FMT_PDML):
 		fputs("</pdml>\n", fh);
+        break;
+    case(PR_FMT_PSML):
+		fputs("</psml>\n", fh);
+        break;
+    default:
+		g_assert_not_reached();
 	}
 }
 
 void
-print_line(FILE *fh, int indent, gint format, char *line)
+print_line(FILE *fh, int indent, print_format_e format, char *line)
 {
 	char		space[MAX_INDENT+1];
 	int		i;
 	int		num_spaces;
 	char		psbuffer[MAX_PS_LINE_LENGTH]; /* static sized buffer! */
 
-	if (format == PR_FMT_PS) {
-		ps_clean_string(psbuffer, line, MAX_PS_LINE_LENGTH);
-		fprintf(fh, "%d (%s) putline\n", indent, psbuffer);
-	}
-	else if (format == PR_FMT_TEXT) {
+    switch(format) {
+    case(PR_FMT_TEXT):
 		/* Prepare the tabs for printing, depending on tree level */
 		num_spaces = indent * 4;
 		if (num_spaces > MAX_INDENT) {
@@ -700,8 +848,18 @@ print_line(FILE *fh, int indent, gint format, char *line)
 		fputs(space, fh);
 		fputs(line, fh);
 		putc('\n', fh);
-	}
-	else {
+        break;
+    case(PR_FMT_PS):
+		ps_clean_string(psbuffer, line, MAX_PS_LINE_LENGTH);
+		fprintf(fh, "%d (%s) putline\n", indent, psbuffer);
+        break;
+    case(PR_FMT_PDML):
+        /* do nothing */
+        break;
+    case(PR_FMT_PSML):
+        /* do nothing */
+        break;
+    default:
 		g_assert_not_reached();
-	}
+    }
 }
