@@ -1,7 +1,7 @@
 /* util.c
  * Utility routines
  *
- * $Id: util.c,v 1.78 2004/03/12 17:23:56 gerald Exp $
+ * $Id: util.c,v 1.81 2004/06/25 07:00:54 jmayer Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -56,6 +56,8 @@ typedef int mode_t;	/* for win32 */
 #endif /* __MINGW32__ */
 #endif /* HAVE_IO_H */
 
+#include <epan/resolv.h>
+
 /*
  * This has to come after the include of <pcap.h>, as the include of
  * <pcap.h> might cause <winsock2.h> to be included, and if we've
@@ -114,14 +116,14 @@ setup_tmpdir(char *dir)
 	int len = strlen(dir);
 	char *newdir;
 
-	/* Append slash if necessary */
-	if (dir[len - 1] == '/') {
+	/* Append path separator if necessary */
+	if (dir[len - 1] == G_DIR_SEPARATOR) {
 		newdir = dir;
 	}
 	else {
 		newdir = g_malloc(len + 2);
 		strcpy(newdir, dir);
-		strcat(newdir, "/");
+		strcat(newdir, G_DIR_SEPARATOR_S);
 	}
 	return newdir;
 }
@@ -369,7 +371,7 @@ compute_timestamp_diff(gint *diffsec, gint *diffusec,
    Return length of result. Taken from rproxy/librsync/base64.c by
    Andrew Tridgell. */
 
-size_t base64_decode(char *s)
+size_t epan_base64_decode(char *s)
 {
 	static const char b64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	int bit_offset, byte_offset, idx, i, n;
@@ -419,21 +421,21 @@ gchar *get_conn_cfilter(void) {
 	if ((env = getenv("SSH_CONNECTION")) != NULL) {
 		tokens = g_strsplit(env, " ", 4);
 		if (tokens[3]) {
-			g_string_sprintf(filter_str, "not (tcp port %s and ip host %s "
-				"and tcp port %s and ip host %s)", tokens[1], tokens[0],
-				tokens[3], tokens[2]);
+			g_string_sprintf(filter_str, "not (tcp port %s and %s host %s "
+							 "and tcp port %s and %s host %s)", tokens[1], host_ip_af(tokens[0]), tokens[0],
+				tokens[3], host_ip_af(tokens[2]), tokens[2]);
 			return filter_str->str;
 		}
 	} else if ((env = getenv("SSH_CLIENT")) != NULL) {
 		tokens = g_strsplit(env, " ", 3);
-		g_string_sprintf(filter_str, "not (tcp port %s and ip host %s "
-			"and tcp port %s)", tokens[1], tokens[0], tokens[2]);
+		g_string_sprintf(filter_str, "not (tcp port %s and %s host %s "
+			"and tcp port %s)", tokens[1], host_ip_af(tokens[0]), tokens[0], tokens[2]);
 		return filter_str->str;
 	} else if ((env = getenv("REMOTEHOST")) != NULL) {
 		if (strcasecmp(env, "localhost") == 0 || strcmp(env, "127.0.0.1") == 0) {
 			return "";
 		}
-		g_string_sprintf(filter_str, "not ip host %s", env);
+		g_string_sprintf(filter_str, "not %s host %s", host_ip_af(env), env);
 		return filter_str->str;
 	} else if ((env = getenv("DISPLAY")) != NULL) {
 		tokens = g_strsplit(env, ":", 2);
@@ -442,13 +444,13 @@ gchar *get_conn_cfilter(void) {
 					strcmp(tokens[0], "127.0.0.1") == 0) {
 				return "";
 			}
-			g_string_sprintf(filter_str, "not ip host %s",
-				tokens[0]);
+			g_string_sprintf(filter_str, "not %s host %s",
+				host_ip_af(tokens[0]), tokens[0]);
 			return filter_str->str;
 		}
 	} else if ((env = getenv("CLIENTNAME")) != NULL) {
 		if (g_strcasecmp("console", env) != 0) {
-			g_string_sprintf(filter_str, "not ip host %s", env);
+			g_string_sprintf(filter_str, "not %s host %s", host_ip_af(env), env);
 			return filter_str->str;
 		}
 	}
