@@ -1,6 +1,6 @@
 /* netxray.c
  *
- * $Id: netxray.c,v 1.79 2003/03/04 02:04:00 guy Exp $
+ * $Id: netxray.c,v 1.90 2004/03/03 22:24:51 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -55,8 +55,8 @@ struct netxray_hdr {
 	guint32	start_offset;	/* offset of first packet in capture */
 	guint32	end_offset;	/* offset after last packet in capture */
 	guint32 xxy[3];		/* unknown */
-	guint16	network;	/* datalink type */
-	guint8	xxz[2];		/* XXX - is this the upper 2 bytes of the datalink type? */
+	guint8	network;	/* datalink type */
+	guint8	xxz[3];		/* XXX - is this the upper 3 bytes of the datalink type? */
 	guint8	timeunit;	/* encodes length of a tick */
 	guint8	xxa[3];		/* XXX - is this the upper 3 bytes of the time units? */
 	guint32	timelo;		/* lower 32 bits of time stamp of capture start */
@@ -84,8 +84,15 @@ struct netxray_hdr {
 
 /*
  * # of ticks that equal 1 second
+ *
+ * XXX - the third item was 1193180.0, presumably because somebody found
+ * it gave the right answer for some captures, but 3 times that, i.e.
+ * 3579540.0, appears to give the right answer for some other captures.
+ *
+ * Is there something else in the file header to indicate which of those
+ * is correct?
  */
-static double TpS[] = { 1e6, 1193000.0, 1193180.0 };
+static double TpS[] = { 1e6, 1193000.0, 3579540.0 };
 #define NUM_NETXRAY_TIMEUNITS (sizeof TpS / sizeof TpS[0])
 
 /* Version number strings. */
@@ -144,12 +151,14 @@ union netxrayrec_hdr {
 	struct netxrayrec_2_x_hdr hdr_2_x;
 };
 
-static gboolean netxray_read(wtap *wth, int *err, long *data_offset);
+static gboolean netxray_read(wtap *wth, int *err, gchar **err_info,
+    long *data_offset);
 static gboolean netxray_seek_read(wtap *wth, long seek_off,
-    union wtap_pseudo_header *pseudo_header, guchar *pd, int length, int *err);
+    union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
+    int *err, gchar **err_info);
 static int netxray_read_rec_header(wtap *wth, FILE_T fh,
     union netxrayrec_hdr *hdr, int *err);
-static void netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
+static guint netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
     union wtap_pseudo_header *pseudo_header, union netxrayrec_hdr *hdr);
 static gboolean netxray_read_rec_data(FILE_T fh, guint8 *data_ptr,
     guint32 packet_size, int *err);
@@ -163,17 +172,19 @@ static gboolean netxray_dump_2_0(wtap_dumper *wdh,
     const union wtap_pseudo_header *pseudo_header, const guchar *pd, int *err);
 static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err);
 
-int netxray_open(wtap *wth, int *err)
+int netxray_open(wtap *wth, int *err, gchar **err_info)
 {
 	int bytes_read;
 	char magic[sizeof netxray_magic];
 	gboolean is_old;
 	struct netxray_hdr hdr;
+	guint network_type;
 	double timeunit;
-	int version_major;
+	int version_major, version_minor;
 	int file_type;
-	double t;
+	double start_timestamp;
 	static const int netxray_encap[] = {
+		WTAP_ENCAP_UNKNOWN,
 		WTAP_ENCAP_ETHERNET,
 		WTAP_ENCAP_TOKEN_RING,
 		WTAP_ENCAP_FDDI_BITSWAPPED,
@@ -231,8 +242,8 @@ int netxray_open(wtap *wth, int *err)
 	wth->data_offset += sizeof hdr;
 
 	if (is_old) {
-		timeunit = 1000.0;
 		version_major = 0;
+		version_minor = 0;
 		file_type = WTAP_FILE_NETXRAY_OLD;
 	} else {
 		/* It appears that version 1.1 files (as produced by Windows
@@ -245,52 +256,148 @@ int netxray_open(wtap *wth, int *err)
 		if (memcmp(hdr.version, vers_1_0, sizeof vers_1_0) == 0) {
 			timeunit = 1000.0;
 			version_major = 1;
+			version_minor = 0;
 			file_type = WTAP_FILE_NETXRAY_1_0;
 		} else if (memcmp(hdr.version, vers_1_1, sizeof vers_1_1) == 0) {
 			timeunit = 1000000.0;
 			version_major = 1;
+			version_minor = 1;
 			file_type = WTAP_FILE_NETXRAY_1_1;
-		} else if (memcmp(hdr.version, vers_2_000, sizeof vers_2_000) == 0
-		    || memcmp(hdr.version, vers_2_001, sizeof vers_2_001) == 0
-		    || memcmp(hdr.version, vers_2_002, sizeof vers_2_002) == 0) {
-			if (hdr.timeunit > NUM_NETXRAY_TIMEUNITS) {
-				g_message("netxray: Unknown timeunit %u",
-					  hdr.timeunit);
-				*err = WTAP_ERR_UNSUPPORTED;
-				return -1;
-			}
-			timeunit = TpS[hdr.timeunit];
+		} else if (memcmp(hdr.version, vers_2_000, sizeof vers_2_000) == 0) {
 			version_major = 2;
+			version_minor = 0;
+			file_type = WTAP_FILE_NETXRAY_2_00x;
+		} else if (memcmp(hdr.version, vers_2_001, sizeof vers_2_001) == 0) {
+			version_major = 2;
+			version_minor = 1;
+			file_type = WTAP_FILE_NETXRAY_2_00x;
+		} else if (memcmp(hdr.version, vers_2_002, sizeof vers_2_002) == 0) {
+			version_major = 2;
+			version_minor = 2;
 			file_type = WTAP_FILE_NETXRAY_2_00x;
-
-			/*
-			 * It appears that if hdr.xxb[20] is 2, that indicates
-			 * that it's a gigabit Ethernet capture, possibly
-			 * from a special whizzo gigabit pod, and also
-			 * indicates that the time stamps have nanosecond
-			 * resolution (or, at least, 1000 times greater
-			 * resolution), possibly thanks to a high-resolution
-			 * timer on the pod.
-			 */
-			if (hdr.xxb[20] == CAPTYPE_GIGPOD)
-				timeunit = timeunit*1000.0;
 		} else {
-			g_message("netxray: version \"%.8s\" unsupported", hdr.version);
 			*err = WTAP_ERR_UNSUPPORTED;
+			*err_info = g_strdup_printf("netxray: version \"%.8s\" unsupported", hdr.version);
 			return -1;
 		}
 	}
 
-	hdr.network = pletohs(&hdr.network);
-	if (hdr.network >= NUM_NETXRAY_ENCAPS
-	    || netxray_encap[hdr.network] == WTAP_ENCAP_UNKNOWN) {
-		g_message("netxray: network type %u unknown or unsupported",
-		    hdr.network);
-		*err = WTAP_ERR_UNSUPPORTED_ENCAP;
+	switch (hdr.xxz[0]) {
+
+	case 0:
+		/*
+		 * The byte after hdr.network is usually 0, in which case
+		 * the hdr.network byte is an NDIS network type value - 1.
+		 */
+		network_type = hdr.network + 1;
+		break;
+
+	case 2:
+		/*
+		 * However, in some Ethernet captures, it's 2, and the
+		 * hdr.network byte is 1 rather than 0.  We assume
+		 * that if there's a byte after hdr.network with the value
+		 * 2, the hdr.network byte is an NDIS network type, rather
+		 * than an NDIS network type - 1.
+		 */
+		network_type = hdr.network;
+		break;
+
+	default:
+		*err = WTAP_ERR_UNSUPPORTED;
+		*err_info = g_strdup_printf("netxray: the byte after the network type has the value %u, which I don't understand",
+		    hdr.xxz[0]);
 		return -1;
 	}
 
-	if (hdr.network == 3) {
+	if (network_type >= NUM_NETXRAY_ENCAPS
+	    || netxray_encap[network_type] == WTAP_ENCAP_UNKNOWN) {
+		*err = WTAP_ERR_UNSUPPORTED_ENCAP;
+		*err_info = g_strdup_printf("netxray: network type %u (%u) unknown or unsupported",
+		    network_type, hdr.xxz[0]);
+		return -1;
+	}
+
+	/*
+	 * Figure out the time stamp units and start time stamp.
+	 */
+	start_timestamp = (double)pletohl(&hdr.timelo)
+	    + (double)pletohl(&hdr.timehi)*4294967296.0;
+	switch (file_type) {
+
+	case WTAP_FILE_NETXRAY_OLD:
+		timeunit = 1000.0;
+		break;
+
+	case WTAP_FILE_NETXRAY_1_0:
+		timeunit = 1000.0;
+		break;
+
+	case WTAP_FILE_NETXRAY_1_1:
+		timeunit = 1000000.0;
+		break;
+
+	case WTAP_FILE_NETXRAY_2_00x:
+		/*
+		 * It appears that, at least for Ethernet
+		 * captures, if hdr.xxb[20] is 2, that indicates
+		 * that it's a gigabit Ethernet capture, possibly
+		 * from a special whizzo gigabit pod, and also
+		 * indicates that the time stamps have some
+		 * higher resolution than in other captures,
+		 * possibly thanks to a high-resolution
+		 * timer on the pod.
+		 *
+		 * It also appears that the time units might differ
+		 * for gigabit pod captures between version 002.001
+		 * and 002.002.
+		 */
+		if (network_type == 1 && hdr.xxb[20] == CAPTYPE_GIGPOD) {
+			if (version_minor == 1) {
+				/*
+				 * It appears that the time units for
+				 * these captures are nanoseconds, unless
+				 * hdr.timeunit is 2, in which case it's
+				 * 1/31250000.0 of a second.
+				 */
+				if (hdr.timeunit == 2)
+					timeunit = 31250000.0;
+				else
+					timeunit = 1e9;
+			} else {
+				/*
+				 * These just seem to be 1000 times
+				 * the regular time stamp units, unless
+				 * hdr.timeunit is 2, in which case it's
+				 * 1/1125000 of a second.
+				 *
+				 * In addition, the start timestamp
+				 * appears to be 0.
+				 */
+				if (hdr.timeunit == 2)
+					timeunit = 1250000.0;
+				else
+					timeunit = TpS[hdr.timeunit]*1000.0;
+				start_timestamp = 0.0;
+			}
+		} else {
+			if (hdr.timeunit > NUM_NETXRAY_TIMEUNITS) {
+				*err = WTAP_ERR_UNSUPPORTED;
+				*err_info = g_strdup_printf("netxray: Unknown timeunit %u",
+				    hdr.timeunit);
+				return -1;
+			}
+			timeunit = TpS[hdr.timeunit];
+		}
+		break;
+
+	default:
+		g_assert_not_reached();
+		timeunit = 0.0;
+	}
+	start_timestamp = start_timestamp/timeunit;
+
+	if (network_type == 4) {
 		/*
 		 * In version 0 and 1, we assume, for now, that all
 		 * WAN captures have frames that look like Ethernet
@@ -337,9 +444,9 @@ int netxray_open(wtap *wth, int *err)
 					break;
 
 				default:
-					g_message("netxray: WAN HDLC capture subsubtype 0x%02x unknown or unsupported",
-					   hdr.xxb[28]);
 					*err = WTAP_ERR_UNSUPPORTED_ENCAP;
+					*err_info = g_strdup_printf("netxray: WAN HDLC capture subsubtype 0x%02x unknown or unsupported",
+					   hdr.xxb[28]);
 					return -1;
 				}
 				break;
@@ -352,15 +459,15 @@ int netxray_open(wtap *wth, int *err)
 				break;
 
 			default:
-				g_message("netxray: WAN capture subtype 0x%02x unknown or unsupported",
-				   hdr.xxb[20]);
 				*err = WTAP_ERR_UNSUPPORTED_ENCAP;
+				*err_info = g_strdup_printf("netxray: WAN capture subtype 0x%02x unknown or unsupported",
+				   hdr.xxb[20]);
 				return -1;
 			}
 		} else
 			file_encap = WTAP_ENCAP_ETHERNET;
 	} else
-		file_encap = netxray_encap[hdr.network];
+		file_encap = netxray_encap[network_type];
 
 	/* This is a netxray file */
 	wth->file_type = file_type;
@@ -372,38 +479,47 @@ int netxray_open(wtap *wth, int *err)
 	wth->snapshot_length = 0;	/* not available in header */
 	wth->capture.netxray->start_time = pletohl(&hdr.start_time);
 	wth->capture.netxray->timeunit = timeunit;
-	t = (double)pletohl(&hdr.timelo)
-	    + (double)pletohl(&hdr.timehi)*4294967296.0;
-	t = t/timeunit;
-	wth->capture.netxray->start_timestamp = t;
+	wth->capture.netxray->start_timestamp = start_timestamp;
 	wth->capture.netxray->version_major = version_major;
 
 	/*
-	 * End-of-packet padding.  802.11 captures appear to have four
-	 * bytes of it, as do some ISDN captures; those 4 bytes don't
-	 * show up as frame data.
-	 *
-	 * We've seen what appears to be an FCS at the end of some frames
-	 * in some Ethernet captures, but this stuff appears to be just
-	 * padding - Sniffers don't show it, and it doesn't have values
-	 * that look like FCS values, so it looks like padding.
+	 * If frames have an extra 4 bytes of stuff at the end, is
+	 * it an FCS, or just junk?
 	 */
-	wth->capture.netxray->padding = 0;
+	wth->capture.netxray->fcs_valid = FALSE;
 	switch (file_encap) {
 
+	case WTAP_ENCAP_ETHERNET:
 	case WTAP_ENCAP_IEEE_802_11_WITH_RADIO:
-		wth->capture.netxray->padding = 4;
-		break;
-
 	case WTAP_ENCAP_ISDN:
 		/*
-		 * The only captures we've seen with padding are PRI
-		 * captures; until we see PRI captures with no padding,
-		 * or BRI captures with padding, we assume that PRI
-		 * captures have padding and BRI captures don't.
+		 * It appears that, in at least some version 2 Ethernet
+		 * captures, for frames that have 0xff in hdr_2_x.xxx[2]
+		 * and hdr_2_x.xxx[3] in the per-packet header:
+		 *
+		 *	if, in the file header, hdr.xxb[13] is 0x34 and
+		 *	hdr.xxb[14] is 0x12, the frames have an FCS at
+		 *	the end;
+		 *
+		 *	otherwise, they have 4 bytes of junk at the end.
+		 *
+		 * For now, we assume that to be true for 802.11 captures
+		 * as well; it appears to be the case for at least one
+		 * such capture.
+		 *
+		 * For ISDN captures, it appears, at least in some
+		 * captures, to be similar, although I haven't yet
+		 * checked whether it's a valid FCS.
+		 *
+		 * XXX - should we do this for all encapsulation types?
+		 *
+		 * XXX - is that actually a 2-byte little-endian 0x1234?
+		 * What does that field signify?
 		 */
-		if (isdn_type == 1 || isdn_type == 2)
-			wth->capture.netxray->padding = 4;
+		if (version_major == 2) {
+			if (hdr.xxb[13] == 0x34 && hdr.xxb[14] == 0x12)
+				wth->capture.netxray->fcs_valid = TRUE;
+		}
 		break;
 	}
 
@@ -430,13 +546,15 @@ int netxray_open(wtap *wth, int *err)
 }
 
 /* Read the next packet */
-static gboolean netxray_read(wtap *wth, int *err, long *data_offset)
+static gboolean netxray_read(wtap *wth, int *err, gchar **err_info _U_,
+    long *data_offset)
 {
 	guint32	packet_size;
 	union netxrayrec_hdr hdr;
 	int	hdr_size;
 	double	t;
 	guint8  *pd;
+	guint	padding;
 
 reread:
 	/* Have we reached the end of the packet data? */
@@ -496,8 +614,8 @@ reread:
 	/*
 	 * Set the pseudo-header.
 	 */
-	netxray_set_pseudo_header(wth, pd, packet_size, &wth->pseudo_header,
-	    &hdr);
+	padding = netxray_set_pseudo_header(wth, pd, packet_size,
+	    &wth->pseudo_header, &hdr);
 
 	if (wth->capture.netxray->version_major == 0) {
 		t = (double)pletohl(&hdr.old_hdr.timelo)
@@ -511,7 +629,7 @@ reread:
 		 * We subtract the padding from the packet size, so our caller
 		 * doesn't see it.
 		 */
-		wth->phdr.caplen = packet_size - wth->capture.netxray->padding;
+		wth->phdr.caplen = packet_size - padding;
 		wth->phdr.len = wth->phdr.caplen;
 	} else {
 		t = (double)pletohl(&hdr.hdr_1_x.timelo)
@@ -525,17 +643,17 @@ reread:
 		 * We subtract the padding from the packet size, so our caller
 		 * doesn't see it.
 		 */
-		wth->phdr.caplen = packet_size - wth->capture.netxray->padding;
-		wth->phdr.len = pletohs(&hdr.hdr_1_x.orig_len) - wth->capture.netxray->padding;
+		wth->phdr.caplen = packet_size - padding;
+		wth->phdr.len = pletohs(&hdr.hdr_1_x.orig_len) - padding;
 	}
-	wth->phdr.pkt_encap = wth->file_encap;
 
 	return TRUE;
 }
 
 static gboolean
 netxray_seek_read(wtap *wth, long seek_off,
-    union wtap_pseudo_header *pseudo_header, guchar *pd, int length, int *err)
+    union wtap_pseudo_header *pseudo_header, guchar *pd, int length,
+    int *err, gchar **err_info _U_)
 {
 	union netxrayrec_hdr hdr;
 	gboolean ret;
@@ -561,6 +679,7 @@ netxray_seek_read(wtap *wth, long seek_off,
 	ret = netxray_read_rec_data(wth->random_fh, pd, length, err);
 	if (!ret)
 		return FALSE;
+
 	/*
 	 * Set the pseudo-header.
 	 */
@@ -610,17 +729,103 @@ netxray_read_rec_header(wtap *wth, FILE_T fh, union netxrayrec_hdr *hdr,
 	return hdr_size;
 }
 
-static void
+static guint
 netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
     union wtap_pseudo_header *pseudo_header, union netxrayrec_hdr *hdr)
 {
+	guint padding = 0;
+
 	/*
-	 * If this is 802.11, ISDN, X.25, or ATM, set the pseudo-header.
+	 * If this is Ethernet, 802.11, ISDN, X.25, or ATM, set the
+	 * pseudo-header.
 	 */
-	if (wth->capture.netxray->version_major == 2) {
+	switch (wth->capture.netxray->version_major) {
+
+	case 1:
 		switch (wth->file_encap) {
 
+		case WTAP_ENCAP_ETHERNET:
+			/*
+			 * XXX - if hdr->hdr_1_x.xxx[15] is 1
+			 * the frame appears not to have any extra
+			 * stuff at the end, but if it's 0,
+			 * there appears to be 4 bytes of stuff
+			 * at the end, but it's not an FCS.
+			 *
+			 * Or is that just the low-order bit?
+			 *
+			 * For now, we just say "no FCS".
+			 */
+			pseudo_header->eth.fcs_len = 0;
+			break;
+		}
+		break;
+
+	case 2:
+		switch (wth->file_encap) {
+
+		case WTAP_ENCAP_ETHERNET:
+			/*
+			 * It appears, at least with version 2 captures,
+			 * that we have 4 bytes of stuff (which might be
+			 * a valid FCS or might be junk) at the end of
+			 * the packet if hdr->hdr_2_x.xxx[2] and
+			 * hdr->hdr_2_x.xxx[3] are 0xff, and we don't if
+			 * they don't.
+			 *
+			 * It also appears that if the low-order bit of
+			 * hdr->hdr_2_x.xxx[8] is set, the packet has a
+			 * bad FCS.
+			 */
+			if (hdr->hdr_2_x.xxx[2] == 0xff &&
+			    hdr->hdr_2_x.xxx[3] == 0xff) {
+				/*
+				 * We have 4 bytes of stuff at the
+				 * end of the frame - FCS, or junk?
+				 */
+			    	if (wth->capture.netxray->fcs_valid) {
+					/*
+					 * FCS.
+					 */
+					pseudo_header->eth.fcs_len = 4;
+				} else {
+					/*
+					 * Junk.
+					 */
+					padding = 4;
+				}
+			} else
+				pseudo_header->eth.fcs_len = 0;
+			break;
+
 		case WTAP_ENCAP_IEEE_802_11_WITH_RADIO:
+			/*
+			 * It appears, in one 802.11 capture, that
+			 * we have 4 bytes of junk at the ends of
+			 * frames in which hdr->hdr_2_x.xxx[2] and
+			 * hdr->hdr_2_x.xxx[3] are 0xff; we assume
+			 * for now that it works like Ethernet.
+			 */
+			if (hdr->hdr_2_x.xxx[2] == 0xff &&
+			    hdr->hdr_2_x.xxx[3] == 0xff) {
+				/*
+				 * We have 4 bytes of stuff at the
+				 * end of the frame - FCS, or junk?
+				 */
+			    	if (wth->capture.netxray->fcs_valid) {
+					/*
+					 * FCS.
+					 */
+					pseudo_header->ieee_802_11.fcs_len = 4;
+				} else {
+					/*
+					 * Junk.
+					 */
+					padding = 4;
+				}
+			} else
+				pseudo_header->ieee_802_11.fcs_len = 0;
+
 			pseudo_header->ieee_802_11.channel =
 			    hdr->hdr_2_x.xxx[12];
 			pseudo_header->ieee_802_11.data_rate =
@@ -673,6 +878,28 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 					pseudo_header->isdn.channel -= 1;
 				break;
 			}
+
+			/*
+			 * It appears, at least with version 2 captures,
+			 * that we have 4 bytes of stuff (which might be
+			 * a valid FCS or might be junk) at the end of
+			 * the packet if hdr->hdr_2_x.xxx[2] and
+			 * hdr->hdr_2_x.xxx[3] are 0xff, and we don't if
+			 * they don't.
+			 *
+			 * XXX - does the low-order bit of hdr->hdr_2_x.xxx[8]
+			 * indicate a bad FCS, as is the case with
+			 * Ethernet?
+			 */
+			if (hdr->hdr_2_x.xxx[2] == 0xff &&
+			    hdr->hdr_2_x.xxx[3] == 0xff) {
+				/*
+				 * FCS, or junk, at the end.
+				 * XXX - is it an FCS if "fcs_valid" is
+				 * true?
+				 */
+				padding = 4;
+			}
 			break;
 
 		case WTAP_ENCAP_LAPB:
@@ -778,7 +1005,9 @@ netxray_set_pseudo_header(wtap *wth, const guint8 *pd, int len,
 			}
 			break;
 		}
+		break;
 	}
+	return padding;
 }
 
 static gboolean
@@ -970,7 +1199,7 @@ static gboolean netxray_dump_close_1_1(wtap_dumper *wdh, int *err)
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
-    file_hdr.network = htoles(wtap_encap_to_netxray_1_1_encap(wdh->encap));
+    file_hdr.network = (guint8) htoles(wtap_encap_to_netxray_1_1_encap(wdh->encap));
     file_hdr.timelo = htolel(0);
     file_hdr.timehi = htolel(0);
 
@@ -1177,7 +1406,7 @@ static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err)
     file_hdr.nframes = htolel(netxray->nframes);
     file_hdr.start_offset = htolel(CAPTUREFILE_HEADER_SIZE);
     file_hdr.end_offset = htolel(filelen);
-    file_hdr.network = htoles(wtap_encap_to_netxray_2_0_encap(wdh->encap));
+    file_hdr.network = (guint8) htoles(wtap_encap_to_netxray_2_0_encap(wdh->encap));
     file_hdr.timelo = htolel(0);
     file_hdr.timehi = htolel(0);
     switch (wdh->encap) {
@@ -1198,6 +1427,10 @@ static gboolean netxray_dump_close_2_0(wtap_dumper *wdh, int *err)
     case WTAP_ENCAP_SDLC:
 	file_hdr.xxb[20] = CAPTYPE_SDLC;
 	break;
+
+    default:
+	file_hdr.xxb[20] = CAPTYPE_NDIS;
+	break;
     }
 
     memset(hdr_buf, '\0', sizeof hdr_buf);
