@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c,v 1.92 2004/03/23 01:02:41 guy Exp $
+ * $Id: wtap.c,v 1.94 2004/06/07 20:28:51 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -20,12 +20,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include <string.h>
-#include <errno.h>
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <string.h>
+#include <errno.h>
+
 #include "wtap-int.h"
 #include "file_wrappers.h"
 #include "buffer.h"
@@ -247,6 +248,9 @@ static const struct encap_type_info {
 
 	/* WTAP_ENCAP_APPLE_IP_OVER_IEEE1394 */
 	{ "Apple IP-over-IEEE 1394", "ap1394" },
+
+	/* WTAP_ENCAP_BACNET_MS_TP */
+	{ "BACnet MS/TP", "bacnet-ms-tp" },
 };
 
 /* Name that should be somewhat descriptive. */
